# (C) Copyright 2005- ECMWF.

# TEMPLATE 6.1, Individual ensemble forecast or analysis

# Process type and identifier
include "grib3/template.component.6.0.def";

# Ensemble size
include "grib3/template.component.6.1.def";

# Ensemble member
include "grib3/template.component.6.2.def";
