# (C) Copyright 2005- ECMWF.

# Section 10 - Data Section

position offsetSection10;
section_length[4] section10Length;
meta section10Pointer section_pointer(offsetSection10,section10Length,10);
unsigned[1] numberOfSection = 10: read_only;

# Data in a format described by data template 10.X,
# where X is the Data Template number given in bytes 12-13 of Section 8
position offsetBeforeData;
template dataValues "grib3/template.10.[dataRepresentationTemplateNumber:i].def";

meta changeDecimalPrecision decimal_precision(bitsPerValue,decimalScaleFactor,changingPrecision,values): edition_specific;
meta decimalPrecision decimal_precision(bitsPerValue,decimalScaleFactor,changingPrecision): edition_specific;
alias setDecimalPrecision=changeDecimalPrecision;

meta setBitsPerValue bits_per_value(values,bitsPerValue)  : edition_specific;

meta getNumberOfValues size(values) : edition_specific,dump ;

meta scaleValuesBy scale_values(values,missingValue)  : edition_specific;
meta offsetValuesBy offset_values(values,missingValue)  : edition_specific;

position offsetAfterData;
meta md5Section10 md5(offsetSection10, section10Length);
alias md5DataSection = md5Section10;
