# (C) Copyright 2005- ECMWF.

# Predefined grid 21

constant Ni = 37;
constant Nj = 37;

constant longitudeOfFirstGridPoint = 0;
constant longitudeOfLastGridPoint = 180000;

constant latitudeOfFirstGridPoint = 0;
constant latitudeOfLastGridPoint = 90000;

constant iDirectionIncrement = 5000;
constant jDirectionIncrement = 2500;

constant numberOfDataPoints=1369;
constant numberOfValues=1333 ;

