    rename(g,CCCC);
		modify CCCC : dump;
    group g : dump ; non_alpha na;
    if (is_in_dict(CCCC,"metar/nsd_cccc.txt")) {
        constant coordinatesPresent=one : dump;
        latitudeSexagesimal=dict_search("metar/nsd_cccc.txt",CCCC,7) : string_type,hidden ;
        longitudeSexagesimal=dict_search("metar/nsd_cccc.txt",CCCC,8) : string_type,hidden ;
				latitude = sexagesimal2decimal(latitudeSexagesimal) : dump;
				longitude = sexagesimal2decimal(longitudeSexagesimal) : dump;
        elevation=dict_search("metar/nsd_cccc.txt",CCCC,11) : dump ;
        name=dict_search("metar/nsd_cccc.txt",CCCC,3) : string_type,dump ;
        country=dict_search("metar/nsd_cccc.txt",CCCC,5) : string_type,dump ;
    } else {
        constant coordinatesPresent=zero : dump;
    }
