alias mars.anoffset = offsetToEndOf4DvarWindow;

transient maxobscutoff = max(0, hoursAfterDataCutoff - offsetToEndOf4DvarWindow) : hidden;

meta obscutoff time(maxobscutoff,minutesAfterDataCutoff,zero) : no_copy,read_only;
alias mars.obscutoff = obscutoff;
