# Automatically generated by ./create_defs.grib2.lte33.pl, do not edit
#Mean discharge in the last 6 hours
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 6 ;
	}
#Mean discharge in the last 24 hours
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Total precipitation in the last 6 hours
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 lengthOfTimeRange = 6 ;
	}
#Total precipitation in the last 24 hours
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 lengthOfTimeRange = 24 ;
}
