# CARRA/CERRA local

codetable[2] suiteName "grib2/crra_ci_suiteName.table" : dump;
alias crraSuiteID = suiteName;

if (centre isnot "ecmf") {
    codetable[2] marsClass  "mars/class.table" = "rr" : dump,string_type,lowercase;
    unalias mars.class;
    alias mars.class = marsClass;
}

ksec1expver[4] experimentVersionNumber = "0002" : dump;
alias marsExpver = experimentVersionNumber;
