# (C) Copyright 2005- ECMWF.

# GRID DEFINITION Stretched and rotated spherical harmonic coefficients

# grib 1 -> 2
constant gridDefinitionTemplateNumber     = 53;

template commonBlock "grib1/grid_definition_spherical_harmonics.def";

# Rotation parameters
include "grib1/grid_rotation.def"

# Stretching parameters
include "grib1/grid_stretching.def"
