# Automatically generated by ./create_defs.grib1.pl, do not edit
#Total convective Precipitation
'kg m**-2' = {
	 table2Version = 1 ;
	 indicatorOfParameter = 156 ;
	 stepType = "accum" ;
	}
#Total large scale precipitation
'kg m**-2' = {
	 table2Version = 1 ;
	 indicatorOfParameter = 157 ;
	 stepType = "accum" ;
	}
#Convective Available Potential Energy instantaneous
'm**2 s**-2' = {
	 table2Version = 1 ;
	 indicatorOfParameter = 160 ;
}
