# Automatically generated by ./create_def.pl, do not edit
#Sea-ice cover
'ci' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	}
#Snow density
'rsn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	}
#Sea surface temperature
'sst' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil type
'slt' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	}
#10 metre wind gust since previous post-processing
'fg10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 typeOfStatisticalProcessing = 2 ;
	 is_uerra = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	}
#Specific rain water content
'crwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	}
#Specific snow water content
'cswc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	}
#Eta-coordinate vertical velocity
'etadot' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	}
#Surface solar radiation downwards
'ssrd' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface thermal radiation downwards
'strd' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Eastward turbulent surface stress
'ewss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 38 ;
	}
#Northward turbulent surface stress
'nsss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 37 ;
	}
#Maximum temperature at 2 metres since previous post-processing
'mx2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	 is_uerra = 1 ;
	}
#Minimum temperature at 2 metres since previous post-processing
'mn2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	 is_uerra = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	}
#Ozone mass mixing ratio
'o3' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	}
#Specific cloud liquid water content
'clwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	}
#Specific cloud ice water content
'ciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	}
#Fraction of cloud cover
'cc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	}
#large scale precipitation
'lsp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snow depth
'sde' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	}
#Low cloud cover
'lcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	}
#Medium cloud cover
'mcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	}
#High cloud cover
'hcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	}
#10 metre wind gust in the last 3 hours
'fg310' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 is_uerra = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 typeOfStatisticalProcessing = 2 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 lengthOfTimeRange = 3 ;
	}
#Relative humidity with respect to water
'rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	}
#Relative humidity with respect to ice
'rhi' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 94 ;
	}
#Snow albedo
'asn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	}
#Fraction of stratiform precipitation cover
'fspc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	}
#Fraction of convective precipitation cover
'fcpc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 37 ;
	}
#Height of convective cloud top
'hcct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	}
#Unbalanced component of specific humidity
'ucq' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 118 ;
	}
#Unbalanced component of specific cloud liquid water content
'ucclwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 119 ;
	}
#Unbalanced component of specific cloud ice water content
'ucciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 120 ;
	}
#Soil moisture top 20 cm
'sm20' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfFirstFixedSurface = 106 ;
	}
#Soil moisture top 100 cm
'sm100' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 scaledValueOfSecondFixedSurface = 10 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfFirstFixedSurface = 106 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 typeOfSecondFixedSurface = 106 ;
	}
#Soil temperature top 20 cm
'st20' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfFirstFixedSurface = 106 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Soil temperature top 100 cm
'st100' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfFirstFixedSurface = 106 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfSecondFixedSurface = 10 ;
	}
#Convective precipitation
'cp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Water runoff and drainage
'ro' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	}
#Mean temperature tendency due to short-wave radiation
'mttswr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean temperature tendency due to long-wave radiation
'mttlwr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean temperature tendency due to short-wave radiation, clear sky
'mttswrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean temperature tendency due to long-wave radiation, clear sky
'mttlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean temperature tendency due to parametrisations
'mttpm' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean specific humidity tendency due to parametrisations
'mqtpm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean eastward wind tendency due to parametrisations
'mutpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean northward wind tendency due to parametrisations
'mvtpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean updraught mass flux
'mumf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean downdraught mass flux
'mdmf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean updraught detrainment rate
'mudr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean downdraught detrainment rate
'mddr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean total precipitation flux
'mtpf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean turbulent diffusion coefficient for heat
'mtdch' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Cross sectional area of flow in channel
'chcross' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	}
#Side flow into river channel
'chside' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Discharge from rivers or streams
'dis' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#River storage of water
'rivsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Floodplain storage of water
'fldsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Water fraction
'fldfrc' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#Days since last observation
'dslr' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 3 ;
	}
#Frost index
'frost' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 24 ;
	}
#Depth of water on soil surface
'woss' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Upstream accumulated precipitation
'tpups' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Upstream accumulated snow melt
'smups' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Snow depth at elevation bands
'sd_elev' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	}
#Groundwater upper storage
'gwus' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Groundwater lower storage
'gwls' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Surface air relative humidity
'r2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Apparent temperature
'aptmp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Haines Index
'hindex' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 2 ;
	}
#Cloud cover
'ccl' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 22 ;
	}
#Evaporation rate
'evarate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	}
#Evaporation
'eva' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#10 metre wind direction
'wdir10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	}
#Direct short wave radiation flux
'dirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	}
#Diffuse short wave radiation flux
'difswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	}
#Time-integrated surface direct short wave radiation flux
'tidirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil temperature
'sot' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	}
#Downward short-wave radiation flux, clear sky
'dswrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	}
#Upward short-wave radiation flux, clear sky
'uswrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 53 ;
	}
#Downward long-wave radiation flux, clear sky
'dlwrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	}
#Soil heat flux
'sohf' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 26 ;
	}
#Percolation rate
'percr' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Soil depth
'sod' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	}
#Accumulated surface downward short-wave radiation flux, clear sky
'adswrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Accumulated surface upward short-wave radiation flux, clear sky
'auswrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated surface downward long-wave radiation flux, clear sky
'adlwrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Percolation
'perc' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 177 ;
	}
#Cloudy brightness temperature
'clbt' = {
         discipline = 3 ;
         parameterCategory = 1 ;
         parameterNumber = 14 ;
        }
#Clear-sky brightness temperature
'csbt' = {
         discipline = 3 ;
         parameterCategory = 1 ;
         parameterNumber = 15 ;
        }
#Scaled radiance
'p260530' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Scaled albedo
'p260531' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Scaled brightness temperature
'p260532' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Scaled precipitable water
'p260533' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Scaled lifted index
'p260534' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Scaled cloud top pressure
'p260535' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Scaled skin temperature
'p260536' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Cloud mask
'p260537' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Pixel scene type
'p260538' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Fire detection indicator
'p260539' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Cloudy radiance (with respect to wave number)
'p260550' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 16 ;
	}
#Clear-sky radiance (with respect to wave number)
'p260551' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 17 ;
	}
#Wind speed
'p260552' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	}
#Aerosol optical thickness at 0.635 um
'p260553' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 20 ;
	}
#Aerosol optical thickness at 0.810 um
'p260554' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 21 ;
	}
#Aerosol optical thickness at 1.640 um
'p260555' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 22 ;
	}
#Angstrom coefficient
'p260556' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 23 ;
	}
#Virtual temperature
'vtmp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Virtual potential temperature
'vptmp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Pseudo-adiabatic potential temperature
'papt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Wind direction
'wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	}
#Significant height of combined wind waves and swell
'swh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Mean wave direction
'mwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Mean wave period
'mwp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Surface runoff
'sro' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	}
#Total precipitation of at least 10 mm
'tpg10' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 probabilityType = 3 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 productDefinitionTemplateNumber = 9 ;
	}
#Total precipitation of at least 20 mm
'tpg20' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 productDefinitionTemplateNumber = 9 ;
	 scaleFactorOfLowerLimit = 0 ;
	 scaledValueOfLowerLimit = 20 ;
	 typeOfStatisticalProcessing = 1 ;
	 probabilityType = 3 ;
	}
#Stream function
'strf' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	}
#Velocity potential
'vp' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	}
#Potential temperature
'pt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Wind speed
'ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	}
#Pressure
'pres' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	}
#Convective available potential energy
'cape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Potential vorticity
'pv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	}
#Maximum temperature at 2 metres in the last 6 hours
'mx2t6' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 is_uerra = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 lengthOfTimeRange = 6 ;
	}
#Minimum temperature at 2 metres in the last 6 hours
'mn2t6' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 is_uerra = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfStatisticalProcessing = 3 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 lengthOfTimeRange = 6 ;
	}
#Geopotential
'z' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Temperature
't' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#U component of wind
'u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#V component of wind
'v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	}
#Specific humidity
'q' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Surface pressure
'sp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Vertical velocity
'w' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	}
#Total column water
'tcw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Vorticity (relative)
'vo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	}
#Boundary layer dissipation
'bld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	}
#Surface sensible heat flux
'sshf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface latent heat flux
'slhf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Mean sea level pressure
'msl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	}
#Divergence
'd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	}
#Geopotential Height
'gh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Relative humidity
'r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#10 metre U wind component
'u10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	}
#10 metre V wind component
'v10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	}
#2 metre temperature
't2m' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	}
#2 metre dewpoint temperature
'd2m' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	}
#Land-sea mask
'lsm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness
'sr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Surface net solar radiation
'ssr' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net thermal radiation
'str' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Top net thermal radiation
'ttr' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Sunshine duration
'sund' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Brightness temperature
'btmp' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 4 ;
	}
#10 metre wind speed
'si10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	}
#Skin temperature
'skt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Latent heat net flux
'lhtfl' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Sensible heat net flux
'shtfl' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Heat index
'heatx' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Wind chill factor
'wcf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Minimum dew point depression
'mindpd' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Snow phase change heat flux
'snohf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Vapor pressure
'vapp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Large scale precipitation (non-convective)
'ncpcp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Snowfall rate water equivalent
'srweq' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 12 ;
	}
#Convective snow
'snoc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 14 ;
	}
#Large scale snow
'snol' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 15 ;
	}
#Snow age
'snoag' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 17 ;
	}
#Absolute humidity
'absh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 18 ;
	}
#Precipitation type
'ptype' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	}
#Integrated liquid water
'iliqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 20 ;
	}
#Condensate
'tcond' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 21 ;
	}
#Cloud mixing ratio
'clwmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 22 ;
	}
#Ice water mixing ratio
'icmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 23 ;
	}
#Rain mixing ratio
'rwmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	}
#Snow mixing ratio
'snmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 25 ;
	}
#Horizontal moisture convergence
'mconv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 26 ;
	}
#Maximum relative humidity
'maxrh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 27 ;
	}
#Maximum absolute humidity
'maxah' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 28 ;
	}
#Total snowfall
'asnow' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 29 ;
	}
#Precipitable water category
'pwcat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 30 ;
	}
#Hail
'hail' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 31 ;
	}
#Graupel (snow pellets)
'grle' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 32 ;
	}
#Categorical rain
'crain' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 33 ;
	}
#Categorical freezing rain
'cfrzr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 34 ;
	}
#Categorical ice pellets
'cicep' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 35 ;
	}
#Categorical snow
'csnow' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 36 ;
	}
#Convective precipitation rate
'cprat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	}
#Horizontal moisture divergence
'mdiv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 38 ;
	}
#Percent frozen precipitation
'cpofp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 39 ;
	}
#Potential evaporation
'pevap' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 40 ;
	}
#Potential evaporation rate
'pevpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 41 ;
	}
#Snow cover
'snowc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 42 ;
	}
#Rain fraction of total cloud water
'frain' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 43 ;
	}
#Rime factor
'rime' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 44 ;
	}
#Total column integrated rain
'tcolr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	}
#Total column integrated snow
'tcols' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	}
#Large scale water precipitation (non-convective)
'lswp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 47 ;
	}
#Convective water precipitation
'cwp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 48 ;
	}
#Total water precipitation
'twatp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 49 ;
	}
#Total snow precipitation
'tsnowp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 50 ;
	}
#Total column water (Vertically integrated total water (vapour + cloud water/ice))
'tcwat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	}
#Total precipitation rate
'tprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	}
#Total snowfall rate water equivalent
'tsrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	}
#Large scale precipitation rate
'lsprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	}
#Convective snowfall rate water equivalent
'csrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	}
#Large scale snowfall rate water equivalent
'lssrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	}
#Total snowfall rate
'tsrate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 57 ;
	}
#Convective snowfall rate
'csrate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 58 ;
	}
#Large scale snowfall rate
'lssrate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 59 ;
	}
#Water equivalent of accumulated snow depth
'sdwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 13 ;
	}
#Total column integrated water vapour
'tciwv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	}
#Rain precipitation rate
'rprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 65 ;
	}
#Snow precipitation rate
'sprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	}
#Freezing rain precipitation rate
'fprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	}
#Ice pellets precipitation rate
'iprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 68 ;
	}
#Momentum flux, u component
'uflx' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	}
#Momentum flux, v component
'vflx' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	}
#Maximum wind speed
'maxgust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	}
#Wind speed (gust)
'gust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	}
#u-component of wind (gust)
'ugust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	}
#v-component of wind (gust)
'vgust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	}
#Vertical speed shear
'vwsh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	}
#Horizontal momentum flux
'mflx' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 26 ;
	}
#U-component storm motion
'ustm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 27 ;
	}
#V-component storm motion
'vstm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 28 ;
	}
#Drag coefficient
'cd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	}
#Frictional velocity
'fricv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	}
#Pressure reduced to MSL
'prmsl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Geometric height
'dist' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Altimeter setting
'alts' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	}
#Thickness
'thick' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	}
#Pressure altitude
'presalt' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	}
#Density altitude
'denalt' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#5-wave geopotential height
'wavh5' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Zonal flux of gravity wave stress
'p260081' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	}
#Meridional flux of gravity wave stress
'p260082' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	}
#Planetary boundary layer height
'hpbl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	}
#5-wave geopotential height anomaly
'p260084' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	}
#Standard deviation of sub-grid scale orography
'sdsgso' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	}
#Net short-wave radiation flux (top of atmosphere)
'nswrt' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 1 ;
	}
#Downward short-wave radiation flux
'dswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	}
#Upward short-wave radiation flux
'uswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 8 ;
	}
#Net short wave radiation flux
'nswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	}
#Photosynthetically active radiation
'photar' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	}
#Net short-wave radiation flux, clear sky
'nswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	}
#Downward UV radiation
'dwuvr' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	}
#UV index (under clear sky)
'uviucs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	}
#UV index 
'uvi' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 51 ;
	}
#Net long wave radiation flux (surface)
'nlwrs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 0 ;
	}
#Net long wave radiation flux (top of atmosphere)
'nlwrt' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 1 ;
	}
#Downward long-wave radiation flux
'dlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	}
#Upward long-wave radiation flux
'ulwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 4 ;
	}
#Net long wave radiation flux
'nlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	}
#Net long-wave radiation flux, clear sky
'nlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	}
#Cloud Ice
'cice' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	}
#Cloud water
'cwat' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 6 ;
	}
#Cloud amount
'cdca' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 7 ;
	}
#Cloud type
'cdct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 8 ;
	}
#Thunderstorm maximum tops
'tmaxt' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 9 ;
	}
#Thunderstorm coverage
'thunc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 10 ;
	}
#Cloud base
'cdcb' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	}
#Cloud top
'cdct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 12 ;
	}
#Ceiling
'ceil' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	}
#Non-convective cloud cover
'cdlyr' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 14 ;
	}
#Cloud work function
'cwork' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 15 ;
	}
#Convective cloud efficiency
'cuefi' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 16 ;
	}
#Total condensate
'tcond' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 17 ;
	}
#Total column-integrated cloud water
'tcolw' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 18 ;
	}
#Total column-integrated cloud ice
'tcoli' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 19 ;
	}
#Total column-integrated condensate
'tcolc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 20 ;
	}
#Ice fraction of total condensate
'fice' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 21 ;
	}
#Cloud ice mixing ratio
'cdcimr' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 23 ;
	}
#Sunshine
'suns' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	}
#Horizontal extent of cumulonimbus (CB)
'p260120' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 25 ;
	}
#K index
'kx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	}
#KO index
'kox' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 3 ;
	}
#Total totals index
'totalx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	}
#Sweat index
'sx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 5 ;
	}
#Storm relative helicity
'hlcy' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 8 ;
	}
#Energy helicity index
'ehlx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 9 ;
	}
#Surface lifted index
'lftx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 10 ;
	}
#Best (4-layer) lifted index
'lftx4' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 11 ;
	}
#Aerosol type
'aerot' = {
	 discipline = 0 ;
	 parameterCategory = 13 ;
	 parameterNumber = 0 ;
	}
#Total ozone
'tozne' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 0 ;
	}
#Total column integrated ozone
'tcioz' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	}
#Base spectrum width
'bswid' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 0 ;
	}
#Base reflectivity
'bref' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 1 ;
	}
#Base radial velocity
'brvel' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 2 ;
	}
#Vertically-integrated liquid
'veril' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 3 ;
	}
#Layer-maximum base reflectivity
'lmaxbr' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 4 ;
	}
#Precipitation
'prec' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 5 ;
	}
#Air concentration of Caesium 137
'acces' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 0 ;
	}
#Air concentration of Iodine 131
'aciod' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 1 ;
	}
#Air concentration of radioactive pollutant
'acradp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 2 ;
	}
#Ground deposition of Caesium 137
'gdces' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 3 ;
	}
#Ground deposition of Iodine 131
'gdiod' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 4 ;
	}
#Ground deposition of radioactive pollutant
'gdradp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 5 ;
	}
#Time-integrated air concentration of caesium pollutant
'tiaccp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 6 ;
	}
#Time-integrated air concentration of iodine pollutant
'tiacip' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 7 ;
	}
#Time-integrated air concentration of radioactive pollutant
'tiacrp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 8 ;
	}
#Volcanic ash
'volash' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 4 ;
	}
#Icing top
'icit' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 5 ;
	}
#Icing base
'icib' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 6 ;
	}
#Icing
'ici' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 7 ;
	}
#Turbulence top
'turbt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 8 ;
	}
#Turbulence base
'turbb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 9 ;
	}
#Turbulence
'turb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 10 ;
	}
#Turbulent kinetic energy
'tke' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 11 ;
	}
#Planetary boundary layer regime
'pblreg' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 12 ;
	}
#Contrail intensity
'conti' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 13 ;
	}
#Contrail engine type
'contet' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 14 ;
	}
#Contrail top
'contt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 15 ;
	}
#Contrail base
'contb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 16 ;
	}
#Maximum snow albedo
'mxsalb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 17 ;
	}
#Snow free albedo
'snfalb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 18 ;
	}
#Icing
'p260163' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 20 ;
	}
#In-cloud turbulence
'p260164' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 21 ;
	}
#Clear air turbulence (CAT)
'cat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 22 ;
	}
#Supercooled large droplet probability (see Note 4)
'p260166' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 23 ;
	}
#Arbitrary text string
'var190m0' = {
	 discipline = 0 ;
	 parameterCategory = 190 ;
	 parameterNumber = 0 ;
	}
#Seconds prior to initial reference time (defined in Section 1)
'tsec' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 0 ;
	}
#Flash flood guidance (Encoded as an accumulation over a floating subinterval of time between the ref
'ffldg' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Flash flood runoff (Encoded as an accumulation over a floating subinterval of time)
'ffldro' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Remotely sensed snow cover
'rssc' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Elevation of snow covered terrain
'esct' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Snow water equivalent percent of normal
'swepon' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Baseflow-groundwater runoff
'bgrun' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Storm surface runoff
'ssrun' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Conditional percent precipitation amount fractile for an overall period (Encoded as an accumulation)
'cppop' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Percent precipitation in a sub-period of an overall period (Encoded as per cent accumulation over th
'pposp' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Probability of 0.01 inch of precipitation (POP)
'pop' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Land cover (1=land, 0=sea)
'land' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Vegetation
'veg' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Water runoff
'watr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Evapotranspiration
'evapt' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Model terrain height
'mterh' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Land use
'landu' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Volumetric soil moisture content
'soilw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Ground heat flux
'gflux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Moisture availability
'mstav' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Exchange coefficient
'sfexc' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Plant canopy surface water
'cnwat' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Blackadar mixing length scale
'bmixl' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Canopy conductance
'ccond' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Minimal stomatal resistance
'rsmin' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Solar parameter in canopy conductance
'rcs' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	}
#Temperature parameter in canopy conductance
'rct' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	}
#Soil moisture parameter in canopy conductance
'rcsol' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#Humidity parameter in canopy conductance
'rcq' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Column-integrated soil water
'cisoilw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Heat flux
'hflux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Volumetric soil moisture
'vsw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Volumetric wilting point
'vwiltm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Upper layer soil temperature
'uplst' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Upper layer soil moisture
'uplsm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	}
#Lower layer soil moisture
'lowlsm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Bottom layer soil temperature
'botlst' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Liquid volumetric soil moisture (non-frozen)
'soill' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Number of soil layers in root zone
'rlyrs' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Transpiration stress-onset (soil moisture)
'smref' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	}
#Direct evaporation cease (soil moisture)
'smdry' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	}
#Soil porosity
'poros' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	}
#Liquid volumetric soil moisture (non-frozen)
'liqvsm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	}
#Volumetric transpiration stress-onset (soil moisture)
'voltso' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	}
#Transpiration stress-onset (soil moisture)
'transo' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	}
#Volumetric direct evaporation cease (soil moisture)
'voldec' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	}
#Direct evaporation cease (soil moisture)
'direc' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#Soil porosity
'soilp' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Volumetric saturation of soil moisture
'vsosm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	}
#Saturation of soil moisture
'satosm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	}
#Estimated precipitation
'estp' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Instantaneous rain rate
'irrate' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Cloud top height
'ctoph' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Cloud top height quality indicator
'ctophqi' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Estimated u component of wind 
'estu' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Estimated v component of wind
'estv' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Number of pixels used
'npixu' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 6 ;
	}
#Solar zenith angle
'solza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Relative azimuth angle
'raza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 8 ;
	}
#Reflectance in 0.6 micron channel
'rfl06' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Reflectance in 0.8 micron channel
'rfl08' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 10 ;
	}
#Reflectance in 1.6 micron channel
'rfl16' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	}
#Reflectance in 3.9 micron channel
'rfl39' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 12 ;
	}
#Atmospheric divergence
'atmdiv' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 13 ;
	}
#Direction of wind waves
'wvdir' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Primary wave direction
'dirpw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Primary wave mean period
'perpw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Secondary wave mean period
'persw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Current direction
'dirc' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Current speed
'spc' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Geometric vertical velocity
'wz' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 9 ;
	}
#Ice temperature
'ist' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	}
#Deviation of sea level from mean
'dslm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Seconds prior to initial reference time (defined in Section 1)
'tsec' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 0 ;
	}
#Albedo
'al' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	}
#Pressure tendency
'ptend' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	}
#ICAO Standard Atmosphere reference height
'icaht' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Geometrical height
'h' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Standard deviation of height
'hstdv' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	}
#Maximum temperature
'tmax' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Minimum temperature
'tmin' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Dew point temperature
'dpt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Lapse rate
'lapr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Visibility
'vis' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	}
#Radar spectra (1)
'rdsp1' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 6 ;
	}
#Radar spectra (2)
'rdsp2' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 7 ;
	}
#Radar spectra (3)
'rdsp3' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 8 ;
	}
#Parcel lifted index (to 500 hPa)
'pli' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	}
#Temperature anomaly
'ta' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Pressure anomaly
'presa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	}
#Geopotential height anomaly
'gpa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	}
#Wave spectra (1)
'wvsp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Wave spectra (2)
'wvsp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Wave spectra (3)
'wvsp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Montgomery stream Function
'mntsf' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	}
#Sigma coordinate vertical velocity
'sgcvv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 7 ;
	}
#Absolute vorticity
'absv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	}
#Absolute divergence
'absd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 11 ;
	}
#Vertical u-component shear
'vucsh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	}
#Vertical v-component shear
'vvcsh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	}
#U-component of current 
'ucurr' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#V-component of current 
'vcurr' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Precipitable water
'pwat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Saturation deficit
'satd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Precipitation rate
'prate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Thunderstorm probability
'tstm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 2 ;
	}
#Convective precipitation (water)
'acpcp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 10 ;
	}
#Mixed layer depth
'mld' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	}
#Transient thermocline depth
'tthdp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 2 ;
	}
#Main thermocline depth
'mthd' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	}
#Main thermocline anomaly
'mtha' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 1 ;
	}
#Best lifted index (to 500 hPa)
'bli' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 1 ;
	}
#Soil moisture content
'ssw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Salinity
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Density
'den' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	}
#Ice thickness
'icetk' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	}
#Direction of ice drift
'diced' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#Speed of ice drift
'siced' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	}
#U-component of ice drift
'uice' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	}
#V-component of ice drift
'vice' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	}
#Ice growth rate
'iceg' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	}
#Ice divergence
'iced' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 7 ;
	}
#Snow melt
'snom' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 16 ;
	}
#Significant height of wind waves
'shww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Mean period of wind waves
'mpww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Direction of swell waves
'swdir' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Significant height of swell waves
'swell' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Mean period of swell waves
'swper' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Secondary wave direction
'dirsw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Net short-wave radiation flux (surface)
'nswrs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	}
#Global radiation flux
'grad' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Radiance (with respect to wave number)
'lwrad' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 5 ;
	}
#Radiance (with respect to wave length)
'swrad' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 6 ;
	}
#Wind mixing energy
'wmixe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	}
#10 metre Wind gust of at least 15 m/s
'fg10g15' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 productDefinitionTemplateNumber = 9 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 probabilityType = 3 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaleFactorOfLowerLimit = 0 ;
	 scaledValueOfLowerLimit = 15 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#10 metre Wind gust of at least 20 m/s
'fg10g20' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	 productDefinitionTemplateNumber = 9 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 probabilityType = 3 ;
	 scaleFactorOfLowerLimit = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 20 ;
	 typeOfFirstFixedSurface = 103 ;
	}
#Convective inhibition
'cin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Orography
'orog' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil Moisture
'sm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#Soil Moisture
'sm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfSecondFixedSurface = 106 ;
	 typeOfFirstFixedSurface = 106 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 is_tigge = 1 ;
	}
#Soil Temperature
'st' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Soil Temperature
'st' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfFirstFixedSurface = 106 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 is_tigge = 1 ;
	}
#Snow depth water equivalent
'sd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Snow Fall water equivalent
'sf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Total Cloud Cover
'tcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Field capacity
'cap' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Wilting point
'wilt' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfSecondFixedSurface = 106 ;
	 typeOfFirstFixedSurface = 106 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	}
#Total Precipitation
'tp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
}
