# Concept typeOfLevel
'flightLevel' = {typeOfFirstFixedSurface=193; typeOfSecondFixedSurface=255; scaleFactorOfSecondFixedSurface=missing(); scaledValueOfSecondFixedSurface=missing();}
'abstractLevel' = {typeOfFirstFixedSurface=254; scaleFactorOfFirstFixedSurface=missing(); scaledValueOfFirstFixedSurface=missing();
                   typeOfSecondFixedSurface=255; scaleFactorOfSecondFixedSurface=missing(); scaledValueOfSecondFixedSurface=missing();}
'abstractLevel' = {typeOfFirstFixedSurface=254; typeOfSecondFixedSurface=255;}
