# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Random pattern 1 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 101 ;
	}
#Random pattern 2 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 102 ;
	}
#Random pattern 3 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 103 ;
	}
#Random pattern 4 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 104 ;
	}
#Random pattern 5 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 105 ;
	}
#Random pattern 6 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 106 ;
	}
#Random pattern 7 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 107 ;
	}
#Random pattern 8 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 108 ;
	}
#Random pattern 9 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 109 ;
	}
#Random pattern 10 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 110 ;
	}
#Random pattern 11 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 111 ;
	}
#Random pattern 12 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 112 ;
	}
#Random pattern 13 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 113 ;
	}
#Random pattern 14 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 114 ;
	}
#Random pattern 15 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 115 ;
	}
#Random pattern 16 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 116 ;
	}
#Random pattern 17 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 117 ;
	}
#Random pattern 18 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 118 ;
	}
#Random pattern 19 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 119 ;
	}
#Random pattern 20 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 120 ;
	}
#Random pattern 21 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 121 ;
	}
#Random pattern 22 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 122 ;
	}
#Random pattern 23 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 123 ;
	}
#Random pattern 24 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 124 ;
	}
#Random pattern 25 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 125 ;
	}
#Random pattern 26 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 126 ;
	}
#Random pattern 27 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 127 ;
	}
#Random pattern 28 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 128 ;
	}
#Random pattern 29 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 129 ;
	}
#Random pattern 30 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 130 ;
	}
#Random pattern 31 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 131 ;
	}
#Random pattern 32 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 132 ;
	}
#Random pattern 33 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 133 ;
	}
#Random pattern 34 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 134 ;
	}
#Random pattern 35 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 135 ;
	}
#Random pattern 36 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 136 ;
	}
#Random pattern 37 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 137 ;
	}
#Random pattern 38 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 138 ;
	}
#Random pattern 39 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 139 ;
	}
#Random pattern 40 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 140 ;
	}
#Random pattern 41 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 141 ;
	}
#Random pattern 42 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 142 ;
	}
#Random pattern 43 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 143 ;
	}
#Random pattern 44 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 144 ;
	}
#Random pattern 45 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 145 ;
	}
#Random pattern 46 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 146 ;
	}
#Random pattern 47 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 147 ;
	}
#Random pattern 48 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 148 ;
	}
#Random pattern 49 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 149 ;
	}
#Random pattern 50 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 150 ;
	}
#Random pattern 51 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 151 ;
	}
#Random pattern 52 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 152 ;
	}
#Random pattern 53 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 153 ;
	}
#Random pattern 54 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 154 ;
	}
#Random pattern 55 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 155 ;
	}
#Random pattern 56 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 156 ;
	}
#Random pattern 57 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 157 ;
	}
#Random pattern 58 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 158 ;
	}
#Random pattern 59 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 159 ;
	}
#Random pattern 60 for SPP scheme
'dimensionless' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 160 ;
}
