# Automatically generated by ./create_defs.grib2.lte33.pl, do not edit
#U component of wind
'U component of wind' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#V component of wind
'V component of wind' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
}
