# Automatically generated by ./tigge_def.pl, do not edit

   '10fgg25' = { parameter = 131071; }
   '10fgg15' = { parameter = 131070; }
   '10v' = { parameter = 166; }
   '10u' = { parameter = 165; }
   '10u' = { parameter = 49; }
   'ci' = { parameter = 228001; }
   'cap' = { parameter = 228170; }
   'cape' = { parameter = 59; }
   'gh' = { parameter = 156; }
   'lsm' = { parameter = 172; }
   'msl' = { parameter = 151; }
   'orog' = { parameter = 228002; }
   'sd' = { parameter = 228141; }
   'mx2t6' = { parameter = 121; }
   '2d' = { parameter = 168; }
   'pv' = { parameter = 60; }
   'pt' = { parameter = 3; }
   'sf' = { parameter = 228144; }
   'skt' = { parameter = 235; }
   'sm' = { parameter = 228039; }
   'str' = { parameter = 177; }
   'sund' = { parameter = 189; }
   'mn2t6' = { parameter = 122; }
   'q' = { parameter = 133; }
   'ssta' = { parameter = 171034; }
   '2t' = { parameter = 167; }
   'tcw' = { parameter = 136; }
   'slhf' = { parameter = 147; }
   'st' = { parameter = 228139; }
   'sshf' = { parameter = 146; }
   'sp' = { parameter = 134; }
   't' = { parameter = 130; }
   'tcc' = { parameter = 228164; }
   'ssr' = { parameter = 176; }
   'tpg10' = { parameter = 131062; }
   'tpg20' = { parameter = 131063; }
   'ttr' = { parameter = 179; }
   'tp' = { parameter = 228228; }
   'u' = { parameter = 131; }
   'v' = { parameter = 132; }
   'wilt' = { parameter = 228171; }
   'default' = { parameter = 99999; }
