# Automatically generated by ./create_defs.grib2.lte33.pl, do not edit
#Surface runoff
'Surface runoff' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 202 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Sub-surface runoff
'Sub-surface runoff' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 204 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#UV visible albedo for direct radiation (climatological)
'UV visible albedo for direct radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 193 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for diffuse radiation (climatological)
'UV visible albedo for diffuse radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 192 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation (climatological)
'Near IR albedo for direct radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 193 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for diffuse radiation (climatological)
'Near IR albedo for diffuse radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 192 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Snow albedo
'Snow albedo' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 192 ;
	}
#Snow evaporation
'Snow evaporation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 197 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snowmelt
'Snowmelt' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 203 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Specific rain water content
'Specific rain water content' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Specific snow water content
'Specific snow water content' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Eta-coordinate vertical velocity
'Eta-coordinate vertical velocity' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Geopotential
'Geopotential' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Temperature
'Temperature' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Specific humidity
'Specific humidity' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Vertical velocity
'Vertical velocity' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Vorticity (relative)
'Vorticity (relative)' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Snow depth
'Snow depth' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 254 ;
	}
#Large-scale precipitation
'Large-scale precipitation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 196 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective precipitation
'Convective precipitation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 195 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snowfall
'Snowfall' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 198 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Logarithm of surface pressure
'Logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	}
#Logarithm of surface pressure
'Logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Divergence
'Divergence' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Albedo (climatological)
'Albedo (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 193 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Evaporation
'Evaporation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 199 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Low cloud cover
'Low cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 193 ;
	}
#Medium cloud cover
'Medium cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 194 ;
	}
#High cloud cover
'High cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 195 ;
	}
#Skin reservoir content
'Skin reservoir content' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 205 ;
	}
#Ozone mass mixing ratio
'Ozone mass mixing ratio' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Runoff
'Runoff' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 201 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Total column ozone
'Total column ozone' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total precipitation
'Total precipitation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective snowfall
'Convective snowfall' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 201 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Large-scale snowfall
'Large-scale snowfall' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 202 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Forecast albedo
'Forecast albedo' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 193 ;
	}
#Specific cloud liquid water content
'Specific cloud liquid water content' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Specific cloud ice water content
'Specific cloud ice water content' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Fraction of cloud cover
'Fraction of cloud cover' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Temperature difference
'Temperature difference' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 130 ;
	}
#Specific humidity difference
'Specific humidity difference' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 133 ;
	}
#Vorticity (relative) difference
'Vorticity (relative) difference' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 138 ;
	}
#Logarithm of surface pressure difference
'Logarithm of surface pressure difference' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 152 ;
	}
#Divergence difference
'Divergence difference' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 155 ;
	}
#Ozone mass mixing ratio difference
'Ozone mass mixing ratio difference' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 203 ;
	}
#Snow depth
'Snow depth' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#UV visible albedo for direct radiation, isotropic component (climatological)
'UV visible albedo for direct radiation, isotropic component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 195 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, volumetric component (climatological)
'UV visible albedo for direct radiation, volumetric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 196 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, geometric component (climatological)
'UV visible albedo for direct radiation, geometric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 194 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, isotropic component (climatological)
'Near IR albedo for direct radiation, isotropic component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 195 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, volumetric component (climatological)
'Near IR albedo for direct radiation, volumetric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 196 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, geometric component (climatological)
'Near IR albedo for direct radiation, geometric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 194 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Random pattern 1 for sppt
'Random pattern 1 for sppt' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 1 ;
	}
#Random pattern 2 for sppt
'Random pattern 2 for sppt' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 2 ;
	}
#Random pattern 3 for sppt
'Random pattern 3 for sppt' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 3 ;
	}
#Evaporation from the top of canopy
'Evaporation from the top of canopy' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 100 ;
	}
#Evaporation from bare soil
'Evaporation from bare soil' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 101 ;
	}
#Evaporation from open water surfaces excluding oceans
'Evaporation from open water surfaces excluding oceans' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 102 ;
	}
#Evaporation from vegetation transpiration
'Evaporation from vegetation transpiration' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 103 ;
	}
#Accumulated freezing rain
'Accumulated freezing rain' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 194 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Potential evaporation
'Potential evaporation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 200 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward ozone flux
'Time-integrated total column vertically-integrated eastward ozone flux' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward ozone flux
'Time-integrated total column vertically-integrated northward ozone flux' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of ozone flux
'Time-integrated total column vertically-integrated divergence of ozone flux' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated net source of ozone
'Time-integrated total column vertically-integrated net source of ozone' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean carbon dioxide net ecosystem exchange flux
'Time-mean carbon dioxide net ecosystem exchange flux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean carbon dioxide gross primary production flux
'Time-mean carbon dioxide gross primary production flux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean carbon dioxide ecosystem respiration flux
'Time-mean carbon dioxide ecosystem respiration flux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total cloud cover
'Time-mean total cloud cover' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Precipitation type
'Precipitation type' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Snow cover
'Snow cover' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#K index
'K index' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Total totals index
'Total totals index' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Snow thickness over sea ice
'Snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	}
#Sea ice salinity
'Sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	}
#Time-mean snow thickness over sea ice
'Time-mean snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Sea ice salinity
'Time-mean Sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Sea water potential temperature
'Sea water potential temperature' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 129 ;
	}
#Sea water practical salinity
'Sea water practical salinity' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 130 ;
	}
#Upward sea water velocity
'Upward sea water velocity' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 133 ;
	}
#Sea water sigma theta
'Sea water sigma theta' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 138 ;
	}
#Surface downward eastward stress
'Surface downward eastward stress' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 153 ;
	}
#Surface downward northward stress
'Surface downward northward stress' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 154 ;
	}
#Surface geopotential
'Surface geopotential' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 51 ;
	}
#Total column vertically-integrated temperature
'Total column vertically-integrated temperature' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 54 ;
	}
#Total column vertically-integrated cloud liquid water
'Total column vertically-integrated cloud liquid water' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 56 ;
	}
#Total column vertically-integrated cloud frozen water
'Total column vertically-integrated cloud frozen water' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 57 ;
	}
#Total column vertically-integrated ozone
'Total column vertically-integrated ozone' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 58 ;
	}
#Total column vertically-integrated energy conversion
'Total column vertically-integrated energy conversion' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 64 ;
	}
#Total column vertically-integrated eastward mass flux
'Total column vertically-integrated eastward mass flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 65 ;
	}
#Total column vertically-integrated northward mass flux
'Total column vertically-integrated northward mass flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 66 ;
	}
#Total column vertically-integrated eastward kinetic energy flux
'Total column vertically-integrated eastward kinetic energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 67 ;
	}
#Total column vertically-integrated northward kinetic energy flux
'Total column vertically-integrated northward kinetic energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 68 ;
	}
#Total column vertically-integrated eastward geopotential flux
'Total column vertically-integrated eastward geopotential flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 73 ;
	}
#Total column vertically-integrated northward geopotential flux
'Total column vertically-integrated northward geopotential flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 74 ;
	}
#Total column vertically-integrated eastward total energy flux
'Total column vertically-integrated eastward total energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 75 ;
	}
#Total column vertically-integrated northward total energy flux
'Total column vertically-integrated northward total energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 76 ;
	}
#Total column vertically-integrated eastward ozone flux
'Total column vertically-integrated eastward ozone flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 77 ;
	}
#Total column vertically-integrated northward ozone flux
'Total column vertically-integrated northward ozone flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 78 ;
	}
#Total column vertically-integrated divergence of mass flux
'Total column vertically-integrated divergence of mass flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 81 ;
	}
#Total column vertically-integrated divergence of kinetic energy flux
'Total column vertically-integrated divergence of kinetic energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 82 ;
	}
#Total column vertically-integrated divergence of thermal energy flux
'Total column vertically-integrated divergence of thermal energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 83 ;
	}
#Total column vertically-integrated divergence of geopotential flux
'Total column vertically-integrated divergence of geopotential flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 85 ;
	}
#Total column vertically-integrated divergence of total energy flux
'Total column vertically-integrated divergence of total energy flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 86 ;
	}
#Total column vertically-integrated divergence of ozone flux
'Total column vertically-integrated divergence of ozone flux' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 87 ;
	}
#Variance of geopotential
'Variance of geopotential' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 206 ;
	}
#Covariance of geopotential/temperature
'Covariance of geopotential/temperature' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 207 ;
	}
#Variance of temperature
'Variance of temperature' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 208 ;
	}
#Covariance of geopotential/specific humidity
'Covariance of geopotential/specific humidity' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 209 ;
	}
#Covariance of temperature/specific humidity
'Covariance of temperature/specific humidity' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 210 ;
	}
#Variance of specific humidity
'Variance of specific humidity' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 211 ;
	}
#Covariance of u component/geopotential
'Covariance of u component/geopotential' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 212 ;
	}
#Covariance of u component/temperature
'Covariance of u component/temperature' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 213 ;
	}
#Covariance of u component/specific humidity
'Covariance of u component/specific humidity' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 214 ;
	}
#Variance of u component
'Variance of u component' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 215 ;
}
