# (C) Copyright 2005- ECMWF.

# TEMPLATE 4.97, Average, accumulation, extreme values or other statistically processed values of post-processing analysis or forecast at a horizontal level or in a horizontal layer at a local time

include "grib2/templates/template.4.parameter.def"
include "grib2/templates/template.4.postproc.def"
include "grib2/templates/template.4.generating_process.def"

#include "grib2/templates/template.4.forecast_time.def"
#include "grib2/templates/template.4.point_in_time.def"

include "grib2/templates/template.4.horizontal.def"
include "grib2/templates/template.4.localtime_statistical.def"
include "grib2/templates/template.4.localtime.def"
