# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Carbon dioxide mass mixing ratio
'mass_fraction_of_carbon_dioxide_in_air' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 3 ;
	 is_chemical = 1 ;
}
