# (C) Copyright 2005- ECMWF.

# TEMPLATE 3.110, Equatorial azimuthal equidistant projection
include "grib3/template.3.shape_of_the_earth.def";

#  Nx - number of points along X-axis
unsigned[4] numberOfPointsAlongXAxis : dump  ;

alias Nx  = numberOfPointsAlongXAxis;
#  Ny - number of points along Y-axis
unsigned[4] numberOfPointsAlongYAxis : dump  ;

alias Ny =  numberOfPointsAlongYAxis;
#  La1 - latitude of tangency point
# (centre of grid)
signed[4] latitudeOfTangencyPoint  : dump ;

alias La1  = latitudeOfTangencyPoint;
#  Lo1 - longitude of tangency point
unsigned[4] longitudeOfTangencyPoint  : dump ;

alias Lo1  = longitudeOfTangencyPoint;
#  Resolution and component flag
flags[1] resolutionAndComponentFlags 'grib3/tables/[tablesVersion]/4.1.table' : dump ;

#  Dx - X-direction grid length in units of 10 -3 m as measured at the point of the axis
unsigned[4] Dx  : dump ;

#  Dy - Y-direction grid length in units of 10 -3 m as measured at the point of the axis
unsigned[4] Dy  : dump ;

#  Projection centre flag
unsigned[1] projectionCentreFlag  : dump ;

include "grib3/template.4.scanning_mode.def";

