# (C) Copyright 2005- ECMWF.

# TEMPLATE 4.10, Percentile forecasts at a horizontal level or in a horizontal layer in a continuous or non-continuous time interval

# This template was not validated at the time of publication and should be used with caution.
transient template_is_experimental = 1 : hidden;

include "grib2/templates/template.4.parameter.def"
include "grib2/templates/template.4.generating_process.def"
include "grib2/templates/template.4.forecast_time.def"
include "grib2/templates/template.4.horizontal.def"
include "grib2/templates/template.4.percentile.def"
include "grib2/templates/template.4.statistical.def"
