meta verificationDate        g1verificationdate(dataDate, dataTime, endStep) : read_only;
meta monthlyVerificationDate g1monthlydate(verificationDate) : dump,no_copy;
alias mars.date            = monthlyVerificationDate;

alias monthlyVerificationTime = zero;

meta verificationYear          evaluate(verificationDate/10000);
meta monthlyVerificationYear   evaluate(monthlyVerificationDate/10000);
meta verificationMonth         evaluate( (verificationDate/100)%100 );
meta monthlyVerificationMonth  evaluate( (monthlyVerificationDate/100)%100 );

unalias mars.step;

meta ldas_configuration sprintf("%s-%s", modelVersion, targetFcstSystem ) : no_copy;

alias mars.configuration = ldas_configuration ;

alias mars.anoffset = anoffset ;
