# (C) Copyright 2005- ECMWF.

# Hours of observational data cut-off after reference time
unsigned[2] hoursAfterDataCutoff = missing() : edition_specific,can_be_missing;
alias hoursAfterReferenceTimeOfDataCutoff=hoursAfterDataCutoff;

# Minutes of observational data cut-off after reference time
unsigned[1]  minutesAfterDataCutoff = missing() : edition_specific,can_be_missing;
alias minutesAfterReferenceTimeOfDataCutoff=minutesAfterDataCutoff;

# Indicator of unit for forecastTime (ECC-1709)
codetable[1] indicatorOfUnitForForecastTime ('4.4.table',masterDir,localDir) : dump;
alias indicatorOfUnitOfTimeRange = indicatorOfUnitForForecastTime;

include "grib2/stepUnits.def"

# Forecast time in units defined by previous octet (GRIB-29: supports negative forecast time)
signed[4] forecastTime : dump;
