# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Instantaneous eastward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Time-mean U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-maximum U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Time-mean geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-maximum geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Minimum dew point depression
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Maximum relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 27 ;
	}
#Maximum absolute humidity
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 28 ;
	}
#Maximum wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	}
#Net long wave radiation flux (surface)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 0 ;
	}
#Net long wave radiation flux (top of atmosphere)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 1 ;
	}
#Precipitation from radar
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 5 ;
	}
#Maximum snow albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 17 ;
	}
#Volumetric soil moisture content
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Upper layer soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Upper layer soil moisture
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	}
#Lower layer soil moisture
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Bottom layer soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Liquid volumetric soil moisture (non-frozen)
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Transpiration stress-onset (soil moisture)
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	}
#Direct evaporation cease (soil moisture)
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	}
#Soil porosity
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	}
#Evaporation in the last 6 hours
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 lengthOfTimeRange = 6 ;
	}
#Time-mean evapotranspiration rate in the last 24h
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-integrated potential evapotranspiration rate in the last 24h
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-mean potential evapotranspiration rate in the last 24h
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-mean volumetric soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-integrated water runoff and drainage rate in the last 24h
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-mean water runoff and drainage rate in the last 24h
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-mean snow depth water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-mean skin temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 24 ;
	}
#Time-integrated snow melt rate in the last 24h
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 lengthOfTimeRange = 24 ;
	}
#Upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Time-mean upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Geometrical height above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Maximum temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Minimum temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Envelop-maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Soil moisture
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#Soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
}
