#alias mars.number=perturbationNumber;
alias mars.anoffset=offsetToEndOf4DvarWindow;

if ( grib2LocalSectionNumber == 45 && MTG2Switch == 0) {
 # mars.number = componentIndex from the other local sections for this type
 # this has always value 1, so we mimick this for pre-mtg2 which 50r1 is part of
 transient pertNumber=1;
 alias mars.number = pertNumber;
 alias mars.coeffindex = fourierCoefficientIndex;
} else {
 alias mars.coeffindex = fourierCoefficientIndex;
}
