# (C) Copyright 2005- ECMWF.

# Atmospheric chemical or physical constituent type
codetable[2] constituentType ('4.230.table',masterDir,localDir) : dump;
meta constituentTypeName codetable_title(constituentType);
