# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Sea ice area fraction
'siconc' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 91 ;
	}
#10 metre U wind component
'u10' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 33 ;
	 indicatorOfTypeOfLevel = 105 ;
	 level = 10 ;
	}
#10 metre V wind component
'v10' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 34 ;
	 indicatorOfTypeOfLevel = 105 ;
	 topLevel = 10 ;
	}
#2 metre temperature
't2m' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 11 ;
	 indicatorOfTypeOfLevel = 105 ;
	 level = 2 ;
	}
#2 metre specific humidity
'sh2' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 51 ;
	 indicatorOfTypeOfLevel = 105 ;
	 level = 2 ;
	}
#2 metre relative humidity
'r2' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 52 ;
	 indicatorOfTypeOfLevel = 105 ;
	 level = 2 ;
}
