# (C) Copyright 2005- ECMWF.

#  Type of ensemble forecast
codetable[1] typeOfEnsembleForecast ('4.6.table',masterDir,localDir) = 255 : dump;

#  Perturbation number
unsigned[4] perturbationNumber  : dump;
alias number=perturbationNumber;

#  Number of forecasts in ensemble
unsigned[4] numberOfForecastsInEnsemble  : dump;
alias totalNumber=numberOfForecastsInEnsemble;

