# GLOFAS seasonal reforecast

alias mars.origin = inputOriginatingCentre;

alias mars.number = perturbationNumber;

# We need this because 'postProcessing' is defined later
meta efas_model sprintf("%s", postProcessing) : no_copy;
alias mars.model = efas_model;

alias mars.date  = dateOfModelVersion;
alias mars.time  = dataTime;
alias mars.hdate = dataDate;
