# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Surface runoff
'Surface runoff' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 202 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Sub-surface runoff
'Sub-surface runoff' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 204 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#UV visible albedo for direct radiation (climatological)
'UV visible albedo for direct radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 193 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for diffuse radiation (climatological)
'UV visible albedo for diffuse radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 192 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation (climatological)
'Near IR albedo for direct radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 193 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for diffuse radiation (climatological)
'Near IR albedo for diffuse radiation (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 192 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Snow albedo
'Snow albedo' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 192 ;
	}
#Snow evaporation
'Snow evaporation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 197 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snowmelt
'Snowmelt' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 203 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snow depth
'Snow depth' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 254 ;
	}
#Large-scale precipitation
'Large-scale precipitation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 196 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective precipitation
'Convective precipitation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 195 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snowfall
'Snowfall' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 198 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Logarithm of surface pressure
'Logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Albedo (climatological)
'Albedo (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 193 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Evaporation
'Evaporation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 199 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Low cloud cover
'Low cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 193 ;
	}
#Medium cloud cover
'Medium cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 194 ;
	}
#High cloud cover
'High cloud cover' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 195 ;
	}
#Skin reservoir content
'Skin reservoir content' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 205 ;
	}
#Runoff
'Runoff' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 201 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Total column ozone
'Total column ozone' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total precipitation
'Total precipitation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective snowfall
'Convective snowfall' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 201 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Large-scale snowfall
'Large-scale snowfall' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 202 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Forecast albedo
'Forecast albedo' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 193 ;
	}
#UV visible albedo for direct radiation, isotropic component (climatological)
'UV visible albedo for direct radiation, isotropic component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 195 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, volumetric component (climatological)
'UV visible albedo for direct radiation, volumetric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 196 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, geometric component (climatological)
'UV visible albedo for direct radiation, geometric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 194 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, isotropic component (climatological)
'Near IR albedo for direct radiation, isotropic component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 195 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, volumetric component (climatological)
'Near IR albedo for direct radiation, volumetric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 196 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, geometric component (climatological)
'Near IR albedo for direct radiation, geometric component (climatological)' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 194 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Accumulated freezing rain
'Accumulated freezing rain' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 194 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Potential evaporation
'Potential evaporation' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 200 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward ozone flux
'Time-integrated total column vertically-integrated eastward ozone flux' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward ozone flux
'Time-integrated total column vertically-integrated northward ozone flux' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of ozone flux
'Time-integrated total column vertically-integrated divergence of ozone flux' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated net source of ozone
'Time-integrated total column vertically-integrated net source of ozone' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean carbon dioxide net ecosystem exchange flux
'Time-mean carbon dioxide net ecosystem exchange flux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean carbon dioxide gross primary production flux
'Time-mean carbon dioxide gross primary production flux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean carbon dioxide ecosystem respiration flux
'Time-mean carbon dioxide ecosystem respiration flux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean logarithm of surface pressure
'Time-mean logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total cloud cover
'Time-mean total cloud cover' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-maximum wind speed
'Time-maximum wind speed' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind speed
'Time-maximum wind speed' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 200 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum logarithm of surface pressure
'Time-maximum logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum wind speed
'Time-minimum wind speed' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind speed
'Time-minimum wind speed' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 200 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum logarithm of surface pressure
'Time-minimum logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation wind speed
'Time-standard-deviation wind speed' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind speed
'Time-standard-deviation wind speed' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 200 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation logarithm of surface pressure
'Time-standard-deviation logarithm of surface pressure' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 105 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Covariance between 2-metre temperature and volumetric soil water layer 1
'Covariance between 2-metre temperature and volumetric soil water layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and volumetric soil water layer 1
'Covariance between 2-metre relative humidity and volumetric soil water layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between surface soil moisture and volumetric soil water layer 1
'Covariance between surface soil moisture and volumetric soil water layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 194 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and volumetric soil water layer 2
'Covariance between 2-metre temperature and volumetric soil water layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 195 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and volumetric soil water layer 2
'Covariance between 2-metre relative humidity and volumetric soil water layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 196 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between surface soil moisture and volumetric soil water layer 2
'Covariance between surface soil moisture and volumetric soil water layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 197 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and volumetric soil water layer 3
'Covariance between 2-metre temperature and volumetric soil water layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 198 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and volumetric soil water layer 3
'Covariance between 2-metre relative humidity and volumetric soil water layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 199 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between surface soil moisture and volumetric soil water layer 3
'Covariance between surface soil moisture and volumetric soil water layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 200 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and soil temperature layer 1
'Covariance between 2-metre temperature and soil temperature layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 201 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and soil temperature layer 1
'Covariance between 2-metre relative humidity and soil temperature layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 202 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and soil temperature layer 2
'Covariance between 2-metre temperature and soil temperature layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 203 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and soil temperature layer 2
'Covariance between 2-metre relative humidity and soil temperature layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 204 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and soil temperature layer 3
'Covariance between 2-metre temperature and soil temperature layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 205 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and soil temperature layer 3
'Covariance between 2-metre relative humidity and soil temperature layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 206 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and temperature of snow layer 1
'Covariance between 2-metre temperature and temperature of snow layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 207 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and temperature of snow layer 1
'Covariance between 2-metre relative humidity and temperature of snow layer 1' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 208 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and temperature of snow layer 2
'Covariance between 2-metre temperature and temperature of snow layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 209 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and temperature of snow layer 2
'Covariance between 2-metre relative humidity and temperature of snow layer 2' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 210 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre temperature and temperature of snow layer 3
'Covariance between 2-metre temperature and temperature of snow layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 211 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Covariance between 2-metre relative humidity and temperature of snow layer 3
'Covariance between 2-metre relative humidity and temperature of snow layer 3' = {
	 localTablesVersion = 1 ;
	 discipline = 254 ;
	 parameterCategory = 254 ;
	 parameterNumber = 212 ;
	 typeOfFirstFixedSurface = 254 ;
	}
#Snow thickness over sea ice
'Snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice salinity
'Sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Time-mean snow thickness over sea ice
'Time-mean snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Sea ice salinity
'Time-mean Sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-maximum snow thickness over sea ice
'Time-maximum snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice salinity
'Time-maximum sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum snow thickness over sea ice
'Time-minimum snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum  sea ice salinity
'Time-minimum  sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation snow thickness over sea ice
'Time-standard-deviation snow thickness over sea ice' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 192 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice salinity
'Time-standard-deviation sea ice salinity' = {
	 localTablesVersion = 1 ;
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 193 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
}
