# (C) Copyright 2005- ECMWF.

# TEMPLATE 4.131, Anomalies significance and other derived products as probability forecasts in relation to a reference period at a horizontal level or in a horizontal layer at a point in time

include "grib2/templates/template.4.parameter.def"
include "grib2/templates/template.4.generating_process.def"
include "grib2/templates/template.4.forecast_time.def"
include "grib2/templates/template.4.horizontal.def"
include "grib2/templates/template.4.point_in_time.def"
include "grib2/templates/template.4.probability.def"
include "grib2/templates/template.4.referenceperiod.def"
