# (C) Copyright 2005- ECMWF.

# TEMPLATE 4.141, Reforecast at a horizontal level or in a horizontal layer at a point in time for wave 2D spectra with explicit list of frequencies and directions

include "grib2/templates/template.4.parameter.def"
include "grib2/templates/template.4.wave.def"
include "grib2/templates/template.4.generating_process.def"
include "grib2/templates/template.4.forecast_time.def"
include "grib2/templates/template.4.point_in_time.def"
include "grib2/templates/template.4.reforecast.def"
include "grib2/templates/template.4.wave_spectra_list.def"
