# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Sea ice area fraction
'siconc' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 31 ;
	}
#Maximum temperature at 2 metres in the last 24 hours
'mx2t24' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	 lengthOfTimeRange = 24 ;
	}
#Minimum temperature at 2 metres in the last 24 hours
'mn2t24' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	 lengthOfTimeRange = 24 ;
	}
#Maximum 10 metre wind gust in the last 6 hours
'fg10_6' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 123 ;
	}
#10 metre wind gust difference
'fgdiff10' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 49 ;
	}
#10 metre wind gust in the last 6 hours difference
'fg6diff10' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 123 ;
	}
#10 metre U wind component difference
'udiff10' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 165 ;
	}
#10 metre V wind component difference
'vdiff10' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 166 ;
	}
#2 metre temperature difference
'difft2' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 167 ;
	}
#10 metre wind speed difference
'sidiff10' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 207 ;
	}
#100 metre U wind component anomaly
'ua100' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 6 ;
	}
#100 metre V wind component anomaly
'va100' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 7 ;
	}
#Maximum temperature at 2 metres in the last 3 hours
'mx2t3' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	 lengthOfTimeRange = 3 ;
	}
#Minimum temperature at 2 metres in the last 3 hours
'mn2t3' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	 lengthOfTimeRange = 3 ;
	}
#Maximum 10 metre wind gust in the last 3 hours
'fg10_3' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 28 ;
	}
#100 metre wind speed
'si100' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 249 ;
	}
#10 metre wind gust gradient
'fggrd10' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 49 ;
	}
#10 metre wind gust in the last 6 hours gradient
'fg6grd10' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 123 ;
	}
#10 metre U wind component gradient
'ugrd10' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 165 ;
	}
#10 metre V wind component gradient
'vgrd10' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 166 ;
	}
#2 metre temperature gradient
'grd2t' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 167 ;
	}
#2 metre dewpoint temperature gradient
'grd2d' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 168 ;
	}
#10 metre wind speed gradient
'sigrd10' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 207 ;
	}
#2m temperature anomaly of at least +2K
't2ag2' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 1 ;
	}
#2m temperature anomaly of at least +1K
't2ag1' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 2 ;
	}
#2m temperature anomaly of at least 0K
't2ag0' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 3 ;
	}
#2m temperature anomaly of at most -1K
't2alm1' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 4 ;
	}
#2m temperature anomaly of at most -2K
't2alm2' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 5 ;
	}
#10 metre wind gust probability
'g10p' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 49 ;
	}
#10 metre Wind speed of at least 10 m/s
'sp10g10' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 68 ;
	}
#10 metre Wind speed of at least 15 m/s
'sp10g15' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 69 ;
	}
#10 metre wind gust of at least 25 m/s
'fg10g25' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#2 metre temperature less than 273.15 K
't2l273' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 73 ;
	}
#10 metre speed probability
'sp10' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 165 ;
	}
#2 metre temperature probability
't2p' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 167 ;
	}
#2m temperature probability less than -10 C
't2plm10' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 1 ;
	}
#2m temperature probability less than -5 C
't2plm5' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 2 ;
	}
#2m temperature probability less than 0 C
't2pl0' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 3 ;
	}
#2m temperature probability less than 5 C
't2pl5' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 4 ;
	}
#2m temperature probability less than 10 C
't2pl10' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 5 ;
	}
#2m temperature probability greater than 25 C
't2pg25' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 6 ;
	}
#2m temperature probability greater than 30 C
't2pg30' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 7 ;
	}
#2m temperature probability greater than 35 C
't2pg35' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 8 ;
	}
#2m temperature probability greater than 40 C
't2pg40' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 9 ;
	}
#2m temperature probability greater than 45 C
't2pg45' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 10 ;
	}
#10 metre wind speed probability of at least 10 m/s
'sp10g10' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 21 ;
	}
#10 metre wind speed probability of at least 15 m/s
'sp10g15' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 22 ;
	}
#10 metre wind speed probability of at least 20 m/s
'sp10g20' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 23 ;
	}
#10 metre wind speed probability of at least 35 m/s
'sp10g35' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 24 ;
	}
#10 metre wind speed probability of at least 50 m/s
'sp10g50' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 25 ;
	}
#10 metre wind gust probability of at least 20 m/s
'gp10g20' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 26 ;
	}
#10 metre wind gust probability of at least 35 m/s
'gp10g35' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 27 ;
	}
#10 metre wind gust probability of at least 50 m/s
'gp10g50' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 28 ;
	}
#10 metre wind gust probability of at least 75 m/s
'gp10g75' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 29 ;
	}
#10 metre wind gust probability of at least 100 m/s
'gp10g100' = {
	 discipline = 192 ;
	 parameterCategory = 133 ;
	 parameterNumber = 30 ;
	}
#2D wave spectra (multiple)
'd2sp' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 250 ;
	}
#2D wave spectra (single)
'd2fd' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 251 ;
	}
#Eastward surface sea water velocity
'uoe' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 131 ;
	}
#Northward surface sea water velocity
'von' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 132 ;
	}
#10 metre wind gust during averaging time
'fgrea10' = {
	 discipline = 192 ;
	 parameterCategory = 160 ;
	 parameterNumber = 49 ;
	}
#10 metre wind speed
'wsrea10' = {
	 discipline = 192 ;
	 parameterCategory = 160 ;
	 parameterNumber = 246 ;
	}
#10 metre wind gust anomaly
'fga10' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 49 ;
	}
#10 metre U wind component anomaly
'ua10' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 165 ;
	}
#10 metre V wind component anomaly
'va10' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 166 ;
	}
#2 metre temperature anomaly
't2a' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 167 ;
	}
#2 metre dewpoint temperature anomaly
'd2a' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 168 ;
	}
#10 metre wind speed anomaly
'sia10' = {
	 discipline = 192 ;
	 parameterCategory = 171 ;
	 parameterNumber = 207 ;
	}
#2 metre dewpoint temperature difference
'ddiff2' = {
	 discipline = 192 ;
	 parameterCategory = 200 ;
	 parameterNumber = 168 ;
	}
#Time-mean 2 metre temperature
'avg_2t' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 4 ;
	}
#Time-mean 10 metre wind speed
'avg_10ws' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 5 ;
	}
#2 metre temperature significance
't2s' = {
	 discipline = 192 ;
	 parameterCategory = 234 ;
	 parameterNumber = 167 ;
	}
#100 metre U wind component
'u100' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 246 ;
	}
#100 metre V wind component
'v100' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 247 ;
}
