# (C) Copyright 2005- ECMWF.

# TEMPLATE 4.203, Satellite Product with channel, bandwidth and polarisation

include "grib2/templates/template.4.parameter.def"

codetable[1] typeOfGeneratingProcess ('4.3.table',masterDir,localDir) : dump;

#  Observation generating process identifier (defined by originating centre)
unsigned[1] observationGeneratingProcessIdentifier  : dump;
alias generatingProcessIdentifier=observationGeneratingProcessIdentifier;

include "grib2/templates/template.4.satellite_extended.def"
