#Methane (chemistry)
'Methane (chemistry)' = {
	localTablesVersion = 1 ;
	constituentType = 2 ;
	}
#Volcanic sulphur dioxide
'Volcanic sulphur dioxide' = {
	localTablesVersion = 1 ;
	constituentType = 65507 ;
	}
#Stratospheric ozone
'Stratospheric ozone' = {
	localTablesVersion = 1 ;
	constituentType = 65524 ;
	}
#PAR budget corrector
'PAR budget corrector' = {
	localTablesVersion = 1 ;
	constituentType = 65520 ;
	}
#NO to NO2 operator
'NO to NO2 operator' = {
	localTablesVersion = 1 ;
	constituentType = 65521 ;
	}
#NO to alkyl nitrate operator
'NO to alkyl nitrate operator' = {
	localTablesVersion = 1 ;
	constituentType = 65522 ;
	}
#Polar stratospheric cloud
'Polar stratospheric cloud' = {
	localTablesVersion = 1 ;
	constituentType = 65516 ;
	}
#Methacrolein MVK
'Methacrolein MVK' = {
	localTablesVersion = 1 ;
	constituentType = 65523 ;
	}
#Acetone product
'Acetone product' = {
	localTablesVersion = 1 ;
	constituentType = 65515 ;
	}
#HYPROPO2
'HYPROPO2' = {
	localTablesVersion = 1 ;
	constituentType = 65519 ;
	}
#Nitrogen oxides Transp
'Nitrogen oxides Transp' = {
	localTablesVersion = 1 ;
	constituentType = 65514 ;
	}
#Carbon dioxide (chemistry)
'Carbon dioxide (chemistry)' = {
	localTablesVersion = 1 ;
	constituentType = 3 ;
	}
#Nitrous oxide (chemistry)
'Nitrous oxide (chemistry)' = {
	localTablesVersion = 1 ;
	constituentType = 6 ;
	}
#Water vapour (chemistry)
'Water vapour (chemistry)' = {
	localTablesVersion = 1 ;
	constituentType = 1 ;
	}
#Toluene and less reactive aromatics
'Toluene and less reactive aromatics' = {
	localTablesVersion = 1 ;
	constituentType = 65529 ;
	}
#Xylene and more reactive aromatics
'Xylene and more reactive aromatics' = {
	localTablesVersion = 1 ;
	constituentType = 65530 ;
	}
#Lumped alkenes
'Lumped alkenes' = {
	localTablesVersion = 1 ;
	constituentType = 60010 ;
	}
#Lumped alkanes
'Lumped alkanes' = {
	localTablesVersion = 1 ;
	constituentType = 60009 ;
	}
#Hydrogen atom
'Hydrogen atom' = {
	localTablesVersion = 1 ;
	constituentType = 65513 ;
	}
#Condensable gas type 1
'Condensable gas type 1' = {
	localTablesVersion = 1 ;
	constituentType = 65526 ;
	}
#Condensable gas type 2a
'Condensable gas type 2a' = {
	localTablesVersion = 1 ;
	constituentType = 65527 ;
	}
#Condensable gas type 2b
'Condensable gas type 2b' = {
	localTablesVersion = 1 ;
	constituentType = 65528 ;
	}
#Asymmetric chlorine dioxide radical
'Asymmetric chlorine dioxide radical' = {
	localTablesVersion = 1 ;
	constituentType = 65518 ;
	}
#GEMS ozone
'GEMS ozone' = {
	localTablesVersion = 1 ;
	constituentType = 0 ;
	}
#Stratospheric aerosol
'Stratospheric aerosol' = {
	localTablesVersion = 1 ;
	constituentType = 65517 ;
	}
#Biogenic secondary organic aerosol precursor
'Biogenic secondary organic aerosol precursor' = {
	localTablesVersion = 1 ;
	constituentType = 65511 ;
	}
#Anthropogenic secondary organic aerosol precursor
'Anthropogenic secondary organic aerosol precursor' = {
	localTablesVersion = 1 ;
	constituentType = 65512 ;
	}
#SO2 precursor
'SO2 precursor' = {
	localTablesVersion = 1 ;
	constituentType = 65510 ;
	}
#Aerosol small mode
'Aerosol small mode' = {
	localTablesVersion = 1 ;
	constituentType = 65508 ;
	}
#Aerosol large mode
'Aerosol large mode' = {
	localTablesVersion = 1 ;
	constituentType = 65509 ;
	}
#Volcanic Sulphate
'Volcanic Sulphate' = {
	localTablesVersion = 1 ;
	constituentType = 65506 ;
	}
