# (C) Copyright 2005- ECMWF.
#
# Section 11 - End Section
constant section11Length=4;
position offsetSection11;
ascii[4] '7777' = "7777" : read_only;
meta section11Pointer section_pointer(offsetSection11,section11Length,11);
