# Concept typeOfLevel for kwbc
'atmosphereSingleLayer'         = {typeOfFirstFixedSurface=200;}
'oceanSingleLayer'              = {typeOfFirstFixedSurface=201;}
'highestTroposphericFreezing'   = {typeOfFirstFixedSurface=204;}
'gridScaleCloudBottom'          = {typeOfFirstFixedSurface=206;}
'gridScaleCloudTop'             = {typeOfFirstFixedSurface=207;}
'boundaryLayerCloudBottom'      = {typeOfFirstFixedSurface=209;}
'boundaryLayerCloudTop'         = {typeOfFirstFixedSurface=210;}
'boundaryLayerCloudLayer'       = {typeOfFirstFixedSurface=211;}
'lowCloudBottom'                = {typeOfFirstFixedSurface=212;}
'lowCloudTop'                   = {typeOfFirstFixedSurface=213;}
'lowCloudLayer'                 = {typeOfFirstFixedSurface=214;}
'cloudCeiling'                  = {typeOfFirstFixedSurface=215;}
'planetaryBoundaryLayer'        = {typeOfFirstFixedSurface=220;}
'layerBetween2Hybrids'          = {typeOfFirstFixedSurface=221;}
'middleCloudBottom'             = {typeOfFirstFixedSurface=222;}
'middleCloudTop'                = {typeOfFirstFixedSurface=223;}
'middleCloudLayer'              = {typeOfFirstFixedSurface=224;}
'highCloudBottom'               = {typeOfFirstFixedSurface=232;}
'highCloudTop'                  = {typeOfFirstFixedSurface=233;}
'highCloudLayer'                = {typeOfFirstFixedSurface=234;}
'oceanIsotherm'                 = {typeOfFirstFixedSurface=235;}
'oceanMixedLayer'               = {typeOfFirstFixedSurface=240;}
'orderedSequenceData'           = {typeOfFirstFixedSurface=241;}
'convectiveCloudBottom'         = {typeOfFirstFixedSurface=242;}
'convectiveCloudTop'            = {typeOfFirstFixedSurface=243;}
'convectiveCloudLayer'          = {typeOfFirstFixedSurface=244;}
'lowestLevelWetBulb0'           = {typeOfFirstFixedSurface=245;}
'equilibrium'                   = {typeOfFirstFixedSurface=247;}
'shallowConvectiveCloudBottom'  = {typeOfFirstFixedSurface=248;}
'shallowConvectiveCloudTop'     = {typeOfFirstFixedSurface=249;}
'deepConvectiveCloudBottom'     = {typeOfFirstFixedSurface=251;}
'deepConvectiveCloudTop'        = {typeOfFirstFixedSurface=252;}
