concept modelName(unknown, "modelNameConcept.def", conceptsDir2, conceptsLocalDirAll): no_copy, dump;

if (modelName isnot "unknown") {
   concept modelVersionConcept(unknown, "modelVersionConcept.[modelName].def", conceptsDir2, conceptsLocalDirAll): no_copy, dump, read_only;
  if ( grib2LocalSectionPresent or localUsePresent ) { # See ECC-2039
      if ( defined(marsClass) ) {
         alias ls.model = modelName;
         alias ls.modelVersion = modelVersionConcept;;
      }
   }
}

concept inputModelName(unknown, "inputModelNameConcept.def", conceptsDir2, conceptsLocalDirAll): no_copy, dump;
concept inputModelVersion(unknown, "inputModelVersionConcept.def", conceptsDir2, conceptsLocalDirAll): no_copy, dump;

if (defined(typeOfPostProcessing) and typeOfPostProcessing == 11){
 if ( inputModelName is "obs" ){
     meta forcingModel sprintf("%s",inputModelName);
   } else {
     meta forcingModel sprintf("%s-%s",centre, inputModelName);
 }
 alias mars.forcing = forcingModel;
 unalias mars.origin;
}
