# (C) Copyright 2005- ECMWF.

# TEMPLATE 4.162, Probability forecasts at a horizontal level or in a horizontal layer at a point in time for waves selected by period range

include "grib2/templates/template.4.parameter.def"
include "grib2/templates/template.4.wave_period_range.def"
include "grib2/templates/template.4.generating_process.def"
include "grib2/templates/template.4.forecast_time.def"
include "grib2/templates/template.4.point_in_time.def"
include "grib2/templates/template.4.horizontal.def"
include "grib2/templates/template.4.probability_large_ensemble.def"
