# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Total precipitation of at least 1 mm
'131060' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 60 ;
	}
#Total precipitation of at least 5 mm
'131061' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 61 ;
	}
#Total precipitation of at least 40 mm
'131082' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 82 ;
	}
#Total precipitation of at least 60 mm
'131083' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 83 ;
	}
#Total precipitation of at least 80 mm
'131084' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 84 ;
	}
#Total precipitation of at least 100 mm
'131085' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 85 ;
	}
#Total precipitation of at least 150 mm
'131086' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 86 ;
	}
#Total precipitation of at least 200 mm
'131087' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 87 ;
	}
#Total precipitation of at least 300 mm
'131088' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 88 ;
	}
#Surface runoff
'8' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 8 ;
	}
#Sub-surface runoff
'9' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 9 ;
	}
#UV visible albedo for direct radiation (climatological)
'15' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 15 ;
	}
#UV visible albedo for diffuse radiation (climatological)
'16' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 16 ;
	}
#Near IR albedo for direct radiation (climatological)
'17' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 17 ;
	}
#Near IR albedo for diffuse radiation (climatological)
'18' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 18 ;
	}
#Surface photosynthetically active radiation, clear sky
'20' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 20 ;
	}
#Unbalanced component of temperature
'21' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 21 ;
	}
#Unbalanced component of logarithm of surface pressure
'22' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 22 ;
	}
#Unbalanced component of divergence
'23' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 23 ;
	}
#Lake cover
'26' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 26 ;
	}
#Low vegetation cover
'27' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 27 ;
	}
#High vegetation cover
'28' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 28 ;
	}
#Type of low vegetation
'29' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 29 ;
	}
#Type of high vegetation
'30' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 30 ;
	}
#Sea ice area fraction
'31' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 31 ;
	}
#Snow albedo
'32' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 32 ;
	}
#Snow density
'33' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 33 ;
	}
#Sea surface temperature
'34' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 34 ;
	}
#Snow evaporation
'44' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 44 ;
	}
#Snowmelt
'45' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 45 ;
	}
#Surface direct normal short-wave (solar) radiation
'47' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 47 ;
	}
#Time-integrated magnitude of turbulent surface stress
'48' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 48 ;
	}
#Large-scale precipitation fraction
'50' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 50 ;
	}
#Montgomery potential
'53' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 53 ;
	}
#Mean temperature at 2 metres in the last 24 hours
'55' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 55 ;
	}
#Mean 2 metre dewpoint temperature in the last 24 hours
'56' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 56 ;
	}
#Surface downward UV radiation
'57' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 57 ;
	}
#Photosynthetically active radiation at the surface
'58' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 58 ;
	}
#Leaf area index, low vegetation
'66' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 66 ;
	}
#Leaf area index, high vegetation
'67' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 67 ;
	}
#Standard deviation of filtered subgrid orography (climatological)
'74' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 74 ;
	}
#Total column cloud liquid water
'78' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 78 ;
	}
#Total column cloud ice water
'79' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 79 ;
	}
#Surface emissivity
'124' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 124 ;
	}
#Total column vertically-integrated water vapour
'137' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 137 ;
	}
#Large-scale precipitation
'142' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 142 ;
	}
#Snowfall
'144' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 144 ;
	}
#Charnock
'148' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 148 ;
	}
#Surface net radiation (SW and LW)
'149' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 149 ;
	}
#Top net radiation (SW and LW)
'150' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 150 ;
	}
#Boundary layer height
'159' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 159 ;
	}
#Standard deviation of sub-gridscale orography
'160' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 160 ;
	}
#Anisotropy of sub-gridscale orography
'161' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 161 ;
	}
#Angle of sub-gridscale orography
'162' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 162 ;
	}
#Slope of sub-gridscale orography
'163' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 163 ;
	}
#Total cloud cover
'164' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 164 ;
	}
#Surface short-wave (solar) radiation downwards
'169' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 169 ;
	}
#Albedo (climatological)
'174' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 174 ;
	}
#Surface long-wave (thermal) radiation downwards
'175' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 175 ;
	}
#Top net short-wave (solar) radiation
'178' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 178 ;
	}
#Time-integrated eastward turbulent surface stress
'180' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 180 ;
	}
#Time-integrated northward turbulent surface stress
'181' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 181 ;
	}
#Evaporation
'182' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 182 ;
	}
#Low cloud cover
'186' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 186 ;
	}
#Medium cloud cover
'187' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 187 ;
	}
#High cloud cover
'188' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 188 ;
	}
#Eastward gravity wave surface stress
'195' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 195 ;
	}
#Northward gravity wave surface stress
'196' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 196 ;
	}
#Gravity wave dissipation
'197' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 197 ;
	}
#Skin reservoir content
'198' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 198 ;
	}
#Runoff
'205' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 205 ;
	}
#Total column ozone
'206' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 206 ;
	}
#Top net short-wave (solar) radiation, clear sky
'208' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 208 ;
	}
#Top net long-wave (thermal) radiation, clear sky
'209' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 209 ;
	}
#Surface net short-wave (solar) radiation, clear sky
'210' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 210 ;
	}
#Surface net long-wave (thermal) radiation, clear sky
'211' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 211 ;
	}
#TOA incident short-wave (solar) radiation
'212' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 212 ;
	}
#Vertically integrated moisture divergence
'213' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 213 ;
	}
#Instantaneous eastward turbulent surface stress
'229' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 229 ;
	}
#Instantaneous northward turbulent surface stress
'230' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 230 ;
	}
#Instantaneous surface sensible heat net flux
'231' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 231 ;
	}
#Instantaneous moisture flux
'232' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 232 ;
	}
#Logarithm of surface roughness length for heat (climatological)
'234' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 234 ;
	}
#Temperature of snow layer
'238' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 238 ;
	}
#Convective snowfall
'239' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 239 ;
	}
#Large-scale snowfall
'240' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 240 ;
	}
#Forecast albedo
'243' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 243 ;
	}
#Forecast surface roughness
'244' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 244 ;
	}
#Forecast logarithm of surface roughness for heat
'245' = {
	 discipline = 192 ;
	 parameterCategory = 128 ;
	 parameterNumber = 245 ;
	}
#Mean surface net radiation flux
'172149' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 149 ;
	}
#Injection height (from IS4FIRES)
'210060' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 60 ;
	}
#Particulate matter d <= 1 um
'210072' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 72 ;
	}
#Particulate matter d <= 2.5 um
'210073' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 73 ;
	}
#Particulate matter d <= 10 um
'210074' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 74 ;
	}
#Mean height of maximum injection
'210119' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 119 ;
	}
#Plume top height above surface
'210120' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 120 ;
	}
#UV visible albedo for direct radiation, isotropic component (climatological)
'210186' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 186 ;
	}
#UV visible albedo for direct radiation, volumetric component (climatological)
'210187' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 187 ;
	}
#UV visible albedo for direct radiation, geometric component (climatological)
'210188' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 188 ;
	}
#Near IR albedo for direct radiation, isotropic component (climatological)
'210189' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 189 ;
	}
#Near IR albedo for direct radiation, volumetric component (climatological)
'210190' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 190 ;
	}
#Near IR albedo for direct radiation, geometric component (climatological)
'210191' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 191 ;
	}
#UV visible albedo for diffuse radiation, isotropic component (climatological)
'210192' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 192 ;
	}
#UV visible albedo for diffuse radiation, volumetric component (climatological)
'210193' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 193 ;
	}
#UV visible albedo for diffuse radiation, geometric component (climatological)
'210194' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 194 ;
	}
#Near IR albedo for diffuse radiation, isotropic component (climatological)
'210195' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 195 ;
	}
#Near IR albedo for diffuse radiation, volumetric component (climatological)
'210196' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 196 ;
	}
#Near IR albedo for diffuse radiation, geometric component (climatological)
'210197' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 197 ;
	}
#Plume bottom height above surface
'210242' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 242 ;
	}
#Random pattern 1 for SPP scheme
'213101' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 101 ;
	}
#Random pattern 2 for SPP scheme
'213102' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 102 ;
	}
#Random pattern 3 for SPP scheme
'213103' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 103 ;
	}
#Random pattern 4 for SPP scheme
'213104' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 104 ;
	}
#Random pattern 5 for SPP scheme
'213105' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 105 ;
	}
#Random pattern 6 for SPP scheme
'213106' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 106 ;
	}
#Random pattern 7 for SPP scheme
'213107' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 107 ;
	}
#Random pattern 8 for SPP scheme
'213108' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 108 ;
	}
#Random pattern 9 for SPP scheme
'213109' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 109 ;
	}
#Random pattern 10 for SPP scheme
'213110' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 110 ;
	}
#Random pattern 11 for SPP scheme
'213111' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 111 ;
	}
#Random pattern 12 for SPP scheme
'213112' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 112 ;
	}
#Random pattern 13 for SPP scheme
'213113' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 113 ;
	}
#Random pattern 14 for SPP scheme
'213114' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 114 ;
	}
#Random pattern 15 for SPP scheme
'213115' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 115 ;
	}
#Random pattern 16 for SPP scheme
'213116' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 116 ;
	}
#Random pattern 17 for SPP scheme
'213117' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 117 ;
	}
#Random pattern 18 for SPP scheme
'213118' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 118 ;
	}
#Random pattern 19 for SPP scheme
'213119' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 119 ;
	}
#Random pattern 20 for SPP scheme
'213120' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 120 ;
	}
#Random pattern 21 for SPP scheme
'213121' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 121 ;
	}
#Random pattern 22 for SPP scheme
'213122' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 122 ;
	}
#Random pattern 23 for SPP scheme
'213123' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 123 ;
	}
#Random pattern 24 for SPP scheme
'213124' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 124 ;
	}
#Random pattern 25 for SPP scheme
'213125' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 125 ;
	}
#Random pattern 26 for SPP scheme
'213126' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 126 ;
	}
#Random pattern 27 for SPP scheme
'213127' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 127 ;
	}
#Random pattern 28 for SPP scheme
'213128' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 128 ;
	}
#Random pattern 29 for SPP scheme
'213129' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 129 ;
	}
#Random pattern 30 for SPP scheme
'213130' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 130 ;
	}
#Random pattern 31 for SPP scheme
'213131' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 131 ;
	}
#Random pattern 32 for SPP scheme
'213132' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 132 ;
	}
#Random pattern 33 for SPP scheme
'213133' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 133 ;
	}
#Random pattern 34 for SPP scheme
'213134' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 134 ;
	}
#Random pattern 35 for SPP scheme
'213135' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 135 ;
	}
#Random pattern 36 for SPP scheme
'213136' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 136 ;
	}
#Random pattern 37 for SPP scheme
'213137' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 137 ;
	}
#Random pattern 38 for SPP scheme
'213138' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 138 ;
	}
#Random pattern 39 for SPP scheme
'213139' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 139 ;
	}
#Random pattern 40 for SPP scheme
'213140' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 140 ;
	}
#Random pattern 41 for SPP scheme
'213141' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 141 ;
	}
#Random pattern 42 for SPP scheme
'213142' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 142 ;
	}
#Random pattern 43 for SPP scheme
'213143' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 143 ;
	}
#Random pattern 44 for SPP scheme
'213144' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 144 ;
	}
#Random pattern 45 for SPP scheme
'213145' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 145 ;
	}
#Random pattern 46 for SPP scheme
'213146' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 146 ;
	}
#Random pattern 47 for SPP scheme
'213147' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 147 ;
	}
#Random pattern 48 for SPP scheme
'213148' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 148 ;
	}
#Random pattern 49 for SPP scheme
'213149' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 149 ;
	}
#Random pattern 50 for SPP scheme
'213150' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 150 ;
	}
#Random pattern 51 for SPP scheme
'213151' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 151 ;
	}
#Random pattern 52 for SPP scheme
'213152' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 152 ;
	}
#Random pattern 53 for SPP scheme
'213153' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 153 ;
	}
#Random pattern 54 for SPP scheme
'213154' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 154 ;
	}
#Random pattern 55 for SPP scheme
'213155' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 155 ;
	}
#Random pattern 56 for SPP scheme
'213156' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 156 ;
	}
#Random pattern 57 for SPP scheme
'213157' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 157 ;
	}
#Random pattern 58 for SPP scheme
'213158' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 158 ;
	}
#Random pattern 59 for SPP scheme
'213159' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 159 ;
	}
#Random pattern 60 for SPP scheme
'213160' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 160 ;
	}
#Random pattern 61 for SPP scheme
'213161' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 161 ;
	}
#Random pattern 62 for SPP scheme
'213162' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 162 ;
	}
#Random pattern 63 for SPP scheme
'213163' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 163 ;
	}
#Random pattern 64 for SPP scheme
'213164' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 164 ;
	}
#Random pattern 65 for SPP scheme
'213165' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 165 ;
	}
#Random pattern 66 for SPP scheme
'213166' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 166 ;
	}
#Random pattern 67 for SPP scheme
'213167' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 167 ;
	}
#Random pattern 68 for SPP scheme
'213168' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 168 ;
	}
#Random pattern 69 for SPP scheme
'213169' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 169 ;
	}
#Random pattern 70 for SPP scheme
'213170' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 170 ;
	}
#Random pattern 71 for SPP scheme
'213171' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 171 ;
	}
#Random pattern 72 for SPP scheme
'213172' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 172 ;
	}
#Random pattern 73 for SPP scheme
'213173' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 173 ;
	}
#Random pattern 74 for SPP scheme
'213174' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 174 ;
	}
#Random pattern 75 for SPP scheme
'213175' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 175 ;
	}
#Random pattern 76 for SPP scheme
'213176' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 176 ;
	}
#Random pattern 77 for SPP scheme
'213177' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 177 ;
	}
#Random pattern 78 for SPP scheme
'213178' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 178 ;
	}
#Random pattern 79 for SPP scheme
'213179' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 179 ;
	}
#Random pattern 80 for SPP scheme
'213180' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 180 ;
	}
#Random pattern 81 for SPP scheme
'213181' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 181 ;
	}
#Random pattern 82 for SPP scheme
'213182' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 182 ;
	}
#Random pattern 83 for SPP scheme
'213183' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 183 ;
	}
#Random pattern 84 for SPP scheme
'213184' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 184 ;
	}
#Random pattern 85 for SPP scheme
'213185' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 185 ;
	}
#Random pattern 86 for SPP scheme
'213186' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 186 ;
	}
#Random pattern 87 for SPP scheme
'213187' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 187 ;
	}
#Random pattern 88 for SPP scheme
'213188' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 188 ;
	}
#Random pattern 89 for SPP scheme
'213189' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 189 ;
	}
#Random pattern 90 for SPP scheme
'213190' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 190 ;
	}
#Random pattern 91 for SPP scheme
'213191' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 191 ;
	}
#Random pattern 92 for SPP scheme
'213192' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 192 ;
	}
#Random pattern 93 for SPP scheme
'213193' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 193 ;
	}
#Random pattern 94 for SPP scheme
'213194' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 194 ;
	}
#Random pattern 95 for SPP scheme
'213195' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 195 ;
	}
#Random pattern 96 for SPP scheme
'213196' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 196 ;
	}
#Random pattern 97 for SPP scheme
'213197' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 197 ;
	}
#Random pattern 98 for SPP scheme
'213198' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 198 ;
	}
#Random pattern 99 for SPP scheme
'213199' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 199 ;
	}
#Random pattern 100 for SPP scheme
'213200' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 200 ;
	}
#Random pattern 101 for SPP scheme
'213201' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 201 ;
	}
#Random pattern 102 for SPP scheme
'213202' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 202 ;
	}
#Random pattern 103 for SPP scheme
'213203' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 203 ;
	}
#Random pattern 104 for SPP scheme
'213204' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 204 ;
	}
#Random pattern 105 for SPP scheme
'213205' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 205 ;
	}
#Random pattern 106 for SPP scheme
'213206' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 206 ;
	}
#Random pattern 107 for SPP scheme
'213207' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 207 ;
	}
#Random pattern 108 for SPP scheme
'213208' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 208 ;
	}
#Random pattern 109 for SPP scheme
'213209' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 209 ;
	}
#Random pattern 110 for SPP scheme
'213210' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 210 ;
	}
#Random pattern 111 for SPP scheme
'213211' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 211 ;
	}
#Random pattern 112 for SPP scheme
'213212' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 212 ;
	}
#Random pattern 113 for SPP scheme
'213213' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 213 ;
	}
#Random pattern 114 for SPP scheme
'213214' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 214 ;
	}
#Random pattern 115 for SPP scheme
'213215' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 215 ;
	}
#Random pattern 116 for SPP scheme
'213216' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 216 ;
	}
#Random pattern 117 for SPP scheme
'213217' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 217 ;
	}
#Random pattern 118 for SPP scheme
'213218' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 218 ;
	}
#Random pattern 119 for SPP scheme
'213219' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 219 ;
	}
#Random pattern 120 for SPP scheme
'213220' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 220 ;
	}
#Random pattern 121 for SPP scheme
'213221' = {
	 discipline = 192 ;
	 parameterCategory = 213 ;
	 parameterNumber = 221 ;
	}
#Cosine of solar zenith angle
'214001' = {
	 discipline = 192 ;
	 parameterCategory = 214 ;
	 parameterNumber = 1 ;
	}
#UV biologically effective dose
'214002' = {
	 discipline = 192 ;
	 parameterCategory = 214 ;
	 parameterNumber = 2 ;
	}
#UV biologically effective dose clear-sky
'214003' = {
	 discipline = 192 ;
	 parameterCategory = 214 ;
	 parameterNumber = 3 ;
	}
#Ammonia mass mixing ratio
'217019' = {
	 discipline = 192 ;
	 parameterCategory = 217 ;
	 parameterNumber = 19 ;
	}
#Nitrogen monoxide mass mixing ratio
'217027' = {
	 discipline = 192 ;
	 parameterCategory = 217 ;
	 parameterNumber = 27 ;
	}
#Surface direct short-wave (solar) radiation
'228021' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 21 ;
	}
#Surface direct short-wave radiation, clear sky
'228022' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 22 ;
	}
#Cloud base height
'228023' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 23 ;
	}
#Geometric height of 0 degrees C atmospheric isothermal level above ground
'228024' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 24 ;
	}
#Maximum 10 metre wind gust in the last 3 hours
'228028' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 28 ;
	}
#Height of zero-degree wet-bulb temperature
'228047' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 47 ;
	}
#Height of one-degree wet-bulb temperature
'228048' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 48 ;
	}
#Instantaneous total lightning flash density
'228050' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 50 ;
	}
#Averaged total lightning flash density in the last hour
'228051' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 51 ;
	}
#Instantaneous cloud-to-ground lightning flash density
'228052' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 52 ;
	}
#Averaged cloud-to-ground lightning flash density in the last hour
'228053' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 53 ;
	}
#Averaged total lightning flash density in the last 3 hours
'228057' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 57 ;
	}
#Averaged total lightning flash density in the last 6 hours
'228058' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 58 ;
	}
#Averaged cloud-to-ground lightning flash density in the last 3 hours
'228059' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 59 ;
	}
#Averaged cloud-to-ground lightning flash density in the last 6 hours
'228060' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 60 ;
	}
#Accumulated Carbon Dioxide Net Ecosystem Exchange
'228080' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 192 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated Carbon Dioxide Gross Primary Production
'228081' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 193 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated Carbon Dioxide Ecosystem Respiration
'228082' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 194 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Carbon dioxide net ecosystem exchange flux
'228083' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 195 ;
	}
#Carbon dioxide gross primary production flux
'228084' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 197 ;
	}
#Carbon dioxide ecosystem respiration flux
'228085' = {
	 localTablesVersion = 1 ;
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 196 ;
	}
#Total column rain water
'228089' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 89 ;
	}
#Total column snow water
'228090' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 90 ;
	}
#Ice temperature
'228094' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 94 ;
	}
#Surface short-wave (solar) radiation downward clear-sky
'228129' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 129 ;
	}
#Surface long-wave (thermal) radiation downward clear-sky
'228130' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 130 ;
	}
#100 metre wind speed
'228249' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 249 ;
	}
#Potential evaporation
'228251' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 251 ;
	}
#Volume-mean total column mixing ratio
'409000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 195 ;
	}
#Emission mass flux from vegetation
'412000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 192 ;
	}
#Emission mass flux from biomass burning
'417000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 193 ;
	}
#Emission mass flux from settlements
'432000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 194 ;
	}
#Emission mass flux from soil
'442000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 195 ;
	}
#Emission mass flux from wild animals
'443000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 196 ;
	}
#Emission mass flux from commercial buildings heating
'463000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 197 ;
	}
#Emission mass flux from residential heating
'464000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 198 ;
	}
#Emission mass flux from oil refineries and transformation industry
'465000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 199 ;
	}
#Emission mass flux from gas production
'466000' = {
	 localTablesVersion = 1 ;
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 77 ;
	 sourceSinkChemicalPhysicalProcess = 200 ;
	}
#Mean 2 metre temperature in the last 24 hours gradient
'129055' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 55 ;
	}
#Mean 2 metre dewpoint temperature in the last 24 hours gradient
'129056' = {
	 discipline = 192 ;
	 parameterCategory = 129 ;
	 parameterNumber = 56 ;
	}
#Mean vertical velocity
'130232' = {
	 discipline = 192 ;
	 parameterCategory = 130 ;
	 parameterNumber = 232 ;
	}
#Total precipitation less than 0.1 mm
'131064' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 64 ;
	}
#Total precipitation rate less than 1 mm/day
'131065' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 65 ;
	}
#Total precipitation rate of at least 3 mm/day
'131066' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 66 ;
	}
#Total precipitation rate of at least 5 mm/day
'131067' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 67 ;
	}
#10 metre Wind speed of at least 10 m/s
'131068' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 68 ;
	}
#10 metre Wind speed of at least 15 m/s
'131069' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 69 ;
	}
#2 metre temperature less than 273.15 K
'131073' = {
	 discipline = 192 ;
	 parameterCategory = 131 ;
	 parameterNumber = 73 ;
	}
#Period corresponding to maximum individual wave height
'140217' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 217 ;
	}
#Envelop-maximum individual wave height
'140218' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 218 ;
	}
#Model bathymetry
'140219' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 219 ;
	}
#Mean wave period based on first moment
'140220' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 220 ;
	}
#Mean zero-crossing wave period
'140221' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 221 ;
	}
#Wave spectral directional width
'140222' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 222 ;
	}
#Mean wave period based on first moment for wind waves
'140223' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 223 ;
	}
#Mean wave period based on second moment for wind waves
'140224' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 224 ;
	}
#Wave spectral directional width for wind waves
'140225' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 225 ;
	}
#Mean wave period based on first moment for swell
'140226' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 226 ;
	}
#Mean wave period based on second moment for swell
'140227' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 227 ;
	}
#Wave spectral directional width for swell
'140228' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 228 ;
	}
#Peak wave period
'140231' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 231 ;
	}
#Coefficient of drag with waves
'140233' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 233 ;
	}
#Mean direction of wind waves
'140235' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 235 ;
	}
#Significant height of total swell
'140237' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 237 ;
	}
#Mean direction of total swell
'140238' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 238 ;
	}
#Mean period of total swell
'140239' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 239 ;
	}
#Standard deviation wave height
'140240' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 240 ;
	}
#Mean of 10 metre wind speed
'140241' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 241 ;
	}
#Mean wind direction
'140242' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 242 ;
	}
#Standard deviation of 10 metre wind speed
'140243' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 243 ;
	}
#Mean square slope of waves
'140244' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 244 ;
	}
#10 metre wind speed
'140245' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 245 ;
	}
#Altimeter wave height
'140246' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 246 ;
	}
#Altimeter corrected wave height
'140247' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 247 ;
	}
#Altimeter range relative correction
'140248' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 248 ;
	}
#10 metre wind direction
'140249' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 249 ;
	}
#2D wave spectra (single)
'140251' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 251 ;
	}
#Wave spectral kurtosis
'140252' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 252 ;
	}
#Benjamin-Feir index
'140253' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 253 ;
	}
#Wave spectral peakedness
'140254' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 254 ;
	}
#Sea water potential temperature
'151129' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 129 ;
	}
#Sea water practical salinity
'151130' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 130 ;
	}
#Eastward surface sea water velocity
'151131' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 131 ;
	}
#Northward surface sea water velocity
'151132' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 132 ;
	}
#Upward sea water velocity
'151133' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 133 ;
	}
#Sea water sigma theta
'151138' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 138 ;
	}
#Sea surface height
'151145' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 145 ;
	}
#Surface downward eastward stress
'151153' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 153 ;
	}
#Surface downward northward stress
'151154' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 154 ;
	}
#Depth of 20C isotherm
'151163' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 163 ;
	}
#Average sea water practical salinity in the upper 300m
'151175' = {
	 discipline = 192 ;
	 parameterCategory = 151 ;
	 parameterNumber = 175 ;
	}
#Skin reservoir content
'160198' = {
	 discipline = 192 ;
	 parameterCategory = 160 ;
	 parameterNumber = 198 ;
	}
#Surface geopotential
'162051' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 51 ;
	}
#Total column vertically-integrated mass of atmosphere
'162053' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 53 ;
	}
#Total column vertically-integrated temperature
'162054' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 54 ;
	}
#Total column vertically-integrated water vapour
'162055' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 55 ;
	}
#Total column vertically-integrated cloud liquid water
'162056' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 56 ;
	}
#Total column vertically-integrated cloud frozen water
'162057' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 57 ;
	}
#Total column vertically-integrated ozone
'162058' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 58 ;
	}
#Total column vertically-integrated kinetic energy
'162059' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 59 ;
	}
#Total column vertically-integrated enthalpy
'162060' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 60 ;
	}
#Total column vertically-integrated potential + internal energy
'162061' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 61 ;
	}
#Total column vertically-integrated potential+internal+latent energy
'162062' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 62 ;
	}
#Total column vertically-integrated total energy
'162063' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 63 ;
	}
#Total column vertically-integrated energy conversion
'162064' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 64 ;
	}
#Total column vertically-integrated eastward mass flux
'162065' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 65 ;
	}
#Total column vertically-integrated northward mass flux
'162066' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 66 ;
	}
#Total column vertically-integrated eastward kinetic energy flux
'162067' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 67 ;
	}
#Total column vertically-integrated northward kinetic energy flux
'162068' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 68 ;
	}
#Total column vertically-integrated eastward heat flux
'162069' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 69 ;
	}
#Total column vertically-integrated northward heat flux
'162070' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 70 ;
	}
#Total column vertically-integrated eastward water vapour flux
'162071' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 71 ;
	}
#Total column vertically-integrated northward water vapour flux
'162072' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 72 ;
	}
#Total column vertically-integrated eastward geopotential flux
'162073' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 73 ;
	}
#Total column vertically-integrated northward geopotential flux
'162074' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 74 ;
	}
#Total column vertically-integrated eastward total energy flux
'162075' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 75 ;
	}
#Total column vertically-integrated northward total energy flux
'162076' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 76 ;
	}
#Total column vertically-integrated eastward ozone flux
'162077' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 77 ;
	}
#Total column vertically-integrated northward ozone flux
'162078' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 78 ;
	}
#Total column vertically-integrated divergence of mass flux
'162081' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 81 ;
	}
#Total column vertically-integrated divergence of kinetic energy flux
'162082' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 82 ;
	}
#Total column vertically-integrated divergence of thermal energy flux
'162083' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 83 ;
	}
#Total column vertically-integrated moisture divergence flux
'162084' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 84 ;
	}
#Total column vertically-integrated divergence of geopotential flux
'162085' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 85 ;
	}
#Total column vertically-integrated divergence of total energy flux
'162086' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 86 ;
	}
#Total column vertically-integrated divergence of ozone flux
'162087' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 87 ;
	}
#Time-integrated temperature tendency due to short-wave radiation
'162100' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 100 ;
	}
#Time-integrated temperature tendency due to long-wave radiation
'162101' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 101 ;
	}
#Time-integrated temperature tendency due to short wave radiation, clear sky
'162102' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 102 ;
	}
#Time-integrated temperature tendency due to long-wave radiation, clear sky
'162103' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 103 ;
	}
#Time-integrated updraught mass flux
'162104' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 104 ;
	}
#Time-integrated downdraught mass flux
'162105' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 105 ;
	}
#Time-integrated updraught detrainment rate
'162106' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 106 ;
	}
#Time-integrated downdraught detrainment rate
'162107' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 107 ;
	}
#Time-integrated total precipitation flux
'162108' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 108 ;
	}
#Time-integrated turbulent diffusion coefficient for heat
'162109' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 109 ;
	}
#Time-integrated temperature tendency due to parametrisations
'162110' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 110 ;
	}
#Time-integrated specific humidity tendency due to parametrisations
'162111' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 111 ;
	}
#Time-integrated eastward wind tendency due to parametrisations
'162112' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 112 ;
	}
#Time-integrated northward wind tendency due to parametrisations
'162113' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 113 ;
	}
#Variance of geopotential
'162206' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 206 ;
	}
#Covariance of geopotential/temperature
'162207' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 207 ;
	}
#Variance of temperature
'162208' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 208 ;
	}
#Covariance of geopotential/specific humidity
'162209' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 209 ;
	}
#Covariance of temperature/specific humidity
'162210' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 210 ;
	}
#Variance of specific humidity
'162211' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 211 ;
	}
#Covariance of u component/geopotential
'162212' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 212 ;
	}
#Covariance of u component/temperature
'162213' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 213 ;
	}
#Covariance of u component/specific humidity
'162214' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 214 ;
	}
#Variance of u component
'162215' = {
	 discipline = 192 ;
	 parameterCategory = 162 ;
	 parameterNumber = 215 ;
	}
#Time-mean large-scale precipitation rate
'172142' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 142 ;
	}
#Time-mean convective precipitation rate
'172143' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 143 ;
	}
#Time-mean total snowfall rate
'172144' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 144 ;
	}
#Time-mean surface sensible heat flux
'172146' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 146 ;
	}
#Time-mean surface latent heat flux
'172147' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 147 ;
	}
#Time-mean surface downward short-wave (solar) radiation flux
'172169' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 169 ;
	}
#Time-mean surface downward long-wave (thermal) radiation flux
'172175' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 175 ;
	}
#Time-mean surface net short-wave (solar) radiation flux
'172176' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 176 ;
	}
#Time-mean surface net long-wave (thermal) radiation flux
'172177' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 177 ;
	}
#Time-mean top net short-wave (solar) radiation flux
'172178' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 178 ;
	}
#Time-mean top net long-wave (thermal) radiation flux
'172179' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 179 ;
	}
#Time-mean eastward turbulent surface stress
'172180' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 180 ;
	}
#Time-mean northward turbulent surface stress
'172181' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 181 ;
	}
#Time-mean evaporation rate
'172182' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 182 ;
	}
#Time-mean runoff rate
'172205' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 205 ;
	}
#Time-mean top net short-wave (solar) radiation flux, clear sky
'172208' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 208 ;
	}
#Time-mean top net long-wave (thermal) radiation flux, clear sky
'172209' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 209 ;
	}
#Time-mean surface net short-wave (solar) radiation flux, clear sky
'172210' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 210 ;
	}
#Time-mean surface net long-wave (thermal) radiation flux, clear sky
'172211' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 211 ;
	}
#Time-mean short-wave (solar) insolation rate
'172212' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 212 ;
	}
#Time-mean total precipitation rate
'172228' = {
	 discipline = 192 ;
	 parameterCategory = 172 ;
	 parameterNumber = 228 ;
	}
#Surface runoff
'174008' = {
	 discipline = 192 ;
	 parameterCategory = 174 ;
	 parameterNumber = 8 ;
	}
#Sea-ice thickness
'174098' = {
	 discipline = 192 ;
	 parameterCategory = 174 ;
	 parameterNumber = 98 ;
	}
#Snow depth
'190141' = {
	 discipline = 192 ;
	 parameterCategory = 190 ;
	 parameterNumber = 141 ;
	}
#Nitrogen dioxide mass mixing ratio
'210121' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 121 ;
	}
#Sulphur dioxide mass mixing ratio
'210122' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 122 ;
	}
#Carbon monoxide mass mixing ratio
'210123' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 123 ;
	}
#Ozone mass mixing ratio (full chemistry scheme)
'210203' = {
	 discipline = 192 ;
	 parameterCategory = 210 ;
	 parameterNumber = 203 ;
	}
#Nitrogen dioxide mass mixing ratio difference
'211121' = {
	 discipline = 192 ;
	 parameterCategory = 211 ;
	 parameterNumber = 121 ;
	}
#Sulphur dioxide mass mixing ratio difference
'211122' = {
	 discipline = 192 ;
	 parameterCategory = 211 ;
	 parameterNumber = 122 ;
	}
#Carbon monoxide mass mixing ratio difference
'211123' = {
	 discipline = 192 ;
	 parameterCategory = 211 ;
	 parameterNumber = 123 ;
	}
#Ozone mass mixing ratio difference (full chemistry scheme)
'211203' = {
	 discipline = 192 ;
	 parameterCategory = 211 ;
	 parameterNumber = 203 ;
	}
#Friction velocity
'228003' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 3 ;
	}
#Time-mean 2 metre temperature
'228004' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 4 ;
	}
#Time-mean 10 metre wind speed
'228005' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 5 ;
	}
#Mean total cloud cover
'228006' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 6 ;
	}
#Lake total depth
'228007' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 7 ;
	}
#Lake mix-layer temperature
'228008' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 8 ;
	}
#Lake mix-layer depth
'228009' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 9 ;
	}
#Lake bottom temperature
'228010' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 10 ;
	}
#Lake total layer temperature
'228011' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 11 ;
	}
#Lake shape factor
'228012' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 12 ;
	}
#Lake ice surface temperature
'228013' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 13 ;
	}
#Lake ice total depth
'228014' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 14 ;
	}
#Minimum vertical gradient of refractivity inside trapping layer
'228015' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 15 ;
	}
#Mean vertical gradient of refractivity inside trapping layer
'228016' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 16 ;
	}
#Duct base height
'228017' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 17 ;
	}
#Trapping layer base height
'228018' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 18 ;
	}
#Trapping layer top height
'228019' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 19 ;
	}
#10 metre u-component of neutral wind
'228131' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 131 ;
	}
#10 metre v-component of neutral wind
'228132' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 132 ;
	}
#U-component surface stokes drift
'140215' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 215 ;
	}
#V-component surface stokes drift
'140216' = {
	 discipline = 192 ;
	 parameterCategory = 140 ;
	 parameterNumber = 216 ;
	}
#100 metre U wind component
'228246' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 246 ;
	}
#100 metre V wind component
'228247' = {
	 discipline = 192 ;
	 parameterCategory = 228 ;
	 parameterNumber = 247 ;
}
