'g'={gridName="N16"; global=1;}
'g'={gridName="N24"; global=1;}
'g'={gridName="N32"; global=1;}
'g'={gridName="N48"; global=1;}
'g'={gridName="N64"; global=1;}
'g'={gridName="N80"; global=1;}
'g'={gridName="N96"; global=1;}
'g'={gridName="N128"; global=1;}
'g'={gridName="N160"; global=1;}
'g'={gridName="N200"; global=1;}
'g'={gridName="N256"; global=1;}
'g'={gridName="N320"; global=1;}
'g'={gridName="N400"; global=1;}
'g'={gridName="N512"; global=1;}
'g'={gridName="N576"; global=1;}
'g'={gridName="N640"; global=1;}
'g'={gridName="N800"; global=1;}
'g'={gridName="N1024"; global=1;}
'g'={gridName="N1280"; global=1;}
'g'={gridName="N1600"; global=1;}
'g'={gridName="N2000"; global=1;}
'g'={gridName="N4000"; global=1;}
'g'={gridName="N8000"; global=1;}
'g'={gridName="O16"; global=1;}
'g'={gridName="O24"; global=1;}
'g'={gridName="O32"; global=1;}
'g'={gridName="O48"; global=1;}
'g'={gridName="O64"; global=1;}
'g'={gridName="O80"; global=1;}
'g'={gridName="O96"; global=1;}
'g'={gridName="O128"; global=1;}
'g'={gridName="O160"; global=1;}
'g'={gridName="O200"; global=1;}
'g'={gridName="O256"; global=1;}
'g'={gridName="O320"; global=1;}
'g'={gridName="O400"; global=1;}
'g'={gridName="O512"; global=1;}
'g'={gridName="O576"; global=1;}
'g'={gridName="O640"; global=1;}
'g'={gridName="O800"; global=1;}
'g'={gridName="O1024"; global=1;}
'g'={gridName="O1280"; global=1;}
'g'={gridName="O1600"; global=1;}
'g'={gridName="O2000"; global=1;}
'g'={gridName="O4000"; global=1;}
'g'={gridName="O8000"; global=1;}
'g'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-81000;longitudeOfLastGridPoint=358500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-81000;longitudeOfLastGridPoint=358500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=358500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359000;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359640;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=357000;}
'g'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=357000;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=359750;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78120;longitudeOfLastGridPoint=359640;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=359500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=359000;}
'g'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=357000;}
'g'={latitudeOfFirstGridPoint=89731;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-89731;longitudeOfLastGridPoint=359648;}

'm'={latitudeOfFirstGridPoint=66000;longitudeOfFirstGridPoint=-6000;latitudeOfLastGridPoint=30000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=66000;longitudeOfFirstGridPoint=354000;latitudeOfLastGridPoint=30000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=-98000;latitudeOfLastGridPoint=5000;longitudeOfLastGridPoint=54000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=-98000;latitudeOfLastGridPoint=9000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=262000;latitudeOfLastGridPoint=9000;longitudeOfLastGridPoint=42000;}

'n'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=0;longitudeOfLastGridPoint=359500;}
's'={latitudeOfFirstGridPoint=-500;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-81000;longitudeOfLastGridPoint=359500;}

'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359750;}
'g' = { generatingProcessIdentifier=104 ;}
'g' = { generatingProcessIdentifier=105 ;}
'g' = { generatingProcessIdentifier=106 ;}
'g' = { generatingProcessIdentifier=107 ;}
'g' = { generatingProcessIdentifier=108 ;}
'g' = { generatingProcessIdentifier=109 ;}
'g' = { generatingProcessIdentifier=110 ;}
'g' = { generatingProcessIdentifier=111 ;}
'g' = { generatingProcessIdentifier=112 ;}
'g' = { generatingProcessIdentifier=113 ;}
'g' = { generatingProcessIdentifier=114 ;}
'g' = { generatingProcessIdentifier=115 ;}
'g' = { generatingProcessIdentifier=116 ;}
'g' = { generatingProcessIdentifier=117 ;}
'g' = { generatingProcessIdentifier=118 ;}
'g' = { generatingProcessIdentifier=119 ;}
'g' = { generatingProcessIdentifier=120 ;}
'm' = { generatingProcessIdentifier=204 ;}
'm' = { generatingProcessIdentifier=205 ;}
'm' = { generatingProcessIdentifier=206 ;}
'm' = { generatingProcessIdentifier=207 ;}
'm' = { generatingProcessIdentifier=208 ;}
'm' = { generatingProcessIdentifier=209 ;}
'm' = { generatingProcessIdentifier=210 ;}
'm' = { generatingProcessIdentifier=211 ;}
'm' = { generatingProcessIdentifier=212 ;}
'm' = { generatingProcessIdentifier=213 ;}
'm' = { generatingProcessIdentifier=214 ;}
'm' = { generatingProcessIdentifier=215 ;}
'm' = { generatingProcessIdentifier=216 ;}
'm' = { generatingProcessIdentifier=217 ;}
'm' = { generatingProcessIdentifier=218 ;}
'm' = { generatingProcessIdentifier=219 ;}
'm' = { generatingProcessIdentifier=220 ;}


