# (C) Copyright 2005- ECMWF.

# # Members iformation of
# # SREPS Short-Range EPS
#
# Last update: 20070323
#

template mars_labeling "grib1/mars_labeling.def";

unsigned[1] perturbationNumber : dump;
alias number = perturbationNumber;

unsigned[1] numberOfForecastsInEnsemble : dump ;

ascii[8]    'Model_Identifier' ;
ascii[8]    'LBC_Initial_Conditions' ;
ascii[4]    'Model_LBC_Member_Identifier' ;
ascii[8]    'Model_Additional_Information' ;

pad padding_loc245_1(20);

unsigned[2]  Extra_Data_FreeFormat_0_none  : dump ;
position offsetFreeFormData;
unsigned[1] freeFormData[Extra_Data_FreeFormat_0_none] : dump ;

padtomultiple padding_loc245_2(offsetSection1,80);
