'g'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-81000;longitudeOfLastGridPoint=358500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-81000;longitudeOfLastGridPoint=358500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=358500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359000;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359640;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=357000;}
'g'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=357000;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=359750;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78120;longitudeOfLastGridPoint=359640;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=359500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=359000;}
'g'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-78000;longitudeOfLastGridPoint=357000;}
'g'={latitudeOfFirstGridPoint=89731;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-89731;longitudeOfLastGridPoint=359648;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359750;}
'm'={latitudeOfFirstGridPoint=66000;longitudeOfFirstGridPoint=-6000;latitudeOfLastGridPoint=30000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=66000;longitudeOfFirstGridPoint=354000;latitudeOfLastGridPoint=30000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=-98000;latitudeOfLastGridPoint=5000;longitudeOfLastGridPoint=54000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=-98000;latitudeOfLastGridPoint=9000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=262000;latitudeOfLastGridPoint=9000;longitudeOfLastGridPoint=42000;}
'm'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=-98000;latitudeOfLastGridPoint=5100;longitudeOfLastGridPoint=53950;}
'n'={latitudeOfFirstGridPoint=81000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=0;longitudeOfLastGridPoint=359500;}
's'={latitudeOfFirstGridPoint=-500;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-81000;longitudeOfLastGridPoint=359500;}
'g'={latitudeOfFirstGridPoint=90000;longitudeOfFirstGridPoint=0;latitudeOfLastGridPoint=-90000;longitudeOfLastGridPoint=359750;}


