# Automatically generated by ./create_defs.grib2.mtg2.pl, do not edit
#Total precipitation of at least 1 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 5 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 5 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 10 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 20 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 20 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 40 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 40 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 60 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 60 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 80 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 80 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 100 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 100 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 150 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 150 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 200 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 200 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 300 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 3 ;
	 scaleFactorOfLowerLimit = -2 ;
	 probabilityType = 3 ;
	}
#Stream function
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	}
#Velocity potential
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	}
#Potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	}
#Surface photosynthetically active radiation, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Unbalanced component of temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	}
#Unbalanced component of logarithm of surface pressure
'~' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 31 ;
	}
#Unbalanced component of divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 45 ;
	}
#Lake cover
'(0 - 1)' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#Low vegetation cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	}
#High vegetation cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	}
#Type of low vegetation
'(Code table 4.234)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	}
#Type of high vegetation
'(Code table 4.234)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	}
#Sea ice area fraction
'(0 - 1)' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Snow density
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	}
#Sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil type
'(Code table 4.213)' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	}
#Surface direct normal short-wave (solar) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated magnitude of turbulent surface stress
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Large-scale precipitation fraction
's' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Montgomery potential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	}
#Pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	}
#Surface downward UV radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Photosynthetically active radiation at the surface
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective available potential energy
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Potential vorticity
'K m**2 kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	}
#Leaf area index, low vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	}
#Leaf area index, high vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	}
#Standard deviation of filtered subgrid orography (climatological)
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 38 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Specific rain water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	}
#Specific snow water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	}
#Eta-coordinate vertical velocity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	}
#Total column cloud liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column cloud ice water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Surface emissivity
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Geopotential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#U component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#V component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	}
#Specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Surface pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Vertical velocity
'Pa s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	}
#Total column water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated water vapour
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Vorticity (relative)
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	}
#Time-integrated boundary layer dissipation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated surface sensible heat net flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated surface latent heat net flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Charnock
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	}
#Surface net radiation (SW and LW)
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net radiation (SW and LW)
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Mean sea level pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	}
#Divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	}
#Geopotential height
'gpm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Boundary layer height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	}
#Standard deviation of sub-gridscale orography
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	}
#Anisotropy of sub-gridscale orography
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 24 ;
	}
#Angle of sub-gridscale orography
'radians' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	}
#Slope of sub-gridscale orography
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	}
#10 metre U wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#10 metre V wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#2 metre temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#2 metre dewpoint temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Surface short-wave (solar) radiation downwards
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Land-sea mask
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness (climatological)
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Surface long-wave (thermal) radiation downwards
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net short-wave (solar) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net long-wave (thermal) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net short-wave (solar) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net long-wave (thermal) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated eastward turbulent surface stress
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward turbulent surface stress
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Sunshine duration
's' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Brightness temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 4 ;
	}
#Eastward gravity wave surface stress
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Northward gravity wave surface stress
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Gravity wave dissipation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Ozone mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	}
#10 metre wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Top net short-wave (solar) radiation, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net long-wave (thermal) radiation, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net short-wave (solar) radiation, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net long-wave (thermal) radiation, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#TOA incident short-wave (solar) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Vertically integrated moisture divergence
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Instantaneous eastward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous surface sensible heat net flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous moisture flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Logarithm of surface roughness length for heat (climatological)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Skin temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Temperature of snow layer
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	}
#Forecast surface roughness
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Forecast logarithm of surface roughness for heat
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Specific cloud liquid water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	}
#Specific cloud ice water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	}
#Fraction of cloud cover
'(0 - 1)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	}
#Large-scale precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snow depth
'm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	}
#Low cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	}
#Medium cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	}
#High cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	}
#Probability of a tropical storm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 6 ;
	 probabilityType = 9 ;
	}
#Probability of a hurricane
'%' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 5 ;
	 probabilityType = 9 ;
	}
#Probability of a tropical depression
'%' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 7 ;
	 probabilityType = 9 ;
	}
#Total precipitation of at least 25 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 50 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 50 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust of at least 10 m/s
'%' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre convective gust of at least 25 m/s
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Hail of at least 2 cm diameter
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 134 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 2 ;
	 probabilityType = 3 ;
	}
#Hail of at least 5 cm diameter
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 134 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 3 ;
	 probabilityType = 3 ;
	}
#Thunderstorm in vicinity (lightning-based)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 0 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Convective available potential energy shear index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Water vapour flux index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 166 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = [102,0] ;
	}
#Convective available potential energy index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Maximum of significant wave height index
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Probability of temperature standardized anomaly greater than 1 standard deviation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Probability of temperature standardized anomaly greater than 1.5 standard deviation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = 15 ;
	 scaleFactorOfLowerLimit = 1 ;
	 probabilityType = 3 ;
	}
#Probability of temperature standardized anomaly greater than 2 standard deviation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = 2 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Probability of temperature standardized anomaly less than -1 standard deviation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = -1 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 0 ;
	}
#Probability of temperature standardized anomaly less than -1.5 standard deviation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = -15 ;
	 scaleFactorOfLowerLimit = 1 ;
	 probabilityType = 0 ;
	}
#Probability of temperature standardized anomaly less than -2 standard deviation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = -2 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 0 ;
	}
#Wave induced mean sea level correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	}
#Ratio of wave angular and frequency width
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	}
#Number of events in freak waves statistics
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	}
#U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#U-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#V-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Wave turbulent energy flux into ocean
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	}
#Wave directional width of first swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	}
#Wave frequency width of first swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	}
#Wave directional width of second swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	}
#Wave frequency width of second swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	}
#Wave directional width of third swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	}
#Wave frequency width of third swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	}
#Wave energy flux magnitude
'W m**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	}
#Wave energy flux mean direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 12 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 12 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 14 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 14 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 17 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 17 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 21 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 21 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 25 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 25 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 30 ;
	}
#Significant wave height of all waves with period larger than 10s
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 3 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	}
#Significant wave height of first swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	}
#Mean wave direction of first swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	}
#Mean wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	}
#Significant wave height of second swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	}
#Mean wave direction of second swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	}
#Mean wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	}
#Significant wave height of third swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	}
#Mean wave direction of third swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	}
#Mean wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	}
#Time domain maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	}
#Time domain maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	}
#Space time maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	}
#Space time maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	}
#Peak wave period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	}
#Peak wave period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	}
#Peak wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	}
#Peak wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	}
#Peak wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	}
#Peak wave direction (total)
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	}
#Peak wave direction of wind waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	}
#Peak wave direction of total swell
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	}
#Peak wave direction of first swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	}
#Peak wave direction of second swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	}
#Peak wave direction of third swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	}
#Whitecap fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	}
#Benjamin-Feir index 2D
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	}
#Crest-trough correlation
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	}
#X component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	}
#Y component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	}
#Wave Spectral Skewness
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	}
#Free convective velocity over the oceans
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	}
#Air density over the oceans
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	}
#Normalized energy flux into waves
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	}
#Normalized energy flux into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	}
#Normalized stress into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	}
#Time-mean wave induced mean sea level correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ratio of wave angular and frequency width
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean number of events in freak waves statistics
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave turbulent energy flux into ocean
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave directional width of first swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave frequency width of first swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave directional width of second swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave frequency width of second swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave directional width of third swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave frequency width of third swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave energy flux magnitude
'W m**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave energy flux mean direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 12 ;
	}
#Time-mean significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 12 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 14 ;
	}
#Time-mean significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 14 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 17 ;
	}
#Time-mean significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 17 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 21 ;
	}
#Time-mean significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 21 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 25 ;
	}
#Time-mean significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 25 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 30 ;
	}
#Time-mean significant wave height of all waves with period larger than 10s
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfWavePeriodInterval = 3 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	}
#Time-mean significant wave height of first swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction of first swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant wave height of second swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction of second swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant wave height of third swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction of third swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean time domain maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean time domain maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean space time maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean space time maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction (total)
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of wind waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of total swell
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of first swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of second swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of third swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean whitecap fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Benjamin-Feir index 2D
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean crest-trough correlation
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave Spectral Skewness
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean free convective velocity over the oceans
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean air density over the oceans
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean normalized energy flux into waves
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean normalized energy flux into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean normalized stress into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean period corresponding to maximum individual wave height
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean envelop-maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on first moment
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean zero-crossing wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral directional width
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on first moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on second moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral directional width for wind waves
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on first moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on second moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral directional width for swell
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant height of combined wind waves and swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean coefficient of drag with waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant height of wind waves
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean direction of wind waves
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant height of total swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean direction of total swell
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean square slope of waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind speed
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind direction
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral kurtosis
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean benjamin-Feir index
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral peakedness
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-maximum wave induced mean sea level correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ratio of wave angular and frequency width
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum number of events in freak waves statistics
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave turbulent energy flux into ocean
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave directional width of first swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave frequency width of first swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave directional width of second swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave frequency width of second swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave directional width of third swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave frequency width of third swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave energy flux magnitude
'W m**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave energy flux mean direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 12 ;
	}
#Time-maximum significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 12 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 14 ;
	}
#Time-maximum significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 14 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 17 ;
	}
#Time-maximum significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 17 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 21 ;
	}
#Time-maximum significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 21 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 25 ;
	}
#Time-maximum significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 25 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 30 ;
	}
#Time-maximum significant wave height of all waves with period larger than 10s
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfWavePeriodInterval = 3 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	}
#Time-maximum significant wave height of first swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction of first swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant wave height of second swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction of second swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant wave height of third swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction of third swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum time domain maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum time domain maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum space time maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum space time maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction (total)
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of wind waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of total swell
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of first swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of second swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of third swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum whitecap fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Benjamin-Feir index 2D
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum crest-trough correlation
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum X component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Y component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave Spectral Skewness
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum free convective velocity over the oceans
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum air density over the oceans
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum normalized energy flux into waves
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum normalized energy flux into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum normalized stress into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum period corresponding to maximum individual wave height
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum envelop-maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on first moment
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean zero-crossing wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral directional width
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on first moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on second moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral directional width for wind waves
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on first moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on second moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral directional width for swell
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant height of combined wind waves and swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum coefficient of drag with waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant height of wind waves
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean direction of wind waves
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant height of total swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean direction of total swell
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean square slope of waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind speed
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind direction
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral kurtosis
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum benjamin-Feir index
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral peakedness
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum wave induced mean sea level correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ratio of wave angular and frequency width
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum number of events in freak waves statistics
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave turbulent energy flux into ocean
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave directional width of first swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave frequency width of first swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave directional width of second swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave frequency width of second swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave directional width of third swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave frequency width of third swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave energy flux magnitude
'W m**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave energy flux mean direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 12 ;
	}
#Time-minimum significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 12 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 14 ;
	}
#Time-minimum significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 14 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 17 ;
	}
#Time-minimum significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 17 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 21 ;
	}
#Time-minimum significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 21 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 25 ;
	}
#Time-minimum significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 25 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 30 ;
	}
#Time-minimum significant wave height of all waves with period larger than 10s
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfWavePeriodInterval = 3 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	}
#Time-minimum significant wave height of first swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction of first swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant wave height of second swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction of second swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant wave height of third swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction of third swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum time domain maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum time domain maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum space time maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum space time maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction (total)
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of wind waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of total swell
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of first swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of second swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of third swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum whitecap fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Benjamin-Feir index 2D
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum crest-trough correlation
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum X component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Y component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave Spectral Skewness
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum free convective velocity over the oceans
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum air density over the oceans
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum normalized energy flux into waves
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum normalized energy flux into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum normalized stress into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum period corresponding to maximum individual wave height
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum envelop-maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on first moment
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean zero-crossing wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral directional width
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on first moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on second moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral directional width for wind waves
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on first moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on second moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral directional width for swell
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant height of combined wind waves and swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum coefficient of drag with waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant height of wind waves
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean direction of wind waves
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant height of total swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean direction of total swell
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean square slope of waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind speed
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind direction
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral kurtosis
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum benjamin-Feir index
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral peakedness
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation wave induced mean sea level correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ratio of wave angular and frequency width
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation number of events in freak waves statistics
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component of atmospheric surface momentum flux
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component of surface momentum flux into ocean
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave turbulent energy flux into ocean
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave directional width of first swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave frequency width of first swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave directional width of second swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave frequency width of second swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave directional width of third swell partition
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave frequency width of third swell partition
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave energy flux magnitude
'W m**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave energy flux mean direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 12 ;
	}
#Time-standard-deviation significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 12 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 14 ;
	}
#Time-standard-deviation significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 14 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 17 ;
	}
#Time-standard-deviation significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 17 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 21 ;
	}
#Time-standard-deviation significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 21 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 25 ;
	}
#Time-standard-deviation significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 25 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 30 ;
	}
#Time-standard-deviation significant wave height of all waves with period larger than 10s
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfWavePeriodInterval = 3 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	}
#Time-standard-deviation significant wave height of first swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction of first swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant wave height of second swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction of second swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant wave height of third swell partition
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction of third swell partition
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation time domain maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation time domain maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation space time maximum individual crest height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation space time maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of first swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of second swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of third swell partition
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction (total)
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of wind waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of total swell
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of first swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of second swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of third swell partition
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation whitecap fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Benjamin-Feir index 2D
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation crest-trough correlation
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation X component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Y component of the wave radiative stress to sea ice
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave Spectral Skewness
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation free convective velocity over the oceans
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation air density over the oceans
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation normalized energy flux into waves
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation normalized energy flux into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation normalized stress into ocean
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation period corresponding to maximum individual wave height
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation envelop-maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on first moment
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean zero-crossing wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral directional width
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on first moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on second moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral directional width for wind waves
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on first moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on second moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral directional width for swell
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant height of combined wind waves and swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation coefficient of drag with waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant height of wind waves
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean direction of wind waves
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant height of total swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean direction of total swell
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean square slope of waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind speed
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind direction
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral kurtosis
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation benjamin-Feir index
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral peakedness
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Mean sea water temperature in the upper 300 m
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column vertically-integrated divergence of water geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of water enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of water vapour flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of cloud liquid water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of cloud ice water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of rain flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of snow flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated net source of ozone
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Total column vertically-integrated water enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Temperature tendency due to short-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#Temperature tendency due to long-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Temperature tendency due to short-wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Temperature tendency due to long-wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Updraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	}
#Downdraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	}
#Updraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	}
#Downdraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	}
#Total precipitation flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	}
#Turbulent diffusion coefficient for heat
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#Temperature tendency due to parametrisations
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	}
#Specific humidity tendency due to parametrisations
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	}
#Eastward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	}
#Northward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	}
#2 metre specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Injection height (from IS4FIRES)
'm' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	}
#Particulate matter d <= 1 um
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 0 ;
	 scaleFactorOfSecondSize = missing() ;
	 scaledValueOfFirstSize = 1 ;
	 scaleFactorOfFirstSize = 6 ;
	 typeOfSizeInterval = 5 ;
	 aerosolType = 62026 ;
	 scaledValueOfSecondSize = missing() ;
	 is_aerosol = 1 ;
	}
#Particulate matter d <= 2.5 um
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 0 ;
	 scaleFactorOfSecondSize = missing() ;
	 scaledValueOfFirstSize = 25 ;
	 scaleFactorOfFirstSize = 7 ;
	 typeOfSizeInterval = 5 ;
	 aerosolType = 62026 ;
	 scaledValueOfSecondSize = missing() ;
	 is_aerosol = 1 ;
	}
#Particulate matter d <= 10 um
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 0 ;
	 scaleFactorOfSecondSize = missing() ;
	 scaledValueOfFirstSize = 10 ;
	 scaleFactorOfFirstSize = 6 ;
	 typeOfSizeInterval = 5 ;
	 aerosolType = 62026 ;
	 scaledValueOfSecondSize = missing() ;
	 is_aerosol = 1 ;
	}
#Mean height of maximum injection
'm' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	}
#Plume top height above surface
'm' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 42 ;
	}
#Volcanic sulfur dioxide mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#UV visible albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Plume bottom height above surface
'm' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	}
#Ammonium aerosol mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Nitrate aerosol optical depth at 550 nm
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 102 ;
	 scaleFactorOfFirstWavelength = 8 ;
	 typeOfSizeInterval = 255 ;
	 aerosolType = 62004 ;
	 scaledValueOfFirstWavelength = 55 ;
	 typeOfWavelengthInterval = 11 ;
	 is_aerosol_optical = 1 ;
	}
#Ammonium aerosol optical depth at 550 nm
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 102 ;
	 scaleFactorOfFirstWavelength = 8 ;
	 typeOfSizeInterval = 255 ;
	 aerosolType = 62003 ;
	 scaledValueOfFirstWavelength = 55 ;
	 typeOfWavelengthInterval = 11 ;
	 is_aerosol_optical = 1 ;
	}
#Biogenic secondary organic aerosol mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Anthropogenic secondary organic aerosol mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Near IR albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Volcanic sulfur dioxide mass mixing ratio increment
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Ammonium aerosol mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62003 ;
	 typeOfGeneratingProcess = 20 ;
	 is_aerosol = 1 ;
	}
#Biogenic secondary organic aerosol mass mixing ratio increment
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62032 ;
	 typeOfGeneratingProcess = 20 ;
	 is_aerosol = 1 ;
	}
#Anthropogenic secondary organic aerosol mass mixing ratio increment
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62033 ;
	 typeOfGeneratingProcess = 20 ;
	 is_aerosol = 1 ;
	}
#Random pattern 1 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 1 ;
	}
#Random pattern 2 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 2 ;
	}
#Random pattern 3 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 3 ;
	}
#Random pattern 4 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 4 ;
	}
#Random pattern 5 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 5 ;
	}
#Random pattern 6 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 6 ;
	}
#Random pattern 7 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 7 ;
	}
#Random pattern 8 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 8 ;
	}
#Random pattern 9 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 9 ;
	}
#Random pattern 10 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 10 ;
	}
#Random pattern 11 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 11 ;
	}
#Random pattern 12 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 12 ;
	}
#Random pattern 13 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 13 ;
	}
#Random pattern 14 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 14 ;
	}
#Random pattern 15 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 15 ;
	}
#Random pattern 16 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 16 ;
	}
#Random pattern 17 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 17 ;
	}
#Random pattern 18 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 18 ;
	}
#Random pattern 19 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 19 ;
	}
#Random pattern 20 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 20 ;
	}
#Random pattern 21 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 21 ;
	}
#Random pattern 22 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 22 ;
	}
#Random pattern 23 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 23 ;
	}
#Random pattern 24 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 24 ;
	}
#Random pattern 25 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 25 ;
	}
#Random pattern 26 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 26 ;
	}
#Random pattern 27 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 27 ;
	}
#Random pattern 28 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 28 ;
	}
#Random pattern 29 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 29 ;
	}
#Random pattern 30 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 30 ;
	}
#Random pattern 31 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 31 ;
	}
#Random pattern 32 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 32 ;
	}
#Random pattern 33 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 33 ;
	}
#Random pattern 34 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 34 ;
	}
#Random pattern 35 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 35 ;
	}
#Random pattern 36 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 36 ;
	}
#Random pattern 37 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 37 ;
	}
#Random pattern 38 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 38 ;
	}
#Random pattern 39 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 39 ;
	}
#Random pattern 40 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 40 ;
	}
#Random pattern 41 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 41 ;
	}
#Random pattern 42 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 42 ;
	}
#Random pattern 43 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 43 ;
	}
#Random pattern 44 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 44 ;
	}
#Random pattern 45 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 45 ;
	}
#Random pattern 46 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 46 ;
	}
#Random pattern 47 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 47 ;
	}
#Random pattern 48 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 48 ;
	}
#Random pattern 49 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 49 ;
	}
#Random pattern 50 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 50 ;
	}
#Random pattern 51 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 51 ;
	}
#Random pattern 52 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 52 ;
	}
#Random pattern 53 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 53 ;
	}
#Random pattern 54 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 54 ;
	}
#Random pattern 55 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 55 ;
	}
#Random pattern 56 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 56 ;
	}
#Random pattern 57 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 57 ;
	}
#Random pattern 58 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 58 ;
	}
#Random pattern 59 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 59 ;
	}
#Random pattern 60 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 60 ;
	}
#Random pattern 61 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 61 ;
	}
#Random pattern 62 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 62 ;
	}
#Random pattern 63 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 63 ;
	}
#Random pattern 64 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 64 ;
	}
#Random pattern 65 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 65 ;
	}
#Random pattern 66 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 66 ;
	}
#Random pattern 67 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 67 ;
	}
#Random pattern 68 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 68 ;
	}
#Random pattern 69 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 69 ;
	}
#Random pattern 70 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 70 ;
	}
#Random pattern 71 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 71 ;
	}
#Random pattern 72 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 72 ;
	}
#Random pattern 73 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 73 ;
	}
#Random pattern 74 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 74 ;
	}
#Random pattern 75 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 75 ;
	}
#Random pattern 76 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 76 ;
	}
#Random pattern 77 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 77 ;
	}
#Random pattern 78 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 78 ;
	}
#Random pattern 79 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 79 ;
	}
#Random pattern 80 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 80 ;
	}
#Random pattern 81 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 81 ;
	}
#Random pattern 82 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 82 ;
	}
#Random pattern 83 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 83 ;
	}
#Random pattern 84 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 84 ;
	}
#Random pattern 85 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 85 ;
	}
#Random pattern 86 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 86 ;
	}
#Random pattern 87 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 87 ;
	}
#Random pattern 88 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 88 ;
	}
#Random pattern 89 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 89 ;
	}
#Random pattern 90 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 90 ;
	}
#Random pattern 91 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 91 ;
	}
#Random pattern 92 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 92 ;
	}
#Random pattern 93 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 93 ;
	}
#Random pattern 94 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 94 ;
	}
#Random pattern 95 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 95 ;
	}
#Random pattern 96 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 96 ;
	}
#Random pattern 97 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 97 ;
	}
#Random pattern 98 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 98 ;
	}
#Random pattern 99 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 99 ;
	}
#Random pattern 100 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 100 ;
	}
#Random pattern 101 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 101 ;
	}
#Random pattern 102 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 102 ;
	}
#Random pattern 103 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 103 ;
	}
#Random pattern 104 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 104 ;
	}
#Random pattern 105 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 105 ;
	}
#Random pattern 106 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 106 ;
	}
#Random pattern 107 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 107 ;
	}
#Random pattern 108 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 108 ;
	}
#Random pattern 109 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 109 ;
	}
#Random pattern 110 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 110 ;
	}
#Random pattern 111 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 111 ;
	}
#Random pattern 112 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 112 ;
	}
#Random pattern 113 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 113 ;
	}
#Random pattern 114 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 114 ;
	}
#Random pattern 115 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 115 ;
	}
#Random pattern 116 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 116 ;
	}
#Random pattern 117 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 117 ;
	}
#Random pattern 118 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 118 ;
	}
#Random pattern 119 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 119 ;
	}
#Random pattern 120 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 120 ;
	}
#Random pattern 121 for SPP scheme
'dimensionless' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 121 ;
	}
#Cosine of solar zenith angle
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#UV biologically effective dose
'~' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#UV biologically effective dose clear-sky
'~' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Dry deposition of ammonium aerosol
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Sedimentation of ammonium aerosol
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 11 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of ammonium aerosol by large-scale precipitation
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 9 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of ammonium aerosol by convective precipitation
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 10 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Vertically integrated mass of ammonium aerosol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Dry deposition of biogenic secondary organic aerosol
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 12 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Sedimentation of biogenic secondary organic aerosol
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 11 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of biogenic secondary organic aerosol by large-scale precipitation
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 9 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of biogenic secondary organic aerosol by convective precipitation
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 10 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Vertically integrated mass of biogenic secondary organic aerosol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Dry deposition of anthropogenic secondary organic aerosol
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 12 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Sedimentation of anthropogenic secondary organic aerosol
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 11 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of anthropogenic secondary organic aerosol by large-scale precipitation
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 9 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of anthropogenic secondary organic aerosol by convective precipitation
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 10 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Vertically integrated mass of anthropogenic secondary organic aerosol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Secondary organic aerosol optical depth at 550 nm
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 102 ;
	 scaleFactorOfFirstWavelength = 9 ;
	 aerosolType = 62012 ;
	 scaledValueOfFirstWavelength = 550 ;
	 scaledValueOfSecondWavelength = missing() ;
	 typeOfWavelengthInterval = 11 ;
	 scaleFactorOfSecondWavelength = missing() ;
	 is_aerosol_optical = 1 ;
	}
#Ammonia mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 9 ;
	 is_chemical = 1 ;
	}
#Nitrogen monoxide mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 11 ;
	 is_chemical = 1 ;
	}
#Aromatic peroxy radical mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Ethyne mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Acetonitrile mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Methyl peroxy nitrate mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Hydrogen cyanide mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Hydroperoxy aldehydes type 1 mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Hydroperoxy aldehydes type 2 mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Isoprene peroxy type B mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Isoprene peroxy type D mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Volcanic sulfur dioxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Column integrated mass density of Aromatic peroxy radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Ethyne
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Acetonitrile
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Methyl peroxy nitrate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Hydrogen cyanide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Hydroperoxy aldehydes type 1
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Hydroperoxy aldehydes type 2
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Isoprene peroxy type B
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Isoprene peroxy type D
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Volcanic sulfur dioxide
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Atmosphere emission mass flux of Aromatic peroxy radical
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Ethyne
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Acetonitrile
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Methyl peroxy nitrate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Hydrogen cyanide
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Hydroperoxy aldehydes type 1
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Hydroperoxy aldehydes type 2
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Isoprene peroxy type B
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Isoprene peroxy type D
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Volcanic sulfur dioxide
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Dry deposition velocity of Aromatic peroxy radical
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Ethyne
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Acetonitrile
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Methyl peroxy nitrate
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Hydrogen cyanide
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Hydroperoxy aldehydes type 1
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Hydroperoxy aldehydes type 2
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Isoprene peroxy type B
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Isoprene peroxy type D
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ozone
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydrogen peroxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 19 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Carbon monoxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 4 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitric acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 17 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methyl peroxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10002 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Formaldehyde
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 7 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Aldehydes
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60018 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Peroxyacetyl nitrate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10013 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Peroxides
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60019 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Organic nitrates
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60020 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Isoprene
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10017 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Sulphur dioxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ammonia
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 9 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Sulfate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 22 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ammonium
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methyl glyoxal
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10025 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrogen monoxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 11 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydroperoxy radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 14 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methylperoxy radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10001 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrogen dioxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 5 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrate radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 13 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Dinitrogen pentoxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 15 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Pernitric acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 18 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methanol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10004 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Formic acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10005 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methacrylic acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10027 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ethane
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10008 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ethanol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10011 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methacrolein
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10028 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Acetone
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10029 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrous acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 16 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Glycolaldehyde
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10033 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Glyoxal
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10038 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydroxyacetone
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10041 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of all hydroxy-peroxides products of the reaction of hydroxy-isoprene adducts with O2
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60032 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Acetonitrile
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10007 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydrogen cyanide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10006 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ozone
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen peroxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 19 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitric acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 17 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methyl peroxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10002 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Formaldehyde
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 7 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Aldehydes
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60018 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Peroxyacetyl nitrate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10013 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Peroxides
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60019 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Organic nitrates
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60020 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Isoprene
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10017 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Sulphur dioxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ammonia
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 9 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Sulfate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 22 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ammonium
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methane sulfonic acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10024 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methyl glyoxal
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10025 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Carbon monoxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 4 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Lead
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 30296 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrogen monoxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 11 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroperoxy radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 14 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methylperoxy radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10001 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrogen dioxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 5 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrate radical
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 13 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Dinitrogen pentoxide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 15 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Pernitric acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 18 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methanol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10004 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Formic acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10005 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methacrylic acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10027 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ethane
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10008 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ethanol
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10011 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methacrolein
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10028 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Acetone
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10029 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Chlorine nitrate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 30 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hypochlorous acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 29 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen bromide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 35 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hypobromous acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 36 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrous acid
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 16 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Toluene and less reactive aromatics
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10022 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Xylene and more reactive aromatics
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10023 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Glycolaldehyde
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10033 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Glyoxal
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10038 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroxyacetone
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10041 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of all hydroxy-peroxides products of the reaction of hydroxy-isoprene adducts with O2
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60032 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Bromine nitrate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 37 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen chloride
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 20001 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Acetonitrile
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10007 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen cyanide
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10006 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroperoxy aldehydes type 1
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10058 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroperoxy aldehydes type 2
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10059 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Surface direct short-wave (solar) radiation
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface direct short-wave radiation, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Cloud base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	}
#Geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Instantaneous 10 metre wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	}
#Relative humidity with respect to ice
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 94 ;
	}
#Snow albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	}
#Fraction of convective precipitation cover
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 37 ;
	}
#2 metre relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Liquid water content in snow pack
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	}
#Convective available potential energy shear
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Tropopause pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Height of convective cloud top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	}
#Height of zero-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	}
#Height of one-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	}
#Instantaneous total lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Instantaneous cloud-to-ground lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Unbalanced component of specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 118 ;
	}
#Unbalanced component of specific cloud liquid water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 119 ;
	}
#Unbalanced component of specific cloud ice water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 120 ;
	}
#Accumulated Carbon Dioxide Net Ecosystem Exchange
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated Carbon Dioxide Gross Primary Production
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated Carbon Dioxide Ecosystem Respiration
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Carbon dioxide net ecosystem exchange flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	}
#Carbon dioxide gross primary production flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	}
#Carbon dioxide ecosystem respiration flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	}
#Soil moisture top 20 cm
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Soil moisture top 100 cm
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column supercooled liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column rain water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column snow water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Ice temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	}
#Soil temperature top 20 cm
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Soil temperature top 100 cm
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Atmosphere emission mass flux of Methane from Wetlands
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 2 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 10 ;
	}
#Accumulated mass emission of methane from Wetlands
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 2 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 10 ;
	}
#Surface short-wave (solar) radiation downward clear-sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface long-wave (thermal) radiation downward clear-sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Water runoff and drainage
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	}
#Instantaneous large-scale precipitation fraction
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Convective rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Large scale rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Convective snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Large scale snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Mixed-layer CAPE in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Mixed-layer CIN in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Mixed-layer CAPE in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Mixed-layer CIN in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Most-unstable CAPE
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	}
#Most-unstable CIN
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	}
#Departure level of the most unstable parcel expressed as Pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	}
#Urban cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	}
#Road Cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	}
#Building cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 2 ;
	}
#Building height
'm' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	}
#Vertical-to-horizontal area ratio
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	}
#Standard deviation of building height
'm' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	}
#Wetland cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 45 ;
	}
#Wetland type
'(Code table 4.239)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	}
#Irrigation cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	}
#C4 crop cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	}
#C4 grass cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	}
#Accumulated freezing rain water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Runoff water equivalent (surface plus subsurface)
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snow evaporation water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Potential evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Potential evaporation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Tile fraction
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	}
#Tile percentage
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 37 ;
	}
#Forecast logarithm of surface roughness length for moisture
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface runoff rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface runoff
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Sub-surface runoff rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Sub-surface runoff
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Reflectance in 0.4 micron channel
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 30 ;
	}
#Vertical divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 59 ;
	}
#Drag thermal coefficient
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Drag evaporation coefficient
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Pressure departure from hydrostatic state
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 46 ;
	}
#Surface net radiation flux (SW and LW)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Top net radiation flux (SW and LW)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Global irradiance on tilted surfaces
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 47 ;
	}
#Eady growth rate
'day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 22 ;
	}
#Tropical cyclones track density
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	}
#Canopy air temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 63 ;
	}
#Soil wetness index (total layer)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 151 ;
	}
#Soil wetness index (root zone)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	}
#Soil wetness index (layer)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 151 ;
	 typeOfSecondFixedSurface = 151 ;
	}
#Distance downward from roof surface
'm' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 6 ;
	}
#Distance inward from outer wall surface
'm' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 7 ;
	}
#Distance downward from road surface
'm' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 8 ;
	}
#Renewable power capacity
'W' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Renewable power production rate
'W' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Renewable power production
'W s' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Wind power capacity
'W' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Wind power production rate
'W' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Wind power production
'W s' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Solar photovoltaic (PV) power capacity
'W' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Solar photovoltaic (PV) power production rate
'W' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Solar photovoltaic (PV) power production
'W s' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Graupel (snow pellets) precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 75 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total lightning flash density
'km**-2' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Pressure at cloud base
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 2 ;
	}
#Geometric height of adiabatic condensation level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 5 ;
	}
#Geometric height of free convection level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 14 ;
	}
#Geometric height of neutral buoyancy level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 16 ;
	}
#Geometric height of 3 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Roof temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 185 ;
	}
#Wall temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 186 ;
	}
#Road temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 187 ;
	}
#Snow depth water equivalent on roof
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 185 ;
	}
#Snow depth water equivalent on road
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 187 ;
	}
#Urban canyon temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 187 ;
	 typeOfSecondFixedSurface = 185 ;
	}
#Urban canyon specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 187 ;
	 typeOfSecondFixedSurface = 185 ;
	}
#Convective snowfall water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Large-scale snowfall water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Lake surface temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Surface bulk Richardson number
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness for heat
'm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness for moisture
'm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 48 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Snow evaporation rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Radar reflectivity
'dB' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 4 ;
	}
#Pressure at cloud ceiling
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 19 ;
	 scaledValueOfFirstFixedSurface = 50 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Visibility through precipitation
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	}
#Geometric height of 0 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Geometric height of 1 degree C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Geometric height of 1.5 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Volumetric soil moisture (root zone)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	}
#Total column vertical-maximum radar precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertical-maximum radar reflectivity
'dB' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Thunderstorm hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Surface water flooding hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	}
#Polluted air hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	}
#Storm surge hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	}
#Cold wave hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	}
#Frost (hoar frost) hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 43 ;
	}
#Heatwave hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	}
#Wind hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	}
#Wildfires hazard index
'(Code table 4.253)' = {
	 discipline = 20 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	}
#Burned area
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Burning area
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	}
#Burnable area
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	}
#Un-burnable area
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	}
#Fuel load
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	}
#Combustion completeness
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	}
#Fuel moisture content
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	}
#Live leaf fuel load
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	}
#Live wood fuel load 
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 28 ;
	}
#Dead leaf fuel load
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	}
#Dead wood fuel load
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	}
#Live fuel moisture content
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	}
#Fine dead leaf moisture content
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	}
#Dense dead leaf moisture content
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	}
#Fine dead wood moisture content
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	}
#Dense dead wood moisture content
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	}
#Live fuel moisture content in low vegetation
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	}
#Live fuel moisture content in high vegetation
'kg kg**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	}
#Time-integrated total column vertically-integrated eastward geopotential flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward geopotential flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of water geopotential flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of geopotential flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward enthalpy flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward enthalpy flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward kinetic energy flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward kinetic energy flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward total energy flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward total energy flux
'J m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of enthalpy flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of kinetic energy flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of total energy flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of water enthalpy flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of mass flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward mass flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward mass flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of water vapour flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of cloud liquid water flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of cloud ice water flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of rain flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of snow flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward water vapour flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward water vapour flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward cloud liquid water flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward cloud liquid water flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward cloud ice water flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward cloud ice water flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward rain flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward rain flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward snow flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward snow flux
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean temperature tendency due to short-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to long-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to short wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to long-wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to parametrisations
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific humidity tendency due to parametrisations
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean updraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean downdraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean updraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean downdraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total precipitation flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean turbulent diffusion coefficient for heat
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time integral of rain flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of surface eastward momentum flux
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of surface northward momentum flux
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of surface latent heat evaporation flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean surface runoff rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sub-surface runoff rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface photosynthetically active radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow evaporation rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow melt rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean magnitude of turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large-scale precipitation fraction
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward UV radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface photosynthetically active radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large-scale precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean boundary layer dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface sensible heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface latent heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean moisture flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sunshine duration fraction
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Time-mean eastward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean gravity wave dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean runoff rate water equivalent (surface plus subsurface)
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time mean top downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated moisture divergence flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large scale snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean surface diffuse short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean potential evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time integral of surface latent heat sublimation flux
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of evapotranspiration flux
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean evapotranspiration rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time integral of potential evapotranspiration rate
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean potential evapotranspiration rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow depth water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean skin temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow density
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean low vegetation cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean high vegetation cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice area fraction
'(0 - 1)' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean leaf area index, low vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean leaf area index, high vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column cloud ice water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake mix-layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake mix-layer depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fraction of snow cover
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean charnock
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean potential vorticity
'K m**2 kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific rain water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific snow water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eta-coordinate vertical velocity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean visibility
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column snow water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean low cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean medium cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean high cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column rain water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil moisture top 20 cm
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil moisture top 100 cm
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil temperature top 20 cm
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil temperature top 100 cm
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean most-unstable CAPE
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture top 20 cm
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture top 100 cm
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated graupel (snow pellets)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geopotential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertical velocity
'Pa s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated water vapour
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vorticity (relative)
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net radiation flux (SW and LW)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net radiation flux (SW and LW)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean sea level pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean boundary layer height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre U wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre V wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre dewpoint temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sunshine
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ozone mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature of snow layer
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean forecast surface roughness
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean forecast logarithm of surface roughness for heat
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific cloud liquid water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific cloud ice water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fraction of cloud cover
'(0 - 1)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean K index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total totals index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean clear air turbulence (CAT)
'm**2/3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean forecast albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean discharge from rivers or streams
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil wetness index (total layer)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 151 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil wetness index (root zone)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil wetness index(layer)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 151 ;
	 typeOfSecondFixedSurface = 151 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain flooded fraction
'Proportion' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain flooded area
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river fraction
'Proportion' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river area
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fraction of river coverage plus river related flooding
'Proportion' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean area of river coverage plus river related flooding
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean UV visible albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean UV visible albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean snow albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean montgomery potential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective available potential energy
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean skin reservoir content
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column integrated ozone
'DU' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated kinetic energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated potential + internal energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated potential+internal+latent energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated total energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated water enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean UV visible albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean UV visible albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean UV visible albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean convective inhibition
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean friction velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake bottom temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake total layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake shape factor
'dimensionless' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake ice surface temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake ice total depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean minimum vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean duct base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean trapping layer base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean trapping layer top height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean cloud base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective available potential energy shear
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean tropopause pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean height of convective cloud top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean height of zero-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean height of one-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column supercooled liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre U-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre V-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large scale rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CAPE in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CIN in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CAPE in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CIN in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean departure level of the most unstable parcel expressed as Pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ceiling
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column integrated saturation specific humidity with respect to water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct normal short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean cosine of solar zenith angle
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean freezing rain precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of water geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of kinetic energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of total energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of water enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of water vapour flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of cloud liquid water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of cloud ice water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of rain flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of snow flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of ozone flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated net source of ozone
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean visibility through precipitation
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean cloud-to-ground lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean land surface heat net flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct normal short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre convective gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre turbulent gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 0 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 1 degree C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 1.5 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vegetation
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean leaf area index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface roughness for heat
'm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Reciprocal Obukhov length
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 3 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river storage of water
'm**3' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain storage of water
'm**3' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river outflow of water
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain outflow of water
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodpath outflow of water
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface diffuse short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture (root zone)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean liquid water content in snow pack
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean most-unstable CIN
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific rain water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 144 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific snow water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 145 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed layer depth
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean urban cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean water surface elevation
'm' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean groundwater return flow rate
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river and floodplain storage
'm**3' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth-averaged river velocity
'm s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Wind speed threshold for wind erosion
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-integrated land surface heat net flux
'J m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated surface direct normal short-wave radiation flux, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-maximum temperature tendency due to short-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to long-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to short wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to long-wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to parametrisations
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific humidity tendency due to parametrisations
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum updraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum downdraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum updraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum downdraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total precipitation flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum turbulent diffusion coefficient for heat
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface photosynthetically active radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow evaporation rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow melt rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum magnitude of turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large-scale precipitation fraction
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward UV radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface photosynthetically active radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large-scale precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum boundary layer dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface sensible heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface latent heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum moisture flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum gravity wave dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum runoff rate water equivalent (surface plus subsurface)
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time mean top downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated moisture divergence flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large scale snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum carbon dioxide net ecosystem exchange flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum carbon dioxide gross primary production flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum carbon dioxide ecosystem respiration flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum potential evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum volumetric soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow depth water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum skin temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow density
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice area fraction
'(0 - 1)' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum leaf area index, low vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum leaf area index, high vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column cloud liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column cloud ice water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake mix-layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake mix-layer depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum fraction of snow cover
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum charnock
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum potential vorticity
'K m**2 kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific rain water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific snow water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eta-coordinate vertical velocity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum visibility
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column snow water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum low cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum medium cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum high cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column rain water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum most-unstable CAPE
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated graupel (snow pellets)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geopotential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum u component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum v component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertical velocity
'Pa s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated water vapour
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vorticity (relative)
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean sea level pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum boundary layer height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre U wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre V wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre dewpoint temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sunshine
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ozone mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature of snow layer
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum forecast surface roughness
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum forecast logarithm of surface roughness for heat
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific cloud liquid water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific cloud ice water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum fraction of cloud cover
'(0 - 1)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum k index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total totals index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum clear air turbulence (CAT)
'm**2/3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum forecast albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum UV visible albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum UV visible albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum snow albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum montgomery potential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective available potential energy
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum skin reservoir content
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column integrated ozone
'DU' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated kinetic energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated potential + internal energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated potential+internal+latent energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated total energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated water enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum UV visible albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum UV visible albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum UV visible albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum convective inhibition
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum friction velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake bottom temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake total layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake shape factor
'dimensionless' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake ice surface temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake ice total depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum minimum vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum duct base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum trapping layer base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum trapping layer top height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum cloud base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective available potential energy shear
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum tropopause pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum height of convective cloud top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum height of zero-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum height of one-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column supercooled liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre U-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre V-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large scale rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CAPE in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CIN in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CAPE in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CIN in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum departure level of the most unstable parcel expressed as Pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ceiling
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column integrated saturation specific humidity with respect to water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct normal short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum cosine of solar zenith angle
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum freezing rain precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of water geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of kinetic energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of total energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of water enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of water vapour flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of cloud liquid water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of cloud ice water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of rain flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of snow flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of ozone flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated net source of ozone
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum visibility through precipitation
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum cloud-to-ground lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre eastward wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre northward wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum land surface heat net flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct normal short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre convective gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre turbulent gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 0 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 1 degree C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 1.5 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vegetation
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum leaf area index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface roughness for heat
'm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Reciprocal Obukhov length
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 3 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface diffuse short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum volumetric soil moisture (root zone)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum liquid water content in snow pack
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum most-unstable CIN
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific rain water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 144 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific snow water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 145 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed layer depth
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum urban cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Wind speed threshold for wind erosion
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum temperature tendency due to short-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to long-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to short wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to long-wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to parametrisations
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific humidity tendency due to parametrisations
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum updraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum downdraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum updraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum downdraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total precipitation flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum turbulent diffusion coefficient for heat
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface photosynthetically active radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snowmelt rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum magnitude of turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large-scale precipitation fraction
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward UV radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface photosynthetically active radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large-scale precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snowfall rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum boundary layer dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface sensible heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface latent heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum gravity wave dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum runoff rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated moisture divergence
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large scale snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface direct short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface direct short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum carbon dioxide net ecosystem exchange flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum carbon dioxide gross primary production flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum carbon dioxide ecosystem respiration flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum potential evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum volumetric soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow depth water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum skin temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow density
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice area fraction
'(0 - 1)' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum leaf area index, low vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum leaf area index, high vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column cloud liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column cloud ice water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake mix-layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake mix-layer depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum fraction of snow cover
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum charnock
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum potential vorticity
'K m**2 kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific rain water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific snow water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eta-coordinate vertical velocity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum visibility
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column snow water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum low cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum medium cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum high cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column rain water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum most-unstable CAPE
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated graupel (snow pellets)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geopotential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum u component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum v component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertical velocity
'Pa s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated water vapour
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vorticity (relative)
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean sea level pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum boundary layer height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre U wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre V wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre dewpoint temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sunshine
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ozone mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature of snow layer
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum forecast surface roughness
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum forecast logarithm of surface roughness for heat
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific cloud liquid water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific cloud ice water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum fraction of cloud cover
'(0 - 1)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum k index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total totals index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum clear air turbulence (CAT)
'm**2/3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum forecast albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum UV visible albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum UV visible albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum snow albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum montgomery potential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective available potential energy
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum skin reservoir content
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column integrated ozone
'DU' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated kinetic energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated potential + internal energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertical integral of potential+internal+latent energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated total energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated water enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum UV visible albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum UV visible albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum UV visible albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum convective inhibition
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum friction velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake bottom temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake total layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake shape factor
'dimensionless' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake ice surface temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake ice total depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum minimum vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum duct base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum trapping layer base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum trapping layer top height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum cloud base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective available potential energy shear
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum tropopause pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum height of convective cloud top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum height of zero-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum height of one-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column supercooled liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre U-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre V-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large scale rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CAPE in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CIN in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CAPE in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CIN in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum departure level of the most unstable parcel expressed as Pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ceiling
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column integrated saturation specific humidity with respect to water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum direct solar radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum cosine of solar zenith angle
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum freezing rain rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of water geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of kinetic energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of total energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of water enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of water vapour flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of cloud liquid water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of cloud ice water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of rain flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of snow flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of ozone flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated net source of ozone
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum visibility through precipitation
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum cloud-to-ground lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum land surface heat net flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface direct normal short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre convective gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre turbulent gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 0 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 1 degree C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 1.5 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vegetation
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum leaf area index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface roughness for heat
'm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Reciprocal Obukhov length
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 3 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface diffuse short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum volumetric soil moisture (root zone)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum liquid water content in snow pack
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum most-unstable CIN
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific rain water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 144 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific snow water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 145 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed layer depth
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum urban cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Wind speed threshold for wind erosion
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation temperature tendency due to short-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to long-wave radiation
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to short wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to long-wave radiation, clear sky
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to parametrisations
'K s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific humidity tendency due to parametrisations
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward wind tendency due to parametrisations
'm s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation updraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation downdraught mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation updraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation downdraught detrainment rate
'kg m**-3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total precipitation flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation turbulent diffusion coefficient for heat
'm**2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface photosynthetically active radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snowmelt rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation magnitude of turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large-scale precipitation fraction
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward UV radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface photosynthetically active radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large-scale precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snowfall rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation boundary layer dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface sensible heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface latent heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation gravity wave dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation runoff rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated moisture divergence
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large scale snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface direct short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface direct short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation carbon dioxide net ecosystem exchange flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation carbon dioxide gross primary production flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation carbon dioxide ecosystem respiration flux
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation potential evaporation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation volumetric soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow depth water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation skin temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow density
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice area fraction
'(0 - 1)' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation leaf area index, low vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation leaf area index, high vegetation
'm**2 m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column cloud liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column cloud ice water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake mix-layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake mix-layer depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation fraction of snow cover
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation charnock
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation potential vorticity
'K m**2 kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific rain water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific snow water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eta-coordinate vertical velocity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation visibility
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column snow water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation low cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation medium cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation high cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column rain water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation most-unstable CAPE
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated graupel (snow pellets)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geopotential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation u component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation v component of wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific humidity
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertical velocity
'Pa s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated water vapour
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vorticity (relative)
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean sea level pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation boundary layer height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre U wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre V wind component
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre dewpoint temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sunshine
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ozone mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature of snow layer
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation forecast surface roughness
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation forecast logarithm of surface roughness for heat
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific cloud liquid water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific cloud ice water content
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation fraction of cloud cover
'(0 - 1)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation k index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total totals index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation clear air turbulence (CAT)
'm**2/3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation forecast albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation UV visible albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for diffuse radiation (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation snow albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation montgomery potential
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective available potential energy
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation skin reservoir content
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column integrated ozone
'DU' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated kinetic energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated potential + internal energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertical integral of potential+internal+latent energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated total energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated water enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation, isotropic component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation, volumetric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation, geometric component (climatological)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation convective inhibition
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation friction velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake bottom temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake total layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake shape factor
'dimensionless' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake ice surface temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake ice total depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation minimum vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation duct base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation trapping layer base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation trapping layer top height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of -10 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation cloud base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 0 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre relative humidity with respect to water
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective available potential energy shear
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation tropopause pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation height of convective cloud top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation height of zero-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation height of one-degree wet-bulb temperature
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column supercooled liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre U-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre V-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large scale rain rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CAPE in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CIN in the lowest 50 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CAPE in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CIN in the lowest 100 hPa
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation departure level of the most unstable parcel expressed as Pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ceiling
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column integrated saturation specific humidity with respect to water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation direct solar radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation cosine of solar zenith angle
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation freezing rain rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of water geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward enthalpy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of kinetic energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of total energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of water enthalpy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of water vapour flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of cloud liquid water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of cloud ice water flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of rain flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of snow flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward cloud liquid water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward cloud ice water flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward rain flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward snow flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of ozone flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated net source of ozone
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation cloud-to-ground lightning flash density
'km**-2 day**-1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation land surface heat net flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface direct normal short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre convective gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre turbulent gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 0 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 1 degree C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 1.5 degrees C theta level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vegetation
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation leaf area index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface roughness for heat
'm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Reciprocal Obukhov length
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 3 degrees C atmospheric isothermal level above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface diffuse short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation volumetric soil moisture (root zone)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation liquid water content in snow pack
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation most-unstable CIN
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific rain water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 144 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific snow water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 145 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed layer depth
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation urban cover
'(0 - 1)' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Wind speed threshold for wind erosion
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Cross sectional area of flow in channel
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	}
#Side flow into river channel
'm**3 s**-1 m**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Discharge from rivers or streams
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#River storage of water
'm**3' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Floodplain storage of water
'm**3' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Days since last observation
'Integer' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 3 ;
	}
#Frost index
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 24 ;
	}
#Depth of water on soil surface
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Upstream accumulated precipitation
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Upstream accumulated snow melt
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Snow depth at elevation bands
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	}
#Groundwater upper storage
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Groundwater lower storage
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Lake depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	}
#River depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	}
#River outflow of water
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	}
#Floodplain outflow of water
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	}
#Floodpath outflow of water
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	}
#Floodplain depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	}
#Floodplain flooded fraction
'Proportion' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	}
#Floodplain flooded area
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	}
#River fraction
'Proportion' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	}
#River area
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	}
#Fraction of river coverage plus river related flooding
'Proportion' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	}
#Area of river coverage plus river related flooding
'm**2' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	}
#Water surface elevation
'm' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Groundwater return flow rate
'm**3 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#River and floodplain storage
'm**3' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Depth-averaged river velocity
'm s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Latitude
'Degree N' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	}
#Longitude
'Degree E' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	}
#Latitude on T grid
'Degree N' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 1 ;
	}
#Longitude on T grid
'Degree E' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 1 ;
	}
#Latitude on U grid
'Degree N' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 2 ;
	}
#Longitude on U grid
'Degree E' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 2 ;
	}
#Latitude on V grid
'Degree N' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 3 ;
	}
#Longitude on V grid
'Degree E' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 3 ;
	}
#Latitude on W grid
'Degree N' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 4 ;
	}
#Longitude on W grid
'Degree E' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 4 ;
	}
#Latitude on F grid
'Degree N' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 5 ;
	}
#Longitude on F grid
'Degree E' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 5 ;
	}
#Total column vertically-integrated graupel (snow pellets)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Surface latent heat net flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Heat index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Wind chill factor
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Snow phase change heat flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Vapor pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Large scale precipitation (non-convective)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 12 ;
	}
#Convective snow
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 14 ;
	}
#Large scale snow
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 15 ;
	}
#Snow age
'day' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 17 ;
	}
#Absolute humidity
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 18 ;
	}
#Precipitation type
'(Code table 4.201)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	}
#Integrated liquid water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 20 ;
	}
#Condensate
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 21 ;
	}
#Cloud mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 22 ;
	}
#Ice water mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 23 ;
	}
#Rain mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	}
#Snow mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 25 ;
	}
#Horizontal moisture convergence
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 26 ;
	}
#Total snowfall
'm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Precipitable water category
'(Code table 4.202)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 30 ;
	}
#Hail
'm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 31 ;
	}
#Graupel (snow pellets)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 32 ;
	}
#Categorical rain
'(Code table 4.222)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 33 ;
	}
#Categorical freezing rain
'(Code table 4.222)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 34 ;
	}
#Categorical ice pellets
'(Code table 4.222)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 35 ;
	}
#Categorical snow
'(Code table 4.222)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 36 ;
	}
#Convective precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Horizontal moisture divergence
'kg kg**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 38 ;
	}
#Percent frozen precipitation
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 39 ;
	}
#Potential evaporation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 40 ;
	}
#Snow cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 42 ;
	}
#Rain fraction of total cloud water
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 43 ;
	}
#Rime factor
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 44 ;
	}
#Total column integrated rain
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	}
#Total column integrated snow
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	}
#Large scale water precipitation (non-convective)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 47 ;
	}
#Convective water precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 48 ;
	}
#Total water precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 49 ;
	}
#Total snow precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Total column water (Vertically integrated total water (vapour + cloud water/ice))
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	}
#Total precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Total snowfall rate water equivalent
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Large scale precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Total snowfall rate
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 57 ;
	}
#Convective snowfall rate
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 58 ;
	}
#Large scale snowfall rate
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 59 ;
	}
#Water equivalent of accumulated snow depth (deprecated)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 13 ;
	}
#Rain precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 65 ;
	}
#Snow precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	}
#Freezing rain precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Ice pellets precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 68 ;
	}
#Wind speed (gust)
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	}
#u-component of wind (gust)
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	}
#v-component of wind (gust)
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	}
#Vertical speed shear
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	}
#Horizontal momentum flux
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 26 ;
	}
#U-component storm motion
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 27 ;
	}
#V-component storm motion
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 28 ;
	}
#Drag coefficient
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	}
#Frictional velocity
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	}
#Pressure reduced to MSL
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Altimeter setting
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	}
#Thickness
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	}
#Pressure altitude
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	}
#Density altitude
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#5-wave geopotential height
'gpm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Instantaneous eastward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward gravity wave surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#5-wave geopotential height anomaly
'gpm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	}
#Net short-wave radiation flux (top of atmosphere)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 1 ;
	}
#Surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface upward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface photosynthetically active radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface downward UV radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#UV index (under clear sky)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	}
#UV index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 51 ;
	}
#Surface downward long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface upward long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Cloud Ice
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	}
#Cloud water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 6 ;
	}
#Cloud amount
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 7 ;
	}
#Cloud type
'(Code table 4.203)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 8 ;
	}
#Thunderstorm maximum tops
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 9 ;
	}
#Thunderstorm coverage
'(Code table 4.204)' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 10 ;
	}
#Cloud top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 12 ;
	}
#Ceiling
'm' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	}
#Non-convective cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 14 ;
	}
#Cloud work function
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 15 ;
	}
#Convective cloud efficiency
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 16 ;
	}
#Total condensate
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 17 ;
	}
#Total column-integrated cloud water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 18 ;
	}
#Total column-integrated cloud ice
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 19 ;
	}
#Total column-integrated condensate
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 20 ;
	}
#Ice fraction of total condensate
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 21 ;
	}
#Cloud ice mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 23 ;
	}
#Sunshine
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Horizontal extent of cumulonimbus (CB)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 25 ;
	}
#K index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	}
#KO index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 3 ;
	}
#Total totals index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	}
#Sweat index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 5 ;
	}
#Storm relative helicity
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 8 ;
	}
#Energy helicity index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 9 ;
	}
#Surface lifted index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 10 ;
	}
#Best (4-layer) lifted index
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 11 ;
	}
#Aerosol type
'(Code table 4.205)' = {
	 discipline = 0 ;
	 parameterCategory = 13 ;
	 parameterNumber = 0 ;
	}
#Total ozone
'DU' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 0 ;
	}
#Total column integrated ozone
'DU' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Base spectrum width
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 0 ;
	}
#Base reflectivity
'dB' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 1 ;
	}
#Base radial velocity
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 2 ;
	}
#Vertically-integrated liquid
'kg m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 3 ;
	}
#Layer-maximum base reflectivity
'dB' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 4 ;
	}
#Precipitation from radar
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Air concentration of Caesium 137
'Bq m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 0 ;
	}
#Air concentration of Iodine 131
'Bq m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 1 ;
	}
#Air concentration of radioactive pollutant
'Bq m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 2 ;
	}
#Ground deposition of Caesium 137
'Bq m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 3 ;
	}
#Ground deposition of Iodine 131
'Bq m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 4 ;
	}
#Ground deposition of radioactive pollutant
'Bq m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 5 ;
	}
#Time-integrated air concentration of caesium pollutant
'Bq s m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 6 ;
	}
#Time-integrated air concentration of iodine pollutant
'Bq s m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 7 ;
	}
#Time-integrated air concentration of radioactive pollutant
'Bq s m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 8 ;
	}
#Volcanic ash
'(Code table 4.206)' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 4 ;
	}
#Icing top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 5 ;
	}
#Icing base
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 6 ;
	}
#Icing
'(Code table 4.207)' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 7 ;
	}
#Turbulence top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 8 ;
	}
#Turbulence base
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 9 ;
	}
#Turbulence
'(Code table 4.208)' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 10 ;
	}
#Turbulent kinetic energy
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 11 ;
	}
#Planetary boundary layer regime
'(Code table 4.209)' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 12 ;
	}
#Contrail intensity
'(Code table 4.210)' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 13 ;
	}
#Contrail engine type
'(Code table 4.211)' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 14 ;
	}
#Contrail top
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 15 ;
	}
#Contrail base
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 16 ;
	}
#Snow free albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 18 ;
	}
#Icing
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 20 ;
	}
#In-cloud turbulence
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 21 ;
	}
#Relative clear air turbulence (RCAT)
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 22 ;
	}
#Supercooled large droplet probability
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 23 ;
	}
#Arbitrary text string
'CCITTIA5' = {
	 discipline = 0 ;
	 parameterCategory = 190 ;
	 parameterNumber = 0 ;
	}
#Seconds prior to initial reference time (defined in Section 1)
's' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 0 ;
	}
#Flash flood guidance
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Flash flood runoff (Encoded as an accumulation over a floating subinterval of time)
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Remotely sensed snow cover
'(Code table 4.215)' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Elevation of snow covered terrain
'(Code table 4.216)' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Snow water equivalent percent of normal
'%' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Baseflow-groundwater runoff
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Storm surface runoff
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Conditional percent precipitation amount fractile for an overall period (Encoded as an accumulation)
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Percent precipitation in a sub-period of an overall period
'%' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Probability of 0.01 inch of precipitation (POP)
'%' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Vegetation
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Water runoff
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Evapotranspiration
'kg**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Model terrain height
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Land use
'(Code table 4.212)' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Ground heat flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Moisture availability
'%' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Exchange coefficient
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Plant canopy surface water
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Blackadar mixing length scale
'm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Canopy conductance
'm s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Minimal stomatal resistance
's m**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Solar parameter in canopy conductance
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	}
#Temperature parameter in canopy conductance
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	}
#Soil moisture parameter in canopy conductance
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#Humidity parameter in canopy conductance
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Column-integrated soil water
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Heat flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Volumetric soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Volumetric wilting point
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Number of soil layers in root zone
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Liquid volumetric soil moisture (non-frozen)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	}
#Volumetric transpiration stress-onset (soil moisture)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	}
#Transpiration stress-onset (soil moisture)
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	}
#Volumetric direct evaporation cease (soil moisture)
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	}
#Direct evaporation cease (soil moisture)
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#Soil porosity
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Volumetric saturation of soil moisture
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	}
#Saturation of soil moisture
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	}
#Estimated precipitation
'kg m**-2' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Instantaneous rain rate
'kg m**-2 s**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Cloud top height
'm' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Cloud top height quality indicator
'(Code table 4.219)' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Estimated u component of wind
'm s**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Estimated v component of wind
'm s**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Number of pixels used
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 6 ;
	}
#Solar zenith angle
'Degree' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Relative azimuth angle
'Degree' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 8 ;
	}
#Reflectance in 0.6 micron channel
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Reflectance in 0.8 micron channel
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 10 ;
	}
#Reflectance in 1.6 micron channel
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	}
#Reflectance in 3.9 micron channel
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 12 ;
	}
#Atmospheric divergence
's**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 13 ;
	}
#Direction of wind waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Primary wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Primary wave mean period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Secondary wave mean period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Current direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Current speed
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Geometric vertical velocity
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 9 ;
	}
#Seconds prior to initial reference time (defined in Section 1)
's' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 0 ;
	}
#2 metre relative humidity
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Apparent temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Haines Index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 2 ;
	}
#Cloud cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 22 ;
	}
#Evaporation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#10 metre wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Surface direct short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface diffuse short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Fraction of snow cover
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	}
#Clear air turbulence (CAT)
'm**2/3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	}
#Mountain wave turbulence (eddy dissipation rate)
'm**2/3 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 28 ;
	}
#Specific rain water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 144 ;
	}
#Specific snow water content (convective)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 145 ;
	}
#Glacier mask
'Proportion' = {
	 discipline = 2 ;
	 parameterCategory = 5 ;
	 parameterNumber = 0 ;
	}
#Soil temperature
'K' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	}
#Surface downward short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface upward short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface downward long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil heat flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 26 ;
	}
#Percolation rate
'kg m**-2 s**-1' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Soil depth
'm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	}
#Soil moisture
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	}
#Leaf Area Index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Accumulated surface upward short-wave radiation flux, clear sky
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Percolation
'kg m**-2' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 177 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Evapotranspiration rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Potential evapotranspiration rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Runoff rate water equivalent (surface plus subsurface)
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	}
#Snow melt rate
'kg m**-2 s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Forecast albedo
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	}
#Cloudy brightness temperature
'K' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 14 ;
	}
#Clear-sky brightness temperature
'K' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 15 ;
	}
#Cloudy reflectance
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 31 ;
	}
#Clear reflectance
'%' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 32 ;
	}
#Scaled radiance
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Scaled albedo
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Scaled brightness temperature
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Scaled precipitable water
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Scaled lifted index
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Scaled cloud top pressure
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Scaled skin temperature
'Numeric' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Cloud mask
'(Code table 4.217)' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Pixel scene type
'(Code table 4.218)' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Fire detection indicator
'(Code table 4.223)' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Forest fire weather index (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 5 ;
	}
#Fine fuel moisture code (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 6 ;
	}
#Duff moisture code (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	}
#Drought code (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 8 ;
	}
#Initial fire spread index (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	}
#Fire buildup index (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	}
#Fire daily severity rating (as defined by the Canadian Forest Service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	}
#Cloudy radiance (with respect to wave number)
'W m**-1 sr**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 16 ;
	}
#Clear-sky radiance (with respect to wave number)
'W m**-1 sr**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 17 ;
	}
#Wind speed
'm s**-1' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	}
#Aerosol optical thickness at 0.635 um
'dimensionless' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 20 ;
	}
#Aerosol optical thickness at 0.810 um
'dimensionless' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 21 ;
	}
#Aerosol optical thickness at 1.640 um
'dimensionless' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 22 ;
	}
#Angstrom coefficient
'dimensionless' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 23 ;
	}
#Keetch-Byram drought index
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	}
#Drought factor (as defined by the Australian forest service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	}
#Rate of spread (as defined by the Australian forest service)
'm s**-1' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	}
#Fire danger index (as defined by the Australian forest service)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 15 ;
	}
#Spread component (as defined by the U.S Forest Service National Fire-Danger Rating System)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 16 ;
	}
#Burning index (as defined by the U.S Forest Service National Fire-Danger Rating System)
'Numeric' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 17 ;
	}
#Ignition component (as defined by the U.S Forest Service National Fire-Danger Rating System)
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	}
#Energy release component (as defined by the U.S Forest Service National Fire-Danger Rating System)
'J m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 19 ;
	}
#Probability of fire detection
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 43 ;
	}
#Probability of ignition from lightning
'%' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 44 ;
	}
#Volumetric soil ice
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 38 ;
	}
#Time integral of total solid precipitation flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 128 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Fog
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Time-integrated eastward turbulent surface stress due to orographic form drag
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward turbulent surface stress due to orographic form drag
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated eastward turbulent surface stress due to surface roughness
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward turbulent surface stress due to surface roughness
'N m**-2 s' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Saturation specific humidity with respect to water
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 168 ;
	}
#Total column integrated saturation specific humidity with respect to water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Standardised Precipitation Index (SPI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 0 ;
	}
#Standardised Precipitation Evapotranspiration Index (SPEI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 1 ;
	}
#Standardised Streamflow Index (SSFI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 2 ;
	}
#Standardised Reservoir Supply Index (SRSI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 3 ;
	}
#Standardised Water-level Index (SWI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 4 ;
	}
#Standardised Snowmelt and Rain Index (SMRI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 5 ;
	}
#Streamflow Drought Index (SDI)
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 6 ;
	}
#Volumetric soil moisture top 20 cm
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Volumetric soil moisture top 100 cm
'm**3 m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Surface photosynthetically active radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface direct normal short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Instantaneous magnitude of turbulent surface stress
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous boundary layer dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	}
#Top net short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Top net long-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Instantaneous gravity wave dissipation
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	}
#Top net short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Top net long-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Top downward short-wave radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Surface direct short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous eastward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress due to orographic form drag
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous eastward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress due to surface roughness
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Time-severity precipitation type
'(Code table 4.201)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 100 ;
	}
#Time-mode precipitation type
'(Code table 4.201)' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 101 ;
	}
#Land surface heat net flux
'W m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface direct normal short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#10 metre convective gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#10 metre turbulent gust
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Reciprocal Obukhov length
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	}
#Precipitation rate from radar
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 17 ;
	}
#Radar data quality index
'Proportion' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 18 ;
	}
#Radar data quality flag
'(Code table 4.106)' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 19 ;
	}
#Precipitation rate from forecasted radar imagery
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 6 ;
	}
#Precipitation from forecasted radar imagery
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface diffuse short-wave radiation flux, clear sky
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Storm relative helicity - right moving storm
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 23 ;
	}
#Storm relative helicity - left moving storm
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 24 ;
	}
#Effective storm relative helicity - mean flow
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 28 ;
	 typeOfSecondFixedSurface = 29 ;
	}
#Effective storm relative helicity - right moving storm
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 28 ;
	 typeOfSecondFixedSurface = 29 ;
	}
#Effective storm relative helicity - left moving storm
'm**2 s**-2' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 28 ;
	 typeOfSecondFixedSurface = 29 ;
	}
#Universal thermal climate index
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Mean radiant temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Fraction of Malaria cases
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Malaria circumsporozoite protein ratio
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Plasmodium falciparum entomological inoculation rate
'Bites per day per person' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Human bite rate by anopheles vectors
'Bites per day per person' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Malaria immunity ratio
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Falciparum parasite ratio
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Detectable falciparum parasite ratio (after day 10)
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 6 ;
	}
#Anopheles vector to host ratio
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Anopheles vector density
'Number m**-2' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 8 ;
	}
#Fraction of malarial vector reproductive habitat
'Fraction' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Population density
'Person m**-2' = {
	 discipline = 20 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	}
#Wet bulb globe temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Globe temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Humidex
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Effective temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Normal effective temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Standard effective temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Physiological equivalent temperature
'K' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Saturation water vapour pressure
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 32 ;
	}
#Wet-bulb potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	}
#Wet-bulb temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Excess heat factor
'K**2' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Excess cold factor
'K**2' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Sea ice thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	}
#Sea ice area fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Eastward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Northward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice albedo
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice growth
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow volume over sea ice per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Vertically averaged sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow temperature over sea ice
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice temperature at the sea ice and snow interface
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Underside ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow heat content over sea ice
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice freeboard thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Sea ice melt pond fraction
'Proportion' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice melt pond depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice melt pond volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice fraction tendency due to parameterization
's**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#X-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Y-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	}
#Sea ice breakup memory
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	}
#Sea surface practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 14 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 17 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 20 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 26 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 28 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Barotropic stream function
'm**3 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Surface downward heat flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Northward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Eastward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Y-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#X-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by temperature 0.2 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by temperature 0.5 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Average sea water practical salinity in the upper 300 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Average sea water practical salinity in the upper 700 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column average sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Vertically-integrated heat content in the upper 300 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Vertically-integrated heat content in the upper 700 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column of heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Steric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Halosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Thermosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Thermocline depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Bottom pressure equivalent height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Net surface upward water flux
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Fresh water flux into sea water (from rivers)
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Virtual salt flux into sea water
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Heat flux correction
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Fresh water flux correction
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Virtual salt flux correction
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Turbocline depth (kz=5e-4)
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Y-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#X-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Northward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Eastward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#Heat Content surface to 26C isotherm
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	}
#Sea surface height tendency due to parameterization
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea surface height with inverse barometer correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	}
#Average sea water potential temperature in the upper 300m
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Sea surface salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	}
#Vertically integrated sea water practical salinity in the upper 300 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Vertically integrated sea water practical salinity in the upper 700 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column vertically integrated sea water practical salinity
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	}
#Sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water potential temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water sigma theta
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Y-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#X-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Northward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Eastward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea water potential temperature tendency due to newtonian relaxation
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity tendency due to newtonian relaxation
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water temperature tendency due to parameterization
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity tendency due to parameterization
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Eastward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Northward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water temperature tendency due to direct bias correction
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity tendency due to direct bias correction
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Northward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Eastward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water upward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water age since surface contact
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Net short wave radiation rate at sea surface
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Wind stress at sea surface
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Wind speed at 10m above sea surface
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Neutral drag coefficient at 10m above sea surface
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Total precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Eastward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Northward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#U-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#V-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Time-mean sea ice thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice area fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice albedo
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice growth
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow volume over sea ice per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically averaged sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow temperature over sea ice
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice temperature at the sea ice and snow interface
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean underside ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow heat content over sea ice
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice freeboard thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice melt pond fraction
'Proportion' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice melt pond depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice melt pond volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice fraction tendency due to parameterization
's**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice breakup memory
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 14 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 17 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 20 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 26 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 28 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean barotropic stream function
'm**3 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward heat flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time mean Y-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by temperature 0.2 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by temperature 0.5 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean average sea water practical salinity in the upper 300 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean average sea water practical salinity in the upper 700 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column average sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically-integrated heat content in the upper 300 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically-integrated heat content in the upper 700 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean steric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean halosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean thermosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean thermocline depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean bottom pressure equivalent height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean net surface upward water flux
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fresh water flux into sea water (from rivers)
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean virtual salt flux into sea water
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean heat flux correction
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fresh water flux correction
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean virtual salt flux correction
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean turbocline depth (kz=5e-4)
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean heat content surface to 26C isotherm
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface height tendency due to parameterization
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface height with inverse barometer correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean average sea water potential temperature in the upper 300m
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically integrated sea water practical salinity in the upper 300 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically integrated sea water practical salinity in the upper 700 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically integrated sea water practical salinity
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water potential temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water sigma theta
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water potential temperature tendency due to newtonian relaxation
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity tendency due to newtonian relaxation
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water temperature tendency due to parameterization
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity tendency due to parameterization
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water temperature tendency due to direct bias correction
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity tendency due to direct bias correction
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water upward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water age since surface contact
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean net short wave radiation rate at sea surface
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind stress at sea surface
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind speed at 10m above sea surface
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean neutral drag coefficient at 10m above sea surface
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-accumulated net short wave radiation at sea surface
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-accumulated total precipitation at sea surface
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-accumulated snow precipitation at sea surface
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-maximum sea ice thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice area fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice albedo
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice growth
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow volume over sea ice per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertically averaged sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow temperature over sea ice
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice temperature at the sea ice and snow interface
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum underside ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow heat content over sea ice
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice freeboard thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice melt pond fraction
'Proportion' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice melt pond depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice melt pond volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice fraction tendency due to parameterization
's**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum X-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Y-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice breakup memory
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum depth of 14 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum depth of 17 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum depth of 20 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum depth of 26 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum depth of 28 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum barotropic stream function
'm**3 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward heat flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Y-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum X-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ocean mixed layer depth defined by temperature 0.2 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ocean mixed layer depth defined by temperature 0.5 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum average sea water practical salinity in the upper 300 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum average sea water practical salinity in the upper 700 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column average sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertically-integrated heat content in the upper 300 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertically-integrated heat content in the upper 700 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column of heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum steric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum halosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum thermosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum thermocline depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum bottom pressure equivalent height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum net surface upward water flux
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum fresh water flux into sea water (from rivers)
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum virtual salt flux into sea water
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum heat flux correction
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum fresh water flux correction
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum virtual salt flux correction
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum turbocline depth (kz=5e-4)
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Y-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum X-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum heat content surface to 26C isotherm
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface height tendency due to parameterization
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface height with inverse barometer correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum average sea water potential temperature in the upper 300m
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertically integrated sea water practical salinity in the upper 300 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertically integrated sea water practical salinity in the upper 700 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically integrated sea water practical salinity
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water potential temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water sigma theta
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Y-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum X-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water potential temperature tendency due to newtonian relaxation
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water salinity tendency due to newtonian relaxation
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water temperature tendency due to parameterization
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water salinity tendency due to parameterization
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water temperature tendency due to direct bias correction
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water salinity tendency due to direct bias correction
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water upward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water age since surface contact
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum net short wave radiation rate at sea surface
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind stress at sea surface
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind speed at 10m above sea surface
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum neutral drag coefficient at 10m above sea surface
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum sea ice thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice area fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice albedo
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice growth
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow volume over sea ice per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically averaged sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow temperature over sea ice
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice temperature at the sea ice and snow interface
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum underside ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow heat content over sea ice
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice freeboard thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice melt pond fraction
'Proportion' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice melt pond depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice melt pond volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice fraction tendency due to parameterization
's**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum X-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Y-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice breakup memory
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum depth of 14 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum depth of 17 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum depth of 20 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum depth of 26 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum depth of 28 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum barotropic stream function
'm**3 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward heat flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Y-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum X-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ocean mixed layer depth defined by temperature 0.2 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ocean mixed layer depth defined by temperature 0.5 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum average sea water practical salinity in the upper 300 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum average sea water practical salinity in the upper 700 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column average sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically-integrated heat content in the upper 300 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically-integrated heat content in the upper 700 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column of heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum steric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum halosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum thermosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum thermocline depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum bottom pressure equivalent height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum net surface upward water flux
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum fresh water flux into sea water (from rivers)
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum virtual salt flux into sea water
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum heat flux correction
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum fresh water flux correction
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum virtual salt flux correction
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum turbocline depth (kz=5e-4)
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Y-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum X-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum heat content surface to 26C isotherm
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface height tendency due to parameterization
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface height with inverse barometer correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum average sea water potential temperature in the upper 300m
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated sea water practical salinity in the upper 300 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated sea water practical salinity in the upper 700 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically integrated sea water practical salinity
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water potential temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water sigma theta
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Y-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum X-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water potential temperature tendency due to newtonian relaxation
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water salinity tendency due to newtonian relaxation
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water temperature tendency due to parameterization
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water salinity tendency due to parameterization
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water temperature tendency due to direct bias correction
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water salinity tendency due to direct bias correction
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water upward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water age since surface contact
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum net short wave radiation rate at sea surface
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind stress at sea surface
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind speed at 10m above sea surface
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum neutral drag coefficient at 10m above sea surface
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation sea ice thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice area fraction
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice albedo
'Fraction' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice growth
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow volume over sea ice per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically averaged sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow temperature over sea ice
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice temperature at the sea ice and snow interface
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation underside ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow heat content over sea ice
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice freeboard thickness
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice melt pond fraction
'Proportion' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice melt pond depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice melt pond volume per unit area
'm**3 m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice fraction tendency due to parameterization
's**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation X-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Y-component of sea ice velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice breakup memory
'Numeric' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation depth of 14 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation depth of 17 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation depth of 20 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation depth of 26 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation depth of 28 C isotherm
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation barotropic stream function
'm**3 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward heat flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Y-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation X-component of surface stress
'N m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ocean mixed layer depth defined by temperature 0.2 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ocean mixed layer depth defined by temperature 0.5 C
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation average sea water practical salinity in the upper 300 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation average sea water practical salinity in the upper 700 m
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column average sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically-integrated heat content in the upper 300 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically-integrated heat content in the upper 700 m
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column of heat content
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation steric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation halosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation thermosteric change in sea surface height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation thermocline depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation bottom pressure equivalent height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation net surface upward water flux
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation fresh water flux into sea water (from rivers)
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation virtual salt flux into sea water
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation heat flux correction
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation fresh water flux correction
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation virtual salt flux correction
'g kg**-1 m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation turbocline depth (kz=5e-4)
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Y-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation X-component of surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward surface sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation heat content surface to 26C isotherm
'J m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface height tendency due to parameterization
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface height with inverse barometer correction
'm' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation average sea water potential temperature in the upper 300m
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated sea water practical salinity in the upper 300 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated sea water practical salinity in the upper 700 m
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically integrated sea water practical salinity
'g kg**-1 m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water practical salinity
'g kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water potential temperature
'K' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water sigma theta
'kg m**-3' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Y-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation X-component of sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation upward sea water velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water potential temperature tendency due to newtonian relaxation
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water salinity tendency due to newtonian relaxation
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water temperature tendency due to parameterization
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water salinity tendency due to parameterization
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward sea water velocity tendency due to parameterization
'm s**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water temperature tendency due to direct bias correction
'K s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water salinity tendency due to direct bias correction
'g kg**-1 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water upward mass transport
'kg m**-2 s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water age since surface contact
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water downward short-wave radiation flux
'W m**-2' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation net short wave radiation rate at sea surface
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind stress at sea surface
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind speed at 10m above sea surface
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation neutral drag coefficient at 10m above sea surface
'dimensionless' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow precipitation rate at sea surface
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component of wind stress over sea ice
'N m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Virtual temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Pressure tendency
'Pa s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	}
#ICAO Standard Atmosphere reference height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Geometrical height above ground
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	}
#Standard deviation of height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	}
#Virtual potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Pseudo-adiabatic potential temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Dew point temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Lapse rate
'K m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Visibility
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	}
#Radar spectra (1)
'~' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 6 ;
	}
#Radar spectra (2)
'~' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 7 ;
	}
#Radar spectra (3)
'~' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 8 ;
	}
#Parcel lifted index (to 500 hPa)
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	}
#Temperature anomaly
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Pressure anomaly
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	}
#Geopotential height anomaly
'gpm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	}
#Wave spectra (1)
'~' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Wave spectra (2)
'~' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Wave spectra (3)
'~' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Wind direction
'Degree true' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	}
#Sigma coordinate vertical velocity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 7 ;
	}
#Absolute vorticity
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	}
#Absolute divergence
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 11 ;
	}
#Vertical u-component shear
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	}
#Vertical v-component shear
's**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	}
#Precipitable water
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Saturation deficit
'Pa' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Precipitation rate
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Thunderstorm probability
'%' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 2 ;
	}
#Convective precipitation (water)
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 10 ;
	}
#Mixed layer depth
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	}
#Transient thermocline depth
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 2 ;
	}
#Main thermocline anomaly
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 1 ;
	}
#Best lifted index (to 500 hPa)
'K' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 1 ;
	}
#Soil moisture content
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Salinity
'kg kg**-1' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Density
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	}
#Direction of ice drift
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#Speed of ice drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	}
#Ice divergence
's**-1' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 7 ;
	}
#Snowmelt
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Direction of swell waves
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Secondary wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Net short-wave radiation flux (surface)
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	}
#Global radiation flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Radiance (with respect to wave number)
'W m**-1 sr**-1' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 5 ;
	}
#Radiance (with respect to wave length)
'W m**-3 sr**-1' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 6 ;
	}
#Wind mixing energy
'J' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	}
#Total precipitation less than 0.1 mm
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 1 ;
	 probabilityType = 0 ;
	}
#Total precipitation rate less than 1 mm/day
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	 scaledValueOfLowerLimit = 115741 ;
	 scaleFactorOfLowerLimit = 10 ;
	 probabilityType = 0 ;
	}
#Total precipitation rate of at least 3 mm/day
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	 scaledValueOfLowerLimit = 347222 ;
	 scaleFactorOfLowerLimit = 10 ;
	 probabilityType = 3 ;
	}
#Total precipitation rate of at least 5 mm/day
'%' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	 scaledValueOfLowerLimit = 578704 ;
	 scaleFactorOfLowerLimit = 10 ;
	 probabilityType = 3 ;
	}
#10 metre Wind speed of at least 10 m/s
'%' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre Wind speed of at least 15 m/s
'%' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 15 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust of at least 15 m/s
'%' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 15 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust of at least 20 m/s
'%' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 20 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#2 metre temperature less than 273.15 K
'%' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 27315 ;
	 scaleFactorOfLowerLimit = 2 ;
	 probabilityType = 0 ;
	}
#Significant wave height of at least 2 m
'%' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 2 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Significant wave height of at least 4 m
'%' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 4 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Significant wave height of at least 6 m
'%' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 6 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Significant wave height of at least 8 m
'%' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 8 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Snowfall index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = [102,1] ;
	}
#10 metre speed index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,0] ;
	}
#2 metre temperature index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,0] ;
	}
#Maximum temperature at 2 metres index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Minimum temperature at 2 metres index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,3] ;
	}
#Total precipitation index
'Numeric' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = [102,1] ;
	}
#Period corresponding to maximum individual wave height
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Envelop-maximum individual wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	}
#Model bathymetry
'm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	}
#Mean wave period based on first moment
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Mean zero-crossing wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	}
#Wave spectral directional width
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	}
#Mean wave period based on first moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	}
#Mean wave period based on second moment for wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	}
#Wave spectral directional width for wind waves
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	}
#Mean wave period based on first moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Mean wave period based on second moment for swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	}
#Wave spectral directional width for swell
'radians' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	}
#Significant height of combined wind waves and swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Mean wave direction
'Degree true' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Peak wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	}
#Mean wave period
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Coefficient of drag with waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Significant height of wind waves
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Mean direction of wind waves
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	}
#Mean period of wind waves
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Significant height of total swell
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Mean direction of total swell
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	}
#Mean period of total swell
's' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Mean square slope of waves
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#10 metre wind speed
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Altimeter wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 37 ;
	}
#Altimeter corrected wave height
'm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 38 ;
	}
#Altimeter range relative correction
'~' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	}
#10 metre wind direction
'degrees' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#2D wave spectra (single)
'm**2 s radian**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 86 ;
	}
#Wave spectral kurtosis
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	}
#Benjamin-Feir index
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	}
#Wave spectral peakedness
'dimensionless' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	}
#Skin reservoir content
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	}
#Total column vertically-integrated mass of atmosphere
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated kinetic energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated enthalpy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated potential + internal energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated potential+internal+latent energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated total energy
'J m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward mass flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward kinetic energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward heat flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward heat flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward water vapour flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward geopotential flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward total energy flux
'W m**-1' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Total column vertically-integrated northward ozone flux
'kg m**-1 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Total column vertically-integrated divergence of mass flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of kinetic energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated moisture divergence flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of geopotential flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of total energy flux
'W m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of ozone flux
'kg m**-2 s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Time-integrated temperature tendency due to short-wave radiation
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to long-wave radiation
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to short wave radiation, clear sky
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to long-wave radiation, clear sky
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated updraught mass flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated downdraught mass flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated updraught detrainment rate
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated downdraught detrainment rate
'kg m**-3' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total precipitation flux
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated turbulent diffusion coefficient for heat
'm**2' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to parametrisations
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated specific humidity tendency due to parametrisations
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated eastward wind tendency due to parametrisations
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward wind tendency due to parametrisations
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface runoff
'kg m**-2' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	}
#Nitrogen dioxide mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 5 ;
	 is_chemical = 1 ;
	}
#Sulphur dioxide mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 is_chemical = 1 ;
	}
#Carbon monoxide mass mixing ratio
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 4 ;
	 is_chemical = 1 ;
	}
#Ozone mass mixing ratio (full chemistry scheme)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 0 ;
	 is_chemical = 1 ;
	}
#Nitrogen dioxide mass mixing ratio difference
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 5 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Sulphur dioxide mass mixing ratio difference
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Carbon monoxide mass mixing ratio difference
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 4 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Ozone mass mixing ratio difference (full chemistry scheme)
'kg kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 0 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Convective inhibition
'J kg**-1' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Orography
'm' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Friction velocity
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	}
#Time-mean 2 metre temperature
'K' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind speed
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Lake total depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Lake mix-layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake mix-layer depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake bottom temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake total layer temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Lake shape factor
'dimensionless' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	}
#Lake ice surface temperature
'K' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake ice total depth
'm' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Minimum vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	}
#Mean vertical gradient of refractivity inside trapping layer
'm**-1' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	}
#Duct base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	}
#Trapping layer base height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	}
#Trapping layer top height
'm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	}
#10 metre u-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#10 metre v-component of neutral wind
'm s**-1' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Snow depth water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	}
#Snowfall water equivalent
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Total Cloud Cover
'%' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Field capacity
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Wilting point
'kg m**-3' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Total Precipitation
'kg m**-2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#U-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#V-component surface stokes drift
'm s**-1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
}
