# Automatically generated by ./create_defs.grib2.lte33.pl, do not edit
#Total precipitation of at least 1 mm
'tpg1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 5 mm
'tpg5' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 5 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 10 mm
'tpg10' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 20 mm
'tpg20' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 20 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 40 mm
'tpg40' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 40 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 60 mm
'tpg60' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 60 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 80 mm
'tpg80' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 80 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 100 mm
'tpg100' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 100 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 150 mm
'tpg150' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 150 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 200 mm
'tpg200' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 200 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 300 mm
'tpg300' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 3 ;
	 scaleFactorOfLowerLimit = -2 ;
	 probabilityType = 3 ;
	}
#Stream function
'strf' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	}
#Velocity potential
'vp' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	}
#Potential temperature
'pt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Wind speed
'ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	}
#Surface photosynthetically active radiation, clear sky
'parcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Unbalanced component of temperature
'uctp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	}
#Unbalanced component of logarithm of surface pressure
'ucln' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 31 ;
	}
#Unbalanced component of divergence
'ucdv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 45 ;
	}
#Lake cover
'cl' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#Low vegetation cover
'cvl' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	}
#High vegetation cover
'cvh' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	}
#Type of low vegetation
'tvl' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	}
#Type of high vegetation
'tvh' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	}
#Sea ice area fraction
'ci' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Snow density
'rsn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	}
#Sea surface temperature
'sst' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil type
'slt' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	}
#Surface direct normal short-wave (solar) radiation
'dsrp' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated magnitude of turbulent surface stress
'magss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Maximum 10 metre wind gust since previous post-processing
'10fg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Large-scale precipitation fraction
'lspf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Montgomery potential
'mont' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	}
#Pressure
'pres' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	}
#Surface downward UV radiation
'uvb' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Photosynthetically active radiation at the surface
'par' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective available potential energy
'cape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Potential vorticity
'pv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	}
#Leaf area index, low vegetation
'lai_lv' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	}
#Leaf area index, high vegetation
'lai_hv' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	}
#Standard deviation of filtered subgrid orography (climatological)
'sdfor' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 38 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Specific rain water content
'crwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	}
#Specific snow water content
'cswc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	}
#Eta-coordinate vertical velocity
'etadot' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	}
#Total column cloud liquid water
'tclw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column cloud ice water
'tciw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Maximum temperature at 2 metres in the last 6 hours
'mx2t6' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	 lengthOfTimeRange = 6 ;
	}
#Minimum temperature at 2 metres in the last 6 hours
'mn2t6' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	 lengthOfTimeRange = 6 ;
	}
#Surface emissivity
'emis' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Geopotential
'z' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Temperature
't' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#U component of wind
'u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#V component of wind
'v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	}
#Specific humidity
'q' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Surface pressure
'sp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Vertical velocity
'w' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	}
#Total column water
'tcw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated water vapour
'tcwv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Vorticity (relative)
'vo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	}
#Time-integrated boundary layer dissipation
'bld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated surface sensible heat net flux
'sshf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated surface latent heat net flux
'slhf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Charnock
'chnk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	}
#Surface net radiation (SW and LW)
'snr' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net radiation (SW and LW)
'tnr' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Mean sea level pressure
'msl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	}
#Divergence
'd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	}
#Geopotential height
'gh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Relative humidity
'r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Boundary layer height
'blh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	}
#Standard deviation of sub-gridscale orography
'sdor' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	}
#Anisotropy of sub-gridscale orography
'isor' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 24 ;
	}
#Angle of sub-gridscale orography
'anor' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	}
#Slope of sub-gridscale orography
'slor' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	}
#10 metre U wind component
'10u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#10 metre V wind component
'10v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#2 metre temperature
'2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#2 metre dewpoint temperature
'2d' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Surface short-wave (solar) radiation downwards
'ssrd' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Land-sea mask
'lsm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness (climatological)
'sr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Surface long-wave (thermal) radiation downwards
'strd' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net short-wave (solar) radiation
'ssr' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net long-wave (thermal) radiation
'str' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net short-wave (solar) radiation
'tsr' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net long-wave (thermal) radiation
'ttr' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated eastward turbulent surface stress
'ewss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward turbulent surface stress
'nsss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Sunshine duration
'sund' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Brightness temperature
'btmp' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 4 ;
	}
#Eastward gravity wave surface stress
'lgws' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Northward gravity wave surface stress
'mgws' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Gravity wave dissipation
'gwd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Maximum temperature at 2 metres since previous post-processing
'mx2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Minimum temperature at 2 metres since previous post-processing
'mn2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Ozone mass mixing ratio
'o3' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	}
#10 metre wind speed
'10si' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Top net short-wave (solar) radiation, clear sky
'tsrc' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Top net long-wave (thermal) radiation, clear sky
'ttrc' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net short-wave (solar) radiation, clear sky
'ssrc' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface net long-wave (thermal) radiation, clear sky
'strc' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#TOA incident short-wave (solar) radiation
'tisr' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Vertically integrated moisture divergence
'vimd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Instantaneous eastward turbulent surface stress
'iews' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress
'inss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous surface sensible heat net flux
'ishf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous moisture flux
'ie' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Logarithm of surface roughness length for heat (climatological)
'lsrh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Skin temperature
'skt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Temperature of snow layer
'tsn' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	}
#Forecast surface roughness
'fsr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Forecast logarithm of surface roughness for heat
'flsr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Specific cloud liquid water content
'clwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	}
#Specific cloud ice water content
'ciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	}
#Fraction of cloud cover
'cc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	}
#Large-scale precipitation
'lsp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snow depth
'sde' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	}
#Low cloud cover
'lcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	}
#Medium cloud cover
'mcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	}
#High cloud cover
'hcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	}
#Probability of a tropical storm
'pts' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 6 ;
	 probabilityType = 9 ;
	}
#Probability of a hurricane
'ph' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 5 ;
	 probabilityType = 9 ;
	}
#Probability of a tropical depression
'ptd' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 7 ;
	 probabilityType = 9 ;
	}
#Total precipitation of at least 25 mm
'tpg25' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Total precipitation of at least 50 mm
'tpg50' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 50 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust of at least 10 m/s
'10fgg10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre convective gust of at least 25 m/s
'10cogug25' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Hail of at least 2 cm diameter
'hlg2' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 134 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 2 ;
	 probabilityType = 3 ;
	}
#Hail of at least 5 cm diameter
'hlg5' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 134 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 25 ;
	 scaleFactorOfLowerLimit = 3 ;
	 probabilityType = 3 ;
	}
#Thunderstorm in vicinity (lightning-based)
'tsprb' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 0 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Convective available potential energy shear index
'capesi' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Water vapour flux index
'wvfi' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 166 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = [102,0] ;
	}
#Convective available potential energy index
'capei' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Maximum of significant wave height index
'maxswhi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Probability of temperature standardized anomaly greater than 1 standard deviation
'ptsa_gt_1stdev' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Probability of temperature standardized anomaly greater than 1.5 standard deviation
'ptsa_gt_1p5stdev' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = 15 ;
	 scaleFactorOfLowerLimit = 1 ;
	 probabilityType = 3 ;
	}
#Probability of temperature standardized anomaly greater than 2 standard deviation
'ptsa_gt_2stdev' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = 2 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Probability of temperature standardized anomaly less than -1 standard deviation
'ptsa_lt_1stdev' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = -1 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 0 ;
	}
#Probability of temperature standardized anomaly less than -1.5 standard deviation
'ptsa_lt_1p5stdev' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = -15 ;
	 scaleFactorOfLowerLimit = 1 ;
	 probabilityType = 0 ;
	}
#Probability of temperature standardized anomaly less than -2 standard deviation
'ptsa_lt_2stdev' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 10 ;
	 scaledValueOfLowerLimit = -2 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 0 ;
	}
#Wave induced mean sea level correction
'weta' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	}
#Ratio of wave angular and frequency width
'wraf' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	}
#Number of events in freak waves statistics
'wnslc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	}
#U-component of atmospheric surface momentum flux
'utaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#V-component of atmospheric surface momentum flux
'vtaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#U-component of surface momentum flux into ocean
'utauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#V-component of surface momentum flux into ocean
'vtauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Wave turbulent energy flux into ocean
'wphio' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	}
#Wave directional width of first swell partition
'wdw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	}
#Wave frequency width of first swell partition
'wfw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	}
#Wave directional width of second swell partition
'wdw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	}
#Wave frequency width of second swell partition
'wfw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	}
#Wave directional width of third swell partition
'wdw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	}
#Wave frequency width of third swell partition
'wfw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	}
#Wave energy flux magnitude
'wefxm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	}
#Wave energy flux mean direction
'wefxd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'h1012' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 12 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'h1214' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 12 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 14 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'h1417' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 14 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 17 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'h1721' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 17 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 21 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'h2125' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 21 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 25 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'h2530' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 7 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 25 ;
	 scaleFactorOfUpperWavePeriodLimit = 0 ;
	 scaledValueOfUpperWavePeriodLimit = 30 ;
	}
#Significant wave height of all waves with period larger than 10s
'sh10' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfWavePeriodInterval = 3 ;
	 scaleFactorOfLowerWavePeriodLimit = 0 ;
	 scaledValueOfLowerWavePeriodLimit = 10 ;
	}
#Significant wave height of first swell partition
'swh1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	}
#Mean wave direction of first swell partition
'mwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	}
#Mean wave period of first swell partition
'mwp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	}
#Significant wave height of second swell partition
'swh2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	}
#Mean wave direction of second swell partition
'mwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	}
#Mean wave period of second swell partition
'mwp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	}
#Significant wave height of third swell partition
'swh3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	}
#Mean wave direction of third swell partition
'mwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	}
#Mean wave period of third swell partition
'mwp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	}
#Time domain maximum individual crest height
'tdcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	}
#Time domain maximum individual wave height
'tdhmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	}
#Space time maximum individual crest height
'stcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	}
#Space time maximum individual wave height
'sthmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	}
#Peak wave period of wind waves
'pp1dw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	}
#Peak wave period of total swell
'pp1ds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	}
#Peak wave period of first swell partition
'pp1d1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	}
#Peak wave period of second swell partition
'pp1d2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	}
#Peak wave period of third swell partition
'pp1d3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	}
#Peak wave direction (total)
'pwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	}
#Peak wave direction of wind waves
'pwdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	}
#Peak wave direction of total swell
'pwds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	}
#Peak wave direction of first swell partition
'pwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	}
#Peak wave direction of second swell partition
'pwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	}
#Peak wave direction of third swell partition
'pwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	}
#Whitecap fraction
'wcfr' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	}
#Benjamin-Feir index 2D
'bfi2d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	}
#Crest-trough correlation
'ctc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	}
#X component of the wave radiative stress to sea ice
'xwrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	}
#Y component of the wave radiative stress to sea ice
'ywrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	}
#Wave Spectral Skewness
'wss' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	}
#Free convective velocity over the oceans
'wstar' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	}
#Air density over the oceans
'rhoao' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	}
#Normalized energy flux into waves
'phiaw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	}
#Normalized energy flux into ocean
'phioc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	}
#Normalized stress into ocean
'tauoc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	}
#Time-mean wave induced mean sea level correction
'avg_weta' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ratio of wave angular and frequency width
'avg_wraf' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean number of events in freak waves statistics
'avg_wnslc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component of atmospheric surface momentum flux
'avg_utaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of atmospheric surface momentum flux
'avg_vtaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component of surface momentum flux into ocean
'avg_utauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of surface momentum flux into ocean
'avg_vtauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave turbulent energy flux into ocean
'avg_wphio' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave directional width of first swell partition
'avg_wdw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave frequency width of first swell partition
'avg_wfw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave directional width of second swell partition
'avg_wdw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave frequency width of second swell partition
'avg_wfw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave directional width of third swell partition
'avg_wdw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave frequency width of third swell partition
'avg_wfw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave energy flux magnitude
'avg_wefxm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave energy flux mean direction
'avg_wefxd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant wave height of first swell partition
'avg_swh1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction of first swell partition
'avg_mwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period of first swell partition
'avg_mwp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant wave height of second swell partition
'avg_swh2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction of second swell partition
'avg_mwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period of second swell partition
'avg_mwp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant wave height of third swell partition
'avg_swh3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction of third swell partition
'avg_mwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period of third swell partition
'avg_mwp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean time domain maximum individual crest height
'avg_tdcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean time domain maximum individual wave height
'avg_tdhmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean space time maximum individual crest height
'avg_stcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean space time maximum individual wave height
'avg_sthmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of wind waves
'avg_pp1dw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of total swell
'avg_pp1ds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of first swell partition
'avg_pp1d1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of second swell partition
'avg_pp1d2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period of third swell partition
'avg_pp1d3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction (total)
'avg_pwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of wind waves
'avg_pwdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of total swell
'avg_pwds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of first swell partition
'avg_pwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of second swell partition
'avg_pwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave direction of third swell partition
'avg_pwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean whitecap fraction
'avg_wcfr' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Benjamin-Feir index 2D
'avg_bfi2d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean crest-trough correlation
'avg_ctc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X component of the wave radiative stress to sea ice
'avg_xwrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y component of the wave radiative stress to sea ice
'avg_ywrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave Spectral Skewness
'avg_wss' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean free convective velocity over the oceans
'avg_wstar' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean air density over the oceans
'avg_rhoao' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean normalized energy flux into waves
'avg_phiaw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean normalized energy flux into ocean
'avg_phioc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean normalized stress into ocean
'avg_tauoc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component surface stokes drift
'avg_ust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component surface stokes drift
'avg_vst' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean period corresponding to maximum individual wave height
'avg_tmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean envelop-maximum individual wave height
'avg_hmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on first moment
'avg_mp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean zero-crossing wave period
'avg_mp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral directional width
'avg_wdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on first moment for wind waves
'avg_p1ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on second moment for wind waves
'avg_p2ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral directional width for wind waves
'avg_dwww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on first moment for swell
'avg_p1ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period based on second moment for swell
'avg_p2ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral directional width for swell
'avg_dwps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant height of combined wind waves and swell
'avg_swh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave direction
'avg_mwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean peak wave period
'avg_pp1d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean wave period
'avg_mwp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean coefficient of drag with waves
'avg_cdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant height of wind waves
'avg_shww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean direction of wind waves
'avg_mdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean period of wind waves
'avg_mpww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean significant height of total swell
'avg_shts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean direction of total swell
'avg_mdts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean period of total swell
'avg_mpts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean square slope of waves
'avg_msqs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind speed
'avg_wind' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind direction
'avg_dwi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral kurtosis
'avg_wsk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean benjamin-Feir index
'avg_bfi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wave spectral peakedness
'avg_wsp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-maximum wave induced mean sea level correction
'max_weta' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ratio of wave angular and frequency width
'max_wraf' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum number of events in freak waves statistics
'max_wnslc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component of atmospheric surface momentum flux
'max_utaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component of atmospheric surface momentum flux
'max_vtaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component of surface momentum flux into ocean
'max_utauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component of surface momentum flux into ocean
'max_vtauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave turbulent energy flux into ocean
'max_wphio' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave directional width of first swell partition
'max_wdw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave frequency width of first swell partition
'max_wfw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave directional width of second swell partition
'max_wdw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave frequency width of second swell partition
'max_wfw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave directional width of third swell partition
'max_wdw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave frequency width of third swell partition
'max_wfw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave energy flux magnitude
'max_wefxm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave energy flux mean direction
'max_wefxd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant wave height of first swell partition
'max_swh1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction of first swell partition
'max_mwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period of first swell partition
'max_mwp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant wave height of second swell partition
'max_swh2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction of second swell partition
'max_mwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period of second swell partition
'max_mwp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant wave height of third swell partition
'max_swh3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction of third swell partition
'max_mwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period of third swell partition
'max_mwp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum time domain maximum individual crest height
'max_tdcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum time domain maximum individual wave height
'max_tdhmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum space time maximum individual crest height
'max_stcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum space time maximum individual wave height
'max_sthmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of wind waves
'max_pp1dw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of total swell
'max_pp1ds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of first swell partition
'max_pp1d1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of second swell partition
'max_pp1d2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period of third swell partition
'max_pp1d3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction (total)
'max_pwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of wind waves
'max_pwdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of total swell
'max_pwds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of first swell partition
'max_pwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of second swell partition
'max_pwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave direction of third swell partition
'max_pwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum whitecap fraction
'max_wcfr' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Benjamin-Feir index 2D
'max_bfi2d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum crest-trough correlation
'max_ctc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum X component of the wave radiative stress to sea ice
'max_xwrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Y component of the wave radiative stress to sea ice
'max_ywrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave Spectral Skewness
'max_wss' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum free convective velocity over the oceans
'max_wstar' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum air density over the oceans
'max_rhoao' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum normalized energy flux into waves
'max_phiaw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum normalized energy flux into ocean
'max_phioc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum normalized stress into ocean
'max_tauoc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum U-component surface stokes drift
'max_ust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum V-component surface stokes drift
'max_vst' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum period corresponding to maximum individual wave height
'max_tmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum envelop-maximum individual wave height
'max_hmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on first moment
'max_mp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean zero-crossing wave period
'max_mp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral directional width
'max_wdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on first moment for wind waves
'max_p1ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on second moment for wind waves
'max_p2ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral directional width for wind waves
'max_dwww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on first moment for swell
'max_p1ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period based on second moment for swell
'max_p2ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral directional width for swell
'max_dwps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant height of combined wind waves and swell
'max_swh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave direction
'max_mwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum peak wave period
'max_pp1d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean wave period
'max_mwp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum coefficient of drag with waves
'max_cdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant height of wind waves
'max_shww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean direction of wind waves
'max_mdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean period of wind waves
'max_mpww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum significant height of total swell
'max_shts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean direction of total swell
'max_mdts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean period of total swell
'max_mpts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean square slope of waves
'max_msqs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind speed
'max_wind' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind direction
'max_dwi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral kurtosis
'max_wsk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum benjamin-Feir index
'max_bfi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wave spectral peakedness
'max_wsp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum wave induced mean sea level correction
'min_weta' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ratio of wave angular and frequency width
'min_wraf' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum number of events in freak waves statistics
'min_wnslc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component of atmospheric surface momentum flux
'min_utaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component of atmospheric surface momentum flux
'min_vtaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component of surface momentum flux into ocean
'min_utauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component of surface momentum flux into ocean
'min_vtauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave turbulent energy flux into ocean
'min_wphio' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave directional width of first swell partition
'min_wdw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave frequency width of first swell partition
'min_wfw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave directional width of second swell partition
'min_wdw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave frequency width of second swell partition
'min_wfw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave directional width of third swell partition
'min_wdw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave frequency width of third swell partition
'min_wfw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave energy flux magnitude
'min_wefxm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave energy flux mean direction
'min_wefxd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant wave height of first swell partition
'min_swh1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction of first swell partition
'min_mwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period of first swell partition
'min_mwp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant wave height of second swell partition
'min_swh2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction of second swell partition
'min_mwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period of second swell partition
'min_mwp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant wave height of third swell partition
'min_swh3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction of third swell partition
'min_mwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period of third swell partition
'min_mwp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum time domain maximum individual crest height
'min_tdcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum time domain maximum individual wave height
'min_tdhmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum space time maximum individual crest height
'min_stcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum space time maximum individual wave height
'min_sthmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of wind waves
'min_pp1dw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of total swell
'min_pp1ds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of first swell partition
'min_pp1d1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of second swell partition
'min_pp1d2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period of third swell partition
'min_pp1d3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction (total)
'min_pwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of wind waves
'min_pwdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of total swell
'min_pwds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of first swell partition
'min_pwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of second swell partition
'min_pwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave direction of third swell partition
'min_pwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum whitecap fraction
'min_wcfr' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Benjamin-Feir index 2D
'min_bfi2d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum crest-trough correlation
'min_ctc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum X component of the wave radiative stress to sea ice
'min_xwrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Y component of the wave radiative stress to sea ice
'min_ywrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave Spectral Skewness
'min_wss' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum free convective velocity over the oceans
'min_wstar' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum air density over the oceans
'min_rhoao' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum normalized energy flux into waves
'min_phiaw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum normalized energy flux into ocean
'min_phioc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum normalized stress into ocean
'min_tauoc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum U-component surface stokes drift
'min_ust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum V-component surface stokes drift
'min_vst' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum period corresponding to maximum individual wave height
'min_tmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum envelop-maximum individual wave height
'min_hmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on first moment
'min_mp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean zero-crossing wave period
'min_mp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral directional width
'min_wdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on first moment for wind waves
'min_p1ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on second moment for wind waves
'min_p2ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral directional width for wind waves
'min_dwww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on first moment for swell
'min_p1ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period based on second moment for swell
'min_p2ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral directional width for swell
'min_dwps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant height of combined wind waves and swell
'min_swh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave direction
'min_mwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum peak wave period
'min_pp1d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean wave period
'min_mwp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum coefficient of drag with waves
'min_cdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant height of wind waves
'min_shww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean direction of wind waves
'min_mdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean period of wind waves
'min_mpww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum significant height of total swell
'min_shts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean direction of total swell
'min_mdts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean period of total swell
'min_mpts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean square slope of waves
'min_msqs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind speed
'min_wind' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind direction
'min_dwi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral kurtosis
'min_wsk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum benjamin-Feir index
'min_bfi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wave spectral peakedness
'min_wsp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation wave induced mean sea level correction
'std_weta' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 87 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ratio of wave angular and frequency width
'std_wraf' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 80 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation number of events in freak waves statistics
'std_wnslc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 89 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component of atmospheric surface momentum flux
'std_utaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 103 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component of atmospheric surface momentum flux
'std_vtaua' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 104 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component of surface momentum flux into ocean
'std_utauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 90 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component of surface momentum flux into ocean
'std_vtauo' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 91 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave turbulent energy flux into ocean
'std_wphio' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 92 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave directional width of first swell partition
'std_wdw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave frequency width of first swell partition
'std_wfw1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave directional width of second swell partition
'std_wdw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave frequency width of second swell partition
'std_wfw2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave directional width of third swell partition
'std_wdw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave frequency width of third swell partition
'std_wfw3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave energy flux magnitude
'std_wefxm' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 78 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave energy flux mean direction
'std_wefxd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 79 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant wave height of first swell partition
'std_swh1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction of first swell partition
'std_mwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period of first swell partition
'std_mwp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant wave height of second swell partition
'std_swh2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction of second swell partition
'std_mwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period of second swell partition
'std_mwp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant wave height of third swell partition
'std_swh3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction of third swell partition
'std_mwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period of third swell partition
'std_mwp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation time domain maximum individual crest height
'std_tdcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 94 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation time domain maximum individual wave height
'std_tdhmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 95 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation space time maximum individual crest height
'std_stcmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 96 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation space time maximum individual wave height
'std_sthmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 97 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of wind waves
'std_pp1dw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 35 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of total swell
'std_pp1ds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of first swell partition
'std_pp1d1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 65 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of second swell partition
'std_pp1d2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 66 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period of third swell partition
'std_pp1d3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 67 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction (total)
'std_pwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of wind waves
'std_pwdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 71 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of total swell
'std_pwds' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 72 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of first swell partition
'std_pwd1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 68 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of second swell partition
'std_pwd2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 69 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave direction of third swell partition
'std_pwd3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 70 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation whitecap fraction
'std_wcfr' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 73 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Benjamin-Feir index 2D
'std_bfi2d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 99 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation crest-trough correlation
'std_ctc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 100 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation X component of the wave radiative stress to sea ice
'std_xwrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 101 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Y component of the wave radiative stress to sea ice
'std_ywrs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 102 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave Spectral Skewness
'std_wss' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 77 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation free convective velocity over the oceans
'std_wstar' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 81 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation air density over the oceans
'std_rhoao' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 82 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation normalized energy flux into waves
'std_phiaw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation normalized energy flux into ocean
'std_phioc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation normalized stress into ocean
'std_tauoc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation U-component surface stokes drift
'std_ust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation V-component surface stokes drift
'std_vst' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation period corresponding to maximum individual wave height
'std_tmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation envelop-maximum individual wave height
'std_hmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on first moment
'std_mp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean zero-crossing wave period
'std_mp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral directional width
'std_wdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on first moment for wind waves
'std_p1ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on second moment for wind waves
'std_p2ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral directional width for wind waves
'std_dwww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on first moment for swell
'std_p1ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period based on second moment for swell
'std_p2ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral directional width for swell
'std_dwps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant height of combined wind waves and swell
'std_swh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave direction
'std_mwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation peak wave period
'std_pp1d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean wave period
'std_mwp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation coefficient of drag with waves
'std_cdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant height of wind waves
'std_shww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean direction of wind waves
'std_mdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean period of wind waves
'std_mpww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation significant height of total swell
'std_shts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean direction of total swell
'std_mdts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean period of total swell
'std_mpts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean square slope of waves
'std_msqs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind speed
'std_wind' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind direction
'std_dwi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral kurtosis
'std_wsk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation benjamin-Feir index
'std_bfi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wave spectral peakedness
'std_wsp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Mean sea water temperature in the upper 300 m
'mswt300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column vertically-integrated divergence of water geopotential flux
'viwgd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward enthalpy flux
'viee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward enthalpy flux
'vien' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of enthalpy flux
'vied' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of water enthalpy flux
'viwed' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of water vapour flux
'viwvd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of cloud liquid water flux
'viclwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of cloud ice water flux
'viciwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of rain flux
'vird' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of snow flux
'visd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward cloud liquid water flux
'viclwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward cloud liquid water flux
'viclwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward cloud ice water flux
'viciwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward cloud ice water flux
'viciwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward rain flux
'vire' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward rain flux
'virn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward snow flux
'vise' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward snow flux
'visn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated net source of ozone
'vions' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Total column vertically-integrated water enthalpy
'viwe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Temperature tendency due to short-wave radiation
'ttswr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#Temperature tendency due to long-wave radiation
'ttlwr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Temperature tendency due to short-wave radiation, clear sky
'ttswrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Temperature tendency due to long-wave radiation, clear sky
'ttlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Updraught mass flux
'umf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	}
#Downdraught mass flux
'dmf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	}
#Updraught detrainment rate
'udr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	}
#Downdraught detrainment rate
'ddr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	}
#Total precipitation flux
'tpf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	}
#Turbulent diffusion coefficient for heat
'tdch' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#Temperature tendency due to parametrisations
'ttpm' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	}
#Specific humidity tendency due to parametrisations
'qtpm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	}
#Eastward wind tendency due to parametrisations
'utpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	}
#Northward wind tendency due to parametrisations
'vtpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	}
#2 metre specific humidity
'2sh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Injection height (from IS4FIRES)
'injh' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	}
#Particulate matter d <= 1 um
'pm1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 0 ;
	 scaleFactorOfSecondSize = missing() ;
	 scaledValueOfFirstSize = 1 ;
	 scaleFactorOfFirstSize = 6 ;
	 typeOfSizeInterval = 5 ;
	 aerosolType = 62026 ;
	 scaledValueOfSecondSize = missing() ;
	 is_aerosol = 1 ;
	}
#Particulate matter d <= 2.5 um
'pm2p5' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 0 ;
	 scaleFactorOfSecondSize = missing() ;
	 scaledValueOfFirstSize = 25 ;
	 scaleFactorOfFirstSize = 7 ;
	 typeOfSizeInterval = 5 ;
	 aerosolType = 62026 ;
	 scaledValueOfSecondSize = missing() ;
	 is_aerosol = 1 ;
	}
#Particulate matter d <= 10 um
'pm10' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 0 ;
	 scaleFactorOfSecondSize = missing() ;
	 scaledValueOfFirstSize = 10 ;
	 scaleFactorOfFirstSize = 6 ;
	 typeOfSizeInterval = 5 ;
	 aerosolType = 62026 ;
	 scaledValueOfSecondSize = missing() ;
	 is_aerosol = 1 ;
	}
#Mean height of maximum injection
'mami' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	}
#Plume top height above surface
'apt' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 42 ;
	}
#Volcanic sulfur dioxide mass mixing ratio
'VSO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#UV visible albedo for diffuse radiation (climatological)
'aluvd_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation (climatological)
'aluvp_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, geometric component (climatological)
'aluvpg_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, isotropic component (climatological)
'aluvpi_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfGeneratingProcess = 9 ;
	}
#UV visible albedo for direct radiation, volumetric component (climatological)
'aluvpv_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Plume bottom height above surface
'apb' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	}
#Ammonium aerosol mass mixing ratio
'aermr18' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Nitrate aerosol optical depth at 550 nm
'niaod550' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 102 ;
	 scaleFactorOfFirstWavelength = 8 ;
	 typeOfSizeInterval = 255 ;
	 aerosolType = 62004 ;
	 scaledValueOfFirstWavelength = 55 ;
	 typeOfWavelengthInterval = 11 ;
	 is_aerosol_optical = 1 ;
	}
#Ammonium aerosol optical depth at 550 nm
'amaod550' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 102 ;
	 scaleFactorOfFirstWavelength = 8 ;
	 typeOfSizeInterval = 255 ;
	 aerosolType = 62003 ;
	 scaledValueOfFirstWavelength = 55 ;
	 typeOfWavelengthInterval = 11 ;
	 is_aerosol_optical = 1 ;
	}
#Biogenic secondary organic aerosol mass mixing ratio
'aermr19' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Anthropogenic secondary organic aerosol mass mixing ratio
'aermr20' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Near IR albedo for diffuse radiation (climatological)
'alnid_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation (climatological)
'alnip_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, geometric component (climatological)
'alnipg_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, isotropic component (climatological)
'alnipi_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Near IR albedo for direct radiation, volumetric component (climatological)
'alnipv_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Volcanic sulfur dioxide mass mixing ratio increment
'VSO2diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Ammonium aerosol mass mixing ratio
'aermr18diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62003 ;
	 typeOfGeneratingProcess = 20 ;
	 is_aerosol = 1 ;
	}
#Biogenic secondary organic aerosol mass mixing ratio increment
'aermr19diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62032 ;
	 typeOfGeneratingProcess = 20 ;
	 is_aerosol = 1 ;
	}
#Anthropogenic secondary organic aerosol mass mixing ratio increment
'aermr20diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 aerosolType = 62033 ;
	 typeOfGeneratingProcess = 20 ;
	 is_aerosol = 1 ;
	}
#Random pattern 1 for SPP scheme
'spp1' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 1 ;
	}
#Random pattern 2 for SPP scheme
'spp2' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 2 ;
	}
#Random pattern 3 for SPP scheme
'spp3' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 3 ;
	}
#Random pattern 4 for SPP scheme
'spp4' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 4 ;
	}
#Random pattern 5 for SPP scheme
'spp5' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 5 ;
	}
#Random pattern 6 for SPP scheme
'spp6' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 6 ;
	}
#Random pattern 7 for SPP scheme
'spp7' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 7 ;
	}
#Random pattern 8 for SPP scheme
'spp8' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 8 ;
	}
#Random pattern 9 for SPP scheme
'spp9' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 9 ;
	}
#Random pattern 10 for SPP scheme
'spp10' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 10 ;
	}
#Random pattern 11 for SPP scheme
'spp11' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 11 ;
	}
#Random pattern 12 for SPP scheme
'spp12' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 12 ;
	}
#Random pattern 13 for SPP scheme
'spp13' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 13 ;
	}
#Random pattern 14 for SPP scheme
'spp14' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 14 ;
	}
#Random pattern 15 for SPP scheme
'spp15' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 15 ;
	}
#Random pattern 16 for SPP scheme
'spp16' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 16 ;
	}
#Random pattern 17 for SPP scheme
'spp17' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 17 ;
	}
#Random pattern 18 for SPP scheme
'spp18' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 18 ;
	}
#Random pattern 19 for SPP scheme
'spp19' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 19 ;
	}
#Random pattern 20 for SPP scheme
'spp20' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 20 ;
	}
#Random pattern 21 for SPP scheme
'spp21' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 21 ;
	}
#Random pattern 22 for SPP scheme
'spp22' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 22 ;
	}
#Random pattern 23 for SPP scheme
'spp23' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 23 ;
	}
#Random pattern 24 for SPP scheme
'spp24' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 24 ;
	}
#Random pattern 25 for SPP scheme
'spp25' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 25 ;
	}
#Random pattern 26 for SPP scheme
'spp26' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 26 ;
	}
#Random pattern 27 for SPP scheme
'spp27' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 27 ;
	}
#Random pattern 28 for SPP scheme
'spp28' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 28 ;
	}
#Random pattern 29 for SPP scheme
'spp29' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 29 ;
	}
#Random pattern 30 for SPP scheme
'spp30' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 30 ;
	}
#Random pattern 31 for SPP scheme
'spp31' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 31 ;
	}
#Random pattern 32 for SPP scheme
'spp32' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 32 ;
	}
#Random pattern 33 for SPP scheme
'spp33' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 33 ;
	}
#Random pattern 34 for SPP scheme
'spp34' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 34 ;
	}
#Random pattern 35 for SPP scheme
'spp35' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 35 ;
	}
#Random pattern 36 for SPP scheme
'spp36' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 36 ;
	}
#Random pattern 37 for SPP scheme
'spp37' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 37 ;
	}
#Random pattern 38 for SPP scheme
'spp38' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 38 ;
	}
#Random pattern 39 for SPP scheme
'spp39' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 39 ;
	}
#Random pattern 40 for SPP scheme
'spp40' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 40 ;
	}
#Random pattern 41 for SPP scheme
'spp41' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 41 ;
	}
#Random pattern 42 for SPP scheme
'spp42' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 42 ;
	}
#Random pattern 43 for SPP scheme
'spp43' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 43 ;
	}
#Random pattern 44 for SPP scheme
'spp44' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 44 ;
	}
#Random pattern 45 for SPP scheme
'spp45' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 45 ;
	}
#Random pattern 46 for SPP scheme
'spp46' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 46 ;
	}
#Random pattern 47 for SPP scheme
'spp47' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 47 ;
	}
#Random pattern 48 for SPP scheme
'spp48' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 48 ;
	}
#Random pattern 49 for SPP scheme
'spp49' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 49 ;
	}
#Random pattern 50 for SPP scheme
'spp50' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 50 ;
	}
#Random pattern 51 for SPP scheme
'spp51' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 51 ;
	}
#Random pattern 52 for SPP scheme
'spp52' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 52 ;
	}
#Random pattern 53 for SPP scheme
'spp53' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 53 ;
	}
#Random pattern 54 for SPP scheme
'spp54' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 54 ;
	}
#Random pattern 55 for SPP scheme
'spp55' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 55 ;
	}
#Random pattern 56 for SPP scheme
'spp56' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 56 ;
	}
#Random pattern 57 for SPP scheme
'spp57' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 57 ;
	}
#Random pattern 58 for SPP scheme
'spp58' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 58 ;
	}
#Random pattern 59 for SPP scheme
'spp59' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 59 ;
	}
#Random pattern 60 for SPP scheme
'spp60' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 60 ;
	}
#Random pattern 61 for SPP scheme
'spp61' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 61 ;
	}
#Random pattern 62 for SPP scheme
'spp62' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 62 ;
	}
#Random pattern 63 for SPP scheme
'spp63' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 63 ;
	}
#Random pattern 64 for SPP scheme
'spp64' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 64 ;
	}
#Random pattern 65 for SPP scheme
'spp65' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 65 ;
	}
#Random pattern 66 for SPP scheme
'spp66' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 66 ;
	}
#Random pattern 67 for SPP scheme
'spp67' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 67 ;
	}
#Random pattern 68 for SPP scheme
'spp68' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 68 ;
	}
#Random pattern 69 for SPP scheme
'spp69' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 69 ;
	}
#Random pattern 70 for SPP scheme
'spp70' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 70 ;
	}
#Random pattern 71 for SPP scheme
'spp71' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 71 ;
	}
#Random pattern 72 for SPP scheme
'spp72' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 72 ;
	}
#Random pattern 73 for SPP scheme
'spp73' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 73 ;
	}
#Random pattern 74 for SPP scheme
'spp74' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 74 ;
	}
#Random pattern 75 for SPP scheme
'spp75' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 75 ;
	}
#Random pattern 76 for SPP scheme
'spp76' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 76 ;
	}
#Random pattern 77 for SPP scheme
'spp77' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 77 ;
	}
#Random pattern 78 for SPP scheme
'spp78' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 78 ;
	}
#Random pattern 79 for SPP scheme
'spp79' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 79 ;
	}
#Random pattern 80 for SPP scheme
'spp80' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 80 ;
	}
#Random pattern 81 for SPP scheme
'spp81' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 81 ;
	}
#Random pattern 82 for SPP scheme
'spp82' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 82 ;
	}
#Random pattern 83 for SPP scheme
'spp83' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 83 ;
	}
#Random pattern 84 for SPP scheme
'spp84' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 84 ;
	}
#Random pattern 85 for SPP scheme
'spp85' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 85 ;
	}
#Random pattern 86 for SPP scheme
'spp86' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 86 ;
	}
#Random pattern 87 for SPP scheme
'spp87' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 87 ;
	}
#Random pattern 88 for SPP scheme
'spp88' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 88 ;
	}
#Random pattern 89 for SPP scheme
'spp89' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 89 ;
	}
#Random pattern 90 for SPP scheme
'spp90' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 90 ;
	}
#Random pattern 91 for SPP scheme
'spp91' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 91 ;
	}
#Random pattern 92 for SPP scheme
'spp92' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 92 ;
	}
#Random pattern 93 for SPP scheme
'spp93' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 93 ;
	}
#Random pattern 94 for SPP scheme
'spp94' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 94 ;
	}
#Random pattern 95 for SPP scheme
'spp95' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 95 ;
	}
#Random pattern 96 for SPP scheme
'spp96' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 96 ;
	}
#Random pattern 97 for SPP scheme
'spp97' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 97 ;
	}
#Random pattern 98 for SPP scheme
'spp98' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 98 ;
	}
#Random pattern 99 for SPP scheme
'spp99' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 99 ;
	}
#Random pattern 100 for SPP scheme
'spp100' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 100 ;
	}
#Random pattern 101 for SPP scheme
'spp101' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 101 ;
	}
#Random pattern 102 for SPP scheme
'spp102' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 102 ;
	}
#Random pattern 103 for SPP scheme
'spp103' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 103 ;
	}
#Random pattern 104 for SPP scheme
'spp104' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 104 ;
	}
#Random pattern 105 for SPP scheme
'spp105' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 105 ;
	}
#Random pattern 106 for SPP scheme
'spp106' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 106 ;
	}
#Random pattern 107 for SPP scheme
'spp107' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 107 ;
	}
#Random pattern 108 for SPP scheme
'spp108' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 108 ;
	}
#Random pattern 109 for SPP scheme
'spp109' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 109 ;
	}
#Random pattern 110 for SPP scheme
'spp110' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 110 ;
	}
#Random pattern 111 for SPP scheme
'spp111' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 111 ;
	}
#Random pattern 112 for SPP scheme
'spp112' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 112 ;
	}
#Random pattern 113 for SPP scheme
'spp113' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 113 ;
	}
#Random pattern 114 for SPP scheme
'spp114' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 114 ;
	}
#Random pattern 115 for SPP scheme
'spp115' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 115 ;
	}
#Random pattern 116 for SPP scheme
'spp116' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 116 ;
	}
#Random pattern 117 for SPP scheme
'spp117' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 117 ;
	}
#Random pattern 118 for SPP scheme
'spp118' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 118 ;
	}
#Random pattern 119 for SPP scheme
'spp119' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 119 ;
	}
#Random pattern 120 for SPP scheme
'spp120' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 120 ;
	}
#Random pattern 121 for SPP scheme
'spp121' = {
	 discipline = 191 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 randomFieldNumber = 121 ;
	}
#Cosine of solar zenith angle
'cossza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#UV biologically effective dose
'uvbed' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#UV biologically effective dose clear-sky
'uvbedcs' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Dry deposition of ammonium aerosol
'aerddpam' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Sedimentation of ammonium aerosol
'aersdmam' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 11 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of ammonium aerosol by large-scale precipitation
'aerwdlam' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 9 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of ammonium aerosol by convective precipitation
'aerwdcam' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 10 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Vertically integrated mass of ammonium aerosol
'aermssam' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 aerosolType = 62003 ;
	 is_aerosol = 1 ;
	}
#Dry deposition of biogenic secondary organic aerosol
'aerddpsoab' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 12 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Sedimentation of biogenic secondary organic aerosol
'aersdmsoab' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 11 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of biogenic secondary organic aerosol by large-scale precipitation
'aerwdlsoab' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 9 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of biogenic secondary organic aerosol by convective precipitation
'aerwdcsoab' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 10 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Vertically integrated mass of biogenic secondary organic aerosol
'aermsssoab' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 aerosolType = 62032 ;
	 is_aerosol = 1 ;
	}
#Dry deposition of anthropogenic secondary organic aerosol
'aerddpsoaa' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 12 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Sedimentation of anthropogenic secondary organic aerosol
'aersdmsoaa' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 11 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of anthropogenic secondary organic aerosol by large-scale precipitation
'aerwdlsoaa' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 9 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Wet deposition of anthropogenic secondary organic aerosol by convective precipitation
'aerwdcsoaa' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 10 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Vertically integrated mass of anthropogenic secondary organic aerosol
'aermsssoaa' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 aerosolType = 62033 ;
	 is_aerosol = 1 ;
	}
#Secondary organic aerosol optical depth at 550 nm
'soaod550' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 102 ;
	 scaleFactorOfFirstWavelength = 9 ;
	 aerosolType = 62012 ;
	 scaledValueOfFirstWavelength = 550 ;
	 scaledValueOfSecondWavelength = missing() ;
	 typeOfWavelengthInterval = 11 ;
	 scaleFactorOfSecondWavelength = missing() ;
	 is_aerosol_optical = 1 ;
	}
#Ammonia mass mixing ratio
'nh3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 9 ;
	 is_chemical = 1 ;
	}
#Nitrogen monoxide mass mixing ratio
'no' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 11 ;
	 is_chemical = 1 ;
	}
#Aromatic peroxy radical mass mixing ratio
'AROO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Ethyne mass mixing ratio
'C2H2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Acetonitrile mass mixing ratio
'CH3CN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Methyl peroxy nitrate mass mixing ratio
'CH3O2NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Hydrogen cyanide mass mixing ratio
'HCN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Hydroperoxy aldehydes type 1 mass mixing ratio
'HPALD1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Hydroperoxy aldehydes type 2 mass mixing ratio
'HPALD' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Isoprene peroxy type B mass mixing ratio
'ISOPBO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Isoprene peroxy type D mass mixing ratio
'ISOPDO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Volcanic sulfur dioxide
'tc_VSO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Column integrated mass density of Aromatic peroxy radical
'tc_AROO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Ethyne
'tc_C2H2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Acetonitrile
'tc_CH3CN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Methyl peroxy nitrate
'tc_CH3O2NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Hydrogen cyanide
'tc_HCN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Hydroperoxy aldehydes type 1
'tc_HPALD1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Hydroperoxy aldehydes type 2
'tc_HPALD2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Isoprene peroxy type B
'tc_ISOPBO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Column integrated mass density of Isoprene peroxy type D
'tc_ISOPDO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 1 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Volcanic sulfur dioxide
'e_VSO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Atmosphere emission mass flux of Aromatic peroxy radical
'e_AROO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Ethyne
'e_C2H2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Acetonitrile
'e_CH3CN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Methyl peroxy nitrate
'e_CH3O2NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Hydrogen cyanide
'e_HCN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Hydroperoxy aldehydes type 1
'e_HPALD1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Hydroperoxy aldehydes type 2
'e_HPALD2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Isoprene peroxy type B
'e_ISOPBO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Atmosphere emission mass flux of Isoprene peroxy type D
'e_ISOPDO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Volcanic sulfur dioxide
'dv_VSO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 8 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 7 ;
	}
#Dry deposition velocity of Aromatic peroxy radical
'dv_AROO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60029 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Ethyne
'dv_C2H2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10010 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Acetonitrile
'dv_CH3CN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10007 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Methyl peroxy nitrate
'dv_CH3O2NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10055 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Hydrogen cyanide
'dv_HCN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 10006 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Hydroperoxy aldehydes type 1
'dv_HPALD1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60058 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Hydroperoxy aldehydes type 2
'dv_HPALD2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60059 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Isoprene peroxy type B
'dv_ISOPBO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60056 ;
	 is_chemical = 1 ;
	}
#Dry deposition velocity of Isoprene peroxy type D
'dv_ISOPDO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 15 ;
	 constituentType = 60057 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ozone
'acc_dry_depm_O3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydrogen peroxide
'acc_dry_depm_H2O2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 19 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Carbon monoxide
'acc_dry_depm_CO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 4 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitric acid
'acc_dry_depm_HNO3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 17 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methyl peroxide
'acc_dry_depm_CH3OOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10002 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Formaldehyde
'acc_dry_depm_HCHO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 7 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Aldehydes
'acc_dry_depm_ALD2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60018 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Peroxyacetyl nitrate
'acc_dry_depm_PAN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10013 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Peroxides
'acc_dry_depm_ROOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60019 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Organic nitrates
'acc_dry_depm_ONIT' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60020 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Isoprene
'acc_dry_depm_C5H8' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10017 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Sulphur dioxide
'acc_dry_depm_SO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ammonia
'acc_dry_depm_NH3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 9 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Sulfate
'acc_dry_depm_SO4' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 22 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ammonium
'acc_dry_depm_NH4' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methyl glyoxal
'acc_dry_depm_CH3COCHO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10025 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrogen monoxide
'acc_dry_depm_NO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 11 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydroperoxy radical
'acc_dry_depm_HO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 14 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methylperoxy radical
'acc_dry_depm_CH3O2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10001 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrogen dioxide
'acc_dry_depm_NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 5 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrate radical
'acc_dry_depm_NO3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 13 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Dinitrogen pentoxide
'acc_dry_depm_N2O5' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 15 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Pernitric acid
'acc_dry_depm_HO2NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 18 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methanol
'acc_dry_depm_CH3OH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10004 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Formic acid
'acc_dry_depm_HCOOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10005 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methacrylic acid
'acc_dry_depm_MCOOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10027 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ethane
'acc_dry_depm_C2H6' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10008 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Ethanol
'acc_dry_depm_C2H5OH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10011 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Methacrolein
'acc_dry_depm_ISPD' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10028 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Acetone
'acc_dry_depm_CH3COCH3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10029 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Nitrous acid
'acc_dry_depm_HONO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 16 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Glycolaldehyde
'acc_dry_depm_GLYALD' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10033 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Glyoxal
'acc_dry_depm_GLY' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10038 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydroxyacetone
'acc_dry_depm_HYAC' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10041 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of all hydroxy-peroxides products of the reaction of hydroxy-isoprene adducts with O2
'acc_dry_depm_ISOPOOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 60032 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Acetonitrile
'acc_dry_depm_CH3CN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10007 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated dry deposition mass flux of Hydrogen cyanide
'acc_dry_depm_HCN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 6 ;
	 constituentType = 10006 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ozone
'acc_wet_depm_O3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen peroxide
'acc_wet_depm_H2O2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 19 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitric acid
'acc_wet_depm_HNO3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 17 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methyl peroxide
'acc_wet_depm_CH3OOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10002 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Formaldehyde
'acc_wet_depm_HCHO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 7 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Aldehydes
'acc_wet_depm_ALD2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60018 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Peroxyacetyl nitrate
'acc_wet_depm_PAN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10013 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Peroxides
'acc_wet_depm_ROOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60019 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Organic nitrates
'acc_wet_depm_ONIT' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60020 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Isoprene
'acc_wet_depm_C5H8' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10017 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Sulphur dioxide
'acc_wet_depm_SO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ammonia
'acc_wet_depm_NH3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 9 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Sulfate
'acc_wet_depm_SO4' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 22 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ammonium
'acc_wet_depm_NH4' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methane sulfonic acid
'acc_wet_depm_MSA' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10024 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methyl glyoxal
'acc_wet_depm_CH3COCHO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10025 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Carbon monoxide
'acc_wet_depm_CO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 4 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Lead
'acc_wet_depm_Pb' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 30296 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrogen monoxide
'acc_wet_depm_NO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 11 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroperoxy radical
'acc_wet_depm_HO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 14 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methylperoxy radical
'acc_wet_depm_CH3O2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10001 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrogen dioxide
'acc_wet_depm_NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 5 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrate radical
'acc_wet_depm_NO3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 13 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Dinitrogen pentoxide
'acc_wet_depm_N2O5' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 15 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Pernitric acid
'acc_wet_depm_HO2NO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 18 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methanol
'acc_wet_depm_CH3OH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10004 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Formic acid
'acc_wet_depm_HCOOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10005 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methacrylic acid
'acc_wet_depm_MCOOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10027 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ethane
'acc_wet_depm_C2H6' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10008 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Ethanol
'acc_wet_depm_C2H5OH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10011 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Methacrolein
'acc_wet_depm_ISPD' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10028 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Acetone
'acc_wet_depm_CH3COCH3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10029 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Chlorine nitrate
'acc_wet_depm_CLONO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 30 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hypochlorous acid
'acc_wet_depm_HOCL' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 29 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen bromide
'acc_wet_depm_HBR' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 35 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hypobromous acid
'acc_wet_depm_HOBR' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 36 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Nitrous acid
'acc_wet_depm_HONO' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 16 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Toluene and less reactive aromatics
'acc_wet_depm_TOL' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10022 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Xylene and more reactive aromatics
'acc_wet_depm_XYL' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10023 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Glycolaldehyde
'acc_wet_depm_GLYALD' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10033 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Glyoxal
'acc_wet_depm_GLY' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10038 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroxyacetone
'acc_wet_depm_HYAC' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10041 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of all hydroxy-peroxides products of the reaction of hydroxy-isoprene adducts with O2
'acc_wet_depm_ISOPOOH' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 60032 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Bromine nitrate
'acc_wet_depm_BRONO2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 37 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen chloride
'acc_wet_depm_HCL' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 20001 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Acetonitrile
'acc_wet_depm_CH3CN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10007 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydrogen cyanide
'acc_wet_depm_HCN' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10006 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroperoxy aldehydes type 1
'acc_wet_depm_HPALD1' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10058 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Time-integrated wet deposition mass flux of Hydroperoxy aldehydes type 2
'acc_wet_depm_HPALD2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 7 ;
	 constituentType = 10059 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical = 1 ;
	}
#Geometric height of -10 degrees C atmospheric isothermal level above ground
'degm10l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Surface direct short-wave (solar) radiation
'fdir' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface direct short-wave radiation, clear sky
'cdir' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Cloud base height
'cbh' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	}
#Geometric height of 0 degrees C atmospheric isothermal level above ground
'deg0l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Maximum 10 metre wind gust in the last 3 hours
'10fg3' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	 lengthOfTimeRange = 3 ;
	}
#Instantaneous 10 metre wind gust
'i10fg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Relative humidity with respect to water
'rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	}
#Relative humidity with respect to ice
'rhi' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 94 ;
	}
#Snow albedo
'asn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	}
#Fraction of convective precipitation cover
'fcpc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 37 ;
	}
#Maximum CAPE in the last 6 hours
'mxcape6' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	 lengthOfTimeRange = 6 ;
	}
#Maximum CAPES in the last 6 hours
'mxcapes6' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	 lengthOfTimeRange = 6 ;
	}
#2 metre relative humidity with respect to water
'2rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Liquid water content in snow pack
'lwcs' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	}
#Convective available potential energy shear
'capes' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Tropopause pressure
'trpp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Height of convective cloud top
'hcct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	}
#Height of zero-degree wet-bulb temperature
'hwbt0' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	}
#Height of one-degree wet-bulb temperature
'hwbt1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	}
#Instantaneous total lightning flash density
'litoti' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Averaged total lightning flash density in the last hour
'litota1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 1 ;
	}
#Instantaneous cloud-to-ground lightning flash density
'licgi' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Averaged cloud-to-ground lightning flash density in the last hour
'licga1' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 1 ;
	}
#Unbalanced component of specific humidity
'ucq' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 118 ;
	}
#Unbalanced component of specific cloud liquid water content
'ucclwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 119 ;
	}
#Unbalanced component of specific cloud ice water content
'ucciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 120 ;
	}
#Averaged total lightning flash density in the last 3 hours
'litota3' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 3 ;
	}
#Averaged total lightning flash density in the last 6 hours
'litota6' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 6 ;
	}
#Averaged cloud-to-ground lightning flash density in the last 3 hours
'licga3' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 3 ;
	}
#Averaged cloud-to-ground lightning flash density in the last 6 hours
'licga6' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	 lengthOfTimeRange = 6 ;
	}
#Accumulated Carbon Dioxide Net Ecosystem Exchange
'aco2nee' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated Carbon Dioxide Gross Primary Production
'aco2gpp' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Accumulated Carbon Dioxide Ecosystem Respiration
'aco2rec' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Carbon dioxide net ecosystem exchange flux
'fco2nee' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	}
#Carbon dioxide gross primary production flux
'fco2gpp' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	}
#Carbon dioxide ecosystem respiration flux
'fco2rec' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	}
#Soil moisture top 20 cm
'sm20' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Soil moisture top 100 cm
'sm100' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column supercooled liquid water
'tcslw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column rain water
'tcrw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column snow water
'tcsw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Ice temperature
'ist' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	}
#Soil temperature top 20 cm
'st20' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Soil temperature top 100 cm
'st100' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Atmosphere emission mass flux of Methane from Wetlands
'e_WLCH4' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 2 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 10 ;
	}
#Accumulated mass emission of methane from Wetlands
'acc_e_WLCH4' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 3 ;
	 constituentType = 2 ;
	 typeOfStatisticalProcessing = 1 ;
	 is_chemical_srcsink = 1 ;
	 sourceSinkChemicalPhysicalProcess = 10 ;
	}
#Surface short-wave (solar) radiation downward clear-sky
'ssrdc' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface long-wave (thermal) radiation downward clear-sky
'strdc' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Convective precipitation
'cp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Water runoff and drainage
'ro' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	}
#Instantaneous large-scale precipitation fraction
'ilspf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Convective rain rate
'crr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Large scale rain rate
'lsrr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Convective snowfall rate water equivalent
'csfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Large scale snowfall rate water equivalent
'lssfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Maximum total precipitation rate since previous post-processing
'mxtpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Minimum total precipitation rate since previous post-processing
'mntpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Mixed-layer CAPE in the lowest 50 hPa
'mlcape50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Mixed-layer CIN in the lowest 50 hPa
'mlcin50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Mixed-layer CAPE in the lowest 100 hPa
'mlcape100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Mixed-layer CIN in the lowest 100 hPa
'mlcin100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Most-unstable CAPE
'mucape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	}
#Most-unstable CIN
'mucin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	}
#Departure level of the most unstable parcel expressed as Pressure
'mudlp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	}
#200 metre U wind component
'200u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 200 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#200 metre V wind component
'200v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 200 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#200 metre wind speed
'200si' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 200 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#100 metre wind speed
'100si' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Urban cover
'cur' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	}
#Road Cover
'cro' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	}
#Building cover
'cbu' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 2 ;
	}
#Building height
'bldh' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	}
#Vertical-to-horizontal area ratio
'hwr' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	}
#Standard deviation of building height
'bhstd' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	}
#Wetland cover
'cwe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 45 ;
	}
#Wetland type
'twe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 46 ;
	}
#Irrigation cover
'cirr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 47 ;
	}
#C4 crop cover
'c4cr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 48 ;
	}
#C4 grass cover
'c4gr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 49 ;
	}
#Accumulated freezing rain water equivalent
'fzrawe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Runoff water equivalent (surface plus subsurface)
'rowe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Snow evaporation water equivalent
'eswe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Potential evaporation rate
'pevr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Potential evaporation
'peva' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Tile fraction
'tifr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 36 ;
	}
#Tile percentage
'tipe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 37 ;
	}
#Forecast logarithm of surface roughness length for moisture
'flsrm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface runoff rate
'surfror' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface runoff
'surfro' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Sub-surface runoff rate
'ssurfror' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Sub-surface runoff
'ssurfro' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Reflectance in 0.4 micron channel
'rfl04' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 30 ;
	}
#Vertical divergence
'vdiv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 59 ;
	}
#Drag thermal coefficient
'dtc' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Drag evaporation coefficient
'dec' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Pressure departure from hydrostatic state
'pdhs' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 46 ;
	}
#Surface net radiation flux (SW and LW)
'snrf' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Top net radiation flux (SW and LW)
'tnrf' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Global irradiance on tilted surfaces
'gits' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 47 ;
	}
#Eady growth rate
'eagr' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 22 ;
	}
#Tropical cyclones track density
'tdtc' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	}
#Canopy air temperature
'cant' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 63 ;
	}
#Soil wetness index (total layer)
'swit' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 151 ;
	}
#Soil wetness index (root zone)
'swir' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	}
#Soil wetness index (layer)
'swil' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 151 ;
	 typeOfSecondFixedSurface = 151 ;
	}
#Distance downward from roof surface
'ddrf' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 6 ;
	}
#Distance inward from outer wall surface
'diws' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 7 ;
	}
#Distance downward from road surface
'ddrd' = {
	 discipline = 2 ;
	 parameterCategory = 6 ;
	 parameterNumber = 8 ;
	}
#Renewable power capacity
'rpc' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Renewable power production rate
'rppr' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Renewable power production
'rpp' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Wind power capacity
'wpc' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Wind power production rate
'wppr' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Wind power production
'wpp' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Solar photovoltaic (PV) power capacity
'pvpc' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Solar photovoltaic (PV) power production rate
'pvppr' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Solar photovoltaic (PV) power production
'pvpp' = {
	 discipline = 20 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Graupel (snow pellets) precipitation
'tgrp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 75 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total lightning flash density
'litotint' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Pressure at cloud base
'pcdb' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 2 ;
	}
#Geometric height of adiabatic condensation level above ground
'hacg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 5 ;
	}
#Geometric height of free convection level above ground
'hfcg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 14 ;
	}
#Geometric height of neutral buoyancy level above ground
'hnbg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 16 ;
	}
#Geometric height of 3 degrees C atmospheric isothermal level above ground
'deg3l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Roof temperature
'rft' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 185 ;
	}
#Wall temperature
'wlt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 186 ;
	}
#Road temperature
'rdt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 187 ;
	}
#Snow depth water equivalent on roof
'sdrf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 185 ;
	}
#Snow depth water equivalent on road
'sdrd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 187 ;
	}
#Urban canyon temperature
'urct' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 187 ;
	 typeOfSecondFixedSurface = 185 ;
	}
#Urban canyon specific humidity
'urcq' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 187 ;
	 typeOfSecondFixedSurface = 185 ;
	}
#Convective snowfall water equivalent
'csfwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Large-scale snowfall water equivalent
'lsfwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Lake surface temperature
'lslt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Surface bulk Richardson number
'sbrn' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness for heat
'srhe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface roughness for moisture
'srmo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 48 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Snow evaporation rate water equivalent
'esrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Radar reflectivity
'rare' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 4 ;
	}
#Pressure at cloud ceiling
'pcdc' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 19 ;
	 scaledValueOfFirstFixedSurface = 50 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Visibility through precipitation
'visp' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	}
#Geometric height of 0 degrees C theta level above ground
'h0thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Geometric height of 1 degree C theta level above ground
'h1thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Geometric height of 1.5 degrees C theta level above ground
'h1p5thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	}
#Volumetric soil moisture (root zone)
'vswrz' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	}
#Thunderstorm hazard index
'hits' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Surface water flooding hazard index
'hiswf' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	}
#Polluted air hazard index
'hipa' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	}
#Storm surge hazard index
'hiss' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	}
#Cold wave hazard index
'hicw' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	}
#Frost (hoar frost) hazard index
'hihf' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 43 ;
	}
#Heatwave hazard index
'hihw' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	}
#Wind hazard index
'hiw' = {
	 discipline = 20 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	}
#Wildfires hazard index
'hiwf' = {
	 discipline = 20 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	}
#Burned area
'fba' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Burning area
'bia' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	}
#Burnable area
'baa' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	}
#Un-burnable area
'ubaa' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	}
#Fuel load
'fuell' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	}
#Combustion completeness
'combc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	}
#Fuel moisture content
'fuelmc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	}
#Live leaf fuel load
'llfl' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	}
#Live wood fuel load 
'lwfl' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 28 ;
	}
#Dead leaf fuel load
'dlfl' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	}
#Dead wood fuel load
'dwfl' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	}
#Live fuel moisture content
'lfmc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	}
#Fine dead leaf moisture content
'fdlmc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	}
#Dense dead leaf moisture content
'ddlmc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	}
#Fine dead wood moisture content
'fdwmc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	}
#Dense dead wood moisture content
'ddwmc' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	}
#Live fuel moisture content in low vegetation
'lfmclv' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	}
#Live fuel moisture content in high vegetation
'lfmchv' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	}
#Time-integrated total column vertically-integrated eastward geopotential flux
'tvige' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward geopotential flux
'tvign' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of water geopotential flux
'tviwgd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of geopotential flux
'tvigd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward enthalpy flux
'tviee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward enthalpy flux
'tvien' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward kinetic energy flux
'tvikee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward kinetic energy flux
'tviken' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward total energy flux
'tvitee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward total energy flux
'tviten' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of enthalpy flux
'tvied' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of kinetic energy flux
'tviked' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of total energy flux
'tvited' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of water enthalpy flux
'tviwed' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of mass flux
'tvimad' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward mass flux
'tvimae' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward mass flux
'tviman' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of water vapour flux
'tviwvd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of cloud liquid water flux
'tviclwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of cloud ice water flux
'tviciwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of rain flux
'tvird' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated divergence of snow flux
'tvisd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward water vapour flux
'tviwve' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward water vapour flux
'tviwvn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward cloud liquid water flux
'tviclwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward cloud liquid water flux
'tviclwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward cloud ice water flux
'tviciwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward cloud ice water flux
'tviciwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward rain flux
'tvire' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward rain flux
'tvirn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated eastward snow flux
'tvise' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total column vertically-integrated northward snow flux
'tvisn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean temperature tendency due to short-wave radiation
'avg_ttswr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to long-wave radiation
'avg_ttlwr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to short wave radiation, clear sky
'avg_ttswrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to long-wave radiation, clear sky
'avg_ttlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature tendency due to parametrisations
'avg_ttpm' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific humidity tendency due to parametrisations
'avg_qtpm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward wind tendency due to parametrisations
'avg_utpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward wind tendency due to parametrisations
'avg_vtpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean updraught mass flux
'avg_umf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean downdraught mass flux
'avg_dmf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean updraught detrainment rate
'avg_udr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean downdraught detrainment rate
'avg_ddr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total precipitation flux
'avg_tpf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean turbulent diffusion coefficient for heat
'avg_tdch' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time integral of rain flux
'tirf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of surface eastward momentum flux
'tisemf' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of surface northward momentum flux
'tisnmf' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of surface latent heat evaporation flux
'tislhef' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean surface runoff rate
'avg_surfror' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sub-surface runoff rate
'avg_ssurfror' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface photosynthetically active radiation flux, clear sky
'avg_parcsf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow evaporation rate water equivalent
'avg_esrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow melt rate
'avg_smr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean magnitude of turbulent surface stress
'avg_imagss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large-scale precipitation fraction
'avg_ilspf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward UV radiation flux
'avg_sduvrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface photosynthetically active radiation flux
'avg_sparf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large-scale precipitation rate
'avg_lsprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective precipitation rate
'avg_cpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total snowfall rate water equivalent
'avg_tsrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean boundary layer dissipation
'avg_ibld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface sensible heat flux
'avg_ishf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface latent heat flux
'avg_slhtf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward short-wave radiation flux
'avg_sdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward long-wave radiation flux
'avg_sdlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net short-wave radiation flux
'avg_snswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net long-wave radiation flux
'avg_snlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net short-wave radiation flux
'avg_tnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net long-wave radiation flux
'avg_tnlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward turbulent surface stress
'avg_iews' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward turbulent surface stress
'avg_inss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean moisture flux
'avg_ie' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sunshine duration fraction
'avg_sdf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Time-mean eastward gravity wave surface stress
'avg_iegwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward gravity wave surface stress
'avg_ingwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean gravity wave dissipation
'avg_igwd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean runoff rate water equivalent (surface plus subsurface)
'avg_rorwe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net short-wave radiation flux, clear sky
'avg_tnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net long-wave radiation flux, clear sky
'avg_tnlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net short-wave radiation flux, clear sky
'avg_snswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net long-wave radiation flux, clear sky
'avg_snlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time mean top downward short-wave radiation flux
'avg_tdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated moisture divergence flux
'avg_vimdf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total precipitation rate
'avg_tprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective snowfall rate water equivalent
'avg_csfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large scale snowfall rate water equivalent
'avg_lssfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct short-wave radiation flux
'avg_sdirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct short-wave radiation flux, clear sky
'avg_sdirswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Mean surface diffuse short-wave radiation flux
'msdfswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward short-wave radiation flux, clear sky
'avg_sdswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward long-wave radiation flux, clear sky
'avg_sdlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean potential evaporation rate
'avg_pevr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time integral of surface latent heat sublimation flux
'tislhsf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time integral of evapotranspiration flux
'tietrf' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean evapotranspiration rate
'avg_etr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time integral of potential evapotranspiration rate
'tipet' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-mean potential evapotranspiration rate
'avg_petr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture
'avg_vsw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow depth water equivalent
'avg_sd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean skin temperature
'avg_skt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow density
'avg_rsn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean low vegetation cover
'avg_cvl' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 53 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean high vegetation cover
'avg_cvh' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 54 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice area fraction
'avg_ci' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface temperature
'avg_sst' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean leaf area index, low vegetation
'avg_lai_lv' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean leaf area index, high vegetation
'avg_lai_hv' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column liquid water
'avg_tclw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column cloud ice water
'avg_tciw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre specific humidity
'avg_2sh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake mix-layer temperature
'avg_lmlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake mix-layer depth
'avg_lmld' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre relative humidity
'avg_2r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fraction of snow cover
'avg_fscov' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil temperature
'avg_sot' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind speed
'avg_ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean pressure
'avg_pres' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean charnock
'avg_chnk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean potential vorticity
'avg_pv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific rain water content
'avg_crwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific snow water content
'avg_cswc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eta-coordinate vertical velocity
'avg_etadot' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean visibility
'avg_vis' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind direction
'avg_wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column snow water
'avg_tcsw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean low cloud cover
'avg_lcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean medium cloud cover
'avg_mcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean high cloud cover
'avg_hcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column rain water
'avg_tcrw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil moisture top 20 cm
'avg_sm20' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil moisture top 100 cm
'avg_sm100' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil temperature top 20 cm
'avg_st20' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil temperature top 100 cm
'avg_st100' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean most-unstable CAPE
'avg_mucape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture top 20 cm
'avg_vsw20' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture top 100 cm
'avg_vsw100' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated graupel (snow pellets)
'avg_tcolg' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geopotential
'avg_z' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature
'avg_t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U component of wind
'avg_u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V component of wind
'avg_v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific humidity
'avg_q' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface pressure
'avg_sp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertical velocity
'avg_w' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column water
'avg_tcw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated water vapour
'avg_tcwv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vorticity (relative)
'avg_vo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface net radiation flux (SW and LW)
'avg_snrf' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean top net radiation flux (SW and LW)
'avg_tnrf' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean sea level pressure
'avg_msl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean divergence
'avg_d' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean relative humidity
'avg_r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean boundary layer height
'avg_blh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre U wind component
'avg_10u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre V wind component
'avg_10v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre dewpoint temperature
'avg_2d' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sunshine
'avg_suns' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ozone mass mixing ratio
'avg_o3' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean temperature of snow layer
'avg_tsn' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean forecast surface roughness
'avg_fsr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean forecast logarithm of surface roughness for heat
'avg_flsr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific cloud liquid water content
'avg_clwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean specific cloud ice water content
'avg_ciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fraction of cloud cover
'avg_cc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean K index
'avg_kx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total totals index
'avg_totalx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind direction
'avg_10wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean clear air turbulence (CAT)
'avg_cat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean forecast albedo
'avg_al' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean potential temperature
'avg_pt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean discharge from rivers or streams
'avg_dis' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil wetness index (total layer)
'avg_swit' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 151 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil wetness index (root zone)
'avg_swir' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean soil wetness index(layer)
'avg_swil' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 151 ;
	 typeOfSecondFixedSurface = 151 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain depth
'avg_flddep' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain flooded fraction
'avg_fldffr' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain flooded area
'avg_fldfar' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river fraction
'avg_rivfr' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river area
'avg_rivar' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fraction of river coverage plus river related flooding
'avg_rivcffr' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean area of river coverage plus river related flooding
'avg_rivcfar' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean UV visible albedo for direct radiation (climatological)
'avg_aluvp_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean UV visible albedo for diffuse radiation (climatological)
'avg_aluvd_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation (climatological)
'avg_alnip_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for diffuse radiation (climatological)
'avg_alnid_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean snow albedo
'avg_asn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean montgomery potential
'avg_mont' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective available potential energy
'avg_cape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean skin reservoir content
'avg_srcon' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column integrated ozone
'avg_tcioz' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated kinetic energy
'avg_vike' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated enthalpy
'avg_vithe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated potential + internal energy
'avg_vipie' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated potential+internal+latent energy
'avg_vipile' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated total energy
'avg_vitoe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated water enthalpy
'avg_viwe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean UV visible albedo for direct radiation, isotropic component (climatological)
'avg_aluvpi_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean UV visible albedo for direct radiation, volumetric component (climatological)
'avg_aluvpv_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean UV visible albedo for direct radiation, geometric component (climatological)
'avg_aluvpg_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation, isotropic component (climatological)
'avg_alnipi_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation, volumetric component (climatological)
'avg_alnipv_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean near IR albedo for direct radiation, geometric component (climatological)
'avg_alnipg_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 0 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-mean convective inhibition
'avg_cin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean friction velocity
'avg_zust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake bottom temperature
'avg_lblt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake total layer temperature
'avg_ltlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake shape factor
'avg_lshf' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake ice surface temperature
'avg_lict' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean lake ice total depth
'avg_licd' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean minimum vertical gradient of refractivity inside trapping layer
'avg_dndzn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mean vertical gradient of refractivity inside trapping layer
'avg_dndza' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean duct base height
'avg_dctb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean trapping layer base height
'avg_tplb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean trapping layer top height
'avg_tplt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of -10 degrees C atmospheric isothermal level above ground
'avg_degm10l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean cloud base height
'avg_cbh' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 0 degrees C atmospheric isothermal level above ground
'avg_deg0l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind gust
'avg_i10fg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean relative humidity with respect to water
'avg_rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 2 metre relative humidity with respect to water
'avg_2rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective available potential energy shear
'avg_capes' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean tropopause pressure
'avg_trpp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean height of convective cloud top
'avg_hcct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean height of zero-degree wet-bulb temperature
'avg_hwbt0' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean height of one-degree wet-bulb temperature
'avg_hwbt1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column supercooled liquid water
'avg_tcslw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre U-component of neutral wind
'avg_u10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre V-component of neutral wind
'avg_v10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean convective rain rate
'avg_crr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean large scale rain rate
'avg_lsrr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CAPE in the lowest 50 hPa
'avg_mlcape50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CIN in the lowest 50 hPa
'avg_mlcin50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CAPE in the lowest 100 hPa
'avg_mlcape100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean mixed-layer CIN in the lowest 100 hPa
'avg_mlcin100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean departure level of the most unstable parcel expressed as Pressure
'avg_mudlp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ceiling
'avg_ceil' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column integrated saturation specific humidity with respect to water
'avg_tcsqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct normal short-wave radiation flux
'avg_sdirnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean cosine of solar zenith angle
'avg_cossza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean freezing rain precipitation rate
'avg_fprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward geopotential flux
'avg_vige' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward geopotential flux
'avg_vign' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of water geopotential flux
'avg_viwgd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of geopotential flux
'avg_vigd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward enthalpy flux
'avg_viee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward enthalpy flux
'avg_vien' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward kinetic energy flux
'avg_vikee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward kinetic energy flux
'avg_viken' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward total energy flux
'avg_vitee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward total energy flux
'avg_viten' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of enthalpy flux
'avg_vied' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of kinetic energy flux
'avg_viked' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of total energy flux
'avg_vited' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of water enthalpy flux
'avg_viwed' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of mass flux
'avg_vimad' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward mass flux
'avg_vimae' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward mass flux
'avg_viman' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of water vapour flux
'avg_viwvd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of cloud liquid water flux
'avg_viclwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of cloud ice water flux
'avg_viciwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of rain flux
'avg_vird' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of snow flux
'avg_visd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward water vapour flux
'avg_viwve' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward water vapour flux
'avg_viwvn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward cloud liquid water flux
'avg_viclwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward cloud liquid water flux
'avg_viclwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward cloud ice water flux
'avg_viciwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward cloud ice water flux
'avg_viciwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward rain flux
'avg_vire' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward rain flux
'avg_virn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward snow flux
'avg_vise' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward snow flux
'avg_visn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated eastward ozone flux
'avg_vioze' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated northward ozone flux
'avg_viozn' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated divergence of ozone flux
'avg_viozd' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically-integrated net source of ozone
'avg_vions' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward turbulent surface stress due to orographic form drag
'avg_ietssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward turbulent surface stress due to orographic form drag
'avg_intssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward turbulent surface stress due to surface roughness
'avg_ietsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward turbulent surface stress due to surface roughness
'avg_intsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean visibility through precipitation
'avg_visp' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean land surface heat net flux
'avg_lshnf' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface direct normal short-wave radiation flux, clear sky
'avg_sdirnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre convective gust
'avg_10cogu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre turbulent gust
'avg_10tugu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 0 degrees C theta level above ground
'avg_h0thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 1 degree C theta level above ground
'avg_h1thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 1.5 degrees C theta level above ground
'avg_h1p5thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vegetation
'avg_veg' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean leaf area index
'avg_lai' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface roughness for heat
'avg_srhe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Reciprocal Obukhov length
'avg_rol' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean geometric height of 3 degrees C atmospheric isothermal level above ground
'avg_deg3l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river storage of water
'avg_rivsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain storage of water
'avg_fldsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river depth
'avg_rivdph' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river outflow of water
'avg_rivout' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodplain outflow of water
'avg_fldout' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean floodpath outflow of water
'avg_pthflw' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface diffuse short-wave radiation flux, clear sky
'avg_sdifswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean volumetric soil moisture (root zone)
'avg_vswrz' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean water surface elevation
'avg_wse' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean groundwater return flow rate
'avg_grfr' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean river and floodplain storage
'avg_rivfldsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth-averaged river velocity
'avg_darv' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-integrated land surface heat net flux
'acc_lshnf' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated surface direct normal short-wave radiation flux, clear sky
'acc_sdirnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-maximum temperature tendency due to short-wave radiation
'max_ttswr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to long-wave radiation
'max_ttlwr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to short wave radiation, clear sky
'max_ttswrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to long-wave radiation, clear sky
'max_ttlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature tendency due to parametrisations
'max_ttpm' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific humidity tendency due to parametrisations
'max_qtpm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward wind tendency due to parametrisations
'max_utpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward wind tendency due to parametrisations
'max_vtpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum updraught mass flux
'max_umf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum downdraught mass flux
'max_dmf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum updraught detrainment rate
'max_udr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum downdraught detrainment rate
'max_ddr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total precipitation flux
'max_tpf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum turbulent diffusion coefficient for heat
'max_tdch' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface photosynthetically active radiation flux, clear sky
'max_parcsf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow evaporation rate water equivalent
'max_esrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow melt rate
'max_smr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum magnitude of turbulent surface stress
'max_imagss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large-scale precipitation fraction
'max_ilspf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward UV radiation flux
'max_sduvrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface photosynthetically active radiation flux
'max_sparf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large-scale precipitation rate
'max_lsprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective precipitation rate
'max_cpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total snowfall rate water equivalent
'max_tsrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum boundary layer dissipation
'max_ibld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface sensible heat flux
'max_ishf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface latent heat flux
'max_slhtf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward short-wave radiation flux
'max_sdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward long-wave radiation flux
'max_sdlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net short-wave radiation flux
'max_snswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net long-wave radiation flux
'max_snlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net short-wave radiation flux
'max_tnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net long-wave radiation flux
'max_tnlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward turbulent surface stress
'max_iews' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward turbulent surface stress
'max_inss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum moisture flux
'max_ie' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward gravity wave surface stress
'max_iegwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward gravity wave surface stress
'max_ingwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum gravity wave dissipation
'max_igwd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum runoff rate water equivalent (surface plus subsurface)
'max_rorwe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net short-wave radiation flux, clear sky
'max_tnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum top net long-wave radiation flux, clear sky
'max_tnlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net short-wave radiation flux, clear sky
'max_snswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface net long-wave radiation flux, clear sky
'max_snlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time mean top downward short-wave radiation flux
'max_tdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated moisture divergence flux
'max_vimdf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective snowfall rate water equivalent
'max_csfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large scale snowfall rate water equivalent
'max_lssfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct short-wave radiation flux
'max_sdirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct short-wave radiation flux, clear sky
'max_sdirswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum carbon dioxide net ecosystem exchange flux
'max_fco2nee' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum carbon dioxide gross primary production flux
'max_fco2gpp' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum carbon dioxide ecosystem respiration flux
'max_fco2rec' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward short-wave radiation flux, clear sky
'max_sdswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface downward long-wave radiation flux, clear sky
'max_sdlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum potential evaporation rate
'max_pevr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum volumetric soil moisture
'max_vsw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow depth water equivalent
'max_sd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum skin temperature
'max_skt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum snow density
'max_rsn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea ice area fraction
'max_ci' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea surface temperature
'max_sst' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column cloud liquid water
'max_tclw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column cloud ice water
'max_tciw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre specific humidity
'max_2sh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake mix-layer temperature
'max_lmlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake mix-layer depth
'max_lmld' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre relative humidity
'max_2r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum soil temperature
'max_sot' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind speed
'max_ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum pressure
'max_pres' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum charnock
'max_chnk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum potential vorticity
'max_pv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific rain water content
'max_crwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific snow water content
'max_cswc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eta-coordinate vertical velocity
'max_etadot' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum visibility
'max_vis' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum wind direction
'max_wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column snow water
'max_tcsw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum low cloud cover
'max_lcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum medium cloud cover
'max_mcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum high cloud cover
'max_hcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column rain water
'max_tcrw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum most-unstable CAPE
'max_mucape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated graupel (snow pellets)
'max_tcolg' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geopotential
'max_z' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature
'max_t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum u component of wind
'max_u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum v component of wind
'max_v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific humidity
'max_q' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface pressure
'max_sp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vertical velocity
'max_w' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column water
'max_tcw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated water vapour
'max_tcwv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vorticity (relative)
'max_vo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean sea level pressure
'max_msl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum divergence
'max_d' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum relative humidity
'max_r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum boundary layer height
'max_blh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre U wind component
'max_10u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre V wind component
'max_10v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre dewpoint temperature
'max_2d' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sunshine
'max_suns' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ozone mass mixing ratio
'max_o3' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind speed
'max_10si' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum temperature of snow layer
'max_tsn' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum forecast surface roughness
'max_fsr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum forecast logarithm of surface roughness for heat
'max_flsr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific cloud liquid water content
'max_clwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum specific cloud ice water content
'max_ciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum fraction of cloud cover
'max_cc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum k index
'max_kx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total totals index
'max_totalx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre wind direction
'max_10wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum clear air turbulence (CAT)
'max_cat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum forecast albedo
'max_al' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum potential temperature
'max_pt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum UV visible albedo for direct radiation (climatological)
'max_aluvp_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum UV visible albedo for diffuse radiation (climatological)
'max_aluvd_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation (climatological)
'max_alnip_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for diffuse radiation (climatological)
'max_alnid_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum snow albedo
'max_asn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum montgomery potential
'max_mont' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total cloud cover
'max_tcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum skin reservoir content
'max_srcon' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column integrated ozone
'max_tcioz' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated kinetic energy
'max_vike' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated enthalpy
'max_vithe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated potential + internal energy
'max_vipie' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated potential+internal+latent energy
'max_vipile' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated total energy
'max_vitoe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated water enthalpy
'max_viwe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum UV visible albedo for direct radiation, isotropic component (climatological)
'max_aluvpi_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum UV visible albedo for direct radiation, volumetric component (climatological)
'max_aluvpv_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum UV visible albedo for direct radiation, geometric component (climatological)
'max_aluvpg_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation, isotropic component (climatological)
'max_alnipi_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation, volumetric component (climatological)
'max_alnipv_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum near IR albedo for direct radiation, geometric component (climatological)
'max_alnipg_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 2 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-maximum convective inhibition
'max_cin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum friction velocity
'max_zust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake bottom temperature
'max_lblt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake total layer temperature
'max_ltlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake shape factor
'max_lshf' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake ice surface temperature
'max_lict' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum lake ice total depth
'max_licd' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum minimum vertical gradient of refractivity inside trapping layer
'max_dndzn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mean vertical gradient of refractivity inside trapping layer
'max_dndza' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum duct base height
'max_dctb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum trapping layer base height
'max_tplb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum trapping layer top height
'max_tplt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of -10 degrees C atmospheric isothermal level above ground
'max_degm10l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum cloud base height
'max_cbh' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 0 degrees C atmospheric isothermal level above ground
'max_deg0l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum relative humidity with respect to water
'max_rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 2 metre relative humidity with respect to water
'max_2rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum tropopause pressure
'max_trpp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum height of convective cloud top
'max_hcct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum height of zero-degree wet-bulb temperature
'max_hwbt0' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum height of one-degree wet-bulb temperature
'max_hwbt1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total lightning flash density
'max_litoti' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column supercooled liquid water
'max_tcslw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre U-component of neutral wind
'max_u10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre V-component of neutral wind
'max_v10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum convective rain rate
'max_crr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum large scale rain rate
'max_lsrr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CAPE in the lowest 50 hPa
'max_mlcape50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CIN in the lowest 50 hPa
'max_mlcin50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CAPE in the lowest 100 hPa
'max_mlcape100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum mixed-layer CIN in the lowest 100 hPa
'max_mlcin100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum departure level of the most unstable parcel expressed as Pressure
'max_mudlp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum ceiling
'max_ceil' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column integrated saturation specific humidity with respect to water
'max_tcsqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct normal short-wave radiation flux
'max_sdirnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum cosine of solar zenith angle
'max_cossza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum freezing rain precipitation rate
'max_fprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward geopotential flux
'max_vige' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward geopotential flux
'max_vign' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of water geopotential flux
'max_viwgd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of geopotential flux
'max_vigd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward enthalpy flux
'max_viee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward enthalpy flux
'max_vien' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward kinetic energy flux
'max_vikee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward kinetic energy flux
'max_viken' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward total energy flux
'max_vitee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward total energy flux
'max_viten' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of enthalpy flux
'max_vied' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of kinetic energy flux
'max_viked' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of total energy flux
'max_vited' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of water enthalpy flux
'max_viwed' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of mass flux
'max_vimad' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward mass flux
'max_vimae' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward mass flux
'max_viman' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of water vapour flux
'max_viwvd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of cloud liquid water flux
'max_viclwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of cloud ice water flux
'max_viciwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of rain flux
'max_vird' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of snow flux
'max_visd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward water vapour flux
'max_viwve' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward water vapour flux
'max_viwvn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward cloud liquid water flux
'max_viclwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward cloud liquid water flux
'max_viclwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward cloud ice water flux
'max_viciwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward cloud ice water flux
'max_viciwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward rain flux
'max_vire' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward rain flux
'max_virn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward snow flux
'max_vise' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward snow flux
'max_visn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated eastward ozone flux
'max_vioze' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated northward ozone flux
'max_viozn' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated divergence of ozone flux
'max_viozd' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum total column vertically-integrated net source of ozone
'max_vions' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward turbulent surface stress due to orographic form drag
'max_ietssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward turbulent surface stress due to orographic form drag
'max_intssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum eastward turbulent surface stress due to surface roughness
'max_ietsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum northward turbulent surface stress due to surface roughness
'max_intsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum visibility through precipitation
'max_visp' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum land surface heat net flux
'max_lshnf' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface direct normal short-wave radiation flux, clear sky
'max_sdirnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre convective gust
'max_10cogu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum 10 metre turbulent gust
'max_10tugu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 0 degrees C theta level above ground
'max_h0thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 1 degree C theta level above ground
'max_h1thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 1.5 degrees C theta level above ground
'max_h1p5thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum vegetation
'max_veg' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum leaf area index
'max_lai' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface roughness for heat
'max_srhe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum Reciprocal Obukhov length
'max_rol' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum geometric height of 3 degrees C atmospheric isothermal level above ground
'max_deg3l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum surface diffuse short-wave radiation flux, clear sky
'max_sdifswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum volumetric soil moisture (root zone)
'max_vswrz' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum temperature tendency due to short-wave radiation
'min_ttswr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to long-wave radiation
'min_ttlwr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to short wave radiation, clear sky
'min_ttswrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to long-wave radiation, clear sky
'min_ttlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature tendency due to parametrisations
'min_ttpm' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific humidity tendency due to parametrisations
'min_qtpm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward wind tendency due to parametrisations
'min_utpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward wind tendency due to parametrisations
'min_vtpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum updraught mass flux
'min_umf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum downdraught mass flux
'min_dmf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum updraught detrainment rate
'min_udr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum downdraught detrainment rate
'min_ddr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total precipitation flux
'min_tpf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum turbulent diffusion coefficient for heat
'min_tdch' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface photosynthetically active radiation flux, clear sky
'min_parcsf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow evaporation rate
'min_esrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snowmelt rate
'min_smr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum magnitude of turbulent surface stress
'min_imagss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large-scale precipitation fraction
'min_ilspf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward UV radiation flux
'min_sduvrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface photosynthetically active radiation flux
'min_sparf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large-scale precipitation rate
'min_lsprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective precipitation rate
'min_cpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snowfall rate
'min_tsrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum boundary layer dissipation
'min_ibld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface sensible heat flux
'min_ishf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface latent heat flux
'min_slhtf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward short-wave radiation flux
'min_sdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward long-wave radiation flux
'min_sdlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net short-wave radiation flux
'min_snswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net long-wave radiation flux
'min_snlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net short-wave radiation flux
'min_tnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net long-wave radiation flux
'min_tnlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward turbulent surface stress
'min_iews' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward turbulent surface stress
'min_inss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum evaporation rate
'min_ie' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward gravity wave surface stress
'min_iegwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward gravity wave surface stress
'min_ingwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum gravity wave dissipation
'min_igwd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum runoff rate
'min_rorwe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net short-wave radiation flux, clear sky
'min_tnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top net long-wave radiation flux, clear sky
'min_tnlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net short-wave radiation flux, clear sky
'min_snswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface net long-wave radiation flux, clear sky
'min_snlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum top downward short-wave radiation flux
'min_tdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated moisture divergence
'min_vimdf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective snowfall rate water equivalent
'min_csfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large scale snowfall rate water equivalent
'min_lssfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface direct short-wave radiation flux
'min_sdirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface direct short-wave radiation flux, clear sky
'min_sdirswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum carbon dioxide net ecosystem exchange flux
'min_fco2nee' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum carbon dioxide gross primary production flux
'min_fco2gpp' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum carbon dioxide ecosystem respiration flux
'min_fco2rec' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward short-wave radiation flux, clear sky
'min_sdswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface downward long-wave radiation flux, clear sky
'min_sdlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum potential evaporation rate
'min_pevr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum volumetric soil moisture
'min_vsw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow depth water equivalent
'min_sd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum skin temperature
'min_skt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum snow density
'min_rsn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea ice area fraction
'min_ci' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea surface temperature
'min_sst' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column cloud liquid water
'min_tclw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column cloud ice water
'min_tciw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre specific humidity
'min_2sh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake mix-layer temperature
'min_lmlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake mix-layer depth
'min_lmld' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre relative humidity
'min_2r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum soil temperature
'min_sot' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind speed
'min_ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum pressure
'min_pres' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum charnock
'min_chnk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum potential vorticity
'min_pv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific rain water content
'min_crwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific snow water content
'min_cswc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eta-coordinate vertical velocity
'min_etadot' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum visibility
'min_vis' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum wind direction
'min_wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column snow water
'min_tcsw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum low cloud cover
'min_lcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum medium cloud cover
'min_mcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum high cloud cover
'min_hcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column rain water
'min_tcrw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum most-unstable CAPE
'min_mucape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated graupel (snow pellets)
'min_tcolg' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geopotential
'min_z' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature
'min_t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum u component of wind
'min_u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum v component of wind
'min_v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific humidity
'min_q' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface pressure
'min_sp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertical velocity
'min_w' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column water
'min_tcw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated water vapour
'min_tcwv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vorticity (relative)
'min_vo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean sea level pressure
'min_msl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum divergence
'min_d' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum relative humidity
'min_r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum boundary layer height
'min_blh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre U wind component
'min_10u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre V wind component
'min_10v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre dewpoint temperature
'min_2d' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sunshine
'min_suns' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ozone mass mixing ratio
'min_o3' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind speed
'min_10si' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum temperature of snow layer
'min_tsn' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum forecast surface roughness
'min_fsr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum forecast logarithm of surface roughness for heat
'min_flsr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific cloud liquid water content
'min_clwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum specific cloud ice water content
'min_ciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum fraction of cloud cover
'min_cc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum k index
'min_kx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total totals index
'min_totalx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind direction
'min_10wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum clear air turbulence (CAT)
'min_cat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum forecast albedo
'min_al' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum potential temperature
'min_pt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum UV visible albedo for direct radiation (climatological)
'min_aluvp_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum UV visible albedo for diffuse radiation (climatological)
'min_aluvd_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation (climatological)
'min_alnip_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for diffuse radiation (climatological)
'min_alnid_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum snow albedo
'min_asn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum montgomery potential
'min_mont' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective available potential energy
'min_cape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total cloud cover
'min_tcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum skin reservoir content
'min_srcon' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column integrated ozone
'min_tcioz' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated kinetic energy
'min_vike' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated enthalpy
'min_vithe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated potential + internal energy
'min_vipie' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertical integral of potential+internal+latent energy
'min_vipile' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated total energy
'min_vitoe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated water enthalpy
'min_viwe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum UV visible albedo for direct radiation, isotropic component (climatological)
'min_aluvpi_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum UV visible albedo for direct radiation, volumetric component (climatological)
'min_aluvpv_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum UV visible albedo for direct radiation, geometric component (climatological)
'min_aluvpg_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation, isotropic component (climatological)
'min_alnipi_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation, volumetric component (climatological)
'min_alnipv_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum near IR albedo for direct radiation, geometric component (climatological)
'min_alnipg_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 3 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-minimum convective inhibition
'min_cin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum friction velocity
'min_zust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake bottom temperature
'min_lblt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake total layer temperature
'min_ltlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake shape factor
'min_lshf' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake ice surface temperature
'min_lict' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum lake ice total depth
'min_licd' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum minimum vertical gradient of refractivity inside trapping layer
'min_dndzn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mean vertical gradient of refractivity inside trapping layer
'min_dndza' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum duct base height
'min_dctb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum trapping layer base height
'min_tplb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum trapping layer top height
'min_tplt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of -10 degrees C atmospheric isothermal level above ground
'min_degm10l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum cloud base height
'min_cbh' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 0 degrees C atmospheric isothermal level above ground
'min_deg0l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre wind gust
'min_i10fg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum relative humidity with respect to water
'min_rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 2 metre relative humidity with respect to water
'min_2rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective available potential energy shear
'min_capes' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum tropopause pressure
'min_trpp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum height of convective cloud top
'min_hcct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum height of zero-degree wet-bulb temperature
'min_hwbt0' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum height of one-degree wet-bulb temperature
'min_hwbt1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total lightning flash density
'min_litoti' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column supercooled liquid water
'min_tcslw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre U-component of neutral wind
'min_u10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre V-component of neutral wind
'min_v10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum convective rain rate
'min_crr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum large scale rain rate
'min_lsrr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CAPE in the lowest 50 hPa
'min_mlcape50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CIN in the lowest 50 hPa
'min_mlcin50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CAPE in the lowest 100 hPa
'min_mlcape100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum mixed-layer CIN in the lowest 100 hPa
'min_mlcin100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum departure level of the most unstable parcel expressed as Pressure
'min_mudlp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum ceiling
'min_ceil' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column integrated saturation specific humidity with respect to water
'min_tcsqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum direct solar radiation flux
'min_sdirnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum cosine of solar zenith angle
'min_cossza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum freezing rain rate water equivalent
'min_fprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward geopotential flux
'min_vige' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward geopotential flux
'min_vign' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of water geopotential flux
'min_viwgd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of geopotential flux
'min_vigd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward enthalpy flux
'min_viee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward enthalpy flux
'min_vien' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward kinetic energy flux
'min_vikee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward kinetic energy flux
'min_viken' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated eastward total energy flux
'min_vitee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated northward total energy flux
'min_viten' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of enthalpy flux
'min_vied' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of kinetic energy flux
'min_viked' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of total energy flux
'min_vited' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum total column vertically-integrated divergence of water enthalpy flux
'min_viwed' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of mass flux
'min_vimad' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward mass flux
'min_vimae' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward mass flux
'min_viman' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of water vapour flux
'min_viwvd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of cloud liquid water flux
'min_viclwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of cloud ice water flux
'min_viciwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of rain flux
'min_vird' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of snow flux
'min_visd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward water vapour flux
'min_viwve' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward water vapour flux
'min_viwvn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward cloud liquid water flux
'min_viclwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward cloud liquid water flux
'min_viclwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward cloud ice water flux
'min_viciwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward cloud ice water flux
'min_viciwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward rain flux
'min_vire' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward rain flux
'min_virn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward snow flux
'min_vise' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward snow flux
'min_visn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated eastward ozone flux
'min_vioze' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated northward ozone flux
'min_viozn' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated divergence of ozone flux
'min_viozd' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vertically integrated net source of ozone
'min_vions' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward turbulent surface stress due to orographic form drag
'min_ietssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward turbulent surface stress due to orographic form drag
'min_intssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum eastward turbulent surface stress due to surface roughness
'min_ietsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum northward turbulent surface stress due to surface roughness
'min_intsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum visibility through precipitation
'min_visp' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 51 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum land surface heat net flux
'min_lshnf' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface direct normal short-wave radiation flux, clear sky
'min_sdirnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre convective gust
'min_10cogu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum 10 metre turbulent gust
'min_10tugu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 0 degrees C theta level above ground
'min_h0thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 1 degree C theta level above ground
'min_h1thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 1.5 degrees C theta level above ground
'min_h1p5thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum vegetation
'min_veg' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum leaf area index
'min_lai' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface roughness for heat
'min_srhe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum Reciprocal Obukhov length
'min_rol' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum geometric height of 3 degrees C atmospheric isothermal level above ground
'min_deg3l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum surface diffuse short-wave radiation flux, clear sky
'min_sdifswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum volumetric soil moisture (root zone)
'min_vswrz' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation temperature tendency due to short-wave radiation
'std_ttswr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to long-wave radiation
'std_ttlwr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to short wave radiation, clear sky
'std_ttswrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to long-wave radiation, clear sky
'std_ttlwrcs' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature tendency due to parametrisations
'std_ttpm' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific humidity tendency due to parametrisations
'std_qtpm' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward wind tendency due to parametrisations
'std_utpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward wind tendency due to parametrisations
'std_vtpm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation updraught mass flux
'std_umf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation downdraught mass flux
'std_dmf' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation updraught detrainment rate
'std_udr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation downdraught detrainment rate
'std_ddr' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total precipitation flux
'std_tpf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation turbulent diffusion coefficient for heat
'std_tdch' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface photosynthetically active radiation flux, clear sky
'std_parcsf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow evaporation rate
'std_esrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 148 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snowmelt rate
'std_smr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation magnitude of turbulent surface stress
'std_imagss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large-scale precipitation fraction
'std_ilspf' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward UV radiation flux
'std_sduvrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface photosynthetically active radiation flux
'std_sparf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large-scale precipitation rate
'std_lsprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective precipitation rate
'std_cpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snowfall rate
'std_tsrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation boundary layer dissipation
'std_ibld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface sensible heat flux
'std_ishf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface latent heat flux
'std_slhtf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward short-wave radiation flux
'std_sdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward long-wave radiation flux
'std_sdlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net short-wave radiation flux
'std_snswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net long-wave radiation flux
'std_snlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net short-wave radiation flux
'std_tnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net long-wave radiation flux
'std_tnlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward turbulent surface stress
'std_iews' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward turbulent surface stress
'std_inss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation evaporation rate
'std_ie' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward gravity wave surface stress
'std_iegwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward gravity wave surface stress
'std_ingwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation gravity wave dissipation
'std_igwd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation runoff rate
'std_rorwe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net short-wave radiation flux, clear sky
'std_tnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top net long-wave radiation flux, clear sky
'std_tnlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net short-wave radiation flux, clear sky
'std_snswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface net long-wave radiation flux, clear sky
'std_snlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation top downward short-wave radiation flux
'std_tdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated moisture divergence
'std_vimdf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total precipitation rate
'std_tprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective snowfall rate water equivalent
'std_csfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 55 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large scale snowfall rate water equivalent
'std_lssfr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface direct short-wave radiation flux
'std_sdirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface direct short-wave radiation flux, clear sky
'std_sdirswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation carbon dioxide net ecosystem exchange flux
'std_fco2nee' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 59 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation carbon dioxide gross primary production flux
'std_fco2gpp' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 60 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation carbon dioxide ecosystem respiration flux
'std_fco2rec' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 61 ;
	 constituentType = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward short-wave radiation flux, clear sky
'std_sdswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface downward long-wave radiation flux, clear sky
'std_sdlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation potential evaporation rate
'std_pevr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 143 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation volumetric soil moisture
'std_vsw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow depth water equivalent
'std_sd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation skin temperature
'std_skt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation snow density
'std_rsn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 61 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea ice area fraction
'std_ci' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea surface temperature
'std_sst' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column cloud liquid water
'std_tclw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column cloud ice water
'std_tciw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 70 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre specific humidity
'std_2sh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake mix-layer temperature
'std_lmlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake mix-layer depth
'std_lmld' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation soil temperature
'std_sot' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind speed
'std_ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation pressure
'std_pres' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation charnock
'std_chnk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 76 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation potential vorticity
'std_pv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific rain water content
'std_crwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 85 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific snow water content
'std_cswc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 86 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eta-coordinate vertical velocity
'std_etadot' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation visibility
'std_vis' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation wind direction
'std_wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column snow water
'std_tcsw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation low cloud cover
'std_lcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation medium cloud cover
'std_mcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation high cloud cover
'std_hcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column rain water
'std_tcrw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation most-unstable CAPE
'std_mucape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated graupel (snow pellets)
'std_tcolg' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geopotential
'std_z' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature
'std_t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation u component of wind
'std_u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation v component of wind
'std_v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific humidity
'std_q' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface pressure
'std_sp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertical velocity
'std_w' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column water
'std_tcw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated water vapour
'std_tcwv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vorticity (relative)
'std_vo' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean sea level pressure
'std_msl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 101 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation divergence
'std_d' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation relative humidity
'std_r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation boundary layer height
'std_blh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre U wind component
'std_10u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre V wind component
'std_10v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre temperature
'std_2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre dewpoint temperature
'std_2d' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sunshine
'std_suns' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ozone mass mixing ratio
'std_o3' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind speed
'std_10si' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation temperature of snow layer
'std_tsn' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation forecast surface roughness
'std_fsr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation forecast logarithm of surface roughness for heat
'std_flsr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific cloud liquid water content
'std_clwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 83 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation specific cloud ice water content
'std_ciwc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 84 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation fraction of cloud cover
'std_cc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 32 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation k index
'std_kx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total totals index
'std_totalx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind direction
'std_10wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation clear air turbulence (CAT)
'std_cat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation forecast albedo
'std_al' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation potential temperature
'std_pt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation (climatological)
'std_aluvp_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation UV visible albedo for diffuse radiation (climatological)
'std_aluvd_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 55 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation (climatological)
'std_alnip_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for diffuse radiation (climatological)
'std_alnid_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 9 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation snow albedo
'std_asn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 19 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation montgomery potential
'std_mont' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective available potential energy
'std_cape' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total cloud cover
'std_tcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation skin reservoir content
'std_srcon' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column integrated ozone
'std_tcioz' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated kinetic energy
'std_vike' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated enthalpy
'std_vithe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated potential + internal energy
'std_vipie' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertical integral of potential+internal+latent energy
'std_vipile' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated total energy
'std_vitoe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated water enthalpy
'std_viwe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation, isotropic component (climatological)
'std_aluvpi_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation, volumetric component (climatological)
'std_aluvpv_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 59 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation UV visible albedo for direct radiation, geometric component (climatological)
'std_aluvpg_p' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation, isotropic component (climatological)
'std_alnipi_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 12 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation, volumetric component (climatological)
'std_alnipv_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation near IR albedo for direct radiation, geometric component (climatological)
'std_alnipg_p' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 11 ;
	 typeOfStatisticalProcessing = 6 ;
	 typeOfGeneratingProcess = 9 ;
	}
#Time-standard-deviation convective inhibition
'std_cin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation friction velocity
'std_zust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake bottom temperature
'std_lblt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake total layer temperature
'std_ltlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake shape factor
'std_lshf' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake ice surface temperature
'std_lict' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation lake ice total depth
'std_licd' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation minimum vertical gradient of refractivity inside trapping layer
'std_dndzn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mean vertical gradient of refractivity inside trapping layer
'std_dndza' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation duct base height
'std_dctb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation trapping layer base height
'std_tplb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation trapping layer top height
'std_tplt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of -10 degrees C atmospheric isothermal level above ground
'std_degm10l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 26315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation cloud base height
'std_cbh' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 0 degrees C atmospheric isothermal level above ground
'std_deg0l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre wind gust
'std_i10fg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation relative humidity with respect to water
'std_rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 2 metre relative humidity with respect to water
'std_2rhw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 93 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective available potential energy shear
'std_capes' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation tropopause pressure
'std_trpp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 7 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation height of convective cloud top
'std_hcct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation height of zero-degree wet-bulb temperature
'std_hwbt0' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 44 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation height of one-degree wet-bulb temperature
'std_hwbt1' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 45 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total lightning flash density
'std_litoti' = {
	 discipline = 0 ;
	 parameterCategory = 17 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column supercooled liquid water
'std_tcslw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 167 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre U-component of neutral wind
'std_u10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre V-component of neutral wind
'std_v10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation convective rain rate
'std_crr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 76 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation large scale rain rate
'std_lsrr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 77 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CAPE in the lowest 50 hPa
'std_mlcape50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CIN in the lowest 50 hPa
'std_mlcin50' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 5000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CAPE in the lowest 100 hPa
'std_mlcape100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation mixed-layer CIN in the lowest 100 hPa
'std_mlcin100' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 18 ;
	 scaledValueOfFirstFixedSurface = 10000 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation departure level of the most unstable parcel expressed as Pressure
'std_mudlp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 17 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation ceiling
'std_ceil' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column integrated saturation specific humidity with respect to water
'std_tcsqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation direct solar radiation flux
'std_sdirnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation cosine of solar zenith angle
'std_cossza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation freezing rain rate water equivalent
'std_fprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward geopotential flux
'std_vige' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward geopotential flux
'std_vign' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of water geopotential flux
'std_viwgd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 42 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of geopotential flux
'std_vigd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward enthalpy flux
'std_viee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward enthalpy flux
'std_vien' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward kinetic energy flux
'std_vikee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward kinetic energy flux
'std_viken' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated eastward total energy flux
'std_vitee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated northward total energy flux
'std_viten' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of enthalpy flux
'std_vied' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of kinetic energy flux
'std_viked' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of total energy flux
'std_vited' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation total column vertically-integrated divergence of water enthalpy flux
'std_viwed' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of mass flux
'std_vimad' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward mass flux
'std_vimae' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward mass flux
'std_viman' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of water vapour flux
'std_viwvd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 160 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of cloud liquid water flux
'std_viclwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 161 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of cloud ice water flux
'std_viciwd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 162 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of rain flux
'std_vird' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 163 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of snow flux
'std_visd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 164 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward water vapour flux
'std_viwve' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward water vapour flux
'std_viwvn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward cloud liquid water flux
'std_viclwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 152 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward cloud liquid water flux
'std_viclwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 153 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward cloud ice water flux
'std_viciwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 154 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward cloud ice water flux
'std_viciwn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 155 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward rain flux
'std_vire' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 156 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward rain flux
'std_virn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 157 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward snow flux
'std_vise' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 158 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward snow flux
'std_visn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 159 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated eastward ozone flux
'std_vioze' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated northward ozone flux
'std_viozn' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated divergence of ozone flux
'std_viozd' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vertically integrated net source of ozone
'std_vions' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 81 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward turbulent surface stress due to orographic form drag
'std_ietssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward turbulent surface stress due to orographic form drag
'std_intssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation eastward turbulent surface stress due to surface roughness
'std_ietsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation northward turbulent surface stress due to surface roughness
'std_intsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation land surface heat net flux
'std_lshnf' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface direct normal short-wave radiation flux, clear sky
'std_sdirnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre convective gust
'std_10cogu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation 10 metre turbulent gust
'std_10tugu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 0 degrees C theta level above ground
'std_h0thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 1 degree C theta level above ground
'std_h1thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27415 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 1.5 degrees C theta level above ground
'std_h1p5thg' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 107 ;
	 scaledValueOfFirstFixedSurface = 27465 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation vegetation
'std_veg' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation leaf area index
'std_lai' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface roughness for heat
'std_srhe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation Reciprocal Obukhov length
'std_rol' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation geometric height of 3 degrees C atmospheric isothermal level above ground
'std_deg3l' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 27615 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation surface diffuse short-wave radiation flux, clear sky
'std_sdifswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation volumetric soil moisture (root zone)
'std_vswrz' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 167 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Cross sectional area of flow in channel
'chcross' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	}
#Side flow into river channel
'chside' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Discharge from rivers or streams
'dis' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#River storage of water
'rivsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Floodplain storage of water
'fldsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Days since last observation
'dslr' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 3 ;
	}
#Frost index
'frost' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 24 ;
	}
#Depth of water on soil surface
'woss' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Upstream accumulated precipitation
'tpups' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Upstream accumulated snow melt
'smups' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Snow depth at elevation bands
'sd_elev' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 25 ;
	}
#Groundwater upper storage
'gwus' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Groundwater lower storage
'gwls' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Lake depth
'lakdph' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	}
#River depth
'rivdph' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	}
#River outflow of water
'rivout' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	}
#Floodplain outflow of water
'fldout' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	}
#Floodpath outflow of water
'pthflw' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	}
#Floodplain depth
'flddep' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	}
#Floodplain flooded fraction
'fldffr' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	}
#Floodplain flooded area
'fldfar' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	}
#River fraction
'rivfr' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	}
#River area
'rivar' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	}
#Fraction of river coverage plus river related flooding
'rivcffr' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	}
#Area of river coverage plus river related flooding
'rivcfar' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	}
#Water surface elevation
'wse' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Groundwater return flow rate
'grfr' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#River and floodplain storage
'rivfldsto' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Depth-averaged river velocity
'darv' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Latitude
'lat' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	}
#Longitude
'lon' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	}
#Latitude on T grid
'tlat' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 1 ;
	}
#Longitude on T grid
'tlon' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 1 ;
	}
#Latitude on U grid
'ulat' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 2 ;
	}
#Longitude on U grid
'ulon' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 2 ;
	}
#Latitude on V grid
'vlat' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 3 ;
	}
#Longitude on V grid
'vlon' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 3 ;
	}
#Latitude on W grid
'wlat' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 4 ;
	}
#Longitude on W grid
'wlon' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 4 ;
	}
#Latitude on F grid
'flat' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 1 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 5 ;
	}
#Longitude on F grid
'flon' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 2 ;
	 gridDefinitionTemplateNumber = 101 ;
	 numberOfGridInReference = 5 ;
	}
#Total column vertically-integrated graupel (snow pellets)
'tcolg' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 74 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Surface latent heat net flux
'slhtf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Heat index
'heatx' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Wind chill factor
'wcf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Snow phase change heat flux
'snohf' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Vapor pressure
'vapp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Large scale precipitation (non-convective)
'ncpcp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Snowfall rate water equivalent
'srweq' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 12 ;
	}
#Convective snow
'snoc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 14 ;
	}
#Large scale snow
'snol' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 15 ;
	}
#Snow age
'snoag' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 17 ;
	}
#Absolute humidity
'absh' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 18 ;
	}
#Precipitation type
'ptype' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	}
#Integrated liquid water
'iliqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 20 ;
	}
#Condensate
'tcond' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 21 ;
	}
#Cloud mixing ratio
'clwmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 22 ;
	}
#Ice water mixing ratio
'icmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 23 ;
	}
#Rain mixing ratio
'rwmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 24 ;
	}
#Snow mixing ratio
'snmr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 25 ;
	}
#Horizontal moisture convergence
'mconv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 26 ;
	}
#Total snowfall
'asnow' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Precipitable water category
'pwcat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 30 ;
	}
#Hail
'hail' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 31 ;
	}
#Graupel (snow pellets)
'grle' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 32 ;
	}
#Categorical rain
'crain' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 33 ;
	}
#Categorical freezing rain
'cfrzr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 34 ;
	}
#Categorical ice pellets
'cicep' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 35 ;
	}
#Categorical snow
'csnow' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 36 ;
	}
#Convective precipitation rate
'cpr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Horizontal moisture divergence
'mdiv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 38 ;
	}
#Percent frozen precipitation
'cpofp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 39 ;
	}
#Potential evaporation
'pevap' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 40 ;
	}
#Snow cover
'snowc' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 42 ;
	}
#Rain fraction of total cloud water
'frain' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 43 ;
	}
#Rime factor
'rime' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 44 ;
	}
#Total column integrated rain
'tcolr' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 45 ;
	}
#Total column integrated snow
'tcols' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 46 ;
	}
#Large scale water precipitation (non-convective)
'lswp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 47 ;
	}
#Convective water precipitation
'cwp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 48 ;
	}
#Total water precipitation
'twatp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 49 ;
	}
#Total snow precipitation
'tsnowp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Total column water (Vertically integrated total water (vapour + cloud water/ice))
'tcwat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 51 ;
	}
#Total precipitation rate
'tprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Total snowfall rate water equivalent
'tsrwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Large scale precipitation rate
'lsprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Total snowfall rate
'tsrate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 57 ;
	}
#Convective snowfall rate
'csrate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 58 ;
	}
#Large scale snowfall rate
'lssrate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 59 ;
	}
#Water equivalent of accumulated snow depth (deprecated)
'sdwe' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 13 ;
	}
#Rain precipitation rate
'rprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 65 ;
	}
#Snow precipitation rate
'sprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	}
#Freezing rain precipitation rate
'fprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Ice pellets precipitation rate
'iprate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 68 ;
	}
#Wind speed (gust)
'gust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	}
#u-component of wind (gust)
'ugust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	}
#v-component of wind (gust)
'vgust' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	}
#Vertical speed shear
'vwsh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	}
#Horizontal momentum flux
'mflx' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 26 ;
	}
#U-component storm motion
'ustm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 27 ;
	}
#V-component storm motion
'vstm' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 28 ;
	}
#Drag coefficient
'cd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	}
#Frictional velocity
'fricv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	}
#Pressure reduced to MSL
'prmsl' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Altimeter setting
'alts' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	}
#Thickness
'thick' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	}
#Pressure altitude
'presalt' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	}
#Density altitude
'denalt' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#5-wave geopotential height
'5wavh' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Instantaneous eastward gravity wave surface stress
'iegwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward gravity wave surface stress
'ingwss' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#5-wave geopotential height anomaly
'5wava' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	}
#Net short-wave radiation flux (top of atmosphere)
'nswrt' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 1 ;
	}
#Surface downward short-wave radiation flux
'sdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface upward short-wave radiation flux
'suswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net short-wave radiation flux
'snswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface photosynthetically active radiation flux
'sparf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net short-wave radiation flux, clear sky
'snswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface downward UV radiation flux
'sduvrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#UV index (under clear sky)
'uviucs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	}
#UV index
'uvi' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 51 ;
	}
#Surface downward long-wave radiation flux
'sdlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface upward long-wave radiation flux
'sulwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net long-wave radiation flux
'snlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface net long-wave radiation flux, clear sky
'snlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Cloud Ice
'cice' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 0 ;
	}
#Cloud water
'cwat' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 6 ;
	}
#Cloud amount
'cdca' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 7 ;
	}
#Cloud type
'cdct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 8 ;
	}
#Thunderstorm maximum tops
'tmaxt' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 9 ;
	}
#Thunderstorm coverage
'thunc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 10 ;
	}
#Cloud top
'cdct' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 12 ;
	}
#Ceiling
'ceil' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 13 ;
	}
#Non-convective cloud cover
'cdlyr' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 14 ;
	}
#Cloud work function
'cwork' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 15 ;
	}
#Convective cloud efficiency
'cuefi' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 16 ;
	}
#Total condensate
'tcond' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 17 ;
	}
#Total column-integrated cloud water
'tcolw' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 18 ;
	}
#Total column-integrated cloud ice
'tcoli' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 19 ;
	}
#Total column-integrated condensate
'tcolc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 20 ;
	}
#Ice fraction of total condensate
'fice' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 21 ;
	}
#Cloud ice mixing ratio
'cdcimr' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 23 ;
	}
#Sunshine
'suns' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Horizontal extent of cumulonimbus (CB)
'~' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 25 ;
	}
#K index
'kx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 2 ;
	}
#KO index
'kox' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 3 ;
	}
#Total totals index
'totalx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 4 ;
	}
#Sweat index
'sx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 5 ;
	}
#Storm relative helicity
'hlcy' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 8 ;
	}
#Energy helicity index
'ehlx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 9 ;
	}
#Surface lifted index
'lftx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 10 ;
	}
#Best (4-layer) lifted index
'4lftx' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 11 ;
	}
#Aerosol type
'aerot' = {
	 discipline = 0 ;
	 parameterCategory = 13 ;
	 parameterNumber = 0 ;
	}
#Total ozone
'tozne' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 0 ;
	}
#Total column integrated ozone
'tcioz' = {
	 discipline = 0 ;
	 parameterCategory = 14 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Base spectrum width
'bswid' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 0 ;
	}
#Base reflectivity
'bref' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 1 ;
	}
#Base radial velocity
'brvel' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 2 ;
	}
#Vertically-integrated liquid
'veril' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 3 ;
	}
#Layer-maximum base reflectivity
'lmaxbr' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 4 ;
	}
#Precipitation from radar
'rdp' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 17 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Air concentration of Caesium 137
'acces' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 0 ;
	}
#Air concentration of Iodine 131
'aciod' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 1 ;
	}
#Air concentration of radioactive pollutant
'acradp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 2 ;
	}
#Ground deposition of Caesium 137
'gdces' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 3 ;
	}
#Ground deposition of Iodine 131
'gdiod' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 4 ;
	}
#Ground deposition of radioactive pollutant
'gdradp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 5 ;
	}
#Time-integrated air concentration of caesium pollutant
'tiaccp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 6 ;
	}
#Time-integrated air concentration of iodine pollutant
'tiacip' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 7 ;
	}
#Time-integrated air concentration of radioactive pollutant
'tiacrp' = {
	 discipline = 0 ;
	 parameterCategory = 18 ;
	 parameterNumber = 8 ;
	}
#Volcanic ash
'volash' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 4 ;
	}
#Icing top
'icit' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 5 ;
	}
#Icing base
'icib' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 6 ;
	}
#Icing
'ici' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 7 ;
	}
#Turbulence top
'turbt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 8 ;
	}
#Turbulence base
'turbb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 9 ;
	}
#Turbulence
'turb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 10 ;
	}
#Turbulent kinetic energy
'tke' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 11 ;
	}
#Planetary boundary layer regime
'pblreg' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 12 ;
	}
#Contrail intensity
'conti' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 13 ;
	}
#Contrail engine type
'contet' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 14 ;
	}
#Contrail top
'contt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 15 ;
	}
#Contrail base
'contb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 16 ;
	}
#Snow free albedo
'snfalb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 18 ;
	}
#Icing
'~' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 20 ;
	}
#In-cloud turbulence
'~' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 21 ;
	}
#Relative clear air turbulence (RCAT)
'rcat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 22 ;
	}
#Supercooled large droplet probability
'sldp' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 23 ;
	}
#Arbitrary text string
'var190m0' = {
	 discipline = 0 ;
	 parameterCategory = 190 ;
	 parameterNumber = 0 ;
	}
#Seconds prior to initial reference time (defined in Section 1)
'tsec' = {
	 discipline = 0 ;
	 parameterCategory = 191 ;
	 parameterNumber = 0 ;
	}
#Flash flood guidance
'ffldg' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Flash flood runoff (Encoded as an accumulation over a floating subinterval of time)
'ffldro' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Remotely sensed snow cover
'rssc' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Elevation of snow covered terrain
'esct' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Snow water equivalent percent of normal
'swepon' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Baseflow-groundwater runoff
'bgrun' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Storm surface runoff
'ssrun' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Conditional percent precipitation amount fractile for an overall period (Encoded as an accumulation)
'cppop' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Percent precipitation in a sub-period of an overall period
'pposp' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Probability of 0.01 inch of precipitation (POP)
'pop' = {
	 discipline = 1 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Vegetation
'veg' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Water runoff
'watr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Evapotranspiration
'evapt' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Model terrain height
'mterh' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Land use
'landu' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Ground heat flux
'gflux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Moisture availability
'mstav' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Exchange coefficient
'sfexc' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Plant canopy surface water
'cnwat' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Blackadar mixing length scale
'bmixl' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Canopy conductance
'ccond' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Minimal stomatal resistance
'rsmin' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Solar parameter in canopy conductance
'rcs' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 18 ;
	}
#Temperature parameter in canopy conductance
'rct' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 19 ;
	}
#Soil moisture parameter in canopy conductance
'rcsol' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#Humidity parameter in canopy conductance
'rcq' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Column-integrated soil water
'cisoilw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Heat flux
'hflux' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	}
#Volumetric soil moisture
'vsw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Volumetric wilting point
'vwiltm' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Number of soil layers in root zone
'rlyrs' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Liquid volumetric soil moisture (non-frozen)
'liqvsm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	}
#Volumetric transpiration stress-onset (soil moisture)
'voltso' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	}
#Transpiration stress-onset (soil moisture)
'transo' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	}
#Volumetric direct evaporation cease (soil moisture)
'voldec' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	}
#Direct evaporation cease (soil moisture)
'direc' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#Soil porosity
'soilp' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Volumetric saturation of soil moisture
'vsosm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	}
#Saturation of soil moisture
'satosm' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	}
#Estimated precipitation
'estp' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Instantaneous rain rate
'irrate' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Cloud top height
'ctoph' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Cloud top height quality indicator
'ctophqi' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Estimated u component of wind
'estu' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Estimated v component of wind
'estv' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Number of pixels used
'npixu' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 6 ;
	}
#Solar zenith angle
'solza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Relative azimuth angle
'raza' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 8 ;
	}
#Reflectance in 0.6 micron channel
'rfl06' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Reflectance in 0.8 micron channel
'rfl08' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 10 ;
	}
#Reflectance in 1.6 micron channel
'rfl16' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 11 ;
	}
#Reflectance in 3.9 micron channel
'rfl39' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 12 ;
	}
#Atmospheric divergence
'atmdiv' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 13 ;
	}
#Direction of wind waves
'wvdir' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Primary wave direction
'dirpw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 10 ;
	}
#Primary wave mean period
'perpw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Secondary wave mean period
'persw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 13 ;
	}
#Current direction
'dirc' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Current speed
'spc' = {
	 discipline = 10 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Geometric vertical velocity
'wz' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 9 ;
	}
#Seconds prior to initial reference time (defined in Section 1)
'tsec' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 0 ;
	}
#2 metre relative humidity
'2r' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Apparent temperature
'aptmp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#Haines Index
'hindex' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 2 ;
	}
#Cloud cover
'ccl' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 22 ;
	}
#Evaporation
'eva' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#10 metre wind direction
'10wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Surface direct short-wave radiation flux
'sdirswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface diffuse short-wave radiation flux
'sdifswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Fraction of snow cover
'fscov' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 121 ;
	}
#Clear air turbulence (CAT)
'cat' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 29 ;
	}
#Mountain wave turbulence (eddy dissipation rate)
'mwt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 28 ;
	}
#Specific rain water content (convective)
'crwc_conv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 144 ;
	}
#Specific snow water content (convective)
'cswc_conv' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 145 ;
	}
#Glacier mask
'glm' = {
	 discipline = 2 ;
	 parameterCategory = 5 ;
	 parameterNumber = 0 ;
	}
#Precipitation type (most severe) in the last 1 hour
'ptype_sev1h' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 100 ;
	 lengthOfTimeRange = 1 ;
	}
#Precipitation type (most severe) in the last 3 hours
'ptype_sev3h' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 100 ;
	 lengthOfTimeRange = 3 ;
	}
#Precipitation type (most frequent) in the last 1 hour
'ptype_freq1h' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 101 ;
	 lengthOfTimeRange = 1 ;
	}
#Precipitation type (most frequent) in the last 3 hours
'ptype_freq3h' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 101 ;
	 lengthOfTimeRange = 3 ;
	}
#Precipitation type (most severe) in the last 6 hours
'ptype_sev6h' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 100 ;
	 lengthOfTimeRange = 6 ;
	}
#Precipitation type (most frequent) in the last 6 hours
'ptype_freq6h' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	 indicatorOfUnitForTimeRange = 1 ;
	 typeOfStatisticalProcessing = 101 ;
	 lengthOfTimeRange = 6 ;
	}
#Soil temperature
'sot' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	}
#Surface downward short-wave radiation flux, clear sky
'sdswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface upward short-wave radiation flux, clear sky
'suswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface downward long-wave radiation flux, clear sky
'sdlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Soil heat flux
'sohf' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 26 ;
	}
#Percolation rate
'percr' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Soil depth
'sod' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	}
#Soil moisture
'som' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	}
#Leaf Area Index
'lai' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Accumulated surface upward short-wave radiation flux, clear sky
'auswrf_cs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Percolation
'perc' = {
	 discipline = 1 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 177 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Evapotranspiration rate
'etr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Potential evapotranspiration rate
'petr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Runoff rate water equivalent (surface plus subsurface)
'rorwe' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 42 ;
	}
#Snow melt rate
'smr' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Forecast albedo
'al' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 1 ;
	}
#Cloudy brightness temperature
'clbt' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 14 ;
	}
#Clear-sky brightness temperature
'csbt' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 15 ;
	}
#Cloudy reflectance
'cdrfl' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 31 ;
	}
#Clear reflectance
'crrfl' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 32 ;
	}
#Scaled radiance
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Scaled albedo
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Scaled brightness temperature
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Scaled precipitable water
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Scaled lifted index
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Scaled cloud top pressure
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Scaled skin temperature
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Cloud mask
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Pixel scene type
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Fire detection indicator
'~' = {
	 discipline = 3 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Forest fire weather index (as defined by the Canadian Forest Service)
'fwinx' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 5 ;
	}
#Fine fuel moisture code (as defined by the Canadian Forest Service)
'ffmcode' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 6 ;
	}
#Duff moisture code (as defined by the Canadian Forest Service)
'dufmcode' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	}
#Drought code (as defined by the Canadian Forest Service)
'drtcode' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 8 ;
	}
#Initial fire spread index (as defined by the Canadian Forest Service)
'infsinx' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	}
#Fire buildup index (as defined by the Canadian Forest Service)
'fbupinx' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 10 ;
	}
#Fire daily severity rating (as defined by the Canadian Forest Service)
'fdsrte' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	}
#Cloudy radiance (with respect to wave number)
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 16 ;
	}
#Clear-sky radiance (with respect to wave number)
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 17 ;
	}
#Wind speed
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 19 ;
	}
#Aerosol optical thickness at 0.635 um
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 20 ;
	}
#Aerosol optical thickness at 0.810 um
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 21 ;
	}
#Aerosol optical thickness at 1.640 um
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 22 ;
	}
#Angstrom coefficient
'~' = {
	 discipline = 3 ;
	 parameterCategory = 1 ;
	 parameterNumber = 23 ;
	}
#Keetch-Byram drought index
'kbdi' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 12 ;
	}
#Drought factor (as defined by the Australian forest service)
'drtmrk' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 13 ;
	}
#Rate of spread (as defined by the Australian forest service)
'rosmrk' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	}
#Fire danger index (as defined by the Australian forest service)
'fdimrk' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 15 ;
	}
#Spread component (as defined by the U.S Forest Service National Fire-Danger Rating System)
'scnfdr' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 16 ;
	}
#Burning index (as defined by the U.S Forest Service National Fire-Danger Rating System)
'buinfdr' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 17 ;
	}
#Ignition component (as defined by the U.S Forest Service National Fire-Danger Rating System)
'icnfdr' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	}
#Energy release component (as defined by the U.S Forest Service National Fire-Danger Rating System)
'ercnfdr' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 19 ;
	}
#Probability of fire detection
'pof' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 43 ;
	}
#Probability of ignition from lightning
'pil' = {
	 discipline = 2 ;
	 parameterCategory = 4 ;
	 parameterNumber = 44 ;
	}
#Volumetric soil ice
'vsi' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 38 ;
	}
#Time integral of total solid precipitation flux
'titspf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 128 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#10 metre eastward wind gust since previous post-processing
'10efg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#10 metre northward wind gust since previous post-processing
'10nfg' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 103 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Fog
'fog' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Time-integrated eastward turbulent surface stress due to orographic form drag
'etssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward turbulent surface stress due to orographic form drag
'ntssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated eastward turbulent surface stress due to surface roughness
'etsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward turbulent surface stress due to surface roughness
'ntsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Saturation specific humidity with respect to water
'sqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 168 ;
	}
#Total column integrated saturation specific humidity with respect to water
'tcsqw' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 169 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Standardised Precipitation Index (SPI)
'spi' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 0 ;
	}
#Standardised Precipitation Evapotranspiration Index (SPEI)
'spei' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 1 ;
	}
#Standardised Streamflow Index (SSFI)
'ssfi' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 2 ;
	}
#Standardised Reservoir Supply Index (SRSI)
'srsi' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 3 ;
	}
#Standardised Water-level Index (SWI)
'swi' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 4 ;
	}
#Standardised Snowmelt and Rain Index (SMRI)
'smri' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 5 ;
	}
#Streamflow Drought Index (SDI)
'sdi' = {
	 discipline = 0 ;
	 parameterCategory = 22 ;
	 parameterNumber = 6 ;
	}
#Volumetric soil moisture top 20 cm
'vsw20' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Volumetric soil moisture top 100 cm
'vsw100' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 1 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Surface photosynthetically active radiation flux, clear sky
'parcsf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 60 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface direct normal short-wave radiation flux
'sdirnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 54 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Instantaneous magnitude of turbulent surface stress
'imagss' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous boundary layer dissipation
'ibld' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	}
#Top net short-wave radiation flux
'tnswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Top net long-wave radiation flux
'tnlwrf' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Instantaneous gravity wave dissipation
'igwd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 23 ;
	}
#Top net short-wave radiation flux, clear sky
'tnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Top net long-wave radiation flux, clear sky
'tnlwrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 5 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Top downward short-wave radiation flux
'tdswrf' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 8 ;
	}
#Surface direct short-wave radiation flux, clear sky
'sdirswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 61 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous eastward turbulent surface stress due to orographic form drag
'ietssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 64 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress due to orographic form drag
'intssofd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 65 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous eastward turbulent surface stress due to surface roughness
'ietsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Instantaneous northward turbulent surface stress due to surface roughness
'intsssr' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 67 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Land surface heat net flux
'lshnf' = {
	 discipline = 2 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Surface direct normal short-wave radiation flux, clear sky
'sdirnswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 62 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#10 metre convective gust
'10cogu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 68 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#10 metre turbulent gust
'10tugu' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 69 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Reciprocal Obukhov length
'rol' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 22 ;
	}
#Precipitation rate from radar
'rdpr' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 17 ;
	}
#Radar data quality index
'rdqi' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 18 ;
	}
#Radar data quality flag
'rdqf' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 19 ;
	}
#Precipitation rate from forecasted radar imagery
'frdpr' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 6 ;
	}
#Precipitation from forecasted radar imagery
'frdp' = {
	 discipline = 0 ;
	 parameterCategory = 16 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface diffuse short-wave radiation flux, clear sky
'sdifswrfcs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 63 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Storm relative helicity - right moving storm
'hlcr' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 23 ;
	}
#Storm relative helicity - left moving storm
'hlcl' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 24 ;
	}
#Effective storm relative helicity - mean flow
'ehlm' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 28 ;
	 typeOfSecondFixedSurface = 29 ;
	}
#Effective storm relative helicity - right moving storm
'ehlr' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 28 ;
	 typeOfSecondFixedSurface = 29 ;
	}
#Effective storm relative helicity - left moving storm
'ehll' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 28 ;
	 typeOfSecondFixedSurface = 29 ;
	}
#Universal thermal climate index
'utci' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Mean radiant temperature
'mrt' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Fraction of Malaria cases
'mal_cases_frac' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 0 ;
	}
#Malaria circumsporozoite protein ratio
'mal_prot_ratio' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 1 ;
	}
#Plasmodium falciparum entomological inoculation rate
'mal_innoc_rate' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 2 ;
	}
#Human bite rate by anopheles vectors
'mal_hbite_rate' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Malaria immunity ratio
'mal_immun_ratio' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 4 ;
	}
#Falciparum parasite ratio
'mal_infect_ratio' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Detectable falciparum parasite ratio (after day 10)
'mal_infect_d10_ratio' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 6 ;
	}
#Anopheles vector to host ratio
'mal_host_ratio' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Anopheles vector density
'mal_vect_dens' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 8 ;
	}
#Fraction of malarial vector reproductive habitat
'mal_hab_frac' = {
	 discipline = 20 ;
	 parameterCategory = 1 ;
	 parameterNumber = 9 ;
	}
#Population density
'pop_dens' = {
	 discipline = 20 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	}
#Wet bulb globe temperature
'wbgt' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Globe temperature
'gt' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Humidex
'hmdx' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 4 ;
	}
#Effective temperature
'efft' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Normal effective temperature
'nefft' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Standard effective temperature
'sefft' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Physiological equivalent temperature
'peqt' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Saturation water vapour pressure
'swvp' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 32 ;
	}
#Wet-bulb potential temperature
'wbpt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	}
#Wet-bulb temperature
'wbt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Excess heat factor
'exhf' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 11 ;
	}
#Excess cold factor
'excf' = {
	 discipline = 20 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Sea ice thickness
'sithick' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	}
#Sea ice area fraction
'siconc' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Eastward sea ice velocity
'siue' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Northward sea ice velocity
'sivn' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice albedo
'sialb' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice surface temperature
'sitemptop' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice growth
'sigrowth' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice volume per unit area
'sivol' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow volume over sea ice per unit area
'snvol' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Vertically averaged sea ice temperature
'vasit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow temperature over sea ice
'sntemp' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice temperature at the sea ice and snow interface
'sisntemp' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Underside ice temperature
'usitemp' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice heat content
'sihc' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow heat content over sea ice
'snhc' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice freeboard thickness
'sifbr' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Sea ice melt pond fraction
'sipf' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice melt pond depth
'sipd' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice melt pond volume per unit area
'sipvol' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice fraction tendency due to parameterization
'bckinsic' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#X-component of sea ice velocity
'six' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Y-component of sea ice velocity
'siy' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea ice temperature
'sit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	}
#Sea ice breakup memory
'sibm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	}
#Sea surface practical salinity
'sos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Sea surface temperature
'tos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 14 C isotherm
't14d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 17 C isotherm
't17d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 20 C isotherm
't20d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 26 C isotherm
't26d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Depth of 28 C isotherm
't28d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Barotropic stream function
'stfbarot' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Surface downward heat flux
'hfds' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	}
#Northward surface stress
'tauvon' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	}
#Eastward surface stress
'tauuoe' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	}
#Y-component of surface stress
'tauvo' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#X-component of surface stress
'tauuo' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'mlotst010' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'mlotst030' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'mlotst125' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by temperature 0.2 C
'mlott02' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Ocean mixed layer depth defined by temperature 0.5 C
'mlott05' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Average sea water practical salinity in the upper 300 m
'sc300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Average sea water practical salinity in the upper 700 m
'sc700m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column average sea water practical salinity
'scbtm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Vertically-integrated heat content in the upper 300 m
'hc300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Vertically-integrated heat content in the upper 700 m
'hc700m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column of heat content
'hcbtm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea surface height
'zos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	}
#Steric change in sea surface height
'stheig' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Halosteric change in sea surface height
'hstheig' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Thermosteric change in sea surface height
'tstheig' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Thermocline depth
'thcline' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Bottom pressure equivalent height
'btp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Net surface upward water flux
'swfup' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Fresh water flux into sea water (from rivers)
'fw2sw' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Virtual salt flux into sea water
'vsf2sw' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Heat flux correction
'hfcorr' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Fresh water flux correction
'fwcorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Virtual salt flux correction
'vsfcorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Turbocline depth (kz=5e-4)
'turbocl' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Y-component of surface sea water velocity
'svy' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#X-component of surface sea water velocity
'svx' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Northward surface sea water velocity
'svn' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	}
#Eastward surface sea water velocity
'sve' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	}
#Heat Content surface to 26C isotherm
'hct26' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	}
#Sea surface height tendency due to parameterization
'bckineta' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea surface height with inverse barometer correction
'zosib' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	}
#Average sea water potential temperature in the upper 300m
'pt300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Sea surface salinity
'sss' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	}
#Vertically integrated sea water practical salinity in the upper 300 m
'sc300v' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Vertically integrated sea water practical salinity in the upper 700 m
'sc700v' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	}
#Total column vertically integrated sea water practical salinity
'scbtv' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea surface downward short-wave radiation flux
'rsdos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	}
#Sea water practical salinity
'so' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water potential temperature
'thetao' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water sigma theta
'sigmat' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Y-component of sea water velocity
'voy' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#X-component of sea water velocity
'uox' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Northward sea water velocity
'von' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Eastward sea water velocity
'uoe' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Upward sea water velocity
'wo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Sea water potential temperature tendency due to newtonian relaxation
'thetaodmp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity tendency due to newtonian relaxation
'sodmp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water temperature tendency due to parameterization
'bckint' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity tendency due to parameterization
'bckins' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Eastward sea water velocity tendency due to parameterization
'bckine' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Northward sea water velocity tendency due to parameterization
'bckinn' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water temperature tendency due to direct bias correction
'tdbiascorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity tendency due to direct bias correction
'sdbiascorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water salinity
'salo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Northward mass transport
'mtn' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Eastward mass transport
'mte' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water depth
'zo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water upward mass transport
'mtup' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water age since surface contact
'agessc' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Sea water downward short-wave radiation flux
'rsdo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	}
#Net short wave radiation rate at sea surface
'ssr_sea' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Wind stress at sea surface
'wst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Wind speed at 10m above sea surface
'10ws_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Neutral drag coefficient at 10m above sea surface
'10nd_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Total precipitation rate at sea surface
'tprate_sea' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Snow precipitation rate at sea surface
'snrate_sea' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Eastward component of wind stress over sea ice
'ewst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Northward component of wind stress over sea ice
'nwst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#U-component of wind stress over sea ice
'uwst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#V-component of wind stress over sea ice
'vwst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Time-mean sea ice thickness
'avg_sithick' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice area fraction
'avg_siconc' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward sea ice velocity
'avg_siue' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward sea ice velocity
'avg_sivn' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice albedo
'avg_sialb' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice surface temperature
'avg_sitemptop' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice growth
'avg_sigrowth' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice volume per unit area
'avg_sivol' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow volume over sea ice per unit area
'avg_snvol' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically averaged sea ice temperature
'avg_vasit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow temperature over sea ice
'avg_sntemp' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice temperature at the sea ice and snow interface
'avg_sisntemp' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 175 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean underside ice temperature
'avg_usitemp' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 176 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice heat content
'avg_sihc' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow heat content over sea ice
'avg_snhc' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 173 ;
	 typeOfSecondFixedSurface = 175 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice freeboard thickness
'avg_sifbr' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = 0 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice melt pond fraction
'avg_sipf' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice melt pond depth
'avg_sipd' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice melt pond volume per unit area
'avg_sipvol' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 188 ;
	 typeOfSecondFixedSurface = 189 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice fraction tendency due to parameterization
'avg_bckinsic' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of sea ice velocity
'avg_six' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y-component of sea ice velocity
'avg_siy' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice temperature
'avg_sit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea ice breakup memory
'avg_sibm' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface practical salinity
'avg_sos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface temperature
'avg_tos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 14 C isotherm
'avg_t14d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 28715 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 17 C isotherm
'avg_t17d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29015 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 20 C isotherm
'avg_t20d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29315 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 26 C isotherm
'avg_t26d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 29915 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean depth of 28 C isotherm
'avg_t28d' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 20 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 30115 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean barotropic stream function
'avg_stfbarot' = {
	 discipline = 10 ;
	 parameterCategory = 191 ;
	 parameterNumber = 4 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean surface downward heat flux
'avg_hfds' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 4 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward surface stress
'avg_tauvon' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 6 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward surface stress
'avg_tauuoe' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time mean Y-component of surface stress
'avg_tauvo' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of surface stress
'avg_tauuo' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by sigma theta 0.01 kg m-3
'avg_mlotst010' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 1 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by sigma theta 0.03 kg m-3
'avg_mlotst030' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 3 ;
	 scaleFactorOfFirstFixedSurface = 2 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by sigma theta 0.125 kg m-3
'avg_mlotst125' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 169 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 125 ;
	 scaleFactorOfFirstFixedSurface = 3 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by temperature 0.2 C
'avg_mlott02' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean ocean mixed layer depth defined by temperature 0.5 C
'avg_mlott05' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 170 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 1 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean average sea water practical salinity in the upper 300 m
'avg_sc300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean average sea water practical salinity in the upper 700 m
'avg_sc700m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column average sea water practical salinity
'avg_scbtm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically-integrated heat content in the upper 300 m
'avg_hc300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically-integrated heat content in the upper 700 m
'avg_hc700m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column heat content
'avg_hcbtm' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface height
'avg_zos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 1 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean steric change in sea surface height
'avg_stheig' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean halosteric change in sea surface height
'avg_hstheig' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean thermosteric change in sea surface height
'avg_tstheig' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean thermocline depth
'avg_thcline' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean bottom pressure equivalent height
'avg_btp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean net surface upward water flux
'avg_swfup' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 13 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fresh water flux into sea water (from rivers)
'avg_fw2sw' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 30 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean virtual salt flux into sea water
'avg_vsf2sw' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 32 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean heat flux correction
'avg_hfcorr' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean fresh water flux correction
'avg_fwcorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 31 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean virtual salt flux correction
'avg_vsfcorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 33 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean turbocline depth (kz=5e-4)
'avg_turbocl' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 171 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 5 ;
	 scaleFactorOfFirstFixedSurface = 4 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y-component of surface sea water velocity
'avg_svy' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of surface sea water velocity
'avg_svx' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward surface sea water velocity
'avg_svn' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 15 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward surface sea water velocity
'avg_sve' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 14 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean heat content surface to 26C isotherm
'avg_hct26' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 20 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 29915 ;
	 scaleFactorOfSecondFixedSurface = 2 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface height tendency due to parameterization
'avg_bckineta' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface height with inverse barometer correction
'avg_zosib' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean average sea water potential temperature in the upper 300m
'avg_pt300m' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface salinity
'avg_sss' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 21 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically integrated sea water practical salinity in the upper 300 m
'avg_sc300v' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 300 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean vertically integrated sea water practical salinity in the upper 700 m
'avg_sc700v' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 160 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 700 ;
	 scaleFactorOfSecondFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total column vertically integrated sea water practical salinity
'avg_scbtv' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 9 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea surface downward short-wave radiation flux
'avg_rsdos' = {
	 discipline = 10 ;
	 parameterCategory = 3 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water practical salinity
'avg_so' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water potential temperature
'avg_thetao' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 18 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water sigma theta
'avg_sigmat' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean Y-component of sea water velocity
'avg_voy' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean X-component of sea water velocity
'avg_uox' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 25 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward sea water velocity
'avg_von' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 24 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward sea water velocity
'avg_uoe' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 23 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean upward sea water velocity
'avg_wo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 27 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water potential temperature tendency due to newtonian relaxation
'avg_thetaodmp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 34 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity tendency due to newtonian relaxation
'avg_sodmp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water temperature tendency due to parameterization
'avg_bckint' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity tendency due to parameterization
'avg_bckins' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward sea water velocity tendency due to parameterization
'avg_bckine' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 38 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward sea water velocity tendency due to parameterization
'avg_bckinn' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water temperature tendency due to direct bias correction
'avg_tdbiascorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity tendency due to direct bias correction
'avg_sdbiascorr' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water salinity
'avg_salo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward mass transport
'avg_mtn' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 46 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward mass transport
'avg_mte' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 47 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water depth
'avg_zo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water upward mass transport
'avg_mtup' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water age since surface contact
'avg_agessc' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean sea water downward short-wave radiation flux
'avg_rsdo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 58 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean net short wave radiation rate at sea surface
'avg_ssr_sea' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind stress at sea surface
'avg_wst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 49 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean wind speed at 10m above sea surface
'avg_10ws_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean neutral drag coefficient at 10m above sea surface
'avg_10nd_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 29 ;
	 typeOfFirstFixedSurface = 102 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean total precipitation rate at sea surface
'avg_tprate_sea' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean snow precipitation rate at sea surface
'avg_snrate_sea' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean eastward component of wind stress over sea ice
'avg_ewst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 50 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean northward component of wind stress over sea ice
'avg_nwst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 51 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean U-component of wind stress over sea ice
'avg_uwst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean V-component of wind stress over sea ice
'avg_vwst_sea' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-accumulated net short wave radiation at sea surface
'acc_ssr_sea' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-accumulated total precipitation at sea surface
'tp_sea' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-accumulated snow precipitation at sea surface
'sn_sea' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 66 ;
	 typeOfFirstFixedSurface = 160 ;
	 typeOfSecondFixedSurface = 255 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-maximum sea ice temperature
'max_sit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-maximum sea water depth
'max_zo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 2 ;
	}
#Time-minimum sea ice temperature
'min_sit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-minimum sea water depth
'min_zo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 3 ;
	}
#Time-standard-deviation sea ice temperature
'std_sit' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 8 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Time-standard-deviation sea water depth
'std_zo' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 14 ;
	 typeOfFirstFixedSurface = 168 ;
	 typeOfSecondFixedSurface = 168 ;
	 typeOfStatisticalProcessing = 6 ;
	}
#Virtual temperature
'vtmp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Pressure tendency
'ptend' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 2 ;
	}
#ICAO Standard Atmosphere reference height
'icaht' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 3 ;
	}
#Geometrical height above ground
'h' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 34 ;
	}
#Standard deviation of height
'hstdv' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 7 ;
	}
#Virtual potential temperature
'vptmp' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Pseudo-adiabatic potential temperature
'papt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Dew point temperature
'dpt' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Lapse rate
'lapr' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Visibility
'vis' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 0 ;
	}
#Radar spectra (1)
'rdsp1' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 6 ;
	}
#Radar spectra (2)
'rdsp2' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 7 ;
	}
#Radar spectra (3)
'rdsp3' = {
	 discipline = 0 ;
	 parameterCategory = 15 ;
	 parameterNumber = 8 ;
	}
#Parcel lifted index (to 500 hPa)
'pli' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 0 ;
	}
#Temperature anomaly
'ta' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Pressure anomaly
'presa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 8 ;
	}
#Geopotential height anomaly
'gpa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 9 ;
	}
#Wave spectra (1)
'wvsp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	}
#Wave spectra (2)
'wvsp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 1 ;
	}
#Wave spectra (3)
'wvsp3' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 2 ;
	}
#Wind direction
'wdir' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	}
#Sigma coordinate vertical velocity
'sgcvv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 7 ;
	}
#Absolute vorticity
'absv' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	}
#Absolute divergence
'absd' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 11 ;
	}
#Vertical u-component shear
'vucsh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 15 ;
	}
#Vertical v-component shear
'vvcsh' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 16 ;
	}
#Precipitable water
'pwat' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 3 ;
	}
#Saturation deficit
'satd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 5 ;
	}
#Precipitation rate
'prate' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 7 ;
	}
#Thunderstorm probability
'tstm' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 2 ;
	}
#Convective precipitation (water)
'acpcp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 10 ;
	}
#Mixed layer depth
'mld' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 3 ;
	}
#Transient thermocline depth
'tthdp' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 2 ;
	}
#Main thermocline anomaly
'mtha' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 1 ;
	}
#Best lifted index (to 500 hPa)
'bli' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 1 ;
	}
#Soil moisture content
'ssw' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Salinity
's' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Density
'den' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 10 ;
	}
#Direction of ice drift
'diced' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	}
#Speed of ice drift
'siced' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	}
#Ice divergence
'iced' = {
	 discipline = 10 ;
	 parameterCategory = 2 ;
	 parameterNumber = 7 ;
	}
#Snowmelt
'snom' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Direction of swell waves
'swdir' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 7 ;
	}
#Secondary wave direction
'dirsw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 12 ;
	}
#Net short-wave radiation flux (surface)
'nswrs' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 0 ;
	}
#Global radiation flux
'grad' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 3 ;
	}
#Radiance (with respect to wave number)
'lwrad' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 5 ;
	}
#Radiance (with respect to wave length)
'swrad' = {
	 discipline = 0 ;
	 parameterCategory = 4 ;
	 parameterNumber = 6 ;
	}
#Wind mixing energy
'wmixe' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 19 ;
	}
#Total precipitation less than 0.1 mm
'tpl01' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 1 ;
	 scaledValueOfLowerLimit = 1 ;
	 scaleFactorOfLowerLimit = 1 ;
	 probabilityType = 0 ;
	}
#Total precipitation rate less than 1 mm/day
'tprl1' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	 scaledValueOfLowerLimit = 115741 ;
	 scaleFactorOfLowerLimit = 10 ;
	 probabilityType = 0 ;
	}
#Total precipitation rate of at least 3 mm/day
'tprg3' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	 scaledValueOfLowerLimit = 347222 ;
	 scaleFactorOfLowerLimit = 10 ;
	 probabilityType = 3 ;
	}
#Total precipitation rate of at least 5 mm/day
'tprg5' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 typeOfStatisticalProcessing = 0 ;
	 scaledValueOfLowerLimit = 578704 ;
	 scaleFactorOfLowerLimit = 10 ;
	 probabilityType = 3 ;
	}
#10 metre Wind speed of at least 10 m/s
'10spg10' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 10 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre Wind speed of at least 15 m/s
'10spg15' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 15 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust of at least 15 m/s
'10fgg15' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 15 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust of at least 20 m/s
'10fgg20' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 2 ;
	 scaledValueOfLowerLimit = 20 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#2 metre temperature less than 273.15 K
'2tl273' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfLowerLimit = 27315 ;
	 scaleFactorOfLowerLimit = 2 ;
	 probabilityType = 0 ;
	}
#Significant wave height of at least 2 m
'swhg2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 2 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Significant wave height of at least 4 m
'swhg4' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 4 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Significant wave height of at least 6 m
'swhg6' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 6 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#Significant wave height of at least 8 m
'swhg8' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	 scaledValueOfLowerLimit = 8 ;
	 scaleFactorOfLowerLimit = 0 ;
	 probabilityType = 3 ;
	}
#10 metre wind gust index
'10fgi' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 22 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Snowfall index
'sfi' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = [102,1] ;
	}
#10 metre speed index
'10wsi' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,0] ;
	}
#2 metre temperature index
'2ti' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,0] ;
	}
#Maximum temperature at 2 metres index
'mx2ti' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,2] ;
	}
#Minimum temperature at 2 metres index
'mn2ti' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = [102,3] ;
	}
#Total precipitation index
'tpi' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = [102,1] ;
	}
#Period corresponding to maximum individual wave height
'tmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	}
#Envelop-maximum individual wave height
'hmax' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 93 ;
	}
#Model bathymetry
'wmb' = {
	 discipline = 10 ;
	 parameterCategory = 4 ;
	 parameterNumber = 7 ;
	}
#Mean wave period based on first moment
'mp1' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	}
#Mean zero-crossing wave period
'mp2' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 28 ;
	}
#Wave spectral directional width
'wdw' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 31 ;
	}
#Mean wave period based on first moment for wind waves
'p1ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	}
#Mean wave period based on second moment for wind waves
'p2ww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 29 ;
	}
#Wave spectral directional width for wind waves
'dwww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 32 ;
	}
#Mean wave period based on first moment for swell
'p1ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 27 ;
	}
#Mean wave period based on second moment for swell
'p2ps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 30 ;
	}
#Wave spectral directional width for swell
'dwps' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 33 ;
	}
#Significant height of combined wind waves and swell
'swh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 3 ;
	}
#Mean wave direction
'mwd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 14 ;
	}
#Peak wave period
'pp1d' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	}
#Mean wave period
'mwp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 15 ;
	}
#Coefficient of drag with waves
'cdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 16 ;
	}
#Significant height of wind waves
'shww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 5 ;
	}
#Mean direction of wind waves
'mdww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 75 ;
	}
#Mean period of wind waves
'mpww' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 6 ;
	}
#Significant height of total swell
'shts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 8 ;
	}
#Mean direction of total swell
'mdts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 74 ;
	}
#Mean period of total swell
'mpts' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 9 ;
	}
#Mean square slope of waves
'msqs' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	}
#10 metre wind speed
'wind' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Altimeter wave height
'awh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 37 ;
	}
#Altimeter corrected wave height
'acwh' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 38 ;
	}
#Altimeter range relative correction
'arrc' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 39 ;
	}
#10 metre wind direction
'dwi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 102 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#2D wave spectra (single)
'2dfd' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 86 ;
	}
#Wave spectral kurtosis
'wsk' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 43 ;
	}
#Benjamin-Feir index
'bfi' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 44 ;
	}
#Wave spectral peakedness
'wsp' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 98 ;
	}
#Skin reservoir content
'srcon' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 50 ;
	}
#Total column vertically-integrated mass of atmosphere
'vima' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 39 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated kinetic energy
'vike' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated enthalpy
'vithe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated potential + internal energy
'vipie' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated potential+internal+latent energy
'vipile' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 21 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated total energy
'vitoe' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward mass flux
'vimae' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 36 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward mass flux
'viman' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 37 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward kinetic energy flux
'vikee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 9 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward kinetic energy flux
'viken' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 10 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward heat flux
'vithee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 19 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward heat flux
'vithen' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 20 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward water vapour flux
'viwve' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 150 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward water vapour flux
'viwvn' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 151 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward geopotential flux
'vige' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 40 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward geopotential flux
'vign' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 41 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward total energy flux
'vitee' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 11 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated northward total energy flux
'viten' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated eastward ozone flux
'vioze' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 78 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Total column vertically-integrated northward ozone flux
'viozn' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 79 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Total column vertically-integrated divergence of mass flux
'vimad' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 35 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of kinetic energy flux
'viked' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 16 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated moisture divergence flux
'vimdf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 165 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of geopotential flux
'vigd' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 43 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of total energy flux
'vited' = {
	 discipline = 0 ;
	 parameterCategory = 21 ;
	 parameterNumber = 17 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Total column vertically-integrated divergence of ozone flux
'viozd' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 80 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	 constituentType = 0 ;
	}
#Time-integrated temperature tendency due to short-wave radiation
'srta' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to long-wave radiation
'trta' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 23 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to short wave radiation, clear sky
'srtca' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 24 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to long-wave radiation, clear sky
'trtca' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 25 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated updraught mass flux
'umfa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 27 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated downdraught mass flux
'dmfa' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 28 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated updraught detrainment rate
'udra' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 29 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated downdraught detrainment rate
'ddra' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 30 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated total precipitation flux
'tpfa' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated turbulent diffusion coefficient for heat
'tdcha' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 20 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated temperature tendency due to parametrisations
'ttpha' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated specific humidity tendency due to parametrisations
'qtpha' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 108 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated eastward wind tendency due to parametrisations
'utpha' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 39 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Time-integrated northward wind tendency due to parametrisations
'vtpha' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 40 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Surface runoff
'sro' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 34 ;
	}
#Nitrogen dioxide mass mixing ratio
'no2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 5 ;
	 is_chemical = 1 ;
	}
#Sulphur dioxide mass mixing ratio
'so2' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 is_chemical = 1 ;
	}
#Carbon monoxide mass mixing ratio
'co' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 4 ;
	 is_chemical = 1 ;
	}
#Ozone mass mixing ratio (full chemistry scheme)
'go3' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 0 ;
	 is_chemical = 1 ;
	}
#Nitrogen dioxide mass mixing ratio difference
'no2diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 5 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Sulphur dioxide mass mixing ratio difference
'so2diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 8 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Carbon monoxide mass mixing ratio difference
'codiff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 4 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Ozone mass mixing ratio difference (full chemistry scheme)
'go3diff' = {
	 discipline = 0 ;
	 parameterCategory = 20 ;
	 parameterNumber = 2 ;
	 constituentType = 0 ;
	 typeOfGeneratingProcess = 20 ;
	 is_chemical = 1 ;
	}
#Convective inhibition
'cin' = {
	 discipline = 0 ;
	 parameterCategory = 7 ;
	 parameterNumber = 7 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Orography
'orog' = {
	 discipline = 0 ;
	 parameterCategory = 3 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 1 ;
	}
#Friction velocity
'zust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 17 ;
	}
#Time-mean 2 metre temperature
'avg_2t' = {
	 discipline = 0 ;
	 parameterCategory = 0 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 2 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Time-mean 10 metre wind speed
'avg_10ws' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 typeOfStatisticalProcessing = 0 ;
	}
#Lake total depth
'dl' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Lake mix-layer temperature
'lmlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake mix-layer depth
'lmld' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 0 ;
	 typeOfFirstFixedSurface = 166 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake bottom temperature
'lblt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 162 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake total layer temperature
'ltlt' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 162 ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Lake shape factor
'lshf' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 10 ;
	}
#Lake ice surface temperature
'lict' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 6 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 255 ;
	}
#Lake ice total depth
'licd' = {
	 discipline = 1 ;
	 parameterCategory = 2 ;
	 parameterNumber = 5 ;
	 typeOfFirstFixedSurface = 174 ;
	 typeOfSecondFixedSurface = 176 ;
	 scaledValueOfFirstFixedSurface = missing() ;
	 scaleFactorOfFirstFixedSurface = missing() ;
	 scaledValueOfSecondFixedSurface = missing() ;
	 scaleFactorOfSecondFixedSurface = missing() ;
	}
#Minimum vertical gradient of refractivity inside trapping layer
'dndzn' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 45 ;
	}
#Mean vertical gradient of refractivity inside trapping layer
'dndza' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 44 ;
	}
#Duct base height
'dctb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 41 ;
	}
#Trapping layer base height
'tplb' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 42 ;
	}
#Trapping layer top height
'tplt' = {
	 discipline = 0 ;
	 parameterCategory = 19 ;
	 parameterNumber = 43 ;
	}
#10 metre u-component of neutral wind
'u10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 56 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#10 metre v-component of neutral wind
'v10n' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 57 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 10 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#Snow depth water equivalent
'sd' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 60 ;
	}
#Snowfall water equivalent
'sf' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 53 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#Total Cloud Cover
'tcc' = {
	 discipline = 0 ;
	 parameterCategory = 6 ;
	 parameterNumber = 1 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfSecondFixedSurface = 8 ;
	}
#Field capacity
'cap' = {
	 discipline = 2 ;
	 parameterCategory = 3 ;
	 parameterNumber = 12 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Wilting point
'wilt' = {
	 discipline = 2 ;
	 parameterCategory = 0 ;
	 parameterNumber = 26 ;
	 typeOfFirstFixedSurface = 106 ;
	 typeOfSecondFixedSurface = 106 ;
	 scaledValueOfFirstFixedSurface = 0 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	 scaledValueOfSecondFixedSurface = 2 ;
	 scaleFactorOfSecondFixedSurface = 1 ;
	}
#Total Precipitation
'tp' = {
	 discipline = 0 ;
	 parameterCategory = 1 ;
	 parameterNumber = 52 ;
	 typeOfFirstFixedSurface = 1 ;
	 typeOfStatisticalProcessing = 1 ;
	}
#U-component surface stokes drift
'ust' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 21 ;
	}
#V-component surface stokes drift
'vst' = {
	 discipline = 10 ;
	 parameterCategory = 0 ;
	 parameterNumber = 22 ;
	}
#100 metre U wind component
'100u' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 2 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
	}
#100 metre V wind component
'100v' = {
	 discipline = 0 ;
	 parameterCategory = 2 ;
	 parameterNumber = 3 ;
	 typeOfFirstFixedSurface = 103 ;
	 scaledValueOfFirstFixedSurface = 100 ;
	 scaleFactorOfFirstFixedSurface = 0 ;
}
