# Automatically generated by ./create_defs.grib1.pl, do not edit
#Total precipitation of at least 1 mm
'Total precipitation of at least 1 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 60 ;
	}
#Total precipitation of at least 5 mm
'Total precipitation of at least 5 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 61 ;
	}
#Total precipitation of at least 10 mm
'Total precipitation of at least 10 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 62 ;
	}
#Total precipitation of at least 20 mm
'Total precipitation of at least 20 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 63 ;
	}
#Total precipitation of at least 40 mm
'Total precipitation of at least 40 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 82 ;
	}
#Total precipitation of at least 60 mm
'Total precipitation of at least 60 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 83 ;
	}
#Total precipitation of at least 80 mm
'Total precipitation of at least 80 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 84 ;
	}
#Total precipitation of at least 100 mm
'Total precipitation of at least 100 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 85 ;
	}
#Total precipitation of at least 150 mm
'Total precipitation of at least 150 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 86 ;
	}
#Total precipitation of at least 200 mm
'Total precipitation of at least 200 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 87 ;
	}
#Total precipitation of at least 300 mm
'Total precipitation of at least 300 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 88 ;
	}
#Stream function
'Stream function' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 1 ;
	}
#Velocity potential
'Velocity potential' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 2 ;
	}
#Potential temperature
'Potential temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 3 ;
	}
#Equivalent potential temperature
'Equivalent potential temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 4 ;
	}
#Saturated equivalent potential temperature
'Saturated equivalent potential temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 5 ;
	}
#Soil sand fraction
'Soil sand fraction' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 6 ;
	}
#Soil clay fraction
'Soil clay fraction' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 7 ;
	}
#Surface runoff
'Surface runoff' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 8 ;
	}
#Sub-surface runoff
'Sub-surface runoff' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 9 ;
	}
#Wind speed
'Wind speed' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 10 ;
	}
#U component of divergent wind
'U component of divergent wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 11 ;
	}
#V component of divergent wind
'V component of divergent wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 12 ;
	}
#U component of rotational wind
'U component of rotational wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 13 ;
	}
#V component of rotational wind
'V component of rotational wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 14 ;
	}
#UV visible albedo for direct radiation (climatological)
'UV visible albedo for direct radiation (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 15 ;
	}
#UV visible albedo for diffuse radiation (climatological)
'UV visible albedo for diffuse radiation (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 16 ;
	}
#Near IR albedo for direct radiation (climatological)
'Near IR albedo for direct radiation (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 17 ;
	}
#Near IR albedo for diffuse radiation (climatological)
'Near IR albedo for diffuse radiation (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 18 ;
	}
#Clear sky surface UV
'Clear sky surface UV' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 19 ;
	}
#Surface photosynthetically active radiation, clear sky
'Surface photosynthetically active radiation, clear sky' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 20 ;
	}
#Unbalanced component of temperature
'Unbalanced component of temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 21 ;
	}
#Unbalanced component of logarithm of surface pressure
'Unbalanced component of logarithm of surface pressure' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 22 ;
	}
#Unbalanced component of divergence
'Unbalanced component of divergence' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 23 ;
	}
#Reserved for future unbalanced components
'Reserved for future unbalanced components' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 24 ;
	}
#Reserved for future unbalanced components
'Reserved for future unbalanced components' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 25 ;
	}
#Lake cover
'Lake cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 26 ;
	}
#Low vegetation cover
'Low vegetation cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 27 ;
	}
#High vegetation cover
'High vegetation cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 28 ;
	}
#Type of low vegetation
'Type of low vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 29 ;
	}
#Type of high vegetation
'Type of high vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 30 ;
	}
#Sea ice area fraction
'Sea ice area fraction' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 31 ;
	}
#Snow albedo
'Snow albedo' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 32 ;
	}
#Snow density
'Snow density' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 33 ;
	}
#Sea surface temperature
'Sea surface temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 34 ;
	}
#Ice temperature layer 1
'Ice temperature layer 1' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 35 ;
	}
#Ice temperature layer 2
'Ice temperature layer 2' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 36 ;
	}
#Ice temperature layer 3
'Ice temperature layer 3' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 37 ;
	}
#Ice temperature layer 4
'Ice temperature layer 4' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 38 ;
	}
#Volumetric soil water layer 1
'Volumetric soil water layer 1' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 39 ;
	}
#Volumetric soil water layer 2
'Volumetric soil water layer 2' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 40 ;
	}
#Volumetric soil water layer 3
'Volumetric soil water layer 3' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 41 ;
	}
#Volumetric soil water layer 4
'Volumetric soil water layer 4' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 42 ;
	}
#Soil type
'Soil type' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 43 ;
	}
#Snow evaporation
'Snow evaporation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 44 ;
	}
#Snowmelt
'Snowmelt' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 45 ;
	}
#Solar duration
'Solar duration' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 46 ;
	}
#Surface direct normal short-wave (solar) radiation
'Surface direct normal short-wave (solar) radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 47 ;
	}
#Time-integrated magnitude of turbulent surface stress
'Time-integrated magnitude of turbulent surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 48 ;
	}
#Maximum 10 metre wind gust since previous post-processing
'Maximum 10 metre wind gust since previous post-processing' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 49 ;
	}
#Large-scale precipitation fraction
'Large-scale precipitation fraction' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 50 ;
	}
#Maximum temperature at 2 metres in the last 24 hours
'Maximum temperature at 2 metres in the last 24 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 51 ;
	}
#Minimum temperature at 2 metres in the last 24 hours
'Minimum temperature at 2 metres in the last 24 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 52 ;
	}
#Montgomery potential
'Montgomery potential' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 53 ;
	}
#Pressure
'Pressure' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 54 ;
	}
#Mean temperature at 2 metres in the last 24 hours
'Mean temperature at 2 metres in the last 24 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 55 ;
	}
#Mean 2 metre dewpoint temperature in the last 24 hours
'Mean 2 metre dewpoint temperature in the last 24 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 56 ;
	}
#Surface downward UV radiation
'Surface downward UV radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 57 ;
	}
#Photosynthetically active radiation at the surface
'Photosynthetically active radiation at the surface' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 58 ;
	}
#Convective available potential energy
'Convective available potential energy' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 59 ;
	}
#Potential vorticity
'Potential vorticity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 60 ;
	}
#Observation count
'Observation count' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 62 ;
	}
#Start time for skin temperature difference
'Start time for skin temperature difference' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 63 ;
	}
#Finish time for skin temperature difference
'Finish time for skin temperature difference' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 64 ;
	}
#Skin temperature difference
'Skin temperature difference' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 65 ;
	}
#Leaf area index, low vegetation
'Leaf area index, low vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 66 ;
	}
#Leaf area index, high vegetation
'Leaf area index, high vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 67 ;
	}
#Minimum stomatal resistance, low vegetation
'Minimum stomatal resistance, low vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 68 ;
	}
#Minimum stomatal resistance, high vegetation
'Minimum stomatal resistance, high vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 69 ;
	}
#Biome cover, low vegetation
'Biome cover, low vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 70 ;
	}
#Biome cover, high vegetation
'Biome cover, high vegetation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 71 ;
	}
#Instantaneous surface solar radiation downwards
'Instantaneous surface solar radiation downwards' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 72 ;
	}
#Instantaneous surface thermal radiation downwards
'Instantaneous surface thermal radiation downwards' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 73 ;
	}
#Standard deviation of filtered subgrid orography (climatological)
'Standard deviation of filtered subgrid orography (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 74 ;
	}
#Specific rain water content
'Specific rain water content' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 75 ;
	}
#Specific snow water content
'Specific snow water content' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 76 ;
	}
#Eta-coordinate vertical velocity
'Eta-coordinate vertical velocity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 77 ;
	}
#Total column cloud liquid water
'Total column cloud liquid water' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 78 ;
	}
#Total column cloud ice water
'Total column cloud ice water' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 79 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 80 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 81 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 82 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 83 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 84 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 85 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 86 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 87 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 88 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 89 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 90 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 91 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 92 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 93 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 94 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 95 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 96 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 97 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 98 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 99 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 100 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 101 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 102 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 103 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 104 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 105 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 106 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 107 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 108 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 109 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 110 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 111 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 112 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 113 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 114 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 115 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 116 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 117 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 118 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 119 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 120 ;
	}
#Maximum temperature at 2 metres in the last 6 hours
'Maximum temperature at 2 metres in the last 6 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 121 ;
	}
#Minimum temperature at 2 metres in the last 6 hours
'Minimum temperature at 2 metres in the last 6 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 122 ;
	}
#Maximum 10 metre wind gust in the last 6 hours
'Maximum 10 metre wind gust in the last 6 hours' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 123 ;
	}
#Surface emissivity
'Surface emissivity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 124 ;
	}
#Vertically integrated total energy
'Vertically integrated total energy' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 125 ;
	}
#Generic parameter for sensitive area prediction
'Generic parameter for sensitive area prediction' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 126 ;
	}
#Atmospheric tide
'Atmospheric tide' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 127 ;
	}
#Atmospheric tide
'Atmospheric tide' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 127 ;
	}
#Budget values
'Budget values' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 128 ;
	}
#Budget values
'Budget values' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 128 ;
	}
#Geopotential
'Geopotential' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 129 ;
	}
#Geopotential
'Geopotential' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 129 ;
	}
#Geopotential
'Geopotential' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 129 ;
	}
#Geopotential
'Geopotential' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 129 ;
	}
#Geopotential
'Geopotential' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 129 ;
	}
#Temperature
'Temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 130 ;
	}
#Temperature
'Temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 130 ;
	}
#Temperature
'Temperature' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 130 ;
	}
#Temperature
'Temperature' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 130 ;
	}
#Temperature
'Temperature' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 130 ;
	}
#U component of wind
'U component of wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 131 ;
	}
#U component of wind
'U component of wind' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 131 ;
	}
#U component of wind
'U component of wind' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 131 ;
	}
#U component of wind
'U component of wind' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 131 ;
	}
#U component of wind
'U component of wind' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 131 ;
	}
#V component of wind
'V component of wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 132 ;
	}
#V component of wind
'V component of wind' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 132 ;
	}
#V component of wind
'V component of wind' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 132 ;
	}
#V component of wind
'V component of wind' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 132 ;
	}
#V component of wind
'V component of wind' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 132 ;
	}
#Specific humidity
'Specific humidity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 133 ;
	}
#Specific humidity
'Specific humidity' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 133 ;
	}
#Specific humidity
'Specific humidity' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 133 ;
	}
#Specific humidity
'Specific humidity' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 133 ;
	}
#Specific humidity
'Specific humidity' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 133 ;
	}
#Surface pressure
'Surface pressure' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 134 ;
	}
#Surface pressure
'Surface pressure' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 134 ;
	}
#Surface pressure
'Surface pressure' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 52 ;
	}
#Surface pressure
'Surface pressure' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 134 ;
	}
#Surface pressure
'Surface pressure' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 134 ;
	}
#Vertical velocity
'Vertical velocity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 135 ;
	}
#Vertical velocity
'Vertical velocity' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 135 ;
	}
#Total column water
'Total column water' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 136 ;
	}
#Total column water
'Total column water' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 136 ;
	}
#Total column vertically-integrated water vapour
'Total column vertically-integrated water vapour' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 137 ;
	}
#Total column vertically-integrated water vapour
'Total column vertically-integrated water vapour' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 137 ;
	}
#Vorticity (relative)
'Vorticity (relative)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 138 ;
	}
#Vorticity (relative)
'Vorticity (relative)' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 138 ;
	}
#Vorticity (relative)
'Vorticity (relative)' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 138 ;
	}
#Vorticity (relative)
'Vorticity (relative)' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 138 ;
	}
#Vorticity (relative)
'Vorticity (relative)' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 138 ;
	}
#Soil temperature level 1
'Soil temperature level 1' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 139 ;
	}
#Soil temperature level 1
'Soil temperature level 1' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 139 ;
	}
#Soil temperature level 1
'Soil temperature level 1' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 139 ;
	}
#Soil temperature level 1
'Soil temperature level 1' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 139 ;
	}
#Soil wetness level 1
'Soil wetness level 1' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 140 ;
	}
#Soil wetness level 1
'Soil wetness level 1' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 140 ;
	}
#Snow depth
'Snow depth' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 141 ;
	}
#Snow depth
'Snow depth' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 141 ;
	}
#Snow depth
'Snow depth' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 141 ;
	}
#Large-scale precipitation
'Large-scale precipitation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 142 ;
	}
#Large-scale precipitation
'Large-scale precipitation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 142 ;
	}
#Large-scale precipitation
'Large-scale precipitation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 142 ;
	}
#Convective precipitation
'Convective precipitation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 143 ;
	}
#Convective precipitation
'Convective precipitation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 143 ;
	}
#Convective precipitation
'Convective precipitation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall
'Snowfall' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 144 ;
	}
#Snowfall
'Snowfall' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 144 ;
	}
#Time-integrated boundary layer dissipation
'Time-integrated boundary layer dissipation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 145 ;
	}
#Time-integrated boundary layer dissipation
'Time-integrated boundary layer dissipation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 145 ;
	}
#Time-integrated surface sensible heat net flux
'Time-integrated surface sensible heat net flux' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 146 ;
	}
#Time-integrated surface sensible heat net flux
'Time-integrated surface sensible heat net flux' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 146 ;
	}
#Time-integrated surface sensible heat net flux
'Time-integrated surface sensible heat net flux' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 146 ;
	}
#Time-integrated surface sensible heat net flux
'Time-integrated surface sensible heat net flux' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 146 ;
	}
#Time-integrated surface sensible heat net flux
'Time-integrated surface sensible heat net flux' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 146 ;
	}
#Time-integrated surface latent heat net flux
'Time-integrated surface latent heat net flux' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 147 ;
	}
#Time-integrated surface latent heat net flux
'Time-integrated surface latent heat net flux' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 147 ;
	}
#Time-integrated surface latent heat net flux
'Time-integrated surface latent heat net flux' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 147 ;
	}
#Time-integrated surface latent heat net flux
'Time-integrated surface latent heat net flux' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 147 ;
	}
#Time-integrated surface latent heat net flux
'Time-integrated surface latent heat net flux' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 147 ;
	}
#Charnock
'Charnock' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 148 ;
	}
#Surface net radiation (SW and LW)
'Surface net radiation (SW and LW)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 149 ;
	}
#Top net radiation (SW and LW)
'Top net radiation (SW and LW)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 150 ;
	}
#Mean sea level pressure
'Mean sea level pressure' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 151 ;
	}
#Mean sea level pressure
'Mean sea level pressure' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 151 ;
	}
#Mean sea level pressure
'Mean sea level pressure' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 151 ;
	}
#Mean sea level pressure
'Mean sea level pressure' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 151 ;
	}
#Mean sea level pressure
'Mean sea level pressure' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 151 ;
	}
#Logarithm of surface pressure
'Logarithm of surface pressure' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 152 ;
	}
#Logarithm of surface pressure
'Logarithm of surface pressure' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 152 ;
	}
#Short-wave heating rate
'Short-wave heating rate' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 153 ;
	}
#Long-wave heating rate
'Long-wave heating rate' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 154 ;
	}
#Divergence
'Divergence' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 155 ;
	}
#Divergence
'Divergence' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 155 ;
	}
#Divergence
'Divergence' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 155 ;
	}
#Divergence
'Divergence' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 155 ;
	}
#Divergence
'Divergence' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 155 ;
	}
#Geopotential height
'Geopotential height' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 156 ;
	}
#Relative humidity
'Relative humidity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 157 ;
	}
#Relative humidity
'Relative humidity' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 157 ;
	}
#Relative humidity
'Relative humidity' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 157 ;
	}
#Tendency of surface pressure
'Tendency of surface pressure' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 158 ;
	}
#Tendency of surface pressure
'Tendency of surface pressure' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 158 ;
	}
#Boundary layer height
'Boundary layer height' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 159 ;
	}
#Standard deviation of sub-gridscale orography
'Standard deviation of sub-gridscale orography' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 160 ;
	}
#Anisotropy of sub-gridscale orography
'Anisotropy of sub-gridscale orography' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 161 ;
	}
#Angle of sub-gridscale orography
'Angle of sub-gridscale orography' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 162 ;
	}
#Slope of sub-gridscale orography
'Slope of sub-gridscale orography' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 163 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 164 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 164 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 164 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 164 ;
	}
#Total cloud cover
'Total cloud cover' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 164 ;
	}
#10 metre U wind component
'10 metre U wind component' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre U wind component
'10 metre U wind component' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre U wind component
'10 metre U wind component' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre U wind component
'10 metre U wind component' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre V wind component
'10 metre V wind component' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 166 ;
	}
#10 metre V wind component
'10 metre V wind component' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 166 ;
	}
#10 metre V wind component
'10 metre V wind component' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 166 ;
	}
#10 metre V wind component
'10 metre V wind component' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 166 ;
	}
#2 metre temperature
'2 metre temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 167 ;
	}
#2 metre temperature
'2 metre temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 167 ;
	}
#2 metre temperature
'2 metre temperature' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 167 ;
	}
#2 metre temperature
'2 metre temperature' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 167 ;
	}
#2 metre dewpoint temperature
'2 metre dewpoint temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 168 ;
	}
#2 metre dewpoint temperature
'2 metre dewpoint temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 168 ;
	}
#2 metre dewpoint temperature
'2 metre dewpoint temperature' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 168 ;
	}
#2 metre dewpoint temperature
'2 metre dewpoint temperature' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 168 ;
	}
#Surface short-wave (solar) radiation downwards
'Surface short-wave (solar) radiation downwards' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 169 ;
	}
#Surface short-wave (solar) radiation downwards
'Surface short-wave (solar) radiation downwards' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 169 ;
	}
#Soil temperature level 2
'Soil temperature level 2' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 170 ;
	}
#Soil temperature level 2
'Soil temperature level 2' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 170 ;
	}
#Soil wetness level 2
'Soil wetness level 2' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 171 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 172 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 172 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 172 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 172 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 172 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 172 ;
	}
#Land-sea mask
'Land-sea mask' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 172 ;
	}
#Surface roughness (climatological)
'Surface roughness (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 173 ;
	}
#Surface roughness (climatological)
'Surface roughness (climatological)' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 173 ;
	}
#Albedo (climatological)
'Albedo (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 174 ;
	}
#Albedo (climatological)
'Albedo (climatological)' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 174 ;
	}
#Albedo (climatological)
'Albedo (climatological)' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 174 ;
	}
#Surface long-wave (thermal) radiation downwards
'Surface long-wave (thermal) radiation downwards' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 175 ;
	}
#Surface long-wave (thermal) radiation downwards
'Surface long-wave (thermal) radiation downwards' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 175 ;
	}
#Surface net short-wave (solar) radiation
'Surface net short-wave (solar) radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net short-wave (solar) radiation
'Surface net short-wave (solar) radiation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net short-wave (solar) radiation
'Surface net short-wave (solar) radiation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net short-wave (solar) radiation
'Surface net short-wave (solar) radiation' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net long-wave (thermal) radiation
'Surface net long-wave (thermal) radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 177 ;
	}
#Surface net long-wave (thermal) radiation
'Surface net long-wave (thermal) radiation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 177 ;
	}
#Surface net long-wave (thermal) radiation
'Surface net long-wave (thermal) radiation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 177 ;
	}
#Surface net long-wave (thermal) radiation
'Surface net long-wave (thermal) radiation' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 177 ;
	}
#Top net short-wave (solar) radiation
'Top net short-wave (solar) radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 178 ;
	}
#Top net short-wave (solar) radiation
'Top net short-wave (solar) radiation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 178 ;
	}
#Top net short-wave (solar) radiation
'Top net short-wave (solar) radiation' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 178 ;
	}
#Top net long-wave (thermal) radiation
'Top net long-wave (thermal) radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 179 ;
	}
#Top net long-wave (thermal) radiation
'Top net long-wave (thermal) radiation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 179 ;
	}
#Top net long-wave (thermal) radiation
'Top net long-wave (thermal) radiation' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 179 ;
	}
#Time-integrated eastward turbulent surface stress
'Time-integrated eastward turbulent surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 180 ;
	}
#Time-integrated eastward turbulent surface stress
'Time-integrated eastward turbulent surface stress' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 180 ;
	}
#Time-integrated eastward turbulent surface stress
'Time-integrated eastward turbulent surface stress' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 180 ;
	}
#Time-integrated northward turbulent surface stress
'Time-integrated northward turbulent surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 181 ;
	}
#Time-integrated northward turbulent surface stress
'Time-integrated northward turbulent surface stress' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 181 ;
	}
#Time-integrated northward turbulent surface stress
'Time-integrated northward turbulent surface stress' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 181 ;
	}
#Evaporation
'Evaporation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 182 ;
	}
#Evaporation
'Evaporation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 182 ;
	}
#Evaporation
'Evaporation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 182 ;
	}
#Evaporation
'Evaporation' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 182 ;
	}
#Soil temperature level 3
'Soil temperature level 3' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 183 ;
	}
#Soil temperature level 3
'Soil temperature level 3' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 183 ;
	}
#Soil wetness level 3
'Soil wetness level 3' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 184 ;
	}
#Soil wetness level 3
'Soil wetness level 3' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 184 ;
	}
#Convective cloud cover
'Convective cloud cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 185 ;
	}
#Convective cloud cover
'Convective cloud cover' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 185 ;
	}
#Convective cloud cover
'Convective cloud cover' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 185 ;
	}
#Low cloud cover
'Low cloud cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 186 ;
	}
#Low cloud cover
'Low cloud cover' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 186 ;
	}
#Medium cloud cover
'Medium cloud cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 187 ;
	}
#Medium cloud cover
'Medium cloud cover' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 187 ;
	}
#High cloud cover
'High cloud cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 188 ;
	}
#High cloud cover
'High cloud cover' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 188 ;
	}
#Sunshine duration
'Sunshine duration' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 189 ;
	}
#East-West component of sub-gridscale orographic variance
'East-West component of sub-gridscale orographic variance' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 190 ;
	}
#East-West component of sub-gridscale orographic variance
'East-West component of sub-gridscale orographic variance' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 190 ;
	}
#North-South component of sub-gridscale orographic variance
'North-South component of sub-gridscale orographic variance' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 191 ;
	}
#North-South component of sub-gridscale orographic variance
'North-South component of sub-gridscale orographic variance' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 191 ;
	}
#North-West/South-East component of sub-gridscale orographic variance
'North-West/South-East component of sub-gridscale orographic variance' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 192 ;
	}
#North-West/South-East component of sub-gridscale orographic variance
'North-West/South-East component of sub-gridscale orographic variance' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 192 ;
	}
#North-East/South-West component of sub-gridscale orographic variance
'North-East/South-West component of sub-gridscale orographic variance' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 193 ;
	}
#North-East/South-West component of sub-gridscale orographic variance
'North-East/South-West component of sub-gridscale orographic variance' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 193 ;
	}
#Brightness temperature
'Brightness temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 194 ;
	}
#Eastward gravity wave surface stress
'Eastward gravity wave surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 195 ;
	}
#Eastward gravity wave surface stress
'Eastward gravity wave surface stress' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 195 ;
	}
#Northward gravity wave surface stress
'Northward gravity wave surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 196 ;
	}
#Northward gravity wave surface stress
'Northward gravity wave surface stress' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation
'Gravity wave dissipation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 197 ;
	}
#Gravity wave dissipation
'Gravity wave dissipation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 197 ;
	}
#Skin reservoir content
'Skin reservoir content' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 198 ;
	}
#Vegetation fraction
'Vegetation fraction' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 199 ;
	}
#Variance of sub-gridscale orography
'Variance of sub-gridscale orography' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 200 ;
	}
#Variance of sub-gridscale orography
'Variance of sub-gridscale orography' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 200 ;
	}
#Maximum temperature at 2 metres since previous post-processing
'Maximum temperature at 2 metres since previous post-processing' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 201 ;
	}
#Maximum temperature at 2 metres since previous post-processing
'Maximum temperature at 2 metres since previous post-processing' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 201 ;
	}
#Maximum temperature at 2 metres since previous post-processing
'Maximum temperature at 2 metres since previous post-processing' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 201 ;
	}
#Minimum temperature at 2 metres since previous post-processing
'Minimum temperature at 2 metres since previous post-processing' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 202 ;
	}
#Minimum temperature at 2 metres since previous post-processing
'Minimum temperature at 2 metres since previous post-processing' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 202 ;
	}
#Minimum temperature at 2 metres since previous post-processing
'Minimum temperature at 2 metres since previous post-processing' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 202 ;
	}
#Ozone mass mixing ratio
'Ozone mass mixing ratio' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 203 ;
	}
#Precipitation analysis weights
'Precipitation analysis weights' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 204 ;
	}
#Precipitation analysis weights
'Precipitation analysis weights' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 204 ;
	}
#Runoff
'Runoff' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 205 ;
	}
#Runoff
'Runoff' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 205 ;
	}
#Total column ozone
'Total column ozone' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 206 ;
	}
#10 metre wind speed
'10 metre wind speed' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 207 ;
	}
#Top net short-wave (solar) radiation, clear sky
'Top net short-wave (solar) radiation, clear sky' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 208 ;
	}
#Top net long-wave (thermal) radiation, clear sky
'Top net long-wave (thermal) radiation, clear sky' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 209 ;
	}
#Surface net short-wave (solar) radiation, clear sky
'Surface net short-wave (solar) radiation, clear sky' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 210 ;
	}
#Surface net long-wave (thermal) radiation, clear sky
'Surface net long-wave (thermal) radiation, clear sky' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 211 ;
	}
#TOA incident short-wave (solar) radiation
'TOA incident short-wave (solar) radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 212 ;
	}
#Vertically integrated moisture divergence
'Vertically integrated moisture divergence' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 213 ;
	}
#Diabatic heating by radiation
'Diabatic heating by radiation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 214 ;
	}
#Diabatic heating by vertical diffusion
'Diabatic heating by vertical diffusion' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 215 ;
	}
#Diabatic heating by cumulus convection
'Diabatic heating by cumulus convection' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 216 ;
	}
#Diabatic heating large-scale condensation
'Diabatic heating large-scale condensation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 217 ;
	}
#Vertical diffusion of zonal wind
'Vertical diffusion of zonal wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 218 ;
	}
#Vertical diffusion of meridional wind
'Vertical diffusion of meridional wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 219 ;
	}
#East-West gravity wave drag tendency
'East-West gravity wave drag tendency' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 220 ;
	}
#North-South gravity wave drag tendency
'North-South gravity wave drag tendency' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 221 ;
	}
#Convective tendency of zonal wind
'Convective tendency of zonal wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 222 ;
	}
#Convective tendency of zonal wind
'Convective tendency of zonal wind' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 222 ;
	}
#Convective tendency of meridional wind
'Convective tendency of meridional wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 223 ;
	}
#Convective tendency of meridional wind
'Convective tendency of meridional wind' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 223 ;
	}
#Vertical diffusion of humidity
'Vertical diffusion of humidity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 224 ;
	}
#Humidity tendency by cumulus convection
'Humidity tendency by cumulus convection' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 225 ;
	}
#Humidity tendency by large-scale condensation
'Humidity tendency by large-scale condensation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 226 ;
	}
#Tendency due to removal of negative humidity
'Tendency due to removal of negative humidity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 227 ;
	}
#Tendency due to removal of negative humidity
'Tendency due to removal of negative humidity' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 227 ;
	}
#Total precipitation
'Total precipitation' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 228 ;
	}
#Total precipitation
'Total precipitation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 228 ;
	}
#Total precipitation
'Total precipitation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 228 ;
	}
#Total precipitation
'Total precipitation' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 228 ;
	}
#Instantaneous eastward turbulent surface stress
'Instantaneous eastward turbulent surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 229 ;
	}
#Instantaneous eastward turbulent surface stress
'Instantaneous eastward turbulent surface stress' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 229 ;
	}
#Instantaneous northward turbulent surface stress
'Instantaneous northward turbulent surface stress' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 230 ;
	}
#Instantaneous northward turbulent surface stress
'Instantaneous northward turbulent surface stress' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 230 ;
	}
#Instantaneous surface sensible heat net flux
'Instantaneous surface sensible heat net flux' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 231 ;
	}
#Instantaneous moisture flux
'Instantaneous moisture flux' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 232 ;
	}
#Instantaneous moisture flux
'Instantaneous moisture flux' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 232 ;
	}
#Apparent surface humidity
'Apparent surface humidity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 233 ;
	}
#Apparent surface humidity
'Apparent surface humidity' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 233 ;
	}
#Logarithm of surface roughness length for heat (climatological)
'Logarithm of surface roughness length for heat (climatological)' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 234 ;
	}
#Logarithm of surface roughness length for heat (climatological)
'Logarithm of surface roughness length for heat (climatological)' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 234 ;
	}
#Skin temperature
'Skin temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 235 ;
	}
#Skin temperature
'Skin temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 235 ;
	}
#Soil temperature level 4
'Soil temperature level 4' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 236 ;
	}
#Soil temperature level 4
'Soil temperature level 4' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 236 ;
	}
#Soil wetness level 4
'Soil wetness level 4' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 237 ;
	}
#Soil wetness level 4
'Soil wetness level 4' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 237 ;
	}
#Temperature of snow layer
'Temperature of snow layer' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 238 ;
	}
#Temperature of snow layer
'Temperature of snow layer' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 238 ;
	}
#Convective snowfall
'Convective snowfall' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 239 ;
	}
#Large-scale snowfall
'Large-scale snowfall' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 240 ;
	}
#Accumulated cloud fraction tendency
'Accumulated cloud fraction tendency' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 241 ;
	}
#Accumulated liquid water tendency
'Accumulated liquid water tendency' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 242 ;
	}
#Forecast albedo
'Forecast albedo' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 243 ;
	}
#Forecast surface roughness
'Forecast surface roughness' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 244 ;
	}
#Forecast surface roughness
'Forecast surface roughness' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 244 ;
	}
#Forecast logarithm of surface roughness for heat
'Forecast logarithm of surface roughness for heat' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 245 ;
	}
#Forecast logarithm of surface roughness for heat
'Forecast logarithm of surface roughness for heat' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 245 ;
	}
#Specific cloud liquid water content
'Specific cloud liquid water content' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 246 ;
	}
#Specific cloud ice water content
'Specific cloud ice water content' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 247 ;
	}
#Fraction of cloud cover
'Fraction of cloud cover' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 248 ;
	}
#Accumulated ice water tendency
'Accumulated ice water tendency' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 249 ;
	}
#Ice age
'Ice age' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 250 ;
	}
#Adiabatic tendency of temperature
'Adiabatic tendency of temperature' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 251 ;
	}
#Adiabatic tendency of humidity
'Adiabatic tendency of humidity' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 252 ;
	}
#Adiabatic tendency of zonal wind
'Adiabatic tendency of zonal wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 253 ;
	}
#Adiabatic tendency of meridional wind
'Adiabatic tendency of meridional wind' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 254 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 128 ;
	 indicatorOfParameter = 255 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 255 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 255 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 255 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 255 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 255 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 255 ;
	}
#Stream function difference
'Stream function difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 1 ;
	}
#Velocity potential difference
'Velocity potential difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 2 ;
	}
#Potential temperature difference
'Potential temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 3 ;
	}
#Equivalent potential temperature difference
'Equivalent potential temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 4 ;
	}
#Saturated equivalent potential temperature difference
'Saturated equivalent potential temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 5 ;
	}
#U component of divergent wind difference
'U component of divergent wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 11 ;
	}
#V component of divergent wind difference
'V component of divergent wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 12 ;
	}
#U component of rotational wind difference
'U component of rotational wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 13 ;
	}
#V component of rotational wind difference
'V component of rotational wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 14 ;
	}
#Unbalanced component of temperature difference
'Unbalanced component of temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 21 ;
	}
#Unbalanced component of logarithm of surface pressure difference
'Unbalanced component of logarithm of surface pressure difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 22 ;
	}
#Unbalanced component of divergence difference
'Unbalanced component of divergence difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 23 ;
	}
#Reserved for future unbalanced components
'Reserved for future unbalanced components' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 24 ;
	}
#Reserved for future unbalanced components
'Reserved for future unbalanced components' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 25 ;
	}
#Lake cover difference
'Lake cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 26 ;
	}
#Low vegetation cover difference
'Low vegetation cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 27 ;
	}
#High vegetation cover difference
'High vegetation cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 28 ;
	}
#Type of low vegetation difference
'Type of low vegetation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 29 ;
	}
#Type of high vegetation difference
'Type of high vegetation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 30 ;
	}
#Sea-ice cover difference
'Sea-ice cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 31 ;
	}
#Snow albedo difference
'Snow albedo difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 32 ;
	}
#Snow density difference
'Snow density difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 33 ;
	}
#Sea surface temperature difference
'Sea surface temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 34 ;
	}
#Ice surface temperature layer 1 difference
'Ice surface temperature layer 1 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 35 ;
	}
#Ice surface temperature layer 2 difference
'Ice surface temperature layer 2 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 36 ;
	}
#Ice surface temperature layer 3 difference
'Ice surface temperature layer 3 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 37 ;
	}
#Ice surface temperature layer 4 difference
'Ice surface temperature layer 4 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 38 ;
	}
#Volumetric soil water layer 1 difference
'Volumetric soil water layer 1 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 39 ;
	}
#Volumetric soil water layer 2 difference
'Volumetric soil water layer 2 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 40 ;
	}
#Volumetric soil water layer 3 difference
'Volumetric soil water layer 3 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 41 ;
	}
#Volumetric soil water layer 4 difference
'Volumetric soil water layer 4 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 42 ;
	}
#Soil type difference
'Soil type difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 43 ;
	}
#Snow evaporation difference
'Snow evaporation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 44 ;
	}
#Snowmelt difference
'Snowmelt difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 45 ;
	}
#Solar duration difference
'Solar duration difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 46 ;
	}
#Direct solar radiation difference
'Direct solar radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 47 ;
	}
#Magnitude of turbulent surface stress difference
'Magnitude of turbulent surface stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 48 ;
	}
#10 metre wind gust difference
'10 metre wind gust difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 49 ;
	}
#Large-scale precipitation fraction difference
'Large-scale precipitation fraction difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 50 ;
	}
#Maximum 2 metre temperature difference
'Maximum 2 metre temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 51 ;
	}
#Minimum 2 metre temperature difference
'Minimum 2 metre temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 52 ;
	}
#Montgomery potential difference
'Montgomery potential difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 53 ;
	}
#Pressure difference
'Pressure difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 54 ;
	}
#Mean 2 metre temperature in the last 24 hours difference
'Mean 2 metre temperature in the last 24 hours difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 55 ;
	}
#Mean 2 metre dewpoint temperature in the last 24 hours difference
'Mean 2 metre dewpoint temperature in the last 24 hours difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 56 ;
	}
#Downward UV radiation at the surface difference
'Downward UV radiation at the surface difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 57 ;
	}
#Photosynthetically active radiation at the surface difference
'Photosynthetically active radiation at the surface difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 58 ;
	}
#Convective available potential energy difference
'Convective available potential energy difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 59 ;
	}
#Potential vorticity difference
'Potential vorticity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 60 ;
	}
#Total precipitation from observations difference
'Total precipitation from observations difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 61 ;
	}
#Observation count difference
'Observation count difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 62 ;
	}
#Start time for skin temperature difference
'Start time for skin temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 63 ;
	}
#Finish time for skin temperature difference
'Finish time for skin temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 64 ;
	}
#Skin temperature difference
'Skin temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 65 ;
	}
#Leaf area index, low vegetation
'Leaf area index, low vegetation' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 66 ;
	}
#Leaf area index, high vegetation
'Leaf area index, high vegetation' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 67 ;
	}
#Biome cover, low vegetation
'Biome cover, low vegetation' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 70 ;
	}
#Biome cover, high vegetation
'Biome cover, high vegetation' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 71 ;
	}
#Total column liquid water
'Total column liquid water' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 78 ;
	}
#Total column ice water
'Total column ice water' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 79 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 80 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 81 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 82 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 83 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 84 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 85 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 86 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 87 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 88 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 89 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 90 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 91 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 92 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 93 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 94 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 95 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 96 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 97 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 98 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 99 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 100 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 101 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 102 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 103 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 104 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 105 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 106 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 107 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 108 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 109 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 110 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 111 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 112 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 113 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 114 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 115 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 116 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 117 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 118 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 119 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 120 ;
	}
#Maximum temperature at 2 metres difference
'Maximum temperature at 2 metres difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 121 ;
	}
#Minimum temperature at 2 metres difference
'Minimum temperature at 2 metres difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 122 ;
	}
#10 metre wind gust in the last 6 hours difference
'10 metre wind gust in the last 6 hours difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 123 ;
	}
#Vertically integrated total energy
'Vertically integrated total energy' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 125 ;
	}
#Generic parameter for sensitive area prediction
'Generic parameter for sensitive area prediction' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 126 ;
	}
#Atmospheric tide difference
'Atmospheric tide difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 127 ;
	}
#Budget values difference
'Budget values difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 128 ;
	}
#Geopotential difference
'Geopotential difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 129 ;
	}
#Temperature difference
'Temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 130 ;
	}
#U component of wind difference
'U component of wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 131 ;
	}
#V component of wind difference
'V component of wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 132 ;
	}
#Specific humidity difference
'Specific humidity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 133 ;
	}
#Surface pressure difference
'Surface pressure difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 134 ;
	}
#Vertical velocity (pressure) difference
'Vertical velocity (pressure) difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 135 ;
	}
#Total column water difference
'Total column water difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 136 ;
	}
#Total column water vapour difference
'Total column water vapour difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 137 ;
	}
#Vorticity (relative) difference
'Vorticity (relative) difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 138 ;
	}
#Soil temperature level 1 difference
'Soil temperature level 1 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 139 ;
	}
#Soil wetness level 1 difference
'Soil wetness level 1 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 140 ;
	}
#Snow depth difference
'Snow depth difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 141 ;
	}
#Stratiform precipitation (Large-scale precipitation) difference
'Stratiform precipitation (Large-scale precipitation) difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 142 ;
	}
#Convective precipitation difference
'Convective precipitation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall (convective + stratiform) difference
'Snowfall (convective + stratiform) difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 144 ;
	}
#Boundary layer dissipation difference
'Boundary layer dissipation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 145 ;
	}
#Surface sensible heat flux difference
'Surface sensible heat flux difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 146 ;
	}
#Surface latent heat flux difference
'Surface latent heat flux difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 147 ;
	}
#Charnock difference
'Charnock difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 148 ;
	}
#Surface net radiation difference
'Surface net radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 149 ;
	}
#Top net radiation difference
'Top net radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 150 ;
	}
#Mean sea level pressure difference
'Mean sea level pressure difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 151 ;
	}
#Logarithm of surface pressure difference
'Logarithm of surface pressure difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 152 ;
	}
#Short-wave heating rate difference
'Short-wave heating rate difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 153 ;
	}
#Long-wave heating rate difference
'Long-wave heating rate difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 154 ;
	}
#Divergence difference
'Divergence difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 155 ;
	}
#Height difference
'Height difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 156 ;
	}
#Relative humidity difference
'Relative humidity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 157 ;
	}
#Tendency of surface pressure difference
'Tendency of surface pressure difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 158 ;
	}
#Boundary layer height difference
'Boundary layer height difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 159 ;
	}
#Standard deviation of orography difference
'Standard deviation of orography difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 160 ;
	}
#Anisotropy of sub-gridscale orography difference
'Anisotropy of sub-gridscale orography difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 161 ;
	}
#Angle of sub-gridscale orography difference
'Angle of sub-gridscale orography difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 162 ;
	}
#Slope of sub-gridscale orography difference
'Slope of sub-gridscale orography difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 163 ;
	}
#Total cloud cover difference
'Total cloud cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 164 ;
	}
#10 metre U wind component difference
'10 metre U wind component difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre V wind component difference
'10 metre V wind component difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 166 ;
	}
#2 metre temperature difference
'2 metre temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 167 ;
	}
#Surface solar radiation downwards difference
'Surface solar radiation downwards difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 169 ;
	}
#Soil temperature level 2 difference
'Soil temperature level 2 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 170 ;
	}
#Soil wetness level 2 difference
'Soil wetness level 2 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 171 ;
	}
#Land-sea mask difference
'Land-sea mask difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 172 ;
	}
#Surface roughness difference
'Surface roughness difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 173 ;
	}
#Albedo difference
'Albedo difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 174 ;
	}
#Surface thermal radiation downwards difference
'Surface thermal radiation downwards difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 175 ;
	}
#Surface net solar radiation difference
'Surface net solar radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net thermal radiation difference
'Surface net thermal radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 177 ;
	}
#Top net solar radiation difference
'Top net solar radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 178 ;
	}
#Top net thermal radiation difference
'Top net thermal radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 179 ;
	}
#East-West surface stress difference
'East-West surface stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 180 ;
	}
#North-South surface stress difference
'North-South surface stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 181 ;
	}
#Evaporation difference
'Evaporation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 182 ;
	}
#Soil temperature level 3 difference
'Soil temperature level 3 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 183 ;
	}
#Soil wetness level 3 difference
'Soil wetness level 3 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 184 ;
	}
#Convective cloud cover difference
'Convective cloud cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 185 ;
	}
#Low cloud cover difference
'Low cloud cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 186 ;
	}
#Medium cloud cover difference
'Medium cloud cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 187 ;
	}
#High cloud cover difference
'High cloud cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 188 ;
	}
#Sunshine duration difference
'Sunshine duration difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 189 ;
	}
#East-West component of sub-gridscale orographic variance difference
'East-West component of sub-gridscale orographic variance difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 190 ;
	}
#North-South component of sub-gridscale orographic variance difference
'North-South component of sub-gridscale orographic variance difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 191 ;
	}
#North-West/South-East component of sub-gridscale orographic variance difference
'North-West/South-East component of sub-gridscale orographic variance difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 192 ;
	}
#North-East/South-West component of sub-gridscale orographic variance difference
'North-East/South-West component of sub-gridscale orographic variance difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 193 ;
	}
#Brightness temperature difference
'Brightness temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 194 ;
	}
#Longitudinal component of gravity wave stress difference
'Longitudinal component of gravity wave stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 195 ;
	}
#Meridional component of gravity wave stress difference
'Meridional component of gravity wave stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation difference
'Gravity wave dissipation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 197 ;
	}
#Skin reservoir content difference
'Skin reservoir content difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 198 ;
	}
#Vegetation fraction difference
'Vegetation fraction difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 199 ;
	}
#Variance of sub-gridscale orography difference
'Variance of sub-gridscale orography difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 200 ;
	}
#Maximum temperature at 2 metres since previous post-processing difference
'Maximum temperature at 2 metres since previous post-processing difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 201 ;
	}
#Minimum temperature at 2 metres since previous post-processing difference
'Minimum temperature at 2 metres since previous post-processing difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 202 ;
	}
#Ozone mass mixing ratio difference
'Ozone mass mixing ratio difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 203 ;
	}
#Precipitation analysis weights difference
'Precipitation analysis weights difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 204 ;
	}
#Runoff difference
'Runoff difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 205 ;
	}
#Total column ozone difference
'Total column ozone difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 206 ;
	}
#10 metre wind speed difference
'10 metre wind speed difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 207 ;
	}
#Top net solar radiation, clear sky difference
'Top net solar radiation, clear sky difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 208 ;
	}
#Top net thermal radiation, clear sky difference
'Top net thermal radiation, clear sky difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 209 ;
	}
#Surface net solar radiation, clear sky difference
'Surface net solar radiation, clear sky difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 210 ;
	}
#Surface net thermal radiation, clear sky difference
'Surface net thermal radiation, clear sky difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 211 ;
	}
#TOA incident solar radiation difference
'TOA incident solar radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 212 ;
	}
#Diabatic heating by radiation difference
'Diabatic heating by radiation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 214 ;
	}
#Diabatic heating by vertical diffusion difference
'Diabatic heating by vertical diffusion difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 215 ;
	}
#Diabatic heating by cumulus convection difference
'Diabatic heating by cumulus convection difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 216 ;
	}
#Diabatic heating large-scale condensation difference
'Diabatic heating large-scale condensation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 217 ;
	}
#Vertical diffusion of zonal wind difference
'Vertical diffusion of zonal wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 218 ;
	}
#Vertical diffusion of meridional wind difference
'Vertical diffusion of meridional wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 219 ;
	}
#East-West gravity wave drag tendency difference
'East-West gravity wave drag tendency difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 220 ;
	}
#North-South gravity wave drag tendency difference
'North-South gravity wave drag tendency difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 221 ;
	}
#Convective tendency of zonal wind difference
'Convective tendency of zonal wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 222 ;
	}
#Convective tendency of meridional wind difference
'Convective tendency of meridional wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 223 ;
	}
#Vertical diffusion of humidity difference
'Vertical diffusion of humidity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 224 ;
	}
#Humidity tendency by cumulus convection difference
'Humidity tendency by cumulus convection difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 225 ;
	}
#Humidity tendency by large-scale condensation difference
'Humidity tendency by large-scale condensation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 226 ;
	}
#Change from removal of negative humidity difference
'Change from removal of negative humidity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 227 ;
	}
#Total precipitation difference
'Total precipitation difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 228 ;
	}
#Instantaneous X surface stress difference
'Instantaneous X surface stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 229 ;
	}
#Instantaneous Y surface stress difference
'Instantaneous Y surface stress difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 230 ;
	}
#Instantaneous surface heat flux difference
'Instantaneous surface heat flux difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 231 ;
	}
#Instantaneous moisture flux difference
'Instantaneous moisture flux difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 232 ;
	}
#Apparent surface humidity difference
'Apparent surface humidity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 233 ;
	}
#Logarithm of surface roughness length for heat difference
'Logarithm of surface roughness length for heat difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 234 ;
	}
#Skin temperature difference
'Skin temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 235 ;
	}
#Soil temperature level 4 difference
'Soil temperature level 4 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 236 ;
	}
#Soil wetness level 4 difference
'Soil wetness level 4 difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 237 ;
	}
#Temperature of snow layer difference
'Temperature of snow layer difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 238 ;
	}
#Convective snowfall difference
'Convective snowfall difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 239 ;
	}
#Large scale snowfall difference
'Large scale snowfall difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 240 ;
	}
#Accumulated cloud fraction tendency difference
'Accumulated cloud fraction tendency difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 241 ;
	}
#Accumulated liquid water tendency difference
'Accumulated liquid water tendency difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 242 ;
	}
#Forecast albedo difference
'Forecast albedo difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 243 ;
	}
#Forecast surface roughness difference
'Forecast surface roughness difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 244 ;
	}
#Forecast logarithm of surface roughness for heat difference
'Forecast logarithm of surface roughness for heat difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 245 ;
	}
#Specific cloud liquid water content difference
'Specific cloud liquid water content difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 246 ;
	}
#Specific cloud ice water content difference
'Specific cloud ice water content difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 247 ;
	}
#Cloud cover difference
'Cloud cover difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 248 ;
	}
#Accumulated ice water tendency difference
'Accumulated ice water tendency difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 249 ;
	}
#Ice age difference
'Ice age difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 250 ;
	}
#Adiabatic tendency of temperature difference
'Adiabatic tendency of temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 251 ;
	}
#Adiabatic tendency of humidity difference
'Adiabatic tendency of humidity difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 252 ;
	}
#Adiabatic tendency of zonal wind difference
'Adiabatic tendency of zonal wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 253 ;
	}
#Adiabatic tendency of meridional wind difference
'Adiabatic tendency of meridional wind difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 254 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 255 ;
	}
#Probability of a tropical storm
'Probability of a tropical storm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 89 ;
	}
#Probability of a hurricane
'Probability of a hurricane' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 90 ;
	}
#Probability of a tropical depression
'Probability of a tropical depression' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 91 ;
	}
#Climatological probability of a tropical storm
'Climatological probability of a tropical storm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 92 ;
	}
#Climatological probability of a hurricane
'Climatological probability of a hurricane' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 93 ;
	}
#Climatological probability of a tropical depression
'Climatological probability of a tropical depression' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 94 ;
	}
#Probability anomaly of a tropical storm
'Probability anomaly of a tropical storm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 95 ;
	}
#Probability anomaly of a hurricane
'Probability anomaly of a hurricane' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 96 ;
	}
#Probability anomaly of a tropical depression
'Probability anomaly of a tropical depression' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 97 ;
	}
#Total precipitation of at least 25 mm
'Total precipitation of at least 25 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 98 ;
	}
#Total precipitation of at least 50 mm
'Total precipitation of at least 50 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 99 ;
	}
#10 metre wind gust of at least 10 m/s
'10 metre wind gust of at least 10 m/s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 100 ;
	}
#Convective available potential energy shear index
'Convective available potential energy shear index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 44 ;
	}
#Water vapour flux index
'Water vapour flux index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 45 ;
	}
#Convective available potential energy index
'Convective available potential energy index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 59 ;
	}
#Maximum of significant wave height index
'Maximum of significant wave height index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 216 ;
	}
#Wave experimental parameter 1
'Wave experimental parameter 1' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 80 ;
	}
#Wave experimental parameter 2
'Wave experimental parameter 2' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 81 ;
	}
#Wave experimental parameter 3
'Wave experimental parameter 3' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 82 ;
	}
#Wave experimental parameter 4
'Wave experimental parameter 4' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 83 ;
	}
#Wave experimental parameter 5
'Wave experimental parameter 5' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 84 ;
	}
#Wave induced mean sea level correction
'Wave induced mean sea level correction' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 98 ;
	}
#Ratio of wave angular and frequency width
'Ratio of wave angular and frequency width' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 99 ;
	}
#Number of events in freak waves statistics
'Number of events in freak waves statistics' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 100 ;
	}
#U-component of atmospheric surface momentum flux
'U-component of atmospheric surface momentum flux' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 101 ;
	}
#V-component of atmospheric surface momentum flux
'V-component of atmospheric surface momentum flux' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 102 ;
	}
#U-component of surface momentum flux into ocean
'U-component of surface momentum flux into ocean' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 103 ;
	}
#V-component of surface momentum flux into ocean
'V-component of surface momentum flux into ocean' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 104 ;
	}
#Wave turbulent energy flux into ocean
'Wave turbulent energy flux into ocean' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 105 ;
	}
#Wave directional width of first swell partition
'Wave directional width of first swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 106 ;
	}
#Wave frequency width of first swell partition
'Wave frequency width of first swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 107 ;
	}
#Wave directional width of second swell partition
'Wave directional width of second swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 108 ;
	}
#Wave frequency width of second swell partition
'Wave frequency width of second swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 109 ;
	}
#Wave directional width of third swell partition
'Wave directional width of third swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 110 ;
	}
#Wave frequency width of third swell partition
'Wave frequency width of third swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 111 ;
	}
#Wave energy flux magnitude
'Wave energy flux magnitude' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 112 ;
	}
#Wave energy flux mean direction
'Wave energy flux mean direction' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 113 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds
'Significant wave height of all waves with periods within the inclusive range from 10 to 12 seconds' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 114 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds
'Significant wave height of all waves with periods within the inclusive range from 12 to 14 seconds' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 115 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds
'Significant wave height of all waves with periods within the inclusive range from 14 to 17 seconds' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 116 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds
'Significant wave height of all waves with periods within the inclusive range from 17 to 21 seconds' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 117 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds
'Significant wave height of all waves with periods within the inclusive range from 21 to 25 seconds' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 118 ;
	}
#Significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds
'Significant wave height of all waves with periods within the inclusive range from 25 to 30 seconds' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 119 ;
	}
#Significant wave height of all waves with period larger than 10s
'Significant wave height of all waves with period larger than 10s' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 120 ;
	}
#Significant wave height of first swell partition
'Significant wave height of first swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 121 ;
	}
#Mean wave direction of first swell partition
'Mean wave direction of first swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 122 ;
	}
#Mean wave period of first swell partition
'Mean wave period of first swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 123 ;
	}
#Significant wave height of second swell partition
'Significant wave height of second swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 124 ;
	}
#Mean wave direction of second swell partition
'Mean wave direction of second swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 125 ;
	}
#Mean wave period of second swell partition
'Mean wave period of second swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 126 ;
	}
#Significant wave height of third swell partition
'Significant wave height of third swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 127 ;
	}
#Mean wave direction of third swell partition
'Mean wave direction of third swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 128 ;
	}
#Mean wave period of third swell partition
'Mean wave period of third swell partition' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 129 ;
	}
#Wave Spectral Skewness
'Wave Spectral Skewness' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 207 ;
	}
#Free convective velocity over the oceans
'Free convective velocity over the oceans' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 208 ;
	}
#Air density over the oceans
'Air density over the oceans' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 209 ;
	}
#Mean square wave strain in sea ice
'Mean square wave strain in sea ice' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 210 ;
	}
#Normalized energy flux into waves
'Normalized energy flux into waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 211 ;
	}
#Normalized energy flux into ocean
'Normalized energy flux into ocean' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 212 ;
	}
#Turbulent Langmuir number
'Turbulent Langmuir number' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 213 ;
	}
#Normalized stress into ocean
'Normalized stress into ocean' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 214 ;
	}
#Reserved
'Reserved' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 193 ;
	}
#Water vapour flux
'Water vapour flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 45 ;
	}
#Total column vertically-integrated divergence of cloud liquid water flux
'Total column vertically-integrated divergence of cloud liquid water flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 79 ;
	}
#Total column vertically-integrated divergence of cloud frozen water flux
'Total column vertically-integrated divergence of cloud frozen water flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 80 ;
	}
#Total column vertically-integrated eastward cloud liquid water flux
'Total column vertically-integrated eastward cloud liquid water flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 88 ;
	}
#Total column vertically-integrated northward cloud liquid water flux
'Total column vertically-integrated northward cloud liquid water flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 89 ;
	}
#Total column vertically-integrated eastward cloud frozen water flux
'Total column vertically-integrated eastward cloud frozen water flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 90 ;
	}
#Total column vertically-integrated northward cloud frozen water flux 
'Total column vertically-integrated northward cloud frozen water flux ' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 91 ;
	}
#Total column vertically-integrated mass tendency
'Total column vertically-integrated mass tendency' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 92 ;
	}
#U-tendency from dynamics
'U-tendency from dynamics' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 114 ;
	}
#V-tendency from dynamics
'V-tendency from dynamics' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 115 ;
	}
#T-tendency from dynamics
'T-tendency from dynamics' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 116 ;
	}
#q-tendency from dynamics
'q-tendency from dynamics' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 117 ;
	}
#T-tendency from radiation
'T-tendency from radiation' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 118 ;
	}
#U-tendency from turbulent diffusion + subgrid orography
'U-tendency from turbulent diffusion + subgrid orography' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 119 ;
	}
#V-tendency from turbulent diffusion + subgrid orography
'V-tendency from turbulent diffusion + subgrid orography' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 120 ;
	}
#T-tendency from turbulent diffusion + subgrid orography
'T-tendency from turbulent diffusion + subgrid orography' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 121 ;
	}
#q-tendency from turbulent diffusion
'q-tendency from turbulent diffusion' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 122 ;
	}
#U-tendency from subgrid orography
'U-tendency from subgrid orography' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 123 ;
	}
#V-tendency from subgrid orography
'V-tendency from subgrid orography' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 124 ;
	}
#T-tendency from subgrid orography
'T-tendency from subgrid orography' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 125 ;
	}
#U-tendency from convection (deep+shallow)
'U-tendency from convection (deep+shallow)' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 126 ;
	}
#V-tendency from convection (deep+shallow)
'V-tendency from convection (deep+shallow)' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 127 ;
	}
#T-tendency from convection (deep+shallow)
'T-tendency from convection (deep+shallow)' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 128 ;
	}
#q-tendency from convection (deep+shallow)
'q-tendency from convection (deep+shallow)' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 129 ;
	}
#Liquid Precipitation flux from convection
'Liquid Precipitation flux from convection' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 130 ;
	}
#Ice Precipitation flux from convection
'Ice Precipitation flux from convection' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 131 ;
	}
#T-tendency from cloud scheme
'T-tendency from cloud scheme' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 132 ;
	}
#q-tendency from cloud scheme
'q-tendency from cloud scheme' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 133 ;
	}
#ql-tendency from cloud scheme
'ql-tendency from cloud scheme' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 134 ;
	}
#qi-tendency from cloud scheme
'qi-tendency from cloud scheme' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 135 ;
	}
#Liquid Precip flux from cloud scheme (stratiform)
'Liquid Precip flux from cloud scheme (stratiform)' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 136 ;
	}
#Ice Precip flux from cloud scheme (stratiform)
'Ice Precip flux from cloud scheme (stratiform)' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 137 ;
	}
#U-tendency from shallow convection
'U-tendency from shallow convection' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 138 ;
	}
#V-tendency from shallow convection
'V-tendency from shallow convection' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 139 ;
	}
#T-tendency from shallow convection
'T-tendency from shallow convection' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 140 ;
	}
#q-tendency from shallow convection
'q-tendency from shallow convection' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 141 ;
	}
#Standardised precipitation index valid in the last 3 months
'Standardised precipitation index valid in the last 3 months' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 1 ;
	}
#Standardised precipitation index valid in the last 6 months
'Standardised precipitation index valid in the last 6 months' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 2 ;
	}
#Standardised precipitation index valid in the last 12 months
'Standardised precipitation index valid in the last 12 months' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 3 ;
	}
#100 metre U wind component anomaly
'100 metre U wind component anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 6 ;
	}
#100 metre V wind component anomaly
'100 metre V wind component anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 7 ;
	}
#100 metre wind speed anomaly
'100 metre wind speed anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 8 ;
	}
#Lake mix-layer temperature anomaly
'Lake mix-layer temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 24 ;
	}
#Lake ice depth anomaly
'Lake ice depth anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 25 ;
	}
#Maximum temperature at 2 metres in the last 6 hours anomaly
'Maximum temperature at 2 metres in the last 6 hours anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 121 ;
	}
#Minimum temperature at 2 metres in the last 6 hours anomaly
'Minimum temperature at 2 metres in the last 6 hours anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 122 ;
	}
#Time-mean surface runoff rate
'Time-mean surface runoff rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 8 ;
	}
#Time-mean sub-surface runoff rate
'Time-mean sub-surface runoff rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 9 ;
	}
#Mean surface runoff rate anomaly
'Mean surface runoff rate anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 8 ;
	}
#Mean sub-surface runoff rate anomaly
'Mean sub-surface runoff rate anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 9 ;
	}
#Clear-sky (II) down surface sw flux
'Clear-sky (II) down surface sw flux' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 10 ;
	}
#Clear-sky (II) up surface sw flux
'Clear-sky (II) up surface sw flux' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 13 ;
	}
#Visibility at 1.5m
'Visibility at 1.5m' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 25 ;
	}
#Minimum temperature at 1.5m since previous post-processing
'Minimum temperature at 1.5m since previous post-processing' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 50 ;
	}
#Maximum temperature at 1.5m since previous post-processing
'Maximum temperature at 1.5m since previous post-processing' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 51 ;
	}
#Relative humidity at 1.5m
'Relative humidity at 1.5m' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 52 ;
	}
#2 metre specific humidity
'2 metre specific humidity' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 96 ;
	}
#Sea ice snow thickness
'Sea ice snow thickness' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 97 ;
	}
#Short wave radiation flux at surface
'Short wave radiation flux at surface' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 116 ;
	}
#Short wave radiation flux at top of atmosphere
'Short wave radiation flux at top of atmosphere' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 117 ;
	}
#Total column water vapour
'Total column water vapour' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 137 ;
	}
#Large scale rainfall rate
'Large scale rainfall rate' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 142 ;
	}
#Convective rainfall rate
'Convective rainfall rate' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 143 ;
	}
#Very low cloud amount
'Very low cloud amount' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 186 ;
	}
#Convective snowfall rate
'Convective snowfall rate' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 239 ;
	}
#Large scale snowfall rate
'Large scale snowfall rate' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 240 ;
	}
#Total cloud amount - random overlap
'Total cloud amount - random overlap' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 248 ;
	}
#Total cloud amount in lw radiation
'Total cloud amount in lw radiation' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 249 ;
	}
#Volcanic ash aerosol mixing ratio
'Volcanic ash aerosol mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 13 ;
	}
#Volcanic sulphate aerosol mixing ratio
'Volcanic sulphate aerosol mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 14 ;
	}
#Volcanic SO2 precursor mixing ratio
'Volcanic SO2 precursor mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 15 ;
	}
#SO4 aerosol precursor mass mixing ratio
'SO4 aerosol precursor mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 28 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 1
'Water vapour mixing ratio for hydrophilic aerosols in mode 1' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 29 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 2
'Water vapour mixing ratio for hydrophilic aerosols in mode 2' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 30 ;
	}
#DMS surface emission
'DMS surface emission' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 43 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 3
'Water vapour mixing ratio for hydrophilic aerosols in mode 3' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 44 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 4
'Water vapour mixing ratio for hydrophilic aerosols in mode 4' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 45 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 55 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 56 ;
	}
#Mixing ration of organic carbon aerosol, nucleation mode
'Mixing ration of organic carbon aerosol, nucleation mode' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 57 ;
	}
#Monoterpene precursor mixing ratio
'Monoterpene precursor mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 58 ;
	}
#Secondary organic precursor mixing ratio
'Secondary organic precursor mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 59 ;
	}
#Injection height (from IS4FIRES)
'Injection height (from IS4FIRES)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 60 ;
	}
#Particulate matter d <= 1 um
'Particulate matter d <= 1 um' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 72 ;
	}
#Particulate matter d <= 2.5 um
'Particulate matter d <= 2.5 um' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 73 ;
	}
#Particulate matter d <= 10 um
'Particulate matter d <= 10 um' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 74 ;
	}
#Wildfire viewing angle of observation
'Wildfire viewing angle of observation' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 79 ;
	}
#Wildfire Flux of Ethane (C2H6)
'Wildfire Flux of Ethane (C2H6)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 118 ;
	}
#Mean height of maximum injection
'Mean height of maximum injection' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 119 ;
	}
#Plume top height above surface
'Plume top height above surface' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 120 ;
	}
#UV visible albedo for direct radiation, isotropic component (climatological)
'UV visible albedo for direct radiation, isotropic component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 186 ;
	}
#UV visible albedo for direct radiation, volumetric component (climatological)
'UV visible albedo for direct radiation, volumetric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 187 ;
	}
#UV visible albedo for direct radiation, geometric component (climatological)
'UV visible albedo for direct radiation, geometric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 188 ;
	}
#Near IR albedo for direct radiation, isotropic component (climatological)
'Near IR albedo for direct radiation, isotropic component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 189 ;
	}
#Near IR albedo for direct radiation, volumetric component (climatological)
'Near IR albedo for direct radiation, volumetric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 190 ;
	}
#Near IR albedo for direct radiation, geometric component (climatological)
'Near IR albedo for direct radiation, geometric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 191 ;
	}
#UV visible albedo for diffuse radiation, isotropic component (climatological)
'UV visible albedo for diffuse radiation, isotropic component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 192 ;
	}
#UV visible albedo for diffuse radiation, volumetric component (climatological)
'UV visible albedo for diffuse radiation, volumetric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 193 ;
	}
#UV visible albedo for diffuse radiation, geometric component (climatological)
'UV visible albedo for diffuse radiation, geometric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 194 ;
	}
#Near IR albedo for diffuse radiation, isotropic component (climatological)
'Near IR albedo for diffuse radiation, isotropic component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 195 ;
	}
#Near IR albedo for diffuse radiation, volumetric component (climatological)
'Near IR albedo for diffuse radiation, volumetric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 196 ;
	}
#Near IR albedo for diffuse radiation, geometric component (climatological)
'Near IR albedo for diffuse radiation, geometric component (climatological)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 197 ;
	}
#Total aerosol optical depth at 340 nm
'Total aerosol optical depth at 340 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 217 ;
	}
#Total aerosol optical depth at 355 nm
'Total aerosol optical depth at 355 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 218 ;
	}
#Total aerosol optical depth at 380 nm
'Total aerosol optical depth at 380 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 219 ;
	}
#Total aerosol optical depth at 400 nm
'Total aerosol optical depth at 400 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 220 ;
	}
#Total aerosol optical depth at 440 nm
'Total aerosol optical depth at 440 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 221 ;
	}
#Total aerosol optical depth at 500 nm
'Total aerosol optical depth at 500 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 222 ;
	}
#Total aerosol optical depth at 532 nm
'Total aerosol optical depth at 532 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 223 ;
	}
#Total aerosol optical depth at 645 nm
'Total aerosol optical depth at 645 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 224 ;
	}
#Total aerosol optical depth at 800 nm
'Total aerosol optical depth at 800 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 225 ;
	}
#Total aerosol optical depth at 858 nm
'Total aerosol optical depth at 858 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 226 ;
	}
#Total aerosol optical depth at 1020 nm
'Total aerosol optical depth at 1020 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 227 ;
	}
#Total aerosol optical depth at 1064 nm
'Total aerosol optical depth at 1064 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 228 ;
	}
#Total aerosol optical depth at 1640 nm
'Total aerosol optical depth at 1640 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 229 ;
	}
#Total aerosol optical depth at 2130 nm
'Total aerosol optical depth at 2130 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 230 ;
	}
#Wildfire Flux of Toluene (C7H8)
'Wildfire Flux of Toluene (C7H8)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 231 ;
	}
#Wildfire Flux of Benzene (C6H6)
'Wildfire Flux of Benzene (C6H6)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 232 ;
	}
#Wildfire Flux of Xylene (C8H10)
'Wildfire Flux of Xylene (C8H10)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 233 ;
	}
#Wildfire Flux of Butenes (C4H8)
'Wildfire Flux of Butenes (C4H8)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 234 ;
	}
#Wildfire Flux of Pentenes (C5H10)
'Wildfire Flux of Pentenes (C5H10)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 235 ;
	}
#Wildfire Flux of Hexene (C6H12)
'Wildfire Flux of Hexene (C6H12)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 236 ;
	}
#Wildfire Flux of Octene (C8H16)
'Wildfire Flux of Octene (C8H16)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 237 ;
	}
#Wildfire Flux of Butanes (C4H10)
'Wildfire Flux of Butanes (C4H10)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 238 ;
	}
#Wildfire Flux of Pentanes (C5H12)
'Wildfire Flux of Pentanes (C5H12)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 239 ;
	}
#Wildfire Flux of Hexanes (C6H14)
'Wildfire Flux of Hexanes (C6H14)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 240 ;
	}
#Wildfire Flux of Heptane (C7H16)
'Wildfire Flux of Heptane (C7H16)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 241 ;
	}
#Plume bottom height above surface
'Plume bottom height above surface' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 242 ;
	}
#Volcanic sulphate aerosol optical depth at 550 nm
'Volcanic sulphate aerosol optical depth at 550 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 243 ;
	}
#Volcanic ash optical depth at 550 nm
'Volcanic ash optical depth at 550 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 244 ;
	}
#Profile of total aerosol dry extinction coefficient
'Profile of total aerosol dry extinction coefficient' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 245 ;
	}
#Profile of total aerosol dry absorption coefficient
'Profile of total aerosol dry absorption coefficient' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 246 ;
	}
#Nitrate fine mode aerosol mass mixing ratio
'Nitrate fine mode aerosol mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 247 ;
	}
#Nitrate coarse mode aerosol mass mixing ratio
'Nitrate coarse mode aerosol mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 248 ;
	}
#Ammonium aerosol mass mixing ratio
'Ammonium aerosol mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 249 ;
	}
#Nitrate aerosol optical depth at 550 nm
'Nitrate aerosol optical depth at 550 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 250 ;
	}
#Ammonium aerosol optical depth at 550 nm
'Ammonium aerosol optical depth at 550 nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 251 ;
	}
#Aerosol type 13 mass mixing ratio
'Aerosol type 13 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 13 ;
	}
#Aerosol type 14 mass mixing ratio
'Aerosol type 14 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 14 ;
	}
#Aerosol type 15 mass mixing ratio
'Aerosol type 15 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 15 ;
	}
#SO4 aerosol precursor mass mixing ratio
'SO4 aerosol precursor mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 28 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 1
'Water vapour mixing ratio for hydrophilic aerosols in mode 1' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 29 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 2
'Water vapour mixing ratio for hydrophilic aerosols in mode 2' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 30 ;
	}
#DMS surface emission
'DMS surface emission' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 43 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 3
'Water vapour mixing ratio for hydrophilic aerosols in mode 3' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 44 ;
	}
#Water vapour mixing ratio for hydrophilic aerosols in mode 4
'Water vapour mixing ratio for hydrophilic aerosols in mode 4' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 45 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 55 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 56 ;
	}
#Wildfire Flux of Ethane (C2H6)
'Wildfire Flux of Ethane (C2H6)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 118 ;
	}
#Altitude of emitter
'Altitude of emitter' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 119 ;
	}
#Altitude of plume top
'Altitude of plume top' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 120 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 1 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 2 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 3 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 4 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 5 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 6 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 7 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 8 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 9 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 10 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 11 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 12 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 13 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 14 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 15 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 16 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 17 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 18 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 19 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 20 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 21 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 22 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 23 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 24 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 25 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 26 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 27 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 28 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 29 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 30 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 31 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 32 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 33 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 34 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 35 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 36 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 37 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 38 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 39 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 40 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 41 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 42 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 43 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 44 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 45 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 46 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 47 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 48 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 49 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 50 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 51 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 52 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 53 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 54 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 55 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 56 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 57 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 58 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 59 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 60 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 61 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 62 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 63 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 64 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 65 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 66 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 67 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 68 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 69 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 70 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 71 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 72 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 73 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 74 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 75 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 76 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 77 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 78 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 79 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 80 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 81 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 82 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 83 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 84 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 85 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 86 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 87 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 88 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 89 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 90 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 91 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 92 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 93 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 94 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 95 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 96 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 97 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 98 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 99 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 100 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 101 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 102 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 103 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 104 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 105 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 106 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 107 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 108 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 109 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 110 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 111 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 112 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 113 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 114 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 115 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 116 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 117 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 118 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 119 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 120 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 121 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 122 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 123 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 124 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 125 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 126 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 127 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 128 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 129 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 130 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 131 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 132 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 133 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 134 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 135 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 136 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 137 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 138 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 139 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 140 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 141 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 142 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 143 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 144 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 145 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 146 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 147 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 148 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 149 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 150 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 151 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 152 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 153 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 154 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 155 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 156 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 157 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 158 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 159 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 160 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 161 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 162 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 163 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 164 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 165 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 166 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 167 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 168 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 169 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 170 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 171 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 172 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 173 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 174 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 175 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 176 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 177 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 178 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 179 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 180 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 181 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 182 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 183 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 184 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 185 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 186 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 187 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 188 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 189 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 190 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 191 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 192 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 193 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 194 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 195 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 196 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 197 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 198 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 199 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 200 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 201 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 202 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 203 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 204 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 205 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 206 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 207 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 208 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 209 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 210 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 211 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 212 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 213 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 214 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 215 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 216 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 217 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 218 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 219 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 220 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 221 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 222 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 223 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 224 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 225 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 226 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 227 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 228 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 229 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 230 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 231 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 232 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 233 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 234 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 235 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 236 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 237 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 238 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 239 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 240 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 241 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 242 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 243 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 244 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 245 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 246 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 247 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 248 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 249 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 250 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 251 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 252 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 253 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 254 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 212 ;
	 indicatorOfParameter = 255 ;
	}
#Random pattern 1 for sppt
'Random pattern 1 for sppt' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 1 ;
	}
#Random pattern 2 for sppt
'Random pattern 2 for sppt' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 2 ;
	}
#Random pattern 3 for sppt
'Random pattern 3 for sppt' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 3 ;
	}
#Random pattern 4 for sppt
'Random pattern 4 for sppt' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 4 ;
	}
#Random pattern 5 for sppt
'Random pattern 5 for sppt' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 5 ;
	}
#Random pattern 1 for SPP scheme
'Random pattern 1 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 101 ;
	}
#Random pattern 2 for SPP scheme
'Random pattern 2 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 102 ;
	}
#Random pattern 3 for SPP scheme
'Random pattern 3 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 103 ;
	}
#Random pattern 4 for SPP scheme
'Random pattern 4 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 104 ;
	}
#Random pattern 5 for SPP scheme
'Random pattern 5 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 105 ;
	}
#Random pattern 6 for SPP scheme
'Random pattern 6 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 106 ;
	}
#Random pattern 7 for SPP scheme
'Random pattern 7 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 107 ;
	}
#Random pattern 8 for SPP scheme
'Random pattern 8 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 108 ;
	}
#Random pattern 9 for SPP scheme
'Random pattern 9 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 109 ;
	}
#Random pattern 10 for SPP scheme
'Random pattern 10 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 110 ;
	}
#Random pattern 11 for SPP scheme
'Random pattern 11 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 111 ;
	}
#Random pattern 12 for SPP scheme
'Random pattern 12 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 112 ;
	}
#Random pattern 13 for SPP scheme
'Random pattern 13 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 113 ;
	}
#Random pattern 14 for SPP scheme
'Random pattern 14 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 114 ;
	}
#Random pattern 15 for SPP scheme
'Random pattern 15 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 115 ;
	}
#Random pattern 16 for SPP scheme
'Random pattern 16 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 116 ;
	}
#Random pattern 17 for SPP scheme
'Random pattern 17 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 117 ;
	}
#Random pattern 18 for SPP scheme
'Random pattern 18 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 118 ;
	}
#Random pattern 19 for SPP scheme
'Random pattern 19 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 119 ;
	}
#Random pattern 20 for SPP scheme
'Random pattern 20 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 120 ;
	}
#Random pattern 21 for SPP scheme
'Random pattern 21 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 121 ;
	}
#Random pattern 22 for SPP scheme
'Random pattern 22 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 122 ;
	}
#Random pattern 23 for SPP scheme
'Random pattern 23 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 123 ;
	}
#Random pattern 24 for SPP scheme
'Random pattern 24 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 124 ;
	}
#Random pattern 25 for SPP scheme
'Random pattern 25 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 125 ;
	}
#Random pattern 26 for SPP scheme
'Random pattern 26 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 126 ;
	}
#Random pattern 27 for SPP scheme
'Random pattern 27 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 127 ;
	}
#Random pattern 28 for SPP scheme
'Random pattern 28 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 128 ;
	}
#Random pattern 29 for SPP scheme
'Random pattern 29 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 129 ;
	}
#Random pattern 30 for SPP scheme
'Random pattern 30 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 130 ;
	}
#Random pattern 31 for SPP scheme
'Random pattern 31 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 131 ;
	}
#Random pattern 32 for SPP scheme
'Random pattern 32 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 132 ;
	}
#Random pattern 33 for SPP scheme
'Random pattern 33 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 133 ;
	}
#Random pattern 34 for SPP scheme
'Random pattern 34 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 134 ;
	}
#Random pattern 35 for SPP scheme
'Random pattern 35 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 135 ;
	}
#Random pattern 36 for SPP scheme
'Random pattern 36 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 136 ;
	}
#Random pattern 37 for SPP scheme
'Random pattern 37 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 137 ;
	}
#Random pattern 38 for SPP scheme
'Random pattern 38 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 138 ;
	}
#Random pattern 39 for SPP scheme
'Random pattern 39 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 139 ;
	}
#Random pattern 40 for SPP scheme
'Random pattern 40 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 140 ;
	}
#Random pattern 41 for SPP scheme
'Random pattern 41 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 141 ;
	}
#Random pattern 42 for SPP scheme
'Random pattern 42 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 142 ;
	}
#Random pattern 43 for SPP scheme
'Random pattern 43 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 143 ;
	}
#Random pattern 44 for SPP scheme
'Random pattern 44 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 144 ;
	}
#Random pattern 45 for SPP scheme
'Random pattern 45 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 145 ;
	}
#Random pattern 46 for SPP scheme
'Random pattern 46 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 146 ;
	}
#Random pattern 47 for SPP scheme
'Random pattern 47 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 147 ;
	}
#Random pattern 48 for SPP scheme
'Random pattern 48 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 148 ;
	}
#Random pattern 49 for SPP scheme
'Random pattern 49 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 149 ;
	}
#Random pattern 50 for SPP scheme
'Random pattern 50 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 150 ;
	}
#Random pattern 51 for SPP scheme
'Random pattern 51 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 151 ;
	}
#Random pattern 52 for SPP scheme
'Random pattern 52 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 152 ;
	}
#Random pattern 53 for SPP scheme
'Random pattern 53 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 153 ;
	}
#Random pattern 54 for SPP scheme
'Random pattern 54 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 154 ;
	}
#Random pattern 55 for SPP scheme
'Random pattern 55 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 155 ;
	}
#Random pattern 56 for SPP scheme
'Random pattern 56 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 156 ;
	}
#Random pattern 57 for SPP scheme
'Random pattern 57 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 157 ;
	}
#Random pattern 58 for SPP scheme
'Random pattern 58 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 158 ;
	}
#Random pattern 59 for SPP scheme
'Random pattern 59 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 159 ;
	}
#Random pattern 60 for SPP scheme
'Random pattern 60 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 160 ;
	}
#Random pattern 61 for SPP scheme
'Random pattern 61 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 161 ;
	}
#Random pattern 62 for SPP scheme
'Random pattern 62 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 162 ;
	}
#Random pattern 63 for SPP scheme
'Random pattern 63 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 163 ;
	}
#Random pattern 64 for SPP scheme
'Random pattern 64 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 164 ;
	}
#Random pattern 65 for SPP scheme
'Random pattern 65 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 165 ;
	}
#Random pattern 66 for SPP scheme
'Random pattern 66 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 166 ;
	}
#Random pattern 67 for SPP scheme
'Random pattern 67 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 167 ;
	}
#Random pattern 68 for SPP scheme
'Random pattern 68 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 168 ;
	}
#Random pattern 69 for SPP scheme
'Random pattern 69 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 169 ;
	}
#Random pattern 70 for SPP scheme
'Random pattern 70 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 170 ;
	}
#Random pattern 71 for SPP scheme
'Random pattern 71 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 171 ;
	}
#Random pattern 72 for SPP scheme
'Random pattern 72 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 172 ;
	}
#Random pattern 73 for SPP scheme
'Random pattern 73 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 173 ;
	}
#Random pattern 74 for SPP scheme
'Random pattern 74 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 174 ;
	}
#Random pattern 75 for SPP scheme
'Random pattern 75 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 175 ;
	}
#Random pattern 76 for SPP scheme
'Random pattern 76 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 176 ;
	}
#Random pattern 77 for SPP scheme
'Random pattern 77 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 177 ;
	}
#Random pattern 78 for SPP scheme
'Random pattern 78 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 178 ;
	}
#Random pattern 79 for SPP scheme
'Random pattern 79 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 179 ;
	}
#Random pattern 80 for SPP scheme
'Random pattern 80 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 180 ;
	}
#Random pattern 81 for SPP scheme
'Random pattern 81 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 181 ;
	}
#Random pattern 82 for SPP scheme
'Random pattern 82 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 182 ;
	}
#Random pattern 83 for SPP scheme
'Random pattern 83 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 183 ;
	}
#Random pattern 84 for SPP scheme
'Random pattern 84 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 184 ;
	}
#Random pattern 85 for SPP scheme
'Random pattern 85 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 185 ;
	}
#Random pattern 86 for SPP scheme
'Random pattern 86 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 186 ;
	}
#Random pattern 87 for SPP scheme
'Random pattern 87 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 187 ;
	}
#Random pattern 88 for SPP scheme
'Random pattern 88 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 188 ;
	}
#Random pattern 89 for SPP scheme
'Random pattern 89 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 189 ;
	}
#Random pattern 90 for SPP scheme
'Random pattern 90 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 190 ;
	}
#Random pattern 91 for SPP scheme
'Random pattern 91 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 191 ;
	}
#Random pattern 92 for SPP scheme
'Random pattern 92 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 192 ;
	}
#Random pattern 93 for SPP scheme
'Random pattern 93 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 193 ;
	}
#Random pattern 94 for SPP scheme
'Random pattern 94 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 194 ;
	}
#Random pattern 95 for SPP scheme
'Random pattern 95 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 195 ;
	}
#Random pattern 96 for SPP scheme
'Random pattern 96 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 196 ;
	}
#Random pattern 97 for SPP scheme
'Random pattern 97 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 197 ;
	}
#Random pattern 98 for SPP scheme
'Random pattern 98 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 198 ;
	}
#Random pattern 99 for SPP scheme
'Random pattern 99 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 199 ;
	}
#Random pattern 100 for SPP scheme
'Random pattern 100 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 200 ;
	}
#Random pattern 101 for SPP scheme
'Random pattern 101 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 201 ;
	}
#Random pattern 102 for SPP scheme
'Random pattern 102 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 202 ;
	}
#Random pattern 103 for SPP scheme
'Random pattern 103 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 203 ;
	}
#Random pattern 104 for SPP scheme
'Random pattern 104 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 204 ;
	}
#Random pattern 105 for SPP scheme
'Random pattern 105 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 205 ;
	}
#Random pattern 106 for SPP scheme
'Random pattern 106 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 206 ;
	}
#Random pattern 107 for SPP scheme
'Random pattern 107 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 207 ;
	}
#Random pattern 108 for SPP scheme
'Random pattern 108 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 208 ;
	}
#Random pattern 109 for SPP scheme
'Random pattern 109 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 209 ;
	}
#Random pattern 110 for SPP scheme
'Random pattern 110 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 210 ;
	}
#Random pattern 111 for SPP scheme
'Random pattern 111 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 211 ;
	}
#Random pattern 112 for SPP scheme
'Random pattern 112 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 212 ;
	}
#Random pattern 113 for SPP scheme
'Random pattern 113 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 213 ;
	}
#Random pattern 114 for SPP scheme
'Random pattern 114 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 214 ;
	}
#Random pattern 115 for SPP scheme
'Random pattern 115 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 215 ;
	}
#Random pattern 116 for SPP scheme
'Random pattern 116 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 216 ;
	}
#Random pattern 117 for SPP scheme
'Random pattern 117 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 217 ;
	}
#Random pattern 118 for SPP scheme
'Random pattern 118 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 218 ;
	}
#Random pattern 119 for SPP scheme
'Random pattern 119 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 219 ;
	}
#Random pattern 120 for SPP scheme
'Random pattern 120 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 220 ;
	}
#Random pattern 121 for SPP scheme
'Random pattern 121 for SPP scheme' = {
	 table2Version = 213 ;
	 indicatorOfParameter = 221 ;
	}
#Cosine of solar zenith angle
'Cosine of solar zenith angle' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 1 ;
	}
#UV biologically effective dose
'UV biologically effective dose' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 2 ;
	}
#UV biologically effective dose clear-sky
'UV biologically effective dose clear-sky' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 3 ;
	}
#Total surface UV spectral flux (280-285 nm)
'Total surface UV spectral flux (280-285 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 4 ;
	}
#Total surface UV spectral flux (285-290 nm)
'Total surface UV spectral flux (285-290 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 5 ;
	}
#Total surface UV spectral flux (290-295 nm)
'Total surface UV spectral flux (290-295 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 6 ;
	}
#Total surface UV spectral flux (295-300 nm)
'Total surface UV spectral flux (295-300 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 7 ;
	}
#Total surface UV spectral flux (300-305 nm)
'Total surface UV spectral flux (300-305 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 8 ;
	}
#Total surface UV spectral flux (305-310 nm)
'Total surface UV spectral flux (305-310 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 9 ;
	}
#Total surface UV spectral flux (310-315 nm)
'Total surface UV spectral flux (310-315 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 10 ;
	}
#Total surface UV spectral flux (315-320 nm)
'Total surface UV spectral flux (315-320 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 11 ;
	}
#Total surface UV spectral flux (320-325 nm)
'Total surface UV spectral flux (320-325 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 12 ;
	}
#Total surface UV spectral flux (325-330 nm)
'Total surface UV spectral flux (325-330 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 13 ;
	}
#Total surface UV spectral flux (330-335 nm)
'Total surface UV spectral flux (330-335 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 14 ;
	}
#Total surface UV spectral flux (335-340 nm)
'Total surface UV spectral flux (335-340 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 15 ;
	}
#Total surface UV spectral flux (340-345 nm)
'Total surface UV spectral flux (340-345 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 16 ;
	}
#Total surface UV spectral flux (345-350 nm)
'Total surface UV spectral flux (345-350 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 17 ;
	}
#Total surface UV spectral flux (350-355 nm)
'Total surface UV spectral flux (350-355 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 18 ;
	}
#Total surface UV spectral flux (355-360 nm)
'Total surface UV spectral flux (355-360 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 19 ;
	}
#Total surface UV spectral flux (360-365 nm)
'Total surface UV spectral flux (360-365 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 20 ;
	}
#Total surface UV spectral flux (365-370 nm)
'Total surface UV spectral flux (365-370 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 21 ;
	}
#Total surface UV spectral flux (370-375 nm)
'Total surface UV spectral flux (370-375 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 22 ;
	}
#Total surface UV spectral flux (375-380 nm)
'Total surface UV spectral flux (375-380 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 23 ;
	}
#Total surface UV spectral flux (380-385 nm)
'Total surface UV spectral flux (380-385 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 24 ;
	}
#Total surface UV spectral flux (385-390 nm)
'Total surface UV spectral flux (385-390 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 25 ;
	}
#Total surface UV spectral flux (390-395 nm)
'Total surface UV spectral flux (390-395 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 26 ;
	}
#Total surface UV spectral flux (395-400 nm)
'Total surface UV spectral flux (395-400 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 27 ;
	}
#Clear-sky surface UV spectral flux (280-285 nm)
'Clear-sky surface UV spectral flux (280-285 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 28 ;
	}
#Clear-sky surface UV spectral flux (285-290 nm)
'Clear-sky surface UV spectral flux (285-290 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 29 ;
	}
#Clear-sky surface UV spectral flux (290-295 nm)
'Clear-sky surface UV spectral flux (290-295 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 30 ;
	}
#Clear-sky surface UV spectral flux (295-300 nm)
'Clear-sky surface UV spectral flux (295-300 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 31 ;
	}
#Clear-sky surface UV spectral flux (300-305 nm)
'Clear-sky surface UV spectral flux (300-305 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 32 ;
	}
#Clear-sky surface UV spectral flux (305-310 nm)
'Clear-sky surface UV spectral flux (305-310 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 33 ;
	}
#Clear-sky surface UV spectral flux (310-315 nm)
'Clear-sky surface UV spectral flux (310-315 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 34 ;
	}
#Clear-sky surface UV spectral flux (315-320 nm)
'Clear-sky surface UV spectral flux (315-320 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 35 ;
	}
#Clear-sky surface UV spectral flux (320-325 nm)
'Clear-sky surface UV spectral flux (320-325 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 36 ;
	}
#Clear-sky surface UV spectral flux (325-330 nm)
'Clear-sky surface UV spectral flux (325-330 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 37 ;
	}
#Clear-sky surface UV spectral flux (330-335 nm)
'Clear-sky surface UV spectral flux (330-335 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 38 ;
	}
#Clear-sky surface UV spectral flux (335-340 nm)
'Clear-sky surface UV spectral flux (335-340 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 39 ;
	}
#Clear-sky surface UV spectral flux (340-345 nm)
'Clear-sky surface UV spectral flux (340-345 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 40 ;
	}
#Clear-sky surface UV spectral flux (345-350 nm)
'Clear-sky surface UV spectral flux (345-350 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 41 ;
	}
#Clear-sky surface UV spectral flux (350-355 nm)
'Clear-sky surface UV spectral flux (350-355 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 42 ;
	}
#Clear-sky surface UV spectral flux (355-360 nm)
'Clear-sky surface UV spectral flux (355-360 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 43 ;
	}
#Clear-sky surface UV spectral flux (360-365 nm)
'Clear-sky surface UV spectral flux (360-365 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 44 ;
	}
#Clear-sky surface UV spectral flux (365-370 nm)
'Clear-sky surface UV spectral flux (365-370 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 45 ;
	}
#Clear-sky surface UV spectral flux (370-375 nm)
'Clear-sky surface UV spectral flux (370-375 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 46 ;
	}
#Clear-sky surface UV spectral flux (375-380 nm)
'Clear-sky surface UV spectral flux (375-380 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 47 ;
	}
#Clear-sky surface UV spectral flux (380-385 nm)
'Clear-sky surface UV spectral flux (380-385 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 48 ;
	}
#Clear-sky surface UV spectral flux (385-390 nm)
'Clear-sky surface UV spectral flux (385-390 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 49 ;
	}
#Clear-sky surface UV spectral flux (390-395 nm)
'Clear-sky surface UV spectral flux (390-395 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 50 ;
	}
#Clear-sky surface UV spectral flux (395-400 nm)
'Clear-sky surface UV spectral flux (395-400 nm)' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 51 ;
	}
#Profile of optical thickness at 340 nm
'Profile of optical thickness at 340 nm' = {
	 table2Version = 214 ;
	 indicatorOfParameter = 52 ;
	}
#Source/gain of sea salt aerosol (0.03 - 0.5 um)
'Source/gain of sea salt aerosol (0.03 - 0.5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 1 ;
	}
#Source/gain of sea salt aerosol (0.5 - 5 um)
'Source/gain of sea salt aerosol (0.5 - 5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 2 ;
	}
#Source/gain of sea salt aerosol (5 - 20 um)
'Source/gain of sea salt aerosol (5 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 3 ;
	}
#Dry deposition of sea salt aerosol (0.03 - 0.5 um)
'Dry deposition of sea salt aerosol (0.03 - 0.5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 4 ;
	}
#Dry deposition of sea salt aerosol (0.5 - 5 um)
'Dry deposition of sea salt aerosol (0.5 - 5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 5 ;
	}
#Dry deposition of sea salt aerosol (5 - 20 um)
'Dry deposition of sea salt aerosol (5 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 6 ;
	}
#Sedimentation of sea salt aerosol (0.03 - 0.5 um)
'Sedimentation of sea salt aerosol (0.03 - 0.5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 7 ;
	}
#Sedimentation of sea salt aerosol (0.5 - 5 um)
'Sedimentation of sea salt aerosol (0.5 - 5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 8 ;
	}
#Sedimentation of sea salt aerosol (5 - 20 um)
'Sedimentation of sea salt aerosol (5 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 9 ;
	}
#Wet deposition of sea salt aerosol (0.03 - 0.5 um) by large-scale precipitation
'Wet deposition of sea salt aerosol (0.03 - 0.5 um) by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 10 ;
	}
#Wet deposition of sea salt aerosol (0.5 - 5 um) by large-scale precipitation
'Wet deposition of sea salt aerosol (0.5 - 5 um) by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 11 ;
	}
#Wet deposition of sea salt aerosol (5 - 20 um) by large-scale precipitation
'Wet deposition of sea salt aerosol (5 - 20 um) by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 12 ;
	}
#Wet deposition of sea salt aerosol (0.03 - 0.5 um) by convective precipitation
'Wet deposition of sea salt aerosol (0.03 - 0.5 um) by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 13 ;
	}
#Wet deposition of sea salt aerosol (0.5 - 5 um) by convective precipitation
'Wet deposition of sea salt aerosol (0.5 - 5 um) by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 14 ;
	}
#Wet deposition of sea salt aerosol (5 - 20 um) by convective precipitation
'Wet deposition of sea salt aerosol (5 - 20 um) by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 15 ;
	}
#Negative fixer of sea salt aerosol (0.03 - 0.5 um)
'Negative fixer of sea salt aerosol (0.03 - 0.5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 16 ;
	}
#Negative fixer of sea salt aerosol (0.5 - 5 um)
'Negative fixer of sea salt aerosol (0.5 - 5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 17 ;
	}
#Negative fixer of sea salt aerosol (5 - 20 um)
'Negative fixer of sea salt aerosol (5 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 18 ;
	}
#Vertically integrated mass of sea salt aerosol (0.03 - 0.5 um)
'Vertically integrated mass of sea salt aerosol (0.03 - 0.5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 19 ;
	}
#Vertically integrated mass of sea salt aerosol (0.5 - 5 um)
'Vertically integrated mass of sea salt aerosol (0.5 - 5 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 20 ;
	}
#Vertically integrated mass of sea salt aerosol (5 - 20 um)
'Vertically integrated mass of sea salt aerosol (5 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 21 ;
	}
#Sea salt aerosol (0.03 - 0.5 um) optical depth
'Sea salt aerosol (0.03 - 0.5 um) optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 22 ;
	}
#Sea salt aerosol (0.5 - 5 um) optical depth
'Sea salt aerosol (0.5 - 5 um) optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 23 ;
	}
#Sea salt aerosol (5 - 20 um) optical depth
'Sea salt aerosol (5 - 20 um) optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 24 ;
	}
#Source/gain of dust aerosol (0.03 - 0.55 um)
'Source/gain of dust aerosol (0.03 - 0.55 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 25 ;
	}
#Source/gain of dust aerosol (0.55 - 0.9 um)
'Source/gain of dust aerosol (0.55 - 0.9 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 26 ;
	}
#Source/gain of dust aerosol (0.9 - 20 um)
'Source/gain of dust aerosol (0.9 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 27 ;
	}
#Dry deposition of dust aerosol (0.03 - 0.55 um)
'Dry deposition of dust aerosol (0.03 - 0.55 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 28 ;
	}
#Dry deposition of dust aerosol (0.55 - 0.9 um)
'Dry deposition of dust aerosol (0.55 - 0.9 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 29 ;
	}
#Dry deposition of dust aerosol (0.9 - 20 um)
'Dry deposition of dust aerosol (0.9 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 30 ;
	}
#Sedimentation of dust aerosol (0.03 - 0.55 um)
'Sedimentation of dust aerosol (0.03 - 0.55 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 31 ;
	}
#Sedimentation of dust aerosol (0.55 - 0.9 um)
'Sedimentation of dust aerosol (0.55 - 0.9 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 32 ;
	}
#Sedimentation of dust aerosol (0.9 - 20 um)
'Sedimentation of dust aerosol (0.9 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 33 ;
	}
#Wet deposition of dust aerosol (0.03 - 0.55 um) by large-scale precipitation
'Wet deposition of dust aerosol (0.03 - 0.55 um) by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 34 ;
	}
#Wet deposition of dust aerosol (0.55 - 0.9 um) by large-scale precipitation
'Wet deposition of dust aerosol (0.55 - 0.9 um) by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 35 ;
	}
#Wet deposition of dust aerosol (0.9 - 20 um) by large-scale precipitation
'Wet deposition of dust aerosol (0.9 - 20 um) by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 36 ;
	}
#Wet deposition of dust aerosol (0.03 - 0.55 um) by convective precipitation
'Wet deposition of dust aerosol (0.03 - 0.55 um) by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 37 ;
	}
#Wet deposition of dust aerosol (0.55 - 0.9 um) by convective precipitation
'Wet deposition of dust aerosol (0.55 - 0.9 um) by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 38 ;
	}
#Wet deposition of dust aerosol (0.9 - 20 um) by convective precipitation
'Wet deposition of dust aerosol (0.9 - 20 um) by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 39 ;
	}
#Negative fixer of dust aerosol (0.03 - 0.55 um)
'Negative fixer of dust aerosol (0.03 - 0.55 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 40 ;
	}
#Negative fixer of dust aerosol (0.55 - 0.9 um)
'Negative fixer of dust aerosol (0.55 - 0.9 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 41 ;
	}
#Negative fixer of dust aerosol (0.9 - 20 um)
'Negative fixer of dust aerosol (0.9 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 42 ;
	}
#Vertically integrated mass of dust aerosol (0.03 - 0.55 um)
'Vertically integrated mass of dust aerosol (0.03 - 0.55 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 43 ;
	}
#Vertically integrated mass of dust aerosol (0.55 - 0.9 um)
'Vertically integrated mass of dust aerosol (0.55 - 0.9 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 44 ;
	}
#Vertically integrated mass of dust aerosol (0.9 - 20 um)
'Vertically integrated mass of dust aerosol (0.9 - 20 um)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 45 ;
	}
#Dust aerosol (0.03 - 0.55 um) optical depth
'Dust aerosol (0.03 - 0.55 um) optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 46 ;
	}
#Dust aerosol (0.55 - 0.9 um) optical depth
'Dust aerosol (0.55 - 0.9 um) optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 47 ;
	}
#Dust aerosol (0.9 - 20 um) optical depth
'Dust aerosol (0.9 - 20 um) optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 48 ;
	}
#Source/gain of hydrophobic organic matter aerosol
'Source/gain of hydrophobic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 49 ;
	}
#Source/gain of hydrophilic organic matter aerosol
'Source/gain of hydrophilic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 50 ;
	}
#Dry deposition of hydrophobic organic matter aerosol
'Dry deposition of hydrophobic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 51 ;
	}
#Dry deposition of hydrophilic organic matter aerosol
'Dry deposition of hydrophilic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 52 ;
	}
#Sedimentation of hydrophobic organic matter aerosol
'Sedimentation of hydrophobic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 53 ;
	}
#Sedimentation of hydrophilic organic matter aerosol
'Sedimentation of hydrophilic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 54 ;
	}
#Wet deposition of hydrophobic organic matter aerosol by large-scale precipitation
'Wet deposition of hydrophobic organic matter aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 55 ;
	}
#Wet deposition of hydrophilic organic matter aerosol by large-scale precipitation
'Wet deposition of hydrophilic organic matter aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 56 ;
	}
#Wet deposition of hydrophobic organic matter aerosol by convective precipitation
'Wet deposition of hydrophobic organic matter aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 57 ;
	}
#Wet deposition of hydrophilic organic matter aerosol by convective precipitation
'Wet deposition of hydrophilic organic matter aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 58 ;
	}
#Negative fixer of hydrophobic organic matter aerosol
'Negative fixer of hydrophobic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 59 ;
	}
#Negative fixer of hydrophilic organic matter aerosol
'Negative fixer of hydrophilic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 60 ;
	}
#Vertically integrated mass of hydrophobic organic matter aerosol
'Vertically integrated mass of hydrophobic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 61 ;
	}
#Vertically integrated mass of hydrophilic organic matter aerosol
'Vertically integrated mass of hydrophilic organic matter aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 62 ;
	}
#Hydrophobic organic matter aerosol optical depth
'Hydrophobic organic matter aerosol optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 63 ;
	}
#Hydrophilic organic matter aerosol optical depth
'Hydrophilic organic matter aerosol optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 64 ;
	}
#Source/gain of hydrophobic black carbon aerosol
'Source/gain of hydrophobic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 65 ;
	}
#Source/gain of hydrophilic black carbon aerosol
'Source/gain of hydrophilic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 66 ;
	}
#Dry deposition of hydrophobic black carbon aerosol
'Dry deposition of hydrophobic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 67 ;
	}
#Dry deposition of hydrophilic black carbon aerosol
'Dry deposition of hydrophilic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 68 ;
	}
#Sedimentation of hydrophobic black carbon aerosol
'Sedimentation of hydrophobic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 69 ;
	}
#Sedimentation of hydrophilic black carbon aerosol
'Sedimentation of hydrophilic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 70 ;
	}
#Wet deposition of hydrophobic black carbon aerosol by large-scale precipitation
'Wet deposition of hydrophobic black carbon aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 71 ;
	}
#Wet deposition of hydrophilic black carbon aerosol by large-scale precipitation
'Wet deposition of hydrophilic black carbon aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 72 ;
	}
#Wet deposition of hydrophobic black carbon aerosol by convective precipitation
'Wet deposition of hydrophobic black carbon aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 73 ;
	}
#Wet deposition of hydrophilic black carbon aerosol by convective precipitation
'Wet deposition of hydrophilic black carbon aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 74 ;
	}
#Negative fixer of hydrophobic black carbon aerosol
'Negative fixer of hydrophobic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 75 ;
	}
#Negative fixer of hydrophilic black carbon aerosol
'Negative fixer of hydrophilic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 76 ;
	}
#Vertically integrated mass of hydrophobic black carbon aerosol
'Vertically integrated mass of hydrophobic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 77 ;
	}
#Vertically integrated mass of hydrophilic black carbon aerosol
'Vertically integrated mass of hydrophilic black carbon aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 78 ;
	}
#Hydrophobic black carbon aerosol optical depth
'Hydrophobic black carbon aerosol optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 79 ;
	}
#Hydrophilic black carbon aerosol optical depth
'Hydrophilic black carbon aerosol optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 80 ;
	}
#Source/gain of sulphate aerosol
'Source/gain of sulphate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 81 ;
	}
#Dry deposition of sulphate aerosol
'Dry deposition of sulphate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 82 ;
	}
#Sedimentation of sulphate aerosol
'Sedimentation of sulphate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 83 ;
	}
#Wet deposition of sulphate aerosol by large-scale precipitation
'Wet deposition of sulphate aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 84 ;
	}
#Wet deposition of sulphate aerosol by convective precipitation
'Wet deposition of sulphate aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 85 ;
	}
#Negative fixer of sulphate aerosol
'Negative fixer of sulphate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 86 ;
	}
#Vertically integrated mass of sulphate aerosol
'Vertically integrated mass of sulphate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 87 ;
	}
#Sulphate aerosol optical depth
'Sulphate aerosol optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 88 ;
	}
#Accumulated total aerosol optical depth at 550 nm
'Accumulated total aerosol optical depth at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 89 ;
	}
#Effective (snow effect included) UV visible albedo for direct radiation
'Effective (snow effect included) UV visible albedo for direct radiation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 90 ;
	}
#10 metre wind speed dust emission potential
'10 metre wind speed dust emission potential' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 91 ;
	}
#10 metre wind gustiness dust emission potential
'10 metre wind gustiness dust emission potential' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 92 ;
	}
#Total aerosol optical thickness at 532 nm
'Total aerosol optical thickness at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 93 ;
	}
#Natural (sea-salt and dust) aerosol optical thickness at 532 nm
'Natural (sea-salt and dust) aerosol optical thickness at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 94 ;
	}
#Anthropogenic (black carbon, organic matter, sulphate) aerosol optical thickness at 532 nm
'Anthropogenic (black carbon, organic matter, sulphate) aerosol optical thickness at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 95 ;
	}
#Total absorption aerosol optical depth at 340 nm
'Total absorption aerosol optical depth at 340 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 96 ;
	}
#Total absorption aerosol optical depth at 355 nm
'Total absorption aerosol optical depth at 355 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 97 ;
	}
#Total absorption aerosol optical depth at 380 nm
'Total absorption aerosol optical depth at 380 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 98 ;
	}
#Total absorption aerosol optical depth at 400 nm
'Total absorption aerosol optical depth at 400 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 99 ;
	}
#Total absorption aerosol optical depth at 440 nm
'Total absorption aerosol optical depth at 440 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 100 ;
	}
#Total absorption aerosol optical depth at 469 nm
'Total absorption aerosol optical depth at 469 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 101 ;
	}
#Total absorption aerosol optical depth at 500 nm
'Total absorption aerosol optical depth at 500 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 102 ;
	}
#Total absorption aerosol optical depth at 532 nm
'Total absorption aerosol optical depth at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 103 ;
	}
#Total absorption aerosol optical depth at 550 nm
'Total absorption aerosol optical depth at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 104 ;
	}
#Total absorption aerosol optical depth at 645 nm
'Total absorption aerosol optical depth at 645 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 105 ;
	}
#Total absorption aerosol optical depth at 670 nm
'Total absorption aerosol optical depth at 670 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 106 ;
	}
#Total absorption aerosol optical depth at 800 nm
'Total absorption aerosol optical depth at 800 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 107 ;
	}
#Total absorption aerosol optical depth at 858 nm
'Total absorption aerosol optical depth at 858 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 108 ;
	}
#Total absorption aerosol optical depth at 865 nm
'Total absorption aerosol optical depth at 865 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 109 ;
	}
#Total absorption aerosol optical depth at 1020 nm
'Total absorption aerosol optical depth at 1020 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 110 ;
	}
#Total absorption aerosol optical depth at 1064 nm
'Total absorption aerosol optical depth at 1064 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 111 ;
	}
#Total absorption aerosol optical depth at 1240 nm
'Total absorption aerosol optical depth at 1240 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 112 ;
	}
#Total absorption aerosol optical depth at 1640 nm
'Total absorption aerosol optical depth at 1640 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 113 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 340 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 340 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 114 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 355 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 355 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 115 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 380 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 380 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 116 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 400 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 400 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 117 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 440 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 440 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 118 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 469 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 469 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 119 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 500 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 500 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 120 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 532 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 121 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 550 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 122 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 645 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 645 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 123 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 670 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 670 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 124 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 800 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 800 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 125 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 858 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 858 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 126 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 865 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 865 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 127 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 1020 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 1020 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 128 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 1064 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 1064 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 129 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 1240 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 1240 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 130 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 1640 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 1640 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 131 ;
	}
#Single scattering albedo at 340 nm
'Single scattering albedo at 340 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 132 ;
	}
#Single scattering albedo at 355 nm
'Single scattering albedo at 355 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 133 ;
	}
#Single scattering albedo at 380 nm
'Single scattering albedo at 380 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 134 ;
	}
#Single scattering albedo at 400 nm
'Single scattering albedo at 400 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 135 ;
	}
#Single scattering albedo at 440 nm
'Single scattering albedo at 440 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 136 ;
	}
#Single scattering albedo at 469 nm
'Single scattering albedo at 469 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 137 ;
	}
#Single scattering albedo at 500 nm
'Single scattering albedo at 500 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 138 ;
	}
#Single scattering albedo at 532 nm
'Single scattering albedo at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 139 ;
	}
#Single scattering albedo at 550 nm
'Single scattering albedo at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 140 ;
	}
#Single scattering albedo at 645 nm
'Single scattering albedo at 645 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 141 ;
	}
#Single scattering albedo at 670 nm
'Single scattering albedo at 670 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 142 ;
	}
#Single scattering albedo at 800 nm
'Single scattering albedo at 800 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 143 ;
	}
#Single scattering albedo at 858 nm
'Single scattering albedo at 858 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 144 ;
	}
#Single scattering albedo at 865 nm
'Single scattering albedo at 865 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 145 ;
	}
#Single scattering albedo at 1020 nm
'Single scattering albedo at 1020 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 146 ;
	}
#Single scattering albedo at 1064 nm
'Single scattering albedo at 1064 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 147 ;
	}
#Single scattering albedo at 1240 nm
'Single scattering albedo at 1240 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 148 ;
	}
#Single scattering albedo at 1640 nm
'Single scattering albedo at 1640 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 149 ;
	}
#Asymmetry factor at 340 nm
'Asymmetry factor at 340 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 150 ;
	}
#Asymmetry factor at 355 nm
'Asymmetry factor at 355 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 151 ;
	}
#Asymmetry factor at 380 nm
'Asymmetry factor at 380 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 152 ;
	}
#Asymmetry factor at 400 nm
'Asymmetry factor at 400 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 153 ;
	}
#Asymmetry factor at 440 nm
'Asymmetry factor at 440 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 154 ;
	}
#Asymmetry factor at 469 nm
'Asymmetry factor at 469 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 155 ;
	}
#Asymmetry factor at 500 nm
'Asymmetry factor at 500 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 156 ;
	}
#Asymmetry factor at 532 nm
'Asymmetry factor at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 157 ;
	}
#Asymmetry factor at 550 nm
'Asymmetry factor at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 158 ;
	}
#Asymmetry factor at 645 nm
'Asymmetry factor at 645 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 159 ;
	}
#Asymmetry factor at 670 nm
'Asymmetry factor at 670 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 160 ;
	}
#Asymmetry factor at 800 nm
'Asymmetry factor at 800 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 161 ;
	}
#Asymmetry factor at 858 nm
'Asymmetry factor at 858 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 162 ;
	}
#Asymmetry factor at 865 nm
'Asymmetry factor at 865 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 163 ;
	}
#Asymmetry factor at 1020 nm
'Asymmetry factor at 1020 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 164 ;
	}
#Asymmetry factor at 1064 nm
'Asymmetry factor at 1064 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 165 ;
	}
#Asymmetry factor at 1240 nm
'Asymmetry factor at 1240 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 166 ;
	}
#Asymmetry factor at 1640 nm
'Asymmetry factor at 1640 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 167 ;
	}
#Source/gain of sulphur dioxide
'Source/gain of sulphur dioxide' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 168 ;
	}
#Dry deposition of sulphur dioxide
'Dry deposition of sulphur dioxide' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 169 ;
	}
#Sedimentation of sulphur dioxide
'Sedimentation of sulphur dioxide' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 170 ;
	}
#Wet deposition of sulphur dioxide by large-scale precipitation
'Wet deposition of sulphur dioxide by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 171 ;
	}
#Wet deposition of sulphur dioxide by convective precipitation
'Wet deposition of sulphur dioxide by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 172 ;
	}
#Negative fixer of sulphur dioxide
'Negative fixer of sulphur dioxide' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 173 ;
	}
#Vertically integrated mass of sulphur dioxide
'Vertically integrated mass of sulphur dioxide' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 174 ;
	}
#Sulphur dioxide optical depth
'Sulphur dioxide optical depth' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 175 ;
	}
#Total absorption aerosol optical depth at 2130 nm
'Total absorption aerosol optical depth at 2130 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 176 ;
	}
#Total fine mode (r < 0.5 um) aerosol optical depth at 2130 nm
'Total fine mode (r < 0.5 um) aerosol optical depth at 2130 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 177 ;
	}
#Single scattering albedo at 2130 nm
'Single scattering albedo at 2130 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 178 ;
	}
#Asymmetry factor at 2130 nm
'Asymmetry factor at 2130 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 179 ;
	}
#Aerosol extinction coefficient at 355 nm
'Aerosol extinction coefficient at 355 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 180 ;
	}
#Aerosol extinction coefficient at 532 nm
'Aerosol extinction coefficient at 532 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 181 ;
	}
#Aerosol extinction coefficient at 1064 nm
'Aerosol extinction coefficient at 1064 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 182 ;
	}
#Aerosol backscatter coefficient at 355 nm (from top of atmosphere)
'Aerosol backscatter coefficient at 355 nm (from top of atmosphere)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 183 ;
	}
#Aerosol backscatter coefficient at 532 nm (from top of atmosphere)
'Aerosol backscatter coefficient at 532 nm (from top of atmosphere)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 184 ;
	}
#Aerosol backscatter coefficient at 1064 nm (from top of atmosphere)
'Aerosol backscatter coefficient at 1064 nm (from top of atmosphere)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 185 ;
	}
#Aerosol backscatter coefficient at 355 nm (from ground)
'Aerosol backscatter coefficient at 355 nm (from ground)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 186 ;
	}
#Aerosol backscatter coefficient at 532 nm (from ground)
'Aerosol backscatter coefficient at 532 nm (from ground)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 187 ;
	}
#Aerosol backscatter coefficient at 1064 nm (from ground)
'Aerosol backscatter coefficient at 1064 nm (from ground)' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 188 ;
	}
#Source/gain of fine-mode nitrate aerosol
'Source/gain of fine-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 189 ;
	}
#Source/gain of coarse-mode nitrate aerosol
'Source/gain of coarse-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 190 ;
	}
#Dry deposition of fine-mode nitrate aerosol
'Dry deposition of fine-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 191 ;
	}
#Dry deposition of coarse-mode nitrate aerosol
'Dry deposition of coarse-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 192 ;
	}
#Sedimentation of fine-mode nitrate aerosol
'Sedimentation of fine-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 193 ;
	}
#Sedimentation of coarse-mode nitrate aerosol
'Sedimentation of coarse-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 194 ;
	}
#Wet deposition of fine-mode nitrate aerosol by large-scale precipitation
'Wet deposition of fine-mode nitrate aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 195 ;
	}
#Wet deposition of coarse-mode nitrate aerosol by large-scale precipitation
'Wet deposition of coarse-mode nitrate aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 196 ;
	}
#Wet deposition of fine-mode nitrate aerosol by convective precipitation
'Wet deposition of fine-mode nitrate aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 197 ;
	}
#Wet deposition of coarse-mode nitrate aerosol by convective precipitation
'Wet deposition of coarse-mode nitrate aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 198 ;
	}
#Negative fixer of fine-mode nitrate aerosol
'Negative fixer of fine-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 199 ;
	}
#Negative fixer of coarse-mode nitrate aerosol
'Negative fixer of coarse-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 200 ;
	}
#Vertically integrated mass of fine-mode nitrate aerosol
'Vertically integrated mass of fine-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 201 ;
	}
#Vertically integrated mass of coarse-mode nitrate aerosol
'Vertically integrated mass of coarse-mode nitrate aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 202 ;
	}
#Fine-mode nitrate aerosol optical depth at 550 nm
'Fine-mode nitrate aerosol optical depth at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 203 ;
	}
#Coarse-mode nitrate aerosol optical depth at 550 nm
'Coarse-mode nitrate aerosol optical depth at 550 nm' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 204 ;
	}
#Source/gain of ammonium aerosol
'Source/gain of ammonium aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 205 ;
	}
#Dry deposition of ammonium aerosol
'Dry deposition of ammonium aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 206 ;
	}
#Sedimentation of ammonium aerosol
'Sedimentation of ammonium aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 207 ;
	}
#Wet deposition of ammonium aerosol by large-scale precipitation
'Wet deposition of ammonium aerosol by large-scale precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 208 ;
	}
#Wet deposition of ammonium aerosol by convective precipitation
'Wet deposition of ammonium aerosol by convective precipitation' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 209 ;
	}
#Negative fixer of ammonium aerosol
'Negative fixer of ammonium aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 210 ;
	}
#Vertically integrated mass of ammonium aerosol
'Vertically integrated mass of ammonium aerosol' = {
	 table2Version = 215 ;
	 indicatorOfParameter = 211 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 1 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 2 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 3 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 4 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 5 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 6 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 7 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 8 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 9 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 10 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 11 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 12 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 13 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 14 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 15 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 16 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 17 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 18 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 19 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 20 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 21 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 22 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 23 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 24 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 25 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 26 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 27 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 28 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 29 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 30 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 31 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 32 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 33 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 34 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 35 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 36 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 37 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 38 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 39 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 40 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 41 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 42 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 43 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 44 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 45 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 46 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 47 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 48 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 49 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 50 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 51 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 52 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 53 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 54 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 55 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 56 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 57 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 58 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 59 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 60 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 61 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 62 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 63 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 64 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 65 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 66 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 67 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 68 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 69 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 70 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 71 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 72 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 73 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 74 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 75 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 76 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 77 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 78 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 79 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 80 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 81 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 82 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 83 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 84 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 85 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 86 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 87 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 88 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 89 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 90 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 91 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 92 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 93 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 94 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 95 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 96 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 97 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 98 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 99 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 100 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 101 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 102 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 103 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 104 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 105 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 106 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 107 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 108 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 109 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 110 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 111 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 112 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 113 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 114 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 115 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 116 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 117 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 118 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 119 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 120 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 121 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 122 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 123 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 124 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 125 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 126 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 127 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 128 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 129 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 130 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 131 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 132 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 133 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 134 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 135 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 136 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 137 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 138 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 139 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 140 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 141 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 142 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 143 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 144 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 145 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 146 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 147 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 148 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 149 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 150 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 151 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 152 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 153 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 154 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 155 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 156 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 157 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 158 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 159 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 160 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 161 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 162 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 163 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 164 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 165 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 166 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 167 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 168 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 169 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 170 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 171 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 172 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 173 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 174 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 175 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 176 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 177 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 178 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 179 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 180 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 181 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 182 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 183 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 184 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 185 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 186 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 187 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 188 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 189 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 190 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 191 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 192 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 193 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 194 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 195 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 196 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 197 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 198 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 199 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 200 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 201 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 202 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 203 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 204 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 205 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 206 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 207 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 208 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 209 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 210 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 211 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 212 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 213 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 214 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 215 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 216 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 217 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 218 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 219 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 220 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 221 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 222 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 223 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 224 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 225 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 226 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 227 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 228 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 229 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 230 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 231 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 232 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 233 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 234 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 235 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 236 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 237 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 238 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 239 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 240 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 241 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 242 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 243 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 244 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 245 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 246 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 247 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 248 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 249 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 250 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 251 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 252 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 253 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 254 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 216 ;
	 indicatorOfParameter = 255 ;
	}
#Hydrogen peroxide
'Hydrogen peroxide' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 3 ;
	}
#Methane (chemistry)
'Methane (chemistry)' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 4 ;
	}
#Nitric acid
'Nitric acid' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 6 ;
	}
#Methyl peroxide
'Methyl peroxide' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 7 ;
	}
#Paraffins
'Paraffins' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 9 ;
	}
#Ethene
'Ethene' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 10 ;
	}
#Olefins
'Olefins' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 11 ;
	}
#Aldehydes
'Aldehydes' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 12 ;
	}
#Peroxyacetyl nitrate
'Peroxyacetyl nitrate' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 13 ;
	}
#Peroxides
'Peroxides' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 14 ;
	}
#Organic nitrates
'Organic nitrates' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 15 ;
	}
#Isoprene
'Isoprene' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 16 ;
	}
#Dimethyl sulfide
'Dimethyl sulfide' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 18 ;
	}
#Ammonia mass mixing ratio
'Ammonia mass mixing ratio' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 19 ;
	}
#Sulfate
'Sulfate' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 20 ;
	}
#Ammonium
'Ammonium' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 21 ;
	}
#Methane sulfonic acid
'Methane sulfonic acid' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 22 ;
	}
#Methyl glyoxal
'Methyl glyoxal' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 23 ;
	}
#Stratospheric ozone
'Stratospheric ozone' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 24 ;
	}
#Lead
'Lead' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 26 ;
	}
#Nitrogen monoxide mass mixing ratio
'Nitrogen monoxide mass mixing ratio' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 27 ;
	}
#Hydroperoxy radical
'Hydroperoxy radical' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 28 ;
	}
#Methylperoxy radical
'Methylperoxy radical' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 29 ;
	}
#Hydroxyl radical
'Hydroxyl radical' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 30 ;
	}
#Nitrate radical
'Nitrate radical' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 32 ;
	}
#Dinitrogen pentoxide
'Dinitrogen pentoxide' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 33 ;
	}
#Pernitric acid
'Pernitric acid' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 34 ;
	}
#Peroxy acetyl radical
'Peroxy acetyl radical' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 35 ;
	}
#Organic ethers
'Organic ethers' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 36 ;
	}
#PAR budget corrector
'PAR budget corrector' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 37 ;
	}
#NO to NO2 operator
'NO to NO2 operator' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 38 ;
	}
#NO to alkyl nitrate operator
'NO to alkyl nitrate operator' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 39 ;
	}
#Amine
'Amine' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 40 ;
	}
#Polar stratospheric cloud
'Polar stratospheric cloud' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 41 ;
	}
#Methanol
'Methanol' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 42 ;
	}
#Formic acid
'Formic acid' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 43 ;
	}
#Methacrylic acid
'Methacrylic acid' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 44 ;
	}
#Ethane
'Ethane' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 45 ;
	}
#Ethanol
'Ethanol' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 46 ;
	}
#Propane
'Propane' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 47 ;
	}
#Propene
'Propene' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 48 ;
	}
#Terpenes
'Terpenes' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 49 ;
	}
#Methacrolein MVK
'Methacrolein MVK' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 50 ;
	}
#Nitrate
'Nitrate' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 51 ;
	}
#Acetone
'Acetone' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 52 ;
	}
#Acetone product
'Acetone product' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 53 ;
	}
#IC3H7O2
'IC3H7O2' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 54 ;
	}
#HYPROPO2
'HYPROPO2' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 55 ;
	}
#Nitrogen oxides Transp
'Nitrogen oxides Transp' = {
	 table2Version = 217 ;
	 indicatorOfParameter = 56 ;
	}
#Total column hydrogen peroxide
'Total column hydrogen peroxide' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 3 ;
	}
#Total column methane
'Total column methane' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 4 ;
	}
#Total column nitric acid
'Total column nitric acid' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 6 ;
	}
#Total column methyl peroxide
'Total column methyl peroxide' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 7 ;
	}
#Total column paraffins
'Total column paraffins' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 9 ;
	}
#Total column ethene
'Total column ethene' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 10 ;
	}
#Total column olefins
'Total column olefins' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 11 ;
	}
#Total column aldehydes
'Total column aldehydes' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 12 ;
	}
#Total column  peroxyacetyl nitrate
'Total column  peroxyacetyl nitrate' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 13 ;
	}
#Total column peroxides
'Total column peroxides' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 14 ;
	}
#Total column organic nitrates
'Total column organic nitrates' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 15 ;
	}
#Total column  isoprene
'Total column  isoprene' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 16 ;
	}
#Total column dimethyl sulfide
'Total column dimethyl sulfide' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 18 ;
	}
#Total column ammonia
'Total column ammonia' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 19 ;
	}
#Total column  sulfate
'Total column  sulfate' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 20 ;
	}
#Total column ammonium
'Total column ammonium' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 21 ;
	}
#Total column  methane sulfonic acid
'Total column  methane sulfonic acid' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 22 ;
	}
#Total column methyl glyoxal
'Total column methyl glyoxal' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 23 ;
	}
#Total column stratospheric ozone
'Total column stratospheric ozone' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 24 ;
	}
#Total column  lead
'Total column  lead' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 26 ;
	}
#Total column nitrogen monoxide
'Total column nitrogen monoxide' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 27 ;
	}
#Total column hydroperoxy radical
'Total column hydroperoxy radical' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 28 ;
	}
#Total column methylperoxy radical
'Total column methylperoxy radical' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 29 ;
	}
#Total column hydroxyl radical
'Total column hydroxyl radical' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 30 ;
	}
#Total column nitrate radical
'Total column nitrate radical' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 32 ;
	}
#Total column dinitrogen pentoxide
'Total column dinitrogen pentoxide' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 33 ;
	}
#Total column pernitric acid
'Total column pernitric acid' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 34 ;
	}
#Total column peroxy acetyl radical
'Total column peroxy acetyl radical' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 35 ;
	}
#Total column  organic ethers
'Total column  organic ethers' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 36 ;
	}
#Total column PAR budget corrector
'Total column PAR budget corrector' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 37 ;
	}
#Total column NO to NO2 operator
'Total column NO to NO2 operator' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 38 ;
	}
#Total column NO to alkyl nitrate operator
'Total column NO to alkyl nitrate operator' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 39 ;
	}
#Total column amine
'Total column amine' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 40 ;
	}
#Total column  polar stratospheric cloud
'Total column  polar stratospheric cloud' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 41 ;
	}
#Total column methanol
'Total column methanol' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 42 ;
	}
#Total column formic acid
'Total column formic acid' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 43 ;
	}
#Total column  methacrylic acid
'Total column  methacrylic acid' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 44 ;
	}
#Total column  ethane
'Total column  ethane' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 45 ;
	}
#Total column ethanol
'Total column ethanol' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 46 ;
	}
#Total column propane
'Total column propane' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 47 ;
	}
#Total column propene
'Total column propene' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 48 ;
	}
#Total column terpenes
'Total column terpenes' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 49 ;
	}
#Total column methacrolein MVK
'Total column methacrolein MVK' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 50 ;
	}
#Total column nitrate
'Total column nitrate' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 51 ;
	}
#Total column acetone
'Total column acetone' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 52 ;
	}
#Total column acetone product
'Total column acetone product' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 53 ;
	}
#Total column IC3H7O2
'Total column IC3H7O2' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 54 ;
	}
#Total column HYPROPO2
'Total column HYPROPO2' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 55 ;
	}
#Total column nitrogen oxides Transp
'Total column nitrogen oxides Transp' = {
	 table2Version = 218 ;
	 indicatorOfParameter = 56 ;
	}
#Ozone emissions
'Ozone emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 1 ;
	}
#Nitrogen oxides emissions
'Nitrogen oxides emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 2 ;
	}
#Hydrogen peroxide emissions
'Hydrogen peroxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 3 ;
	}
#Methane emissions
'Methane emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 4 ;
	}
#Carbon monoxide emissions
'Carbon monoxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 5 ;
	}
#Nitric acid emissions
'Nitric acid emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 6 ;
	}
#Methyl peroxide emissions
'Methyl peroxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 7 ;
	}
#Formaldehyde emissions
'Formaldehyde emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 8 ;
	}
#Paraffins emissions
'Paraffins emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 9 ;
	}
#Ethene emissions
'Ethene emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 10 ;
	}
#Olefins emissions
'Olefins emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 11 ;
	}
#Aldehydes emissions
'Aldehydes emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 12 ;
	}
#Peroxyacetyl nitrate emissions
'Peroxyacetyl nitrate emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 13 ;
	}
#Peroxides emissions
'Peroxides emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 14 ;
	}
#Organic nitrates emissions
'Organic nitrates emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 15 ;
	}
#Isoprene emissions
'Isoprene emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 16 ;
	}
#Sulfur dioxide emissions
'Sulfur dioxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 17 ;
	}
#Dimethyl sulfide emissions
'Dimethyl sulfide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 18 ;
	}
#Ammonia emissions
'Ammonia emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 19 ;
	}
#Sulfate emissions
'Sulfate emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 20 ;
	}
#Ammonium emissions
'Ammonium emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 21 ;
	}
#Methane sulfonic acid emissions
'Methane sulfonic acid emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 22 ;
	}
#Methyl glyoxal emissions
'Methyl glyoxal emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 23 ;
	}
#Stratospheric ozone emissions
'Stratospheric ozone emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 24 ;
	}
#Radon emissions
'Radon emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 25 ;
	}
#Lead emissions
'Lead emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 26 ;
	}
#Nitrogen monoxide emissions
'Nitrogen monoxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 27 ;
	}
#Hydroperoxy radical emissions
'Hydroperoxy radical emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 28 ;
	}
#Methylperoxy radical emissions
'Methylperoxy radical emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 29 ;
	}
#Hydroxyl radical emissions
'Hydroxyl radical emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 30 ;
	}
#Nitrogen dioxide emissions
'Nitrogen dioxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 31 ;
	}
#Nitrate radical emissions
'Nitrate radical emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 32 ;
	}
#Dinitrogen pentoxide emissions
'Dinitrogen pentoxide emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 33 ;
	}
#Pernitric acid emissions
'Pernitric acid emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 34 ;
	}
#Peroxy acetyl radical emissions
'Peroxy acetyl radical emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 35 ;
	}
#Organic ethers emissions
'Organic ethers emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 36 ;
	}
#PAR budget corrector emissions
'PAR budget corrector emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 37 ;
	}
#NO to NO2 operator emissions
'NO to NO2 operator emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 38 ;
	}
#NO to alkyl nitrate operator emissions
'NO to alkyl nitrate operator emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 39 ;
	}
#Amine emissions
'Amine emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 40 ;
	}
#Polar stratospheric cloud emissions
'Polar stratospheric cloud emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 41 ;
	}
#Methanol emissions
'Methanol emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 42 ;
	}
#Formic acid emissions
'Formic acid emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 43 ;
	}
#Methacrylic acid emissions
'Methacrylic acid emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 44 ;
	}
#Ethane emissions
'Ethane emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 45 ;
	}
#Ethanol emissions
'Ethanol emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 46 ;
	}
#Propane emissions
'Propane emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 47 ;
	}
#Propene emissions
'Propene emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 48 ;
	}
#Terpenes emissions
'Terpenes emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 49 ;
	}
#Methacrolein MVK  emissions
'Methacrolein MVK  emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 50 ;
	}
#Nitrate emissions
'Nitrate emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 51 ;
	}
#Acetone emissions
'Acetone emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 52 ;
	}
#Acetone product emissions
'Acetone product emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 53 ;
	}
#IC3H7O2 emissions
'IC3H7O2 emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 54 ;
	}
#HYPROPO2 emissions
'HYPROPO2 emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 55 ;
	}
#Nitrogen oxides Transp emissions
'Nitrogen oxides Transp emissions' = {
	 table2Version = 219 ;
	 indicatorOfParameter = 56 ;
	}
#Ozone deposition velocity
'Ozone deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 1 ;
	}
#Nitrogen oxides deposition velocity
'Nitrogen oxides deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 2 ;
	}
#Hydrogen peroxide deposition velocity
'Hydrogen peroxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 3 ;
	}
#Methane deposition velocity
'Methane deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 4 ;
	}
#Carbon monoxide deposition velocity
'Carbon monoxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 5 ;
	}
#Nitric acid deposition velocity
'Nitric acid deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 6 ;
	}
#Methyl peroxide deposition velocity
'Methyl peroxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 7 ;
	}
#Formaldehyde deposition velocity
'Formaldehyde deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 8 ;
	}
#Paraffins deposition velocity
'Paraffins deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 9 ;
	}
#Ethene deposition velocity
'Ethene deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 10 ;
	}
#Olefins deposition velocity
'Olefins deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 11 ;
	}
#Aldehydes deposition velocity
'Aldehydes deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 12 ;
	}
#Peroxyacetyl nitrate deposition velocity
'Peroxyacetyl nitrate deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 13 ;
	}
#Peroxides deposition velocity
'Peroxides deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 14 ;
	}
#Organic nitrates deposition velocity
'Organic nitrates deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 15 ;
	}
#Isoprene deposition velocity
'Isoprene deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 16 ;
	}
#Sulfur dioxide deposition velocity
'Sulfur dioxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 17 ;
	}
#Dimethyl sulfide deposition velocity
'Dimethyl sulfide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 18 ;
	}
#Ammonia deposition velocity
'Ammonia deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 19 ;
	}
#Sulfate deposition velocity
'Sulfate deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 20 ;
	}
#Ammonium deposition velocity
'Ammonium deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 21 ;
	}
#Methane sulfonic acid deposition velocity
'Methane sulfonic acid deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 22 ;
	}
#Methyl glyoxal deposition velocity
'Methyl glyoxal deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 23 ;
	}
#Stratospheric ozone deposition velocity
'Stratospheric ozone deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 24 ;
	}
#Radon deposition velocity
'Radon deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 25 ;
	}
#Lead deposition velocity
'Lead deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 26 ;
	}
#Nitrogen monoxide deposition velocity
'Nitrogen monoxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 27 ;
	}
#Hydroperoxy radical deposition velocity
'Hydroperoxy radical deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 28 ;
	}
#Methylperoxy radical deposition velocity
'Methylperoxy radical deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 29 ;
	}
#Hydroxyl radical deposition velocity
'Hydroxyl radical deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 30 ;
	}
#Nitrogen dioxide deposition velocity
'Nitrogen dioxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 31 ;
	}
#Nitrate radical deposition velocity
'Nitrate radical deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 32 ;
	}
#Dinitrogen pentoxide deposition velocity
'Dinitrogen pentoxide deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 33 ;
	}
#Pernitric acid deposition velocity
'Pernitric acid deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 34 ;
	}
#Peroxy acetyl radical deposition velocity
'Peroxy acetyl radical deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 35 ;
	}
#Organic ethers deposition velocity
'Organic ethers deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 36 ;
	}
#PAR budget corrector deposition velocity
'PAR budget corrector deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 37 ;
	}
#NO to NO2 operator deposition velocity
'NO to NO2 operator deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 38 ;
	}
#NO to alkyl nitrate operator deposition velocity
'NO to alkyl nitrate operator deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 39 ;
	}
#Amine deposition velocity
'Amine deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 40 ;
	}
#Polar stratospheric cloud deposition velocity
'Polar stratospheric cloud deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 41 ;
	}
#Methanol deposition velocity
'Methanol deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 42 ;
	}
#Formic acid deposition velocity
'Formic acid deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 43 ;
	}
#Methacrylic acid deposition velocity
'Methacrylic acid deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 44 ;
	}
#Ethane deposition velocity
'Ethane deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 45 ;
	}
#Ethanol deposition velocity
'Ethanol deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 46 ;
	}
#Propane deposition velocity
'Propane deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 47 ;
	}
#Propene deposition velocity
'Propene deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 48 ;
	}
#Terpenes deposition velocity
'Terpenes deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 49 ;
	}
#Methacrolein MVK  deposition velocity
'Methacrolein MVK  deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 50 ;
	}
#Nitrate deposition velocity
'Nitrate deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 51 ;
	}
#Acetone deposition velocity
'Acetone deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 52 ;
	}
#Acetone product deposition velocity
'Acetone product deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 53 ;
	}
#IC3H7O2 deposition velocity
'IC3H7O2 deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 54 ;
	}
#HYPROPO2 deposition velocity
'HYPROPO2 deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 55 ;
	}
#Nitrogen oxides Transp deposition velocity
'Nitrogen oxides Transp deposition velocity' = {
	 table2Version = 221 ;
	 indicatorOfParameter = 56 ;
	}
#Geometric height of -10 degrees C atmospheric isothermal level above ground
'Geometric height of -10 degrees C atmospheric isothermal level above ground' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 20 ;
	}
#Surface direct short-wave (solar) radiation
'Surface direct short-wave (solar) radiation' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 21 ;
	}
#Surface direct short-wave radiation, clear sky
'Surface direct short-wave radiation, clear sky' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 22 ;
	}
#Cloud base height
'Cloud base height' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 23 ;
	}
#Geometric height of 0 degrees C atmospheric isothermal level above ground
'Geometric height of 0 degrees C atmospheric isothermal level above ground' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 24 ;
	}
#Horizontal visibility
'Horizontal visibility' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 25 ;
	}
#Maximum temperature at 2 metres in the last 3 hours
'Maximum temperature at 2 metres in the last 3 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 26 ;
	}
#Minimum temperature at 2 metres in the last 3 hours
'Minimum temperature at 2 metres in the last 3 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 27 ;
	}
#Maximum 10 metre wind gust in the last 3 hours
'Maximum 10 metre wind gust in the last 3 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 28 ;
	}
#Instantaneous 10 metre wind gust
'Instantaneous 10 metre wind gust' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 29 ;
	}
#2 metre relative humidity with respect to water
'2 metre relative humidity with respect to water' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 37 ;
	}
#Soil wetness index in layer 1
'Soil wetness index in layer 1' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 40 ;
	}
#Soil wetness index in layer 2
'Soil wetness index in layer 2' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 41 ;
	}
#Soil wetness index in layer 3
'Soil wetness index in layer 3' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 42 ;
	}
#Soil wetness index in layer 4
'Soil wetness index in layer 4' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 43 ;
	}
#Convective available potential energy shear
'Convective available potential energy shear' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 44 ;
	}
#Height of convective cloud top
'Height of convective cloud top' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 46 ;
	}
#Height of zero-degree wet-bulb temperature
'Height of zero-degree wet-bulb temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 47 ;
	}
#Height of one-degree wet-bulb temperature
'Height of one-degree wet-bulb temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 48 ;
	}
#Instantaneous total lightning flash density
'Instantaneous total lightning flash density' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 50 ;
	}
#Averaged total lightning flash density in the last hour
'Averaged total lightning flash density in the last hour' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 51 ;
	}
#Instantaneous cloud-to-ground lightning flash density
'Instantaneous cloud-to-ground lightning flash density' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 52 ;
	}
#Averaged cloud-to-ground lightning flash density in the last hour
'Averaged cloud-to-ground lightning flash density in the last hour' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 53 ;
	}
#SMOS observed soil moisture retrieved using neural network
'SMOS observed soil moisture retrieved using neural network' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 70 ;
	}
#SMOS observed soil moisture uncertainty retrieved using neural network
'SMOS observed soil moisture uncertainty retrieved using neural network' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 71 ;
	}
#SMOS radio frequency interference probability
'SMOS radio frequency interference probability' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 72 ;
	}
#SMOS number of observations per grid point
'SMOS number of observations per grid point' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 73 ;
	}
#SMOS observation time for the satellite soil moisture data
'SMOS observation time for the satellite soil moisture data' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 74 ;
	}
#GPP coefficient from Biogenic Flux Adjustment System
'GPP coefficient from Biogenic Flux Adjustment System' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 78 ;
	}
#Rec coefficient from Biogenic Flux Adjustment System
'Rec coefficient from Biogenic Flux Adjustment System' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 79 ;
	}
#Accumulated Carbon Dioxide Net Ecosystem Exchange
'Accumulated Carbon Dioxide Net Ecosystem Exchange' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 80 ;
	}
#Accumulated Carbon Dioxide Gross Primary Production
'Accumulated Carbon Dioxide Gross Primary Production' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 81 ;
	}
#Accumulated Carbon Dioxide Ecosystem Respiration
'Accumulated Carbon Dioxide Ecosystem Respiration' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 82 ;
	}
#Carbon dioxide net ecosystem exchange flux
'Carbon dioxide net ecosystem exchange flux' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 83 ;
	}
#Carbon dioxide gross primary production flux
'Carbon dioxide gross primary production flux' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 84 ;
	}
#Carbon dioxide ecosystem respiration flux
'Carbon dioxide ecosystem respiration flux' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 85 ;
	}
#Total column supercooled liquid water
'Total column supercooled liquid water' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 88 ;
	}
#Total column rain water
'Total column rain water' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 89 ;
	}
#Total column snow water
'Total column snow water' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 90 ;
	}
#Canopy cover fraction
'Canopy cover fraction' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 91 ;
	}
#Soil texture fraction
'Soil texture fraction' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 92 ;
	}
#Volumetric soil moisture
'Volumetric soil moisture' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 93 ;
	}
#Ice temperature
'Ice temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 94 ;
	}
#Surface short-wave (solar) radiation downward clear-sky
'Surface short-wave (solar) radiation downward clear-sky' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 129 ;
	}
#Surface long-wave (thermal) radiation downward clear-sky
'Surface long-wave (thermal) radiation downward clear-sky' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 130 ;
	}
#Accumulated freezing rain
'Accumulated freezing rain' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 216 ;
	}
#Instantaneous large-scale precipitation fraction
'Instantaneous large-scale precipitation fraction' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 217 ;
	}
#Convective rain rate
'Convective rain rate' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 218 ;
	}
#Large scale rain rate
'Large scale rain rate' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 219 ;
	}
#Convective snowfall rate water equivalent
'Convective snowfall rate water equivalent' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 220 ;
	}
#Large scale snowfall rate water equivalent
'Large scale snowfall rate water equivalent' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 221 ;
	}
#Maximum total precipitation rate in the last 3 hours
'Maximum total precipitation rate in the last 3 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 222 ;
	}
#Minimum total precipitation rate in the last 3 hours
'Minimum total precipitation rate in the last 3 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 223 ;
	}
#Maximum total precipitation rate in the last 6 hours
'Maximum total precipitation rate in the last 6 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 224 ;
	}
#Minimum total precipitation rate in the last 6 hours
'Minimum total precipitation rate in the last 6 hours' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 225 ;
	}
#Maximum total precipitation rate since previous post-processing
'Maximum total precipitation rate since previous post-processing' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 226 ;
	}
#Minimum total precipitation rate since previous post-processing
'Minimum total precipitation rate since previous post-processing' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 227 ;
	}
#SMOS first Brightness Temperature Bias Correction parameter
'SMOS first Brightness Temperature Bias Correction parameter' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 229 ;
	}
#SMOS second Brightness Temperature Bias Correction parameter
'SMOS second Brightness Temperature Bias Correction parameter' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 230 ;
	}
#200 metre U wind component
'200 metre U wind component' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 239 ;
	}
#200 metre V wind component
'200 metre V wind component' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 240 ;
	}
#200 metre wind speed
'200 metre wind speed' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 241 ;
	}
#Surface solar radiation diffuse total sky
'Surface solar radiation diffuse total sky' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 242 ;
	}
#Surface solar radiation diffuse clear-sky
'Surface solar radiation diffuse clear-sky' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 243 ;
	}
#Surface albedo of direct radiation
'Surface albedo of direct radiation' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 244 ;
	}
#Surface albedo of diffuse radiation
'Surface albedo of diffuse radiation' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 245 ;
	}
#100 metre wind speed
'100 metre wind speed' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 249 ;
	}
#Irrigation fraction
'Irrigation fraction' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 250 ;
	}
#Potential evaporation
'Potential evaporation' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 251 ;
	}
#Irrigation
'Irrigation' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 252 ;
	}
#Surface runoff (variable resolution)
'Surface runoff (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 8 ;
	}
#Sub-surface runoff (variable resolution)
'Sub-surface runoff (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 9 ;
	}
#Clear sky surface photosynthetically active radiation (variable resolution)
'Clear sky surface photosynthetically active radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 20 ;
	}
#Total sky direct solar radiation at surface (variable resolution)
'Total sky direct solar radiation at surface (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 21 ;
	}
#Clear-sky direct solar radiation at surface (variable resolution)
'Clear-sky direct solar radiation at surface (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 22 ;
	}
#Direct solar radiation (variable resolution)
'Direct solar radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 47 ;
	}
#Large-scale precipitation fraction (variable resolution)
'Large-scale precipitation fraction (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 50 ;
	}
#Accumulated Carbon Dioxide Net Ecosystem Exchange (variable resolution)
'Accumulated Carbon Dioxide Net Ecosystem Exchange (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 80 ;
	}
#Accumulated Carbon Dioxide Gross Primary Production (variable resolution)
'Accumulated Carbon Dioxide Gross Primary Production (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 81 ;
	}
#Accumulated Carbon Dioxide Ecosystem Respiration (variable resolution)
'Accumulated Carbon Dioxide Ecosystem Respiration (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 82 ;
	}
#Surface solar radiation downward clear-sky (variable resolution)
'Surface solar radiation downward clear-sky (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 129 ;
	}
#Surface thermal radiation downward clear-sky (variable resolution)
'Surface thermal radiation downward clear-sky (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 130 ;
	}
#Albedo (variable resolution)
'Albedo (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 174 ;
	}
#Vertically integrated moisture divergence (variable resolution)
'Vertically integrated moisture divergence (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 213 ;
	}
#Accumulated freezing rain (variable resolution)
'Accumulated freezing rain (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 216 ;
	}
#Total precipitation (variable resolution)
'Total precipitation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 228 ;
	}
#Convective snowfall (variable resolution)
'Convective snowfall (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 239 ;
	}
#Large-scale snowfall (variable resolution)
'Large-scale snowfall (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 240 ;
	}
#Potential evaporation (variable resolution)
'Potential evaporation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 251 ;
	}
#Time-mean surface runoff rate
'Time-mean surface runoff rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 20 ;
	}
#Time-mean sub-surface runoff rate
'Time-mean sub-surface runoff rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 21 ;
	}
#Time-mean surface photosynthetically active radiation flux, clear sky
'Time-mean surface photosynthetically active radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 22 ;
	}
#Time-mean snow evaporation rate water equivalent
'Time-mean snow evaporation rate water equivalent' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 23 ;
	}
#Time-mean snow melt rate
'Time-mean snow melt rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 24 ;
	}
#Time-mean magnitude of turbulent surface stress
'Time-mean magnitude of turbulent surface stress' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 25 ;
	}
#Time-mean large-scale precipitation fraction
'Time-mean large-scale precipitation fraction' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 26 ;
	}
#Time-mean surface downward UV radiation flux
'Time-mean surface downward UV radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 27 ;
	}
#Time-mean surface photosynthetically active radiation flux
'Time-mean surface photosynthetically active radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 28 ;
	}
#Time-mean large-scale precipitation rate
'Time-mean large-scale precipitation rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 29 ;
	}
#Time-mean convective precipitation rate
'Time-mean convective precipitation rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 30 ;
	}
#Time-mean total snowfall rate water equivalent
'Time-mean total snowfall rate water equivalent' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 31 ;
	}
#Time-mean boundary layer dissipation
'Time-mean boundary layer dissipation' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 32 ;
	}
#Time-mean surface sensible heat flux
'Time-mean surface sensible heat flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 33 ;
	}
#Time-mean surface latent heat flux
'Time-mean surface latent heat flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 34 ;
	}
#Time-mean surface downward short-wave radiation flux
'Time-mean surface downward short-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 35 ;
	}
#Time-mean surface downward long-wave radiation flux
'Time-mean surface downward long-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 36 ;
	}
#Time-mean surface net short-wave radiation flux
'Time-mean surface net short-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 37 ;
	}
#Time-mean surface net long-wave radiation flux
'Time-mean surface net long-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 38 ;
	}
#Time-mean top net short-wave radiation flux
'Time-mean top net short-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 39 ;
	}
#Time-mean top net long-wave radiation flux
'Time-mean top net long-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 40 ;
	}
#Time-mean eastward turbulent surface stress
'Time-mean eastward turbulent surface stress' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 41 ;
	}
#Time-mean northward turbulent surface stress
'Time-mean northward turbulent surface stress' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 42 ;
	}
#Time-mean moisture flux
'Time-mean moisture flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 43 ;
	}
#Time-mean sunshine duration fraction
'Time-mean sunshine duration fraction' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 44 ;
	}
#Time-mean eastward gravity wave surface stress
'Time-mean eastward gravity wave surface stress' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 45 ;
	}
#Time-mean northward gravity wave surface stress
'Time-mean northward gravity wave surface stress' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 46 ;
	}
#Time-mean gravity wave dissipation
'Time-mean gravity wave dissipation' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 47 ;
	}
#Time-mean runoff rate water equivalent (surface plus subsurface)
'Time-mean runoff rate water equivalent (surface plus subsurface)' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 48 ;
	}
#Time-mean top net short-wave radiation flux, clear sky
'Time-mean top net short-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 49 ;
	}
#Time-mean top net long-wave radiation flux, clear sky
'Time-mean top net long-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 50 ;
	}
#Time-mean surface net short-wave radiation flux, clear sky
'Time-mean surface net short-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 51 ;
	}
#Time-mean surface net long-wave radiation flux, clear sky
'Time-mean surface net long-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 52 ;
	}
#Time mean top downward short-wave radiation flux
'Time mean top downward short-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 53 ;
	}
#Time-mean total column vertically-integrated moisture divergence flux
'Time-mean total column vertically-integrated moisture divergence flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 54 ;
	}
#Time-mean total precipitation rate
'Time-mean total precipitation rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 55 ;
	}
#Time-mean convective snowfall rate water equivalent
'Time-mean convective snowfall rate water equivalent' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 56 ;
	}
#Time-mean large scale snowfall rate water equivalent
'Time-mean large scale snowfall rate water equivalent' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 57 ;
	}
#Time-mean surface direct short-wave radiation flux
'Time-mean surface direct short-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 58 ;
	}
#Time-mean surface direct short-wave radiation flux, clear sky
'Time-mean surface direct short-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 59 ;
	}
#Mean surface diffuse short-wave radiation flux
'Mean surface diffuse short-wave radiation flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 60 ;
	}
#Time-mean surface diffuse short-wave radiation flux, clear sky
'Time-mean surface diffuse short-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 61 ;
	}
#Time-mean carbon dioxide net ecosystem exchange flux
'Time-mean carbon dioxide net ecosystem exchange flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 62 ;
	}
#Time-mean carbon dioxide gross primary production flux
'Time-mean carbon dioxide gross primary production flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 63 ;
	}
#Time-mean carbon dioxide ecosystem respiration flux
'Time-mean carbon dioxide ecosystem respiration flux' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 64 ;
	}
#Time-mean surface downward short-wave radiation flux, clear sky
'Time-mean surface downward short-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 68 ;
	}
#Time-mean surface downward long-wave radiation flux, clear sky
'Time-mean surface downward long-wave radiation flux, clear sky' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 69 ;
	}
#Time-mean potential evaporation rate
'Time-mean potential evaporation rate' = {
	 table2Version = 235 ;
	 indicatorOfParameter = 70 ;
	}
#Total precipitation rate
'Total precipitation rate' = {
	 table2Version = 254 ;
	 indicatorOfParameter = 48 ;
	}
#Ceiling
'Ceiling' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 109 ;
	}
#K index
'K index' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 121 ;
	}
#Total totals index
'Total totals index' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 123 ;
	}
#Stream function gradient
'Stream function gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 1 ;
	}
#Velocity potential gradient
'Velocity potential gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 2 ;
	}
#Potential temperature gradient
'Potential temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 3 ;
	}
#Equivalent potential temperature gradient
'Equivalent potential temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 4 ;
	}
#Saturated equivalent potential temperature gradient
'Saturated equivalent potential temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 5 ;
	}
#U component of divergent wind gradient
'U component of divergent wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 11 ;
	}
#V component of divergent wind gradient
'V component of divergent wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 12 ;
	}
#U component of rotational wind gradient
'U component of rotational wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 13 ;
	}
#V component of rotational wind gradient
'V component of rotational wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 14 ;
	}
#Unbalanced component of temperature gradient
'Unbalanced component of temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 21 ;
	}
#Unbalanced component of logarithm of surface pressure gradient
'Unbalanced component of logarithm of surface pressure gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 22 ;
	}
#Unbalanced component of divergence gradient
'Unbalanced component of divergence gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 23 ;
	}
#Reserved for future unbalanced components
'Reserved for future unbalanced components' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 24 ;
	}
#Reserved for future unbalanced components
'Reserved for future unbalanced components' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 25 ;
	}
#Lake cover gradient
'Lake cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 26 ;
	}
#Low vegetation cover gradient
'Low vegetation cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 27 ;
	}
#High vegetation cover gradient
'High vegetation cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 28 ;
	}
#Type of low vegetation gradient
'Type of low vegetation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 29 ;
	}
#Type of high vegetation gradient
'Type of high vegetation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 30 ;
	}
#Sea-ice cover gradient
'Sea-ice cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 31 ;
	}
#Snow albedo gradient
'Snow albedo gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 32 ;
	}
#Snow density gradient
'Snow density gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 33 ;
	}
#Sea surface temperature gradient
'Sea surface temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 34 ;
	}
#Ice surface temperature layer 1 gradient
'Ice surface temperature layer 1 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 35 ;
	}
#Ice surface temperature layer 2 gradient
'Ice surface temperature layer 2 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 36 ;
	}
#Ice surface temperature layer 3 gradient
'Ice surface temperature layer 3 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 37 ;
	}
#Ice surface temperature layer 4 gradient
'Ice surface temperature layer 4 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 38 ;
	}
#Volumetric soil water layer 1 gradient
'Volumetric soil water layer 1 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 39 ;
	}
#Volumetric soil water layer 2 gradient
'Volumetric soil water layer 2 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 40 ;
	}
#Volumetric soil water layer 3 gradient
'Volumetric soil water layer 3 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 41 ;
	}
#Volumetric soil water layer 4 gradient
'Volumetric soil water layer 4 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 42 ;
	}
#Soil type gradient
'Soil type gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 43 ;
	}
#Snow evaporation gradient
'Snow evaporation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 44 ;
	}
#Snowmelt gradient
'Snowmelt gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 45 ;
	}
#Solar duration gradient
'Solar duration gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 46 ;
	}
#Direct solar radiation gradient
'Direct solar radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 47 ;
	}
#Magnitude of turbulent surface stress gradient
'Magnitude of turbulent surface stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 48 ;
	}
#10 metre wind gust gradient
'10 metre wind gust gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 49 ;
	}
#Large-scale precipitation fraction gradient
'Large-scale precipitation fraction gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 50 ;
	}
#Maximum 2 metre temperature gradient
'Maximum 2 metre temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 51 ;
	}
#Minimum 2 metre temperature gradient
'Minimum 2 metre temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 52 ;
	}
#Montgomery potential gradient
'Montgomery potential gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 53 ;
	}
#Pressure gradient
'Pressure gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 54 ;
	}
#Mean 2 metre temperature in the last 24 hours gradient
'Mean 2 metre temperature in the last 24 hours gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 55 ;
	}
#Mean 2 metre dewpoint temperature in the last 24 hours gradient
'Mean 2 metre dewpoint temperature in the last 24 hours gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 56 ;
	}
#Downward UV radiation at the surface gradient
'Downward UV radiation at the surface gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 57 ;
	}
#Photosynthetically active radiation at the surface gradient
'Photosynthetically active radiation at the surface gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 58 ;
	}
#Convective available potential energy gradient
'Convective available potential energy gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 59 ;
	}
#Potential vorticity gradient
'Potential vorticity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 60 ;
	}
#Total precipitation from observations gradient
'Total precipitation from observations gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 61 ;
	}
#Observation count gradient
'Observation count gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 62 ;
	}
#Start time for skin temperature difference
'Start time for skin temperature difference' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 63 ;
	}
#Finish time for skin temperature difference
'Finish time for skin temperature difference' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 64 ;
	}
#Skin temperature difference
'Skin temperature difference' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 65 ;
	}
#Leaf area index, low vegetation
'Leaf area index, low vegetation' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 66 ;
	}
#Leaf area index, high vegetation
'Leaf area index, high vegetation' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 67 ;
	}
#Biome cover, low vegetation
'Biome cover, low vegetation' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 70 ;
	}
#Biome cover, high vegetation
'Biome cover, high vegetation' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 71 ;
	}
#Total column liquid water
'Total column liquid water' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 78 ;
	}
#Total column ice water
'Total column ice water' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 79 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 80 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 81 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 82 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 83 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 84 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 85 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 86 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 87 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 88 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 89 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 90 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 91 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 92 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 93 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 94 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 95 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 96 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 97 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 98 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 99 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 100 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 101 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 102 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 103 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 104 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 105 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 106 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 107 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 108 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 109 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 110 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 111 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 112 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 113 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 114 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 115 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 116 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 117 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 118 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 119 ;
	}
#Experimental product
'Experimental product' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 120 ;
	}
#Maximum temperature at 2 metres gradient
'Maximum temperature at 2 metres gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 121 ;
	}
#Minimum temperature at 2 metres gradient
'Minimum temperature at 2 metres gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 122 ;
	}
#10 metre wind gust in the last 6 hours gradient
'10 metre wind gust in the last 6 hours gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 123 ;
	}
#Vertically integrated total energy
'Vertically integrated total energy' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 125 ;
	}
#Generic parameter for sensitive area prediction
'Generic parameter for sensitive area prediction' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 126 ;
	}
#Atmospheric tide gradient
'Atmospheric tide gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 127 ;
	}
#Budget values gradient
'Budget values gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 128 ;
	}
#Geopotential gradient
'Geopotential gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 129 ;
	}
#Temperature gradient
'Temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 130 ;
	}
#U component of wind gradient
'U component of wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 131 ;
	}
#V component of wind gradient
'V component of wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 132 ;
	}
#Specific humidity gradient
'Specific humidity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 133 ;
	}
#Surface pressure gradient
'Surface pressure gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 134 ;
	}
#vertical velocity (pressure) gradient
'vertical velocity (pressure) gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 135 ;
	}
#Total column water gradient
'Total column water gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 136 ;
	}
#Total column water vapour gradient
'Total column water vapour gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 137 ;
	}
#Vorticity (relative) gradient
'Vorticity (relative) gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 138 ;
	}
#Soil temperature level 1 gradient
'Soil temperature level 1 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 139 ;
	}
#Soil wetness level 1 gradient
'Soil wetness level 1 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 140 ;
	}
#Snow depth gradient
'Snow depth gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 141 ;
	}
#Stratiform precipitation (Large-scale precipitation) gradient
'Stratiform precipitation (Large-scale precipitation) gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 142 ;
	}
#Convective precipitation gradient
'Convective precipitation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall (convective + stratiform) gradient
'Snowfall (convective + stratiform) gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 144 ;
	}
#Boundary layer dissipation gradient
'Boundary layer dissipation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 145 ;
	}
#Surface sensible heat flux gradient
'Surface sensible heat flux gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 146 ;
	}
#Surface latent heat flux gradient
'Surface latent heat flux gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 147 ;
	}
#Charnock gradient
'Charnock gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 148 ;
	}
#Surface net radiation gradient
'Surface net radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 149 ;
	}
#Top net radiation gradient
'Top net radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 150 ;
	}
#Mean sea level pressure gradient
'Mean sea level pressure gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 151 ;
	}
#Logarithm of surface pressure gradient
'Logarithm of surface pressure gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 152 ;
	}
#Short-wave heating rate gradient
'Short-wave heating rate gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 153 ;
	}
#Long-wave heating rate gradient
'Long-wave heating rate gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 154 ;
	}
#Divergence gradient
'Divergence gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 155 ;
	}
#Height gradient
'Height gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 156 ;
	}
#Relative humidity gradient
'Relative humidity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 157 ;
	}
#Tendency of surface pressure gradient
'Tendency of surface pressure gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 158 ;
	}
#Boundary layer height gradient
'Boundary layer height gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 159 ;
	}
#Standard deviation of orography gradient
'Standard deviation of orography gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 160 ;
	}
#Anisotropy of sub-gridscale orography gradient
'Anisotropy of sub-gridscale orography gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 161 ;
	}
#Angle of sub-gridscale orography gradient
'Angle of sub-gridscale orography gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 162 ;
	}
#Slope of sub-gridscale orography gradient
'Slope of sub-gridscale orography gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 163 ;
	}
#Total cloud cover gradient
'Total cloud cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 164 ;
	}
#10 metre U wind component gradient
'10 metre U wind component gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre V wind component gradient
'10 metre V wind component gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 166 ;
	}
#2 metre temperature gradient
'2 metre temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 167 ;
	}
#2 metre dewpoint temperature gradient
'2 metre dewpoint temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 168 ;
	}
#Surface solar radiation downwards gradient
'Surface solar radiation downwards gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 169 ;
	}
#Soil temperature level 2 gradient
'Soil temperature level 2 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 170 ;
	}
#Soil wetness level 2 gradient
'Soil wetness level 2 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 171 ;
	}
#Land-sea mask gradient
'Land-sea mask gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 172 ;
	}
#Surface roughness gradient
'Surface roughness gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 173 ;
	}
#Albedo gradient
'Albedo gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 174 ;
	}
#Surface thermal radiation downwards gradient
'Surface thermal radiation downwards gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 175 ;
	}
#Surface net solar radiation gradient
'Surface net solar radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net thermal radiation gradient
'Surface net thermal radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 177 ;
	}
#Top net solar radiation gradient
'Top net solar radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 178 ;
	}
#Top net thermal radiation gradient
'Top net thermal radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 179 ;
	}
#East-West surface stress gradient
'East-West surface stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 180 ;
	}
#North-South surface stress gradient
'North-South surface stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 181 ;
	}
#Evaporation gradient
'Evaporation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 182 ;
	}
#Soil temperature level 3 gradient
'Soil temperature level 3 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 183 ;
	}
#Soil wetness level 3 gradient
'Soil wetness level 3 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 184 ;
	}
#Convective cloud cover gradient
'Convective cloud cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 185 ;
	}
#Low cloud cover gradient
'Low cloud cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 186 ;
	}
#Medium cloud cover gradient
'Medium cloud cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 187 ;
	}
#High cloud cover gradient
'High cloud cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 188 ;
	}
#Sunshine duration gradient
'Sunshine duration gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 189 ;
	}
#East-West component of sub-gridscale orographic variance gradient
'East-West component of sub-gridscale orographic variance gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 190 ;
	}
#North-South component of sub-gridscale orographic variance gradient
'North-South component of sub-gridscale orographic variance gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 191 ;
	}
#North-West/South-East component of sub-gridscale orographic variance gradient
'North-West/South-East component of sub-gridscale orographic variance gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 192 ;
	}
#North-East/South-West component of sub-gridscale orographic variance gradient
'North-East/South-West component of sub-gridscale orographic variance gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 193 ;
	}
#Brightness temperature gradient
'Brightness temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 194 ;
	}
#Longitudinal component of gravity wave stress gradient
'Longitudinal component of gravity wave stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 195 ;
	}
#Meridional component of gravity wave stress gradient
'Meridional component of gravity wave stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation gradient
'Gravity wave dissipation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 197 ;
	}
#Skin reservoir content gradient
'Skin reservoir content gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 198 ;
	}
#Vegetation fraction gradient
'Vegetation fraction gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 199 ;
	}
#Variance of sub-gridscale orography gradient
'Variance of sub-gridscale orography gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 200 ;
	}
#Maximum temperature at 2 metres since previous post-processing gradient
'Maximum temperature at 2 metres since previous post-processing gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 201 ;
	}
#Minimum temperature at 2 metres since previous post-processing gradient
'Minimum temperature at 2 metres since previous post-processing gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 202 ;
	}
#Ozone mass mixing ratio gradient
'Ozone mass mixing ratio gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 203 ;
	}
#Precipitation analysis weights gradient
'Precipitation analysis weights gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 204 ;
	}
#Runoff gradient
'Runoff gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 205 ;
	}
#Total column ozone gradient
'Total column ozone gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 206 ;
	}
#10 metre wind speed gradient
'10 metre wind speed gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 207 ;
	}
#Top net solar radiation, clear sky gradient
'Top net solar radiation, clear sky gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 208 ;
	}
#Top net thermal radiation, clear sky gradient
'Top net thermal radiation, clear sky gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 209 ;
	}
#Surface net solar radiation, clear sky gradient
'Surface net solar radiation, clear sky gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 210 ;
	}
#Surface net thermal radiation, clear sky gradient
'Surface net thermal radiation, clear sky gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 211 ;
	}
#TOA incident solar radiation gradient
'TOA incident solar radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 212 ;
	}
#Diabatic heating by radiation gradient
'Diabatic heating by radiation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 214 ;
	}
#Diabatic heating by vertical diffusion gradient
'Diabatic heating by vertical diffusion gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 215 ;
	}
#Diabatic heating by cumulus convection gradient
'Diabatic heating by cumulus convection gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 216 ;
	}
#Diabatic heating large-scale condensation gradient
'Diabatic heating large-scale condensation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 217 ;
	}
#Vertical diffusion of zonal wind gradient
'Vertical diffusion of zonal wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 218 ;
	}
#Vertical diffusion of meridional wind gradient
'Vertical diffusion of meridional wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 219 ;
	}
#East-West gravity wave drag tendency gradient
'East-West gravity wave drag tendency gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 220 ;
	}
#North-South gravity wave drag tendency gradient
'North-South gravity wave drag tendency gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 221 ;
	}
#Convective tendency of zonal wind gradient
'Convective tendency of zonal wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 222 ;
	}
#Convective tendency of meridional wind gradient
'Convective tendency of meridional wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 223 ;
	}
#Vertical diffusion of humidity gradient
'Vertical diffusion of humidity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 224 ;
	}
#Humidity tendency by cumulus convection gradient
'Humidity tendency by cumulus convection gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 225 ;
	}
#Humidity tendency by large-scale condensation gradient
'Humidity tendency by large-scale condensation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 226 ;
	}
#Change from removal of negative humidity gradient
'Change from removal of negative humidity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 227 ;
	}
#Total precipitation gradient
'Total precipitation gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 228 ;
	}
#Instantaneous X surface stress gradient
'Instantaneous X surface stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 229 ;
	}
#Instantaneous Y surface stress gradient
'Instantaneous Y surface stress gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 230 ;
	}
#Instantaneous surface heat flux gradient
'Instantaneous surface heat flux gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 231 ;
	}
#Instantaneous moisture flux gradient
'Instantaneous moisture flux gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 232 ;
	}
#Apparent surface humidity gradient
'Apparent surface humidity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 233 ;
	}
#Logarithm of surface roughness length for heat gradient
'Logarithm of surface roughness length for heat gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 234 ;
	}
#Skin temperature gradient
'Skin temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 235 ;
	}
#Soil temperature level 4 gradient
'Soil temperature level 4 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 236 ;
	}
#Soil wetness level 4 gradient
'Soil wetness level 4 gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 237 ;
	}
#Temperature of snow layer gradient
'Temperature of snow layer gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 238 ;
	}
#Convective snowfall gradient
'Convective snowfall gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 239 ;
	}
#Large scale snowfall gradient
'Large scale snowfall gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 240 ;
	}
#Accumulated cloud fraction tendency gradient
'Accumulated cloud fraction tendency gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 241 ;
	}
#Accumulated liquid water tendency gradient
'Accumulated liquid water tendency gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 242 ;
	}
#Forecast albedo gradient
'Forecast albedo gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 243 ;
	}
#Forecast surface roughness gradient
'Forecast surface roughness gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 244 ;
	}
#Forecast logarithm of surface roughness for heat gradient
'Forecast logarithm of surface roughness for heat gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 245 ;
	}
#Specific cloud liquid water content gradient
'Specific cloud liquid water content gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 246 ;
	}
#Specific cloud ice water content gradient
'Specific cloud ice water content gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 247 ;
	}
#Cloud cover gradient
'Cloud cover gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 248 ;
	}
#Accumulated ice water tendency gradient
'Accumulated ice water tendency gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 249 ;
	}
#Ice age gradient
'Ice age gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 250 ;
	}
#Adiabatic tendency of temperature gradient
'Adiabatic tendency of temperature gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 251 ;
	}
#Adiabatic tendency of humidity gradient
'Adiabatic tendency of humidity gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 252 ;
	}
#Adiabatic tendency of zonal wind gradient
'Adiabatic tendency of zonal wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 253 ;
	}
#Adiabatic tendency of meridional wind gradient
'Adiabatic tendency of meridional wind gradient' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 254 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 129 ;
	 indicatorOfParameter = 255 ;
	}
#Top solar radiation upward
'Top solar radiation upward' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 208 ;
	}
#Top thermal radiation upward
'Top thermal radiation upward' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 209 ;
	}
#Top solar radiation upward, clear sky
'Top solar radiation upward, clear sky' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 210 ;
	}
#Top thermal radiation upward, clear sky
'Top thermal radiation upward, clear sky' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 211 ;
	}
#Cloud liquid water
'Cloud liquid water' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 212 ;
	}
#Cloud fraction
'Cloud fraction' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 213 ;
	}
#Diabatic heating by radiation
'Diabatic heating by radiation' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 214 ;
	}
#Diabatic heating by vertical diffusion
'Diabatic heating by vertical diffusion' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 215 ;
	}
#Diabatic heating by cumulus convection
'Diabatic heating by cumulus convection' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 216 ;
	}
#Diabatic heating by large-scale condensation
'Diabatic heating by large-scale condensation' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 217 ;
	}
#Vertical diffusion of zonal wind
'Vertical diffusion of zonal wind' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 218 ;
	}
#Vertical diffusion of meridional wind
'Vertical diffusion of meridional wind' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 219 ;
	}
#East-West gravity wave drag
'East-West gravity wave drag' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 220 ;
	}
#North-South gravity wave drag
'North-South gravity wave drag' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 221 ;
	}
#Vertical diffusion of humidity
'Vertical diffusion of humidity' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 224 ;
	}
#Humidity tendency by cumulus convection
'Humidity tendency by cumulus convection' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 225 ;
	}
#Humidity tendency by large-scale condensation
'Humidity tendency by large-scale condensation' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 226 ;
	}
#Adiabatic tendency of temperature
'Adiabatic tendency of temperature' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 228 ;
	}
#Adiabatic tendency of humidity
'Adiabatic tendency of humidity' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 229 ;
	}
#Adiabatic tendency of zonal wind
'Adiabatic tendency of zonal wind' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 230 ;
	}
#Adiabatic tendency of meridional wind
'Adiabatic tendency of meridional wind' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 231 ;
	}
#Mean vertical velocity
'Mean vertical velocity' = {
	 table2Version = 130 ;
	 indicatorOfParameter = 232 ;
	}
#2m temperature anomaly of at least +2K
'2m temperature anomaly of at least +2K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 1 ;
	}
#2m temperature anomaly of at least +1K
'2m temperature anomaly of at least +1K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 2 ;
	}
#2m temperature anomaly of at least 0K
'2m temperature anomaly of at least 0K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 3 ;
	}
#2m temperature anomaly of at most -1K
'2m temperature anomaly of at most -1K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 4 ;
	}
#2m temperature anomaly of at most -2K
'2m temperature anomaly of at most -2K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 5 ;
	}
#Total precipitation anomaly of at least 20 mm
'Total precipitation anomaly of at least 20 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 6 ;
	}
#Total precipitation anomaly of at least 10 mm
'Total precipitation anomaly of at least 10 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 7 ;
	}
#Total precipitation anomaly of at least 0 mm
'Total precipitation anomaly of at least 0 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 8 ;
	}
#Surface temperature anomaly of at least 0K
'Surface temperature anomaly of at least 0K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 9 ;
	}
#Mean sea level pressure anomaly of at least 0 Pa
'Mean sea level pressure anomaly of at least 0 Pa' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 10 ;
	}
#Height of 0 degree isotherm probability
'Height of 0 degree isotherm probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 15 ;
	}
#Height of snowfall limit probability
'Height of snowfall limit probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 16 ;
	}
#Showalter index probability
'Showalter index probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 17 ;
	}
#Whiting index probability
'Whiting index probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 18 ;
	}
#Temperature anomaly less than -2 K
'Temperature anomaly less than -2 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 20 ;
	}
#Temperature anomaly of at least +2 K
'Temperature anomaly of at least +2 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 21 ;
	}
#Temperature anomaly less than -8 K
'Temperature anomaly less than -8 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 22 ;
	}
#Temperature anomaly less than -4 K
'Temperature anomaly less than -4 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 23 ;
	}
#Temperature anomaly greater than +4 K
'Temperature anomaly greater than +4 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 24 ;
	}
#Temperature anomaly greater than +8 K
'Temperature anomaly greater than +8 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 25 ;
	}
#10 metre wind gust probability
'10 metre wind gust probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 49 ;
	}
#Convective available potential energy probability
'Convective available potential energy probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 59 ;
	}
#Total precipitation less than 0.1 mm
'Total precipitation less than 0.1 mm' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 64 ;
	}
#Total precipitation rate less than 1 mm/day
'Total precipitation rate less than 1 mm/day' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 65 ;
	}
#Total precipitation rate of at least 3 mm/day
'Total precipitation rate of at least 3 mm/day' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 66 ;
	}
#Total precipitation rate of at least 5 mm/day
'Total precipitation rate of at least 5 mm/day' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 67 ;
	}
#10 metre Wind speed of at least 10 m/s
'10 metre Wind speed of at least 10 m/s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 68 ;
	}
#10 metre Wind speed of at least 15 m/s
'10 metre Wind speed of at least 15 m/s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 69 ;
	}
#10 metre wind gust of at least 15 m/s
'10 metre wind gust of at least 15 m/s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 70 ;
	}
#10 metre wind gust of at least 20 m/s
'10 metre wind gust of at least 20 m/s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 71 ;
	}
#10 metre wind gust of at least 25 m/s
'10 metre wind gust of at least 25 m/s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 72 ;
	}
#2 metre temperature less than 273.15 K
'2 metre temperature less than 273.15 K' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 73 ;
	}
#Significant wave height of at least 2 m
'Significant wave height of at least 2 m' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 74 ;
	}
#Significant wave height of at least 4 m
'Significant wave height of at least 4 m' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 75 ;
	}
#Significant wave height of at least 6 m
'Significant wave height of at least 6 m' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 76 ;
	}
#Significant wave height of at least 8 m
'Significant wave height of at least 8 m' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 77 ;
	}
#Mean wave period of at least 8 s
'Mean wave period of at least 8 s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 78 ;
	}
#Mean wave period of at least 10 s
'Mean wave period of at least 10 s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 79 ;
	}
#Mean wave period of at least 12 s
'Mean wave period of at least 12 s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 80 ;
	}
#Mean wave period of at least 15 s
'Mean wave period of at least 15 s' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 81 ;
	}
#Geopotential probability
'Geopotential probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 129 ;
	}
#Temperature anomaly probability
'Temperature anomaly probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 130 ;
	}
#Soil temperature level 1 probability
'Soil temperature level 1 probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 139 ;
	}
#Snowfall (convective + stratiform) probability
'Snowfall (convective + stratiform) probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 144 ;
	}
#Mean sea level pressure probability
'Mean sea level pressure probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 151 ;
	}
#Total cloud cover probability
'Total cloud cover probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 164 ;
	}
#10 metre speed probability
'10 metre speed probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 165 ;
	}
#2 metre temperature probability
'2 metre temperature probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 167 ;
	}
#Maximum 2 metre temperature probability
'Maximum 2 metre temperature probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 201 ;
	}
#Minimum 2 metre temperature probability
'Minimum 2 metre temperature probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 202 ;
	}
#Total precipitation probability
'Total precipitation probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 228 ;
	}
#Significant wave height probability
'Significant wave height probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 229 ;
	}
#Mean wave period probability
'Mean wave period probability' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 232 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 131 ;
	 indicatorOfParameter = 255 ;
	}
#10 metre wind gust index
'10 metre wind gust index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 49 ;
	}
#Snowfall index
'Snowfall index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 144 ;
	}
#10 metre speed index
'10 metre speed index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 165 ;
	}
#2 metre temperature index
'2 metre temperature index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 167 ;
	}
#Maximum temperature at 2 metres index
'Maximum temperature at 2 metres index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 201 ;
	}
#Minimum temperature at 2 metres index
'Minimum temperature at 2 metres index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 202 ;
	}
#Total precipitation index
'Total precipitation index' = {
	 table2Version = 132 ;
	 indicatorOfParameter = 228 ;
	}
#2m temperature probability less than -10 C
'2m temperature probability less than -10 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 1 ;
	}
#2m temperature probability less than -5 C
'2m temperature probability less than -5 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 2 ;
	}
#2m temperature probability less than 0 C
'2m temperature probability less than 0 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 3 ;
	}
#2m temperature probability less than 5 C
'2m temperature probability less than 5 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 4 ;
	}
#2m temperature probability less than 10 C
'2m temperature probability less than 10 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 5 ;
	}
#2m temperature probability greater than 25 C
'2m temperature probability greater than 25 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 6 ;
	}
#2m temperature probability greater than 30 C
'2m temperature probability greater than 30 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 7 ;
	}
#2m temperature probability greater than 35 C
'2m temperature probability greater than 35 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 8 ;
	}
#2m temperature probability greater than 40 C
'2m temperature probability greater than 40 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 9 ;
	}
#2m temperature probability greater than 45 C
'2m temperature probability greater than 45 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 10 ;
	}
#Minimum 2 metre temperature probability less than -10 C
'Minimum 2 metre temperature probability less than -10 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 11 ;
	}
#Minimum 2 metre temperature probability less than -5 C
'Minimum 2 metre temperature probability less than -5 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 12 ;
	}
#Minimum 2 metre temperature probability less than 0 C
'Minimum 2 metre temperature probability less than 0 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 13 ;
	}
#Minimum 2 metre temperature probability less than 5 C
'Minimum 2 metre temperature probability less than 5 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 14 ;
	}
#Minimum 2 metre temperature probability less than 10 C
'Minimum 2 metre temperature probability less than 10 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 15 ;
	}
#Maximum 2 metre temperature probability greater than 25 C
'Maximum 2 metre temperature probability greater than 25 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 16 ;
	}
#Maximum 2 metre temperature probability greater than 30 C
'Maximum 2 metre temperature probability greater than 30 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 17 ;
	}
#Maximum 2 metre temperature probability greater than 35 C
'Maximum 2 metre temperature probability greater than 35 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 18 ;
	}
#Maximum 2 metre temperature probability greater than 40 C
'Maximum 2 metre temperature probability greater than 40 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 19 ;
	}
#Maximum 2 metre temperature probability greater than 45 C
'Maximum 2 metre temperature probability greater than 45 C' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 20 ;
	}
#10 metre wind speed probability of at least 10 m/s
'10 metre wind speed probability of at least 10 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 21 ;
	}
#10 metre wind speed probability of at least 15 m/s
'10 metre wind speed probability of at least 15 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 22 ;
	}
#10 metre wind speed probability of at least 20 m/s
'10 metre wind speed probability of at least 20 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 23 ;
	}
#10 metre wind speed probability of at least 35 m/s
'10 metre wind speed probability of at least 35 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 24 ;
	}
#10 metre wind speed probability of at least 50 m/s
'10 metre wind speed probability of at least 50 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 25 ;
	}
#10 metre wind gust probability of at least 20 m/s
'10 metre wind gust probability of at least 20 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 26 ;
	}
#10 metre wind gust probability of at least 35 m/s
'10 metre wind gust probability of at least 35 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 27 ;
	}
#10 metre wind gust probability of at least 50 m/s
'10 metre wind gust probability of at least 50 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 28 ;
	}
#10 metre wind gust probability of at least 75 m/s
'10 metre wind gust probability of at least 75 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 29 ;
	}
#10 metre wind gust probability of at least 100 m/s
'10 metre wind gust probability of at least 100 m/s' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 30 ;
	}
#Total precipitation probability of at least 1 mm
'Total precipitation probability of at least 1 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 31 ;
	}
#Total precipitation probability of at least 5 mm
'Total precipitation probability of at least 5 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 32 ;
	}
#Total precipitation probability of at least 10 mm
'Total precipitation probability of at least 10 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 33 ;
	}
#Total precipitation probability of at least 20 mm
'Total precipitation probability of at least 20 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 34 ;
	}
#Total precipitation probability of at least 40 mm
'Total precipitation probability of at least 40 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 35 ;
	}
#Total precipitation probability of at least 60 mm
'Total precipitation probability of at least 60 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 36 ;
	}
#Total precipitation probability of at least 80 mm
'Total precipitation probability of at least 80 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 37 ;
	}
#Total precipitation probability of at least 100 mm
'Total precipitation probability of at least 100 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 38 ;
	}
#Total precipitation probability of at least 150 mm
'Total precipitation probability of at least 150 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 39 ;
	}
#Total precipitation probability of at least 200 mm
'Total precipitation probability of at least 200 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 40 ;
	}
#Total precipitation probability of at least 300 mm
'Total precipitation probability of at least 300 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 41 ;
	}
#Snowfall probability of at least 1 mm
'Snowfall probability of at least 1 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 42 ;
	}
#Snowfall probability of at least 5 mm
'Snowfall probability of at least 5 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 43 ;
	}
#Snowfall probability of at least 10 mm
'Snowfall probability of at least 10 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 44 ;
	}
#Snowfall probability of at least 20 mm
'Snowfall probability of at least 20 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 45 ;
	}
#Snowfall probability of at least 40 mm
'Snowfall probability of at least 40 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 46 ;
	}
#Snowfall probability of at least 60 mm
'Snowfall probability of at least 60 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 47 ;
	}
#Snowfall probability of at least 80 mm
'Snowfall probability of at least 80 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 48 ;
	}
#Snowfall probability of at least 100 mm
'Snowfall probability of at least 100 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 49 ;
	}
#Snowfall probability of at least 150 mm
'Snowfall probability of at least 150 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 50 ;
	}
#Snowfall probability of at least 200 mm
'Snowfall probability of at least 200 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 51 ;
	}
#Snowfall probability of at least 300 mm
'Snowfall probability of at least 300 mm' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 52 ;
	}
#Total Cloud Cover probability greater than 10%
'Total Cloud Cover probability greater than 10%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 53 ;
	}
#Total Cloud Cover probability greater than 20%
'Total Cloud Cover probability greater than 20%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 54 ;
	}
#Total Cloud Cover probability greater than 30%
'Total Cloud Cover probability greater than 30%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 55 ;
	}
#Total Cloud Cover probability greater than 40%
'Total Cloud Cover probability greater than 40%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 56 ;
	}
#Total Cloud Cover probability greater than 50%
'Total Cloud Cover probability greater than 50%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 57 ;
	}
#Total Cloud Cover probability greater than 60%
'Total Cloud Cover probability greater than 60%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 58 ;
	}
#Total Cloud Cover probability greater than 70%
'Total Cloud Cover probability greater than 70%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 59 ;
	}
#Total Cloud Cover probability greater than 80%
'Total Cloud Cover probability greater than 80%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 60 ;
	}
#Total Cloud Cover probability greater than 90%
'Total Cloud Cover probability greater than 90%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 61 ;
	}
#Total Cloud Cover probability greater than 99%
'Total Cloud Cover probability greater than 99%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 62 ;
	}
#High Cloud Cover probability greater than 10%
'High Cloud Cover probability greater than 10%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 63 ;
	}
#High Cloud Cover probability greater than 20%
'High Cloud Cover probability greater than 20%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 64 ;
	}
#High Cloud Cover probability greater than 30%
'High Cloud Cover probability greater than 30%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 65 ;
	}
#High Cloud Cover probability greater than 40%
'High Cloud Cover probability greater than 40%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 66 ;
	}
#High Cloud Cover probability greater than 50%
'High Cloud Cover probability greater than 50%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 67 ;
	}
#High Cloud Cover probability greater than 60%
'High Cloud Cover probability greater than 60%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 68 ;
	}
#High Cloud Cover probability greater than 70%
'High Cloud Cover probability greater than 70%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 69 ;
	}
#High Cloud Cover probability greater than 80%
'High Cloud Cover probability greater than 80%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 70 ;
	}
#High Cloud Cover probability greater than 90%
'High Cloud Cover probability greater than 90%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 71 ;
	}
#High Cloud Cover probability greater than 99%
'High Cloud Cover probability greater than 99%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 72 ;
	}
#Medium Cloud Cover probability greater than 10%
'Medium Cloud Cover probability greater than 10%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 73 ;
	}
#Medium Cloud Cover probability greater than 20%
'Medium Cloud Cover probability greater than 20%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 74 ;
	}
#Medium Cloud Cover probability greater than 30%
'Medium Cloud Cover probability greater than 30%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 75 ;
	}
#Medium Cloud Cover probability greater than 40%
'Medium Cloud Cover probability greater than 40%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 76 ;
	}
#Medium Cloud Cover probability greater than 50%
'Medium Cloud Cover probability greater than 50%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 77 ;
	}
#Medium Cloud Cover probability greater than 60%
'Medium Cloud Cover probability greater than 60%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 78 ;
	}
#Medium Cloud Cover probability greater than 70%
'Medium Cloud Cover probability greater than 70%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 79 ;
	}
#Medium Cloud Cover probability greater than 80%
'Medium Cloud Cover probability greater than 80%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 80 ;
	}
#Medium Cloud Cover probability greater than 90%
'Medium Cloud Cover probability greater than 90%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 81 ;
	}
#Medium Cloud Cover probability greater than 99%
'Medium Cloud Cover probability greater than 99%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 82 ;
	}
#Low Cloud Cover probability greater than 10%
'Low Cloud Cover probability greater than 10%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 83 ;
	}
#Low Cloud Cover probability greater than 20%
'Low Cloud Cover probability greater than 20%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 84 ;
	}
#Low Cloud Cover probability greater than 30%
'Low Cloud Cover probability greater than 30%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 85 ;
	}
#Low Cloud Cover probability greater than 40%
'Low Cloud Cover probability greater than 40%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 86 ;
	}
#Low Cloud Cover probability greater than 50%
'Low Cloud Cover probability greater than 50%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 87 ;
	}
#Low Cloud Cover probability greater than 60%
'Low Cloud Cover probability greater than 60%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 88 ;
	}
#Low Cloud Cover probability greater than 70%
'Low Cloud Cover probability greater than 70%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 89 ;
	}
#Low Cloud Cover probability greater than 80%
'Low Cloud Cover probability greater than 80%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 90 ;
	}
#Low Cloud Cover probability greater than 90%
'Low Cloud Cover probability greater than 90%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 91 ;
	}
#Low Cloud Cover probability greater than 99%
'Low Cloud Cover probability greater than 99%' = {
	 table2Version = 133 ;
	 indicatorOfParameter = 92 ;
	}
#Maximum of significant wave height
'Maximum of significant wave height' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 200 ;
	}
#Period corresponding to maximum individual wave height
'Period corresponding to maximum individual wave height' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 217 ;
	}
#Envelop-maximum individual wave height
'Envelop-maximum individual wave height' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 218 ;
	}
#Model bathymetry
'Model bathymetry' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 219 ;
	}
#Mean wave period based on first moment
'Mean wave period based on first moment' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 220 ;
	}
#Mean zero-crossing wave period
'Mean zero-crossing wave period' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 221 ;
	}
#Wave spectral directional width
'Wave spectral directional width' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 222 ;
	}
#Mean wave period based on first moment for wind waves
'Mean wave period based on first moment for wind waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 223 ;
	}
#Mean wave period based on second moment for wind waves
'Mean wave period based on second moment for wind waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 224 ;
	}
#Wave spectral directional width for wind waves
'Wave spectral directional width for wind waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 225 ;
	}
#Mean wave period based on first moment for swell
'Mean wave period based on first moment for swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 226 ;
	}
#Mean wave period based on second moment for swell
'Mean wave period based on second moment for swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 227 ;
	}
#Wave spectral directional width for swell
'Wave spectral directional width for swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 228 ;
	}
#Significant height of combined wind waves and swell
'Significant height of combined wind waves and swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 229 ;
	}
#Mean wave direction
'Mean wave direction' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 230 ;
	}
#Peak wave period
'Peak wave period' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 231 ;
	}
#Mean wave period
'Mean wave period' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 232 ;
	}
#Coefficient of drag with waves
'Coefficient of drag with waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 233 ;
	}
#Significant height of wind waves
'Significant height of wind waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 234 ;
	}
#Mean direction of wind waves
'Mean direction of wind waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 235 ;
	}
#Mean period of wind waves
'Mean period of wind waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 236 ;
	}
#Significant height of total swell
'Significant height of total swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 237 ;
	}
#Mean direction of total swell
'Mean direction of total swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 238 ;
	}
#Mean period of total swell
'Mean period of total swell' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 239 ;
	}
#Standard deviation wave height
'Standard deviation wave height' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 240 ;
	}
#Mean of 10 metre wind speed
'Mean of 10 metre wind speed' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 241 ;
	}
#Mean wind direction
'Mean wind direction' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 242 ;
	}
#Standard deviation of 10 metre wind speed
'Standard deviation of 10 metre wind speed' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 243 ;
	}
#Mean square slope of waves
'Mean square slope of waves' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 244 ;
	}
#10 metre wind speed
'10 metre wind speed' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 245 ;
	}
#Altimeter wave height
'Altimeter wave height' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 246 ;
	}
#Altimeter corrected wave height
'Altimeter corrected wave height' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 247 ;
	}
#Altimeter range relative correction
'Altimeter range relative correction' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 248 ;
	}
#10 metre wind direction
'10 metre wind direction' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 249 ;
	}
#2D wave spectra (multiple)
'2D wave spectra (multiple)' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 250 ;
	}
#2D wave spectra (single)
'2D wave spectra (single)' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 251 ;
	}
#Wave spectral kurtosis
'Wave spectral kurtosis' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 252 ;
	}
#Benjamin-Feir index
'Benjamin-Feir index' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 253 ;
	}
#Wave spectral peakedness
'Wave spectral peakedness' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 254 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 255 ;
	}
#Ocean potential temperature
'Ocean potential temperature' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 129 ;
	}
#Ocean salinity
'Ocean salinity' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 130 ;
	}
#Ocean potential density
'Ocean potential density' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 131 ;
	}
#Ocean U wind component
'Ocean U wind component' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 133 ;
	}
#Ocean V wind component
'Ocean V wind component' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 134 ;
	}
#Ocean W wind component
'Ocean W wind component' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 135 ;
	}
#Richardson number
'Richardson number' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 137 ;
	}
#U*V product
'U*V product' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 139 ;
	}
#U*T product
'U*T product' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 140 ;
	}
#V*T product
'V*T product' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 141 ;
	}
#U*U product
'U*U product' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 142 ;
	}
#V*V product
'V*V product' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 143 ;
	}
#UV - U~V~
'UV - U~V~' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 144 ;
	}
#UT - U~T~
'UT - U~T~' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 145 ;
	}
#VT - V~T~
'VT - V~T~' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 146 ;
	}
#UU - U~U~
'UU - U~U~' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 147 ;
	}
#VV - V~V~
'VV - V~V~' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 148 ;
	}
#Sea level
'Sea level' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 152 ;
	}
#Barotropic stream function
'Barotropic stream function' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 153 ;
	}
#Mixed layer depth
'Mixed layer depth' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 154 ;
	}
#Depth
'Depth' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 155 ;
	}
#U stress
'U stress' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 168 ;
	}
#V stress
'V stress' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 169 ;
	}
#Turbulent kinetic energy input
'Turbulent kinetic energy input' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 170 ;
	}
#Net surface heat flux
'Net surface heat flux' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 171 ;
	}
#Surface solar radiation
'Surface solar radiation' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 172 ;
	}
#P-E
'P-E' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 173 ;
	}
#Diagnosed sea surface temperature error
'Diagnosed sea surface temperature error' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 180 ;
	}
#Heat flux correction
'Heat flux correction' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 181 ;
	}
#Observed sea surface temperature
'Observed sea surface temperature' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 182 ;
	}
#Observed heat flux
'Observed heat flux' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 183 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 150 ;
	 indicatorOfParameter = 255 ;
	}
#In situ Temperature
'In situ Temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 128 ;
	}
#Sea water potential temperature
'Sea water potential temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 129 ;
	}
#Sea water practical salinity
'Sea water practical salinity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 130 ;
	}
#Eastward surface sea water velocity
'Eastward surface sea water velocity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 131 ;
	}
#Northward surface sea water velocity
'Northward surface sea water velocity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 132 ;
	}
#Upward sea water velocity
'Upward sea water velocity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 133 ;
	}
#Modulus of strain rate tensor
'Modulus of strain rate tensor' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 134 ;
	}
#Vertical viscosity
'Vertical viscosity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 135 ;
	}
#Vertical diffusivity
'Vertical diffusivity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 136 ;
	}
#Bottom level Depth
'Bottom level Depth' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 137 ;
	}
#Sea water sigma theta
'Sea water sigma theta' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 138 ;
	}
#Richardson number
'Richardson number' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 139 ;
	}
#UV product
'UV product' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 140 ;
	}
#UT product
'UT product' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 141 ;
	}
#VT product
'VT product' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 142 ;
	}
#UU product
'UU product' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 143 ;
	}
#VV product
'VV product' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 144 ;
	}
#Sea surface height
'Sea surface height' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 145 ;
	}
#Sea level previous timestep
'Sea level previous timestep' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 146 ;
	}
#Ocean barotropic stream function
'Ocean barotropic stream function' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 147 ;
	}
#Mixed layer depth
'Mixed layer depth' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 148 ;
	}
#Bottom Pressure (equivalent height)
'Bottom Pressure (equivalent height)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 149 ;
	}
#Steric height
'Steric height' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 150 ;
	}
#Curl of Wind Stress
'Curl of Wind Stress' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 151 ;
	}
#Divergence of wind stress
'Divergence of wind stress' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 152 ;
	}
#Surface downward eastward stress
'Surface downward eastward stress' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 153 ;
	}
#Surface downward northward stress
'Surface downward northward stress' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 154 ;
	}
#Turbulent kinetic energy input
'Turbulent kinetic energy input' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 155 ;
	}
#Net surface heat flux
'Net surface heat flux' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 156 ;
	}
#Absorbed solar radiation
'Absorbed solar radiation' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 157 ;
	}
#Precipitation - evaporation
'Precipitation - evaporation' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 158 ;
	}
#Specified sea surface temperature
'Specified sea surface temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 159 ;
	}
#Specified surface heat flux
'Specified surface heat flux' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 160 ;
	}
#Diagnosed sea surface temperature error
'Diagnosed sea surface temperature error' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 161 ;
	}
#Heat flux correction
'Heat flux correction' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 162 ;
	}
#Depth of 20C isotherm
'Depth of 20C isotherm' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 163 ;
	}
#Average potential temperature in the upper 300m
'Average potential temperature in the upper 300m' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 164 ;
	}
#Vertically integrated zonal velocity (previous time step)
'Vertically integrated zonal velocity (previous time step)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 165 ;
	}
#Vertically Integrated meridional velocity (previous time step)
'Vertically Integrated meridional velocity (previous time step)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 166 ;
	}
#Vertically integrated zonal volume transport
'Vertically integrated zonal volume transport' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 167 ;
	}
#Vertically integrated meridional volume transport
'Vertically integrated meridional volume transport' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 168 ;
	}
#Vertically integrated zonal heat transport
'Vertically integrated zonal heat transport' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 169 ;
	}
#Vertically integrated meridional heat transport
'Vertically integrated meridional heat transport' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 170 ;
	}
#U velocity maximum
'U velocity maximum' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 171 ;
	}
#Depth of the velocity maximum
'Depth of the velocity maximum' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 172 ;
	}
#Salinity maximum
'Salinity maximum' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 173 ;
	}
#Depth of salinity maximum
'Depth of salinity maximum' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 174 ;
	}
#Average sea water practical salinity in the upper 300m
'Average sea water practical salinity in the upper 300m' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 175 ;
	}
#Layer Thickness at scalar points
'Layer Thickness at scalar points' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 176 ;
	}
#Layer Thickness at vector points
'Layer Thickness at vector points' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 177 ;
	}
#Potential temperature increment
'Potential temperature increment' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 178 ;
	}
#Potential temperature analysis error
'Potential temperature analysis error' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 179 ;
	}
#Background potential temperature
'Background potential temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 180 ;
	}
#Analysed potential temperature
'Analysed potential temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 181 ;
	}
#Potential temperature background error
'Potential temperature background error' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 182 ;
	}
#Analysed salinity
'Analysed salinity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 183 ;
	}
#Salinity increment
'Salinity increment' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 184 ;
	}
#Estimated Bias in Temperature
'Estimated Bias in Temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 185 ;
	}
#Estimated Bias in Salinity
'Estimated Bias in Salinity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 186 ;
	}
#Zonal Velocity increment (from balance operator)
'Zonal Velocity increment (from balance operator)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 187 ;
	}
#Meridional Velocity increment (from balance operator)
'Meridional Velocity increment (from balance operator)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 188 ;
	}
#Salinity increment (from salinity data)
'Salinity increment (from salinity data)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 190 ;
	}
#Salinity analysis error
'Salinity analysis error' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 191 ;
	}
#Background Salinity
'Background Salinity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 192 ;
	}
#Salinity background error
'Salinity background error' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 194 ;
	}
#Estimated temperature bias from assimilation
'Estimated temperature bias from assimilation' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 199 ;
	}
#Estimated salinity bias from assimilation
'Estimated salinity bias from assimilation' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 200 ;
	}
#Temperature increment from relaxation term
'Temperature increment from relaxation term' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 201 ;
	}
#Salinity increment from relaxation term
'Salinity increment from relaxation term' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 202 ;
	}
#Bias in the zonal pressure gradient (applied)
'Bias in the zonal pressure gradient (applied)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 203 ;
	}
#Bias in the meridional pressure gradient (applied)
'Bias in the meridional pressure gradient (applied)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 204 ;
	}
#Estimated temperature bias from relaxation
'Estimated temperature bias from relaxation' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 205 ;
	}
#Estimated salinity bias from relaxation
'Estimated salinity bias from relaxation' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 206 ;
	}
#First guess bias in temperature
'First guess bias in temperature' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 207 ;
	}
#First guess bias in salinity
'First guess bias in salinity' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 208 ;
	}
#Applied bias in pressure
'Applied bias in pressure' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 209 ;
	}
#FG bias in pressure
'FG bias in pressure' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 210 ;
	}
#Bias in temperature(applied)
'Bias in temperature(applied)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 211 ;
	}
#Bias in salinity (applied)
'Bias in salinity (applied)' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 212 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 151 ;
	 indicatorOfParameter = 255 ;
	}
#10 metre wind gust during averaging time
'10 metre wind gust during averaging time' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 49 ;
	}
#vertical velocity (pressure)
'vertical velocity (pressure)' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 135 ;
	}
#Precipitable water content
'Precipitable water content' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 137 ;
	}
#Soil wetness level 1
'Soil wetness level 1' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 140 ;
	}
#Snow depth
'Snow depth' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 141 ;
	}
#Large-scale precipitation
'Large-scale precipitation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 142 ;
	}
#Convective precipitation
'Convective precipitation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall
'Snowfall' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 144 ;
	}
#Height
'Height' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 156 ;
	}
#Relative humidity
'Relative humidity' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 157 ;
	}
#Soil wetness level 2
'Soil wetness level 2' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 171 ;
	}
#East-West surface stress
'East-West surface stress' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 180 ;
	}
#North-South surface stress
'North-South surface stress' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 181 ;
	}
#Evaporation
'Evaporation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 182 ;
	}
#Soil wetness level 3
'Soil wetness level 3' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 184 ;
	}
#Skin reservoir content
'Skin reservoir content' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 198 ;
	}
#Percentage of vegetation
'Percentage of vegetation' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 199 ;
	}
#Runoff
'Runoff' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 205 ;
	}
#Covariance of temperature and geopotential
'Covariance of temperature and geopotential' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 207 ;
	}
#Covariance of specific humidity and geopotential
'Covariance of specific humidity and geopotential' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 209 ;
	}
#Covariance of specific humidity and temperature
'Covariance of specific humidity and temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 210 ;
	}
#Covariance of U component and geopotential
'Covariance of U component and geopotential' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 212 ;
	}
#Covariance of U component and temperature
'Covariance of U component and temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 213 ;
	}
#Covariance of U component and specific humidity
'Covariance of U component and specific humidity' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 214 ;
	}
#Covariance of V component and geopotential
'Covariance of V component and geopotential' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 216 ;
	}
#Covariance of V component and temperature
'Covariance of V component and temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 217 ;
	}
#Covariance of V component and specific humidity
'Covariance of V component and specific humidity' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 218 ;
	}
#Covariance of V component and U component
'Covariance of V component and U component' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 219 ;
	}
#Covariance of W component and geopotential
'Covariance of W component and geopotential' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 221 ;
	}
#Covariance of W component and temperature
'Covariance of W component and temperature' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 222 ;
	}
#Covariance of W component and specific humidity
'Covariance of W component and specific humidity' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 223 ;
	}
#Covariance of W component and U component
'Covariance of W component and U component' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 224 ;
	}
#Covariance of W component and V component
'Covariance of W component and V component' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 225 ;
	}
#Instantaneous surface heat flux
'Instantaneous surface heat flux' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 231 ;
	}
#Convective snowfall
'Convective snowfall' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 239 ;
	}
#Large scale snowfall
'Large scale snowfall' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 240 ;
	}
#Cloud liquid water content
'Cloud liquid water content' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 241 ;
	}
#Cloud cover
'Cloud cover' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 242 ;
	}
#Forecast albedo
'Forecast albedo' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 243 ;
	}
#10 metre wind speed
'10 metre wind speed' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 246 ;
	}
#Momentum flux
'Momentum flux' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 247 ;
	}
#Gravity wave dissipation flux
'Gravity wave dissipation flux' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 249 ;
	}
#Heaviside beta function
'Heaviside beta function' = {
	 table2Version = 160 ;
	 indicatorOfParameter = 254 ;
	}
#Surface geopotential
'Surface geopotential' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 51 ;
	}
#Total column vertically-integrated mass of atmosphere
'Total column vertically-integrated mass of atmosphere' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 53 ;
	}
#Total column vertically-integrated temperature
'Total column vertically-integrated temperature' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 54 ;
	}
#Total column vertically-integrated water vapour
'Total column vertically-integrated water vapour' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 55 ;
	}
#Total column vertically-integrated cloud liquid water
'Total column vertically-integrated cloud liquid water' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 56 ;
	}
#Total column vertically-integrated cloud frozen water
'Total column vertically-integrated cloud frozen water' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 57 ;
	}
#Total column vertically-integrated ozone
'Total column vertically-integrated ozone' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 58 ;
	}
#Total column vertically-integrated kinetic energy
'Total column vertically-integrated kinetic energy' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 59 ;
	}
#Total column vertically-integrated enthalpy
'Total column vertically-integrated enthalpy' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 60 ;
	}
#Total column vertically-integrated potential + internal energy
'Total column vertically-integrated potential + internal energy' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 61 ;
	}
#Total column vertically-integrated potential+internal+latent energy
'Total column vertically-integrated potential+internal+latent energy' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 62 ;
	}
#Total column vertically-integrated total energy
'Total column vertically-integrated total energy' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 63 ;
	}
#Total column vertically-integrated energy conversion
'Total column vertically-integrated energy conversion' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 64 ;
	}
#Total column vertically-integrated eastward mass flux
'Total column vertically-integrated eastward mass flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 65 ;
	}
#Total column vertically-integrated northward mass flux
'Total column vertically-integrated northward mass flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 66 ;
	}
#Total column vertically-integrated eastward kinetic energy flux
'Total column vertically-integrated eastward kinetic energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 67 ;
	}
#Total column vertically-integrated northward kinetic energy flux
'Total column vertically-integrated northward kinetic energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 68 ;
	}
#Total column vertically-integrated eastward heat flux
'Total column vertically-integrated eastward heat flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 69 ;
	}
#Total column vertically-integrated northward heat flux
'Total column vertically-integrated northward heat flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 70 ;
	}
#Total column vertically-integrated eastward water vapour flux
'Total column vertically-integrated eastward water vapour flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 71 ;
	}
#Total column vertically-integrated northward water vapour flux
'Total column vertically-integrated northward water vapour flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 72 ;
	}
#Total column vertically-integrated eastward geopotential flux
'Total column vertically-integrated eastward geopotential flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 73 ;
	}
#Total column vertically-integrated northward geopotential flux
'Total column vertically-integrated northward geopotential flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 74 ;
	}
#Total column vertically-integrated eastward total energy flux
'Total column vertically-integrated eastward total energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 75 ;
	}
#Total column vertically-integrated northward total energy flux
'Total column vertically-integrated northward total energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 76 ;
	}
#Total column vertically-integrated eastward ozone flux
'Total column vertically-integrated eastward ozone flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 77 ;
	}
#Total column vertically-integrated northward ozone flux
'Total column vertically-integrated northward ozone flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 78 ;
	}
#Total column vertically-integrated divergence of mass flux
'Total column vertically-integrated divergence of mass flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 81 ;
	}
#Total column vertically-integrated divergence of kinetic energy flux
'Total column vertically-integrated divergence of kinetic energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 82 ;
	}
#Total column vertically-integrated divergence of thermal energy flux
'Total column vertically-integrated divergence of thermal energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 83 ;
	}
#Total column vertically-integrated moisture divergence flux
'Total column vertically-integrated moisture divergence flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 84 ;
	}
#Total column vertically-integrated divergence of geopotential flux
'Total column vertically-integrated divergence of geopotential flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 85 ;
	}
#Total column vertically-integrated divergence of total energy flux
'Total column vertically-integrated divergence of total energy flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 86 ;
	}
#Total column vertically-integrated divergence of ozone flux
'Total column vertically-integrated divergence of ozone flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 87 ;
	}
#Time-integrated temperature tendency due to short-wave radiation
'Time-integrated temperature tendency due to short-wave radiation' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 100 ;
	}
#Time-integrated temperature tendency due to long-wave radiation
'Time-integrated temperature tendency due to long-wave radiation' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 101 ;
	}
#Time-integrated temperature tendency due to short wave radiation, clear sky
'Time-integrated temperature tendency due to short wave radiation, clear sky' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 102 ;
	}
#Time-integrated temperature tendency due to long-wave radiation, clear sky
'Time-integrated temperature tendency due to long-wave radiation, clear sky' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 103 ;
	}
#Time-integrated updraught mass flux
'Time-integrated updraught mass flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 104 ;
	}
#Time-integrated downdraught mass flux
'Time-integrated downdraught mass flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 105 ;
	}
#Time-integrated updraught detrainment rate
'Time-integrated updraught detrainment rate' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 106 ;
	}
#Time-integrated downdraught detrainment rate
'Time-integrated downdraught detrainment rate' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 107 ;
	}
#Time-integrated total precipitation flux
'Time-integrated total precipitation flux' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 108 ;
	}
#Time-integrated turbulent diffusion coefficient for heat
'Time-integrated turbulent diffusion coefficient for heat' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 109 ;
	}
#Time-integrated temperature tendency due to parametrisations
'Time-integrated temperature tendency due to parametrisations' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 110 ;
	}
#Time-integrated specific humidity tendency due to parametrisations
'Time-integrated specific humidity tendency due to parametrisations' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 111 ;
	}
#Time-integrated eastward wind tendency due to parametrisations
'Time-integrated eastward wind tendency due to parametrisations' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 112 ;
	}
#Time-integrated northward wind tendency due to parametrisations
'Time-integrated northward wind tendency due to parametrisations' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 113 ;
	}
#Variance of geopotential
'Variance of geopotential' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 206 ;
	}
#Covariance of geopotential/temperature
'Covariance of geopotential/temperature' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 207 ;
	}
#Variance of temperature
'Variance of temperature' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 208 ;
	}
#Covariance of geopotential/specific humidity
'Covariance of geopotential/specific humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 209 ;
	}
#Covariance of temperature/specific humidity
'Covariance of temperature/specific humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 210 ;
	}
#Variance of specific humidity
'Variance of specific humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 211 ;
	}
#Covariance of u component/geopotential
'Covariance of u component/geopotential' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 212 ;
	}
#Covariance of u component/temperature
'Covariance of u component/temperature' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 213 ;
	}
#Covariance of u component/specific humidity
'Covariance of u component/specific humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 214 ;
	}
#Variance of u component
'Variance of u component' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 215 ;
	}
#Covariance of v component/geopotential
'Covariance of v component/geopotential' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 216 ;
	}
#Covariance of v component/temperature
'Covariance of v component/temperature' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 217 ;
	}
#Covariance of v component/specific humidity
'Covariance of v component/specific humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 218 ;
	}
#Covariance of v component/u component
'Covariance of v component/u component' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 219 ;
	}
#Variance of v component
'Variance of v component' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 220 ;
	}
#Covariance of omega/geopotential
'Covariance of omega/geopotential' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 221 ;
	}
#Covariance of omega/temperature
'Covariance of omega/temperature' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 222 ;
	}
#Covariance of omega/specific humidity
'Covariance of omega/specific humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 223 ;
	}
#Covariance of omega/u component
'Covariance of omega/u component' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 224 ;
	}
#Covariance of omega/v component
'Covariance of omega/v component' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 225 ;
	}
#Variance of omega
'Variance of omega' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 226 ;
	}
#Variance of surface pressure
'Variance of surface pressure' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 227 ;
	}
#Variance of relative humidity
'Variance of relative humidity' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 229 ;
	}
#Covariance of u component/ozone
'Covariance of u component/ozone' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 230 ;
	}
#Covariance of v component/ozone
'Covariance of v component/ozone' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 231 ;
	}
#Covariance of omega/ozone
'Covariance of omega/ozone' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 232 ;
	}
#Variance of ozone
'Variance of ozone' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 233 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 162 ;
	 indicatorOfParameter = 255 ;
	}
#Total soil moisture
'Total soil moisture' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 149 ;
	}
#Soil wetness level 2
'Soil wetness level 2' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 171 ;
	}
#Top net thermal radiation
'Top net thermal radiation' = {
	 table2Version = 170 ;
	 indicatorOfParameter = 179 ;
	}
#Stream function anomaly
'Stream function anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 1 ;
	}
#Velocity potential anomaly
'Velocity potential anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 2 ;
	}
#Potential temperature anomaly
'Potential temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 3 ;
	}
#Equivalent potential temperature anomaly
'Equivalent potential temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 4 ;
	}
#Saturated equivalent potential temperature anomaly
'Saturated equivalent potential temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 5 ;
	}
#U component of divergent wind anomaly
'U component of divergent wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 11 ;
	}
#V component of divergent wind anomaly
'V component of divergent wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 12 ;
	}
#U component of rotational wind anomaly
'U component of rotational wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 13 ;
	}
#V component of rotational wind anomaly
'V component of rotational wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 14 ;
	}
#Unbalanced component of temperature anomaly
'Unbalanced component of temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 21 ;
	}
#Unbalanced component of logarithm of surface pressure anomaly
'Unbalanced component of logarithm of surface pressure anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 22 ;
	}
#Unbalanced component of divergence anomaly
'Unbalanced component of divergence anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 23 ;
	}
#Lake cover anomaly
'Lake cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 26 ;
	}
#Low vegetation cover anomaly
'Low vegetation cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 27 ;
	}
#High vegetation cover anomaly
'High vegetation cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 28 ;
	}
#Type of low vegetation anomaly
'Type of low vegetation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 29 ;
	}
#Type of high vegetation anomaly
'Type of high vegetation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 30 ;
	}
#Sea-ice cover anomaly
'Sea-ice cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 31 ;
	}
#Snow albedo anomaly
'Snow albedo anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 32 ;
	}
#Snow density anomaly
'Snow density anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 33 ;
	}
#Sea surface temperature anomaly
'Sea surface temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 34 ;
	}
#Ice surface temperature anomaly layer 1
'Ice surface temperature anomaly layer 1' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 35 ;
	}
#Ice surface temperature anomaly layer 2
'Ice surface temperature anomaly layer 2' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 36 ;
	}
#Ice surface temperature anomaly layer 3
'Ice surface temperature anomaly layer 3' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 37 ;
	}
#Ice surface temperature anomaly layer 4
'Ice surface temperature anomaly layer 4' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 38 ;
	}
#Volumetric soil water anomaly layer 1
'Volumetric soil water anomaly layer 1' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 39 ;
	}
#Volumetric soil water anomaly layer 2
'Volumetric soil water anomaly layer 2' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 40 ;
	}
#Volumetric soil water anomaly layer 3
'Volumetric soil water anomaly layer 3' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 41 ;
	}
#Volumetric soil water anomaly layer 4
'Volumetric soil water anomaly layer 4' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 42 ;
	}
#Soil type anomaly
'Soil type anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 43 ;
	}
#Snow evaporation anomaly
'Snow evaporation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 44 ;
	}
#Snowmelt anomaly
'Snowmelt anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 45 ;
	}
#Solar duration anomaly
'Solar duration anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 46 ;
	}
#Direct solar radiation anomaly
'Direct solar radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 47 ;
	}
#Magnitude of turbulent surface stress anomaly
'Magnitude of turbulent surface stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 48 ;
	}
#10 metre wind gust anomaly
'10 metre wind gust anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 49 ;
	}
#Large-scale precipitation fraction anomaly
'Large-scale precipitation fraction anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 50 ;
	}
#Maximum 2 metre temperature in the last 24 hours anomaly
'Maximum 2 metre temperature in the last 24 hours anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 51 ;
	}
#Minimum 2 metre temperature in the last 24 hours anomaly
'Minimum 2 metre temperature in the last 24 hours anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 52 ;
	}
#Montgomery potential anomaly
'Montgomery potential anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 53 ;
	}
#Pressure anomaly
'Pressure anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 54 ;
	}
#Mean 2 metre temperature in the last 24 hours anomaly
'Mean 2 metre temperature in the last 24 hours anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 55 ;
	}
#Mean 2 metre dewpoint temperature in the last 24 hours anomaly
'Mean 2 metre dewpoint temperature in the last 24 hours anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 56 ;
	}
#Downward UV radiation at the surface anomaly
'Downward UV radiation at the surface anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 57 ;
	}
#Photosynthetically active radiation at the surface anomaly
'Photosynthetically active radiation at the surface anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 58 ;
	}
#Convective available potential energy anomaly
'Convective available potential energy anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 59 ;
	}
#Potential vorticity anomaly
'Potential vorticity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 60 ;
	}
#Total precipitation from observations anomaly
'Total precipitation from observations anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 61 ;
	}
#Observation count anomaly
'Observation count anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 62 ;
	}
#Start time for skin temperature difference anomaly
'Start time for skin temperature difference anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 63 ;
	}
#Finish time for skin temperature difference anomaly
'Finish time for skin temperature difference anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 64 ;
	}
#Skin temperature difference anomaly
'Skin temperature difference anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 65 ;
	}
#Total column liquid water anomaly
'Total column liquid water anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 78 ;
	}
#Total column ice water anomaly
'Total column ice water anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 79 ;
	}
#Vertically integrated total energy anomaly
'Vertically integrated total energy anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 125 ;
	}
#Generic parameter for sensitive area prediction
'Generic parameter for sensitive area prediction' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 126 ;
	}
#Atmospheric tide anomaly
'Atmospheric tide anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 127 ;
	}
#Budget values anomaly
'Budget values anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 128 ;
	}
#Geopotential anomaly
'Geopotential anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 129 ;
	}
#Temperature anomaly
'Temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 130 ;
	}
#U component of wind anomaly
'U component of wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 131 ;
	}
#V component of wind anomaly
'V component of wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 132 ;
	}
#Specific humidity anomaly
'Specific humidity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 133 ;
	}
#Surface pressure anomaly
'Surface pressure anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 134 ;
	}
#Vertical velocity (pressure) anomaly
'Vertical velocity (pressure) anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 135 ;
	}
#Total column water anomaly
'Total column water anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 136 ;
	}
#Total column water vapour anomaly
'Total column water vapour anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 137 ;
	}
#Relative vorticity anomaly
'Relative vorticity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 138 ;
	}
#Soil temperature anomaly level 1
'Soil temperature anomaly level 1' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 139 ;
	}
#Soil wetness anomaly level 1
'Soil wetness anomaly level 1' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 140 ;
	}
#Snow depth anomaly
'Snow depth anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 141 ;
	}
#Stratiform precipitation (Large-scale precipitation) anomaly
'Stratiform precipitation (Large-scale precipitation) anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 142 ;
	}
#Convective precipitation anomaly
'Convective precipitation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall (convective + stratiform) anomaly
'Snowfall (convective + stratiform) anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 144 ;
	}
#Boundary layer dissipation anomaly
'Boundary layer dissipation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 145 ;
	}
#Surface sensible heat flux anomaly
'Surface sensible heat flux anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 146 ;
	}
#Surface latent heat flux anomaly
'Surface latent heat flux anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 147 ;
	}
#Charnock anomaly
'Charnock anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 148 ;
	}
#Surface net radiation anomaly
'Surface net radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 149 ;
	}
#Top net radiation anomaly
'Top net radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 150 ;
	}
#Mean sea level pressure anomaly
'Mean sea level pressure anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 151 ;
	}
#Logarithm of surface pressure anomaly
'Logarithm of surface pressure anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 152 ;
	}
#Short-wave heating rate anomaly
'Short-wave heating rate anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 153 ;
	}
#Long-wave heating rate anomaly
'Long-wave heating rate anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 154 ;
	}
#Relative divergence anomaly
'Relative divergence anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 155 ;
	}
#Height anomaly
'Height anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 156 ;
	}
#Relative humidity anomaly
'Relative humidity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 157 ;
	}
#Tendency of surface pressure anomaly
'Tendency of surface pressure anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 158 ;
	}
#Boundary layer height anomaly
'Boundary layer height anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 159 ;
	}
#Standard deviation of orography anomaly
'Standard deviation of orography anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 160 ;
	}
#Anisotropy of sub-gridscale orography anomaly
'Anisotropy of sub-gridscale orography anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 161 ;
	}
#Angle of sub-gridscale orography anomaly
'Angle of sub-gridscale orography anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 162 ;
	}
#Slope of sub-gridscale orography anomaly
'Slope of sub-gridscale orography anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 163 ;
	}
#Total cloud cover anomaly
'Total cloud cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 164 ;
	}
#10 metre U wind component anomaly
'10 metre U wind component anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 165 ;
	}
#10 metre V wind component anomaly
'10 metre V wind component anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 166 ;
	}
#2 metre temperature anomaly
'2 metre temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 167 ;
	}
#2 metre dewpoint temperature anomaly
'2 metre dewpoint temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 168 ;
	}
#Surface solar radiation downwards anomaly
'Surface solar radiation downwards anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 169 ;
	}
#Soil temperature anomaly level 2
'Soil temperature anomaly level 2' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 170 ;
	}
#Soil wetness anomaly level 2
'Soil wetness anomaly level 2' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 171 ;
	}
#Surface roughness anomaly
'Surface roughness anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 173 ;
	}
#Albedo anomaly
'Albedo anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 174 ;
	}
#Surface thermal radiation downwards anomaly
'Surface thermal radiation downwards anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 175 ;
	}
#Surface net solar radiation anomaly
'Surface net solar radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net thermal radiation anomaly
'Surface net thermal radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 177 ;
	}
#Top net solar radiation anomaly
'Top net solar radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 178 ;
	}
#Top net thermal radiation anomaly
'Top net thermal radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 179 ;
	}
#East-West surface stress anomaly
'East-West surface stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 180 ;
	}
#North-South surface stress anomaly
'North-South surface stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 181 ;
	}
#Evaporation anomaly
'Evaporation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 182 ;
	}
#Soil temperature anomaly level 3
'Soil temperature anomaly level 3' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 183 ;
	}
#Soil wetness anomaly level 3
'Soil wetness anomaly level 3' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 184 ;
	}
#Convective cloud cover anomaly
'Convective cloud cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 185 ;
	}
#Low cloud cover anomaly
'Low cloud cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 186 ;
	}
#Medium cloud cover anomaly
'Medium cloud cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 187 ;
	}
#High cloud cover anomaly
'High cloud cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 188 ;
	}
#Sunshine duration anomaly
'Sunshine duration anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 189 ;
	}
#East-West component of sub-gridscale orographic variance anomaly
'East-West component of sub-gridscale orographic variance anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 190 ;
	}
#North-South component of sub-gridscale orographic variance anomaly
'North-South component of sub-gridscale orographic variance anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 191 ;
	}
#North-West/South-East component of sub-gridscale orographic variance anomaly
'North-West/South-East component of sub-gridscale orographic variance anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 192 ;
	}
#North-East/South-West component of sub-gridscale orographic variance anomaly
'North-East/South-West component of sub-gridscale orographic variance anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 193 ;
	}
#Brightness temperature anomaly
'Brightness temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 194 ;
	}
#Longitudinal component of gravity wave stress anomaly
'Longitudinal component of gravity wave stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 195 ;
	}
#Meridional component of gravity wave stress anomaly
'Meridional component of gravity wave stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation anomaly
'Gravity wave dissipation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 197 ;
	}
#Skin reservoir content anomaly
'Skin reservoir content anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 198 ;
	}
#Vegetation fraction anomaly
'Vegetation fraction anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 199 ;
	}
#Variance of sub-gridscale orography anomaly
'Variance of sub-gridscale orography anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 200 ;
	}
#Maximum temperature at 2 metres anomaly
'Maximum temperature at 2 metres anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 201 ;
	}
#Minimum temperature at 2 metres anomaly
'Minimum temperature at 2 metres anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 202 ;
	}
#Ozone mass mixing ratio anomaly
'Ozone mass mixing ratio anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 203 ;
	}
#Precipitation analysis weights anomaly
'Precipitation analysis weights anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 204 ;
	}
#Runoff anomaly
'Runoff anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 205 ;
	}
#Total column ozone anomaly
'Total column ozone anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 206 ;
	}
#10 metre wind speed anomaly
'10 metre wind speed anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 207 ;
	}
#Top net solar radiation clear sky anomaly
'Top net solar radiation clear sky anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 208 ;
	}
#Top net thermal radiation clear sky anomaly
'Top net thermal radiation clear sky anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 209 ;
	}
#Surface net solar radiation clear sky anomaly
'Surface net solar radiation clear sky anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 210 ;
	}
#Surface net thermal radiation, clear sky anomaly
'Surface net thermal radiation, clear sky anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 211 ;
	}
#Solar insolation anomaly
'Solar insolation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 212 ;
	}
#Diabatic heating by radiation anomaly
'Diabatic heating by radiation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 214 ;
	}
#Diabatic heating by vertical diffusion anomaly
'Diabatic heating by vertical diffusion anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 215 ;
	}
#Diabatic heating by cumulus convection anomaly
'Diabatic heating by cumulus convection anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 216 ;
	}
#Diabatic heating by large-scale condensation anomaly
'Diabatic heating by large-scale condensation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 217 ;
	}
#Vertical diffusion of zonal wind anomaly
'Vertical diffusion of zonal wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 218 ;
	}
#Vertical diffusion of meridional wind anomaly
'Vertical diffusion of meridional wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 219 ;
	}
#East-West gravity wave drag tendency anomaly
'East-West gravity wave drag tendency anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 220 ;
	}
#North-South gravity wave drag tendency anomaly
'North-South gravity wave drag tendency anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 221 ;
	}
#Convective tendency of zonal wind anomaly
'Convective tendency of zonal wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 222 ;
	}
#Convective tendency of meridional wind anomaly
'Convective tendency of meridional wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 223 ;
	}
#Vertical diffusion of humidity anomaly
'Vertical diffusion of humidity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 224 ;
	}
#Humidity tendency by cumulus convection anomaly
'Humidity tendency by cumulus convection anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 225 ;
	}
#Humidity tendency by large-scale condensation anomaly
'Humidity tendency by large-scale condensation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 226 ;
	}
#Change from removal of negative humidity anomaly
'Change from removal of negative humidity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 227 ;
	}
#Total precipitation anomaly
'Total precipitation anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 228 ;
	}
#Instantaneous X surface stress anomaly
'Instantaneous X surface stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 229 ;
	}
#Instantaneous Y surface stress anomaly
'Instantaneous Y surface stress anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 230 ;
	}
#Instantaneous surface heat flux anomaly
'Instantaneous surface heat flux anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 231 ;
	}
#Instantaneous moisture flux anomaly
'Instantaneous moisture flux anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 232 ;
	}
#Apparent surface humidity anomaly
'Apparent surface humidity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 233 ;
	}
#Logarithm of surface roughness length for heat anomaly
'Logarithm of surface roughness length for heat anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 234 ;
	}
#Skin temperature anomaly
'Skin temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 235 ;
	}
#Soil temperature level 4 anomaly
'Soil temperature level 4 anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 236 ;
	}
#Soil wetness level 4 anomaly
'Soil wetness level 4 anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 237 ;
	}
#Temperature of snow layer anomaly
'Temperature of snow layer anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 238 ;
	}
#Convective snowfall anomaly
'Convective snowfall anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 239 ;
	}
#Large scale snowfall anomaly
'Large scale snowfall anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 240 ;
	}
#Accumulated cloud fraction tendency anomaly
'Accumulated cloud fraction tendency anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 241 ;
	}
#Accumulated liquid water tendency anomaly
'Accumulated liquid water tendency anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 242 ;
	}
#Forecast albedo anomaly
'Forecast albedo anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 243 ;
	}
#Forecast surface roughness anomaly
'Forecast surface roughness anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 244 ;
	}
#Forecast logarithm of surface roughness for heat anomaly
'Forecast logarithm of surface roughness for heat anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 245 ;
	}
#Cloud liquid water content anomaly
'Cloud liquid water content anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 246 ;
	}
#Cloud ice water content anomaly
'Cloud ice water content anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 247 ;
	}
#Cloud cover anomaly
'Cloud cover anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 248 ;
	}
#Accumulated ice water tendency anomaly
'Accumulated ice water tendency anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 249 ;
	}
#Ice age anomaly
'Ice age anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 250 ;
	}
#Adiabatic tendency of temperature anomaly
'Adiabatic tendency of temperature anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 251 ;
	}
#Adiabatic tendency of humidity anomaly
'Adiabatic tendency of humidity anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 252 ;
	}
#Adiabatic tendency of zonal wind anomaly
'Adiabatic tendency of zonal wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 253 ;
	}
#Adiabatic tendency of meridional wind anomaly
'Adiabatic tendency of meridional wind anomaly' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 254 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 171 ;
	 indicatorOfParameter = 255 ;
	}
#Time-mean snow evaporation rate
'Time-mean snow evaporation rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 44 ;
	}
#Time-mean snowmelt rate
'Time-mean snowmelt rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 45 ;
	}
#Mean large-scale precipitation fraction
'Mean large-scale precipitation fraction' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 50 ;
	}
#Time-mean large-scale precipitation rate
'Time-mean large-scale precipitation rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 142 ;
	}
#Time-mean convective precipitation rate
'Time-mean convective precipitation rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 143 ;
	}
#Time-mean total snowfall rate
'Time-mean total snowfall rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 144 ;
	}
#Boundary layer dissipation
'Boundary layer dissipation' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 145 ;
	}
#Time-mean surface sensible heat flux
'Time-mean surface sensible heat flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 146 ;
	}
#Time-mean surface latent heat flux
'Time-mean surface latent heat flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 147 ;
	}
#Time-mean short-wave (solar) heating rate
'Time-mean short-wave (solar) heating rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 153 ;
	}
#Time-mean long-wave (thermal) heating rate
'Time-mean long-wave (thermal) heating rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 154 ;
	}
#Time-mean surface downward short-wave (solar) radiation flux
'Time-mean surface downward short-wave (solar) radiation flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 169 ;
	}
#Time-mean surface downward long-wave (thermal) radiation flux
'Time-mean surface downward long-wave (thermal) radiation flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 175 ;
	}
#Time-mean surface net short-wave (solar) radiation flux
'Time-mean surface net short-wave (solar) radiation flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 176 ;
	}
#Time-mean surface net long-wave (thermal) radiation flux
'Time-mean surface net long-wave (thermal) radiation flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 177 ;
	}
#Time-mean top net short-wave (solar) radiation flux
'Time-mean top net short-wave (solar) radiation flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 178 ;
	}
#Time-mean top net long-wave (thermal) radiation flux
'Time-mean top net long-wave (thermal) radiation flux' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 179 ;
	}
#Time-mean eastward turbulent surface stress
'Time-mean eastward turbulent surface stress' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 180 ;
	}
#Time-mean northward turbulent surface stress
'Time-mean northward turbulent surface stress' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 181 ;
	}
#Time-mean evaporation rate
'Time-mean evaporation rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 182 ;
	}
#Mean sunshine duration rate
'Mean sunshine duration rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 189 ;
	}
#Longitudinal component of gravity wave stress
'Longitudinal component of gravity wave stress' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 195 ;
	}
#Meridional component of gravity wave stress
'Meridional component of gravity wave stress' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation
'Gravity wave dissipation' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 197 ;
	}
#Time-mean runoff rate
'Time-mean runoff rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 205 ;
	}
#Time-mean top net short-wave (solar) radiation flux, clear sky
'Time-mean top net short-wave (solar) radiation flux, clear sky' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 208 ;
	}
#Time-mean top net long-wave (thermal) radiation flux, clear sky
'Time-mean top net long-wave (thermal) radiation flux, clear sky' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 209 ;
	}
#Time-mean surface net short-wave (solar) radiation flux, clear sky
'Time-mean surface net short-wave (solar) radiation flux, clear sky' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 210 ;
	}
#Time-mean surface net long-wave (thermal) radiation flux, clear sky
'Time-mean surface net long-wave (thermal) radiation flux, clear sky' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 211 ;
	}
#Time-mean short-wave (solar) insolation rate
'Time-mean short-wave (solar) insolation rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 212 ;
	}
#Time-mean total precipitation rate
'Time-mean total precipitation rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 228 ;
	}
#Time-mean convective snowfall rate
'Time-mean convective snowfall rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 239 ;
	}
#Time-mean large-scale snowfall rate
'Time-mean large-scale snowfall rate' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 240 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 172 ;
	 indicatorOfParameter = 255 ;
	}
#Time-mean snow evaporation anomalous rate of accumulation
'Time-mean snow evaporation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 44 ;
	}
#Time-mean snowmelt anomalous rate of accumulation
'Time-mean snowmelt anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 45 ;
	}
#Magnitude of turbulent surface stress anomaly
'Magnitude of turbulent surface stress anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 48 ;
	}
#Large-scale precipitation fraction anomaly
'Large-scale precipitation fraction anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 50 ;
	}
#Stratiform precipitation (Large-scale precipitation) anomalous rate of accumulation
'Stratiform precipitation (Large-scale precipitation) anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 142 ;
	}
#Mean convective precipitation rate anomaly
'Mean convective precipitation rate anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall (convective + stratiform) anomalous rate of accumulation
'Snowfall (convective + stratiform) anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 144 ;
	}
#Boundary layer dissipation anomaly
'Boundary layer dissipation anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 145 ;
	}
#Surface sensible heat flux anomalous rate of accumulation
'Surface sensible heat flux anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 146 ;
	}
#Surface latent heat flux anomalous rate of accumulation
'Surface latent heat flux anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 147 ;
	}
#Surface net radiation anomaly
'Surface net radiation anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 149 ;
	}
#Short-wave heating rate anomaly
'Short-wave heating rate anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 153 ;
	}
#Long-wave heating rate anomaly
'Long-wave heating rate anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 154 ;
	}
#Surface solar radiation downwards anomalous rate of accumulation
'Surface solar radiation downwards anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 169 ;
	}
#Surface thermal radiation downwards anomalous rate of accumulation
'Surface thermal radiation downwards anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 175 ;
	}
#Surface solar radiation anomalous rate of accumulation
'Surface solar radiation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 176 ;
	}
#Surface thermal radiation anomalous rate of accumulation
'Surface thermal radiation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 177 ;
	}
#Top solar radiation anomalous rate of accumulation
'Top solar radiation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 178 ;
	}
#Top thermal radiation anomalous rate of accumulation
'Top thermal radiation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 179 ;
	}
#East-West surface stress anomalous rate of accumulation
'East-West surface stress anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 180 ;
	}
#North-South surface stress anomalous rate of accumulation
'North-South surface stress anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 181 ;
	}
#Time-mean evaporation anomalous rate of accumulation
'Time-mean evaporation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 182 ;
	}
#Sunshine duration anomalous rate of accumulation
'Sunshine duration anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 189 ;
	}
#Longitudinal component of gravity wave stress anomaly
'Longitudinal component of gravity wave stress anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 195 ;
	}
#Meridional component of gravity wave stress anomaly
'Meridional component of gravity wave stress anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation anomaly
'Gravity wave dissipation anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 197 ;
	}
#Runoff anomalous rate of accumulation
'Runoff anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 205 ;
	}
#Top net solar radiation, clear sky anomaly
'Top net solar radiation, clear sky anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 208 ;
	}
#Top net thermal radiation, clear sky anomaly
'Top net thermal radiation, clear sky anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 209 ;
	}
#Surface net solar radiation, clear sky anomaly
'Surface net solar radiation, clear sky anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 210 ;
	}
#Surface net thermal radiation, clear sky anomaly
'Surface net thermal radiation, clear sky anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 211 ;
	}
#Solar insolation anomalous rate of accumulation
'Solar insolation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 212 ;
	}
#Total precipitation anomalous rate of accumulation
'Total precipitation anomalous rate of accumulation' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 228 ;
	}
#Convective snowfall anomaly
'Convective snowfall anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 239 ;
	}
#Large scale snowfall anomaly
'Large scale snowfall anomaly' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 240 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 173 ;
	 indicatorOfParameter = 255 ;
	}
#Total soil moisture
'Total soil moisture' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 6 ;
	}
#Surface runoff
'Surface runoff' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 8 ;
	}
#Fraction of sea-ice in sea
'Fraction of sea-ice in sea' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 31 ;
	}
#Open-sea surface temperature
'Open-sea surface temperature' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 34 ;
	}
#Volumetric soil water layer 1
'Volumetric soil water layer 1' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 39 ;
	}
#Volumetric soil water layer 2
'Volumetric soil water layer 2' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 40 ;
	}
#Volumetric soil water layer 3
'Volumetric soil water layer 3' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 41 ;
	}
#Volumetric soil water layer 4
'Volumetric soil water layer 4' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 42 ;
	}
#10 metre wind gust in the last 24 hours
'10 metre wind gust in the last 24 hours' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 49 ;
	}
#1.5m temperature - mean in the last 24 hours
'1.5m temperature - mean in the last 24 hours' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 55 ;
	}
#Net primary productivity
'Net primary productivity' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 83 ;
	}
#10m U wind over land
'10m U wind over land' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 85 ;
	}
#10m V wind over land
'10m V wind over land' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 86 ;
	}
#1.5m temperature over land
'1.5m temperature over land' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 87 ;
	}
#1.5m dewpoint temperature over land
'1.5m dewpoint temperature over land' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 88 ;
	}
#Top incoming solar radiation
'Top incoming solar radiation' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 89 ;
	}
#Top outgoing solar radiation
'Top outgoing solar radiation' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 90 ;
	}
#Mean sea surface temperature
'Mean sea surface temperature' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 94 ;
	}
#1.5m specific humidity
'1.5m specific humidity' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 95 ;
	}
#Sea-ice thickness
'Sea-ice thickness' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 98 ;
	}
#Liquid water potential temperature
'Liquid water potential temperature' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 99 ;
	}
#Ocean ice concentration
'Ocean ice concentration' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 110 ;
	}
#Ocean mean ice depth
'Ocean mean ice depth' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 111 ;
	}
#Soil temperature layer 1
'Soil temperature layer 1' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 139 ;
	}
#Average potential temperature in upper 293.4m
'Average potential temperature in upper 293.4m' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 164 ;
	}
#1.5m temperature
'1.5m temperature' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 167 ;
	}
#1.5m dewpoint temperature
'1.5m dewpoint temperature' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 168 ;
	}
#Soil temperature layer 2
'Soil temperature layer 2' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 170 ;
	}
#Average salinity in upper 293.4m
'Average salinity in upper 293.4m' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 175 ;
	}
#Soil temperature layer 3
'Soil temperature layer 3' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 183 ;
	}
#1.5m temperature - maximum in the last 24 hours
'1.5m temperature - maximum in the last 24 hours' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 201 ;
	}
#1.5m temperature - minimum in the last 24 hours
'1.5m temperature - minimum in the last 24 hours' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 202 ;
	}
#Soil temperature layer 4
'Soil temperature layer 4' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 236 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 174 ;
	 indicatorOfParameter = 255 ;
	}
#Total soil moisture
'Total soil moisture' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 6 ;
	}
#Fraction of sea-ice in sea
'Fraction of sea-ice in sea' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 31 ;
	}
#Open-sea surface temperature
'Open-sea surface temperature' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 34 ;
	}
#Volumetric soil water layer 1
'Volumetric soil water layer 1' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 39 ;
	}
#Volumetric soil water layer 2
'Volumetric soil water layer 2' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 40 ;
	}
#Volumetric soil water layer 3
'Volumetric soil water layer 3' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 41 ;
	}
#Volumetric soil water layer 4
'Volumetric soil water layer 4' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 42 ;
	}
#10m wind gust in the last 24 hours
'10m wind gust in the last 24 hours' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 49 ;
	}
#1.5m temperature - mean in the last 24 hours
'1.5m temperature - mean in the last 24 hours' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 55 ;
	}
#Net primary productivity
'Net primary productivity' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 83 ;
	}
#10m U wind over land
'10m U wind over land' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 85 ;
	}
#10m V wind over land
'10m V wind over land' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 86 ;
	}
#1.5m temperature over land
'1.5m temperature over land' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 87 ;
	}
#1.5m dewpoint temperature over land
'1.5m dewpoint temperature over land' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 88 ;
	}
#Top incoming solar radiation
'Top incoming solar radiation' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 89 ;
	}
#Top outgoing solar radiation
'Top outgoing solar radiation' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 90 ;
	}
#Ocean ice concentration
'Ocean ice concentration' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 110 ;
	}
#Ocean mean ice depth
'Ocean mean ice depth' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 111 ;
	}
#Soil temperature layer 1
'Soil temperature layer 1' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 139 ;
	}
#Average potential temperature in upper 293.4m
'Average potential temperature in upper 293.4m' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 164 ;
	}
#1.5m temperature
'1.5m temperature' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 167 ;
	}
#1.5m dewpoint temperature
'1.5m dewpoint temperature' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 168 ;
	}
#Soil temperature layer 2
'Soil temperature layer 2' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 170 ;
	}
#Average salinity in upper 293.4m
'Average salinity in upper 293.4m' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 175 ;
	}
#Soil temperature layer 3
'Soil temperature layer 3' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 183 ;
	}
#1.5m temperature - maximum in the last 24 hours
'1.5m temperature - maximum in the last 24 hours' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 201 ;
	}
#1.5m temperature - minimum in the last 24 hours
'1.5m temperature - minimum in the last 24 hours' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 202 ;
	}
#Soil temperature layer 4
'Soil temperature layer 4' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 236 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 175 ;
	 indicatorOfParameter = 255 ;
	}
#Total soil wetness
'Total soil wetness' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 149 ;
	}
#Surface net solar radiation
'Surface net solar radiation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net thermal radiation
'Surface net thermal radiation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 177 ;
	}
#Top net solar radiation
'Top net solar radiation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 178 ;
	}
#Top net thermal radiation
'Top net thermal radiation' = {
	 table2Version = 180 ;
	 indicatorOfParameter = 179 ;
	}
#Snow depth
'Snow depth' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 141 ;
	}
#Field capacity
'Field capacity' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 170 ;
	}
#Wilting point
'Wilting point' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 171 ;
	}
#Roughness length
'Roughness length' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 173 ;
	}
#Total soil moisture
'Total soil moisture' = {
	 table2Version = 190 ;
	 indicatorOfParameter = 229 ;
	}
#2 metre dewpoint temperature difference
'2 metre dewpoint temperature difference' = {
	 table2Version = 200 ;
	 indicatorOfParameter = 168 ;
	}
#downward shortwave radiant flux density
'downward shortwave radiant flux density' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 1 ;
	}
#upward shortwave radiant flux density
'upward shortwave radiant flux density' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 2 ;
	}
#downward longwave radiant flux density
'downward longwave radiant flux density' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 3 ;
	}
#upward longwave radiant flux density
'upward longwave radiant flux density' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 4 ;
	}
#downwd photosynthetic active radiant flux density
'downwd photosynthetic active radiant flux density' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 5 ;
	}
#net shortwave flux
'net shortwave flux' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 6 ;
	}
#net longwave flux
'net longwave flux' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 7 ;
	}
#total net radiative flux density
'total net radiative flux density' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 8 ;
	}
#downw shortw radiant flux density, cloudfree part
'downw shortw radiant flux density, cloudfree part' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 9 ;
	}
#upw shortw radiant flux density, cloudy part
'upw shortw radiant flux density, cloudy part' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 10 ;
	}
#downw longw radiant flux density, cloudfree part
'downw longw radiant flux density, cloudfree part' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 11 ;
	}
#upw longw radiant flux density, cloudy part
'upw longw radiant flux density, cloudy part' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 12 ;
	}
#shortwave radiative heating rate
'shortwave radiative heating rate' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 13 ;
	}
#longwave radiative heating rate
'longwave radiative heating rate' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 14 ;
	}
#total radiative heating rate
'total radiative heating rate' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 15 ;
	}
#soil heat flux, surface
'soil heat flux, surface' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 16 ;
	}
#soil heat flux, bottom of layer
'soil heat flux, bottom of layer' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 17 ;
	}
#fractional cloud cover
'fractional cloud cover' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 29 ;
	}
#cloud cover, grid scale
'cloud cover, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 30 ;
	}
#specific cloud water content
'specific cloud water content' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 31 ;
	}
#cloud water content, grid scale, vert integrated
'cloud water content, grid scale, vert integrated' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 32 ;
	}
#specific cloud ice content, grid scale
'specific cloud ice content, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 33 ;
	}
#cloud ice content, grid scale, vert integrated
'cloud ice content, grid scale, vert integrated' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 34 ;
	}
#specific rainwater content, grid scale
'specific rainwater content, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 35 ;
	}
#specific snow content, grid scale
'specific snow content, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 36 ;
	}
#specific rainwater content, gs, vert. integrated
'specific rainwater content, gs, vert. integrated' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 37 ;
	}
#specific snow content, gs, vert. integrated
'specific snow content, gs, vert. integrated' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 38 ;
	}
#total column water
'total column water' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 41 ;
	}
#vert. integral of divergence of tot. water content
'vert. integral of divergence of tot. water content' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 42 ;
	}
#cloud covers CH_CM_CL (000...888)
'cloud covers CH_CM_CL (000...888)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 50 ;
	}
#cloud cover CH (0..8)
'cloud cover CH (0..8)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 51 ;
	}
#cloud cover CM (0..8)
'cloud cover CM (0..8)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 52 ;
	}
#cloud cover CL (0..8)
'cloud cover CL (0..8)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 53 ;
	}
#total cloud cover (0..8)
'total cloud cover (0..8)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 54 ;
	}
#fog (0..8)
'fog (0..8)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 55 ;
	}
#fog
'fog' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 56 ;
	}
#cloud cover, convective cirrus
'cloud cover, convective cirrus' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 60 ;
	}
#specific cloud water content, convective clouds
'specific cloud water content, convective clouds' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 61 ;
	}
#cloud water content, conv clouds, vert integrated
'cloud water content, conv clouds, vert integrated' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 62 ;
	}
#specific cloud ice content, convective clouds
'specific cloud ice content, convective clouds' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 63 ;
	}
#cloud ice content, conv clouds, vert integrated
'cloud ice content, conv clouds, vert integrated' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 64 ;
	}
#convective mass flux
'convective mass flux' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 65 ;
	}
#Updraft velocity, convection
'Updraft velocity, convection' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 66 ;
	}
#entrainment parameter, convection
'entrainment parameter, convection' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 67 ;
	}
#cloud base, convective clouds (above msl)
'cloud base, convective clouds (above msl)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 68 ;
	}
#cloud top, convective clouds (above msl)
'cloud top, convective clouds (above msl)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 69 ;
	}
#convective layers (00...77)  (BKE)
'convective layers (00...77)  (BKE)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 70 ;
	}
#KO-index
'KO-index' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 71 ;
	}
#convection base index
'convection base index' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 72 ;
	}
#convection top index
'convection top index' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 73 ;
	}
#convective temperature tendency
'convective temperature tendency' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 74 ;
	}
#convective tendency of specific humidity
'convective tendency of specific humidity' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 75 ;
	}
#convective tendency of total heat
'convective tendency of total heat' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 76 ;
	}
#convective tendency of total water
'convective tendency of total water' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 77 ;
	}
#convective momentum tendency (X-component)
'convective momentum tendency (X-component)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 78 ;
	}
#convective momentum tendency (Y-component)
'convective momentum tendency (Y-component)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 79 ;
	}
#convective vorticity tendency
'convective vorticity tendency' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 80 ;
	}
#convective divergence tendency
'convective divergence tendency' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 81 ;
	}
#top of dry convection (above msl)
'top of dry convection (above msl)' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 82 ;
	}
#dry convection top index
'dry convection top index' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 83 ;
	}
#height of 0 degree Celsius isotherm above msl
'height of 0 degree Celsius isotherm above msl' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 84 ;
	}
#height of snow-fall limit
'height of snow-fall limit' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 85 ;
	}
#spec. content of precip. particles
'spec. content of precip. particles' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 99 ;
	}
#surface precipitation rate, rain, grid scale
'surface precipitation rate, rain, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 100 ;
	}
#surface precipitation rate, snow, grid scale
'surface precipitation rate, snow, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 101 ;
	}
#surface precipitation amount, rain, grid scale
'surface precipitation amount, rain, grid scale' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 102 ;
	}
#surface precipitation rate, rain, convective
'surface precipitation rate, rain, convective' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 111 ;
	}
#surface precipitation rate, snow, convective
'surface precipitation rate, snow, convective' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 112 ;
	}
#surface precipitation amount, rain, convective
'surface precipitation amount, rain, convective' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 113 ;
	}
#coefficient of horizontal diffusion
'coefficient of horizontal diffusion' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 150 ;
	}
#Maximum wind velocity
'Maximum wind velocity' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 187 ;
	}
#water content of interception store
'water content of interception store' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 200 ;
	}
#snow temperature
'snow temperature' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 203 ;
	}
#ice surface temperature
'ice surface temperature' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 215 ;
	}
#convective available potential energy
'convective available potential energy' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 241 ;
	}
#Indicates a missing value
'Indicates a missing value' = {
	 table2Version = 201 ;
	 indicatorOfParameter = 255 ;
	}
#Sea Salt Aerosol (0.03 - 0.5 um) Mixing Ratio
'Sea Salt Aerosol (0.03 - 0.5 um) Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 1 ;
	}
#Sea Salt Aerosol (0.5 - 5 um) Mixing Ratio
'Sea Salt Aerosol (0.5 - 5 um) Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 2 ;
	}
#Sea Salt Aerosol (5 - 20 um) Mixing Ratio
'Sea Salt Aerosol (5 - 20 um) Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 3 ;
	}
#Dust Aerosol (0.03 - 0.55 um) Mixing Ratio
'Dust Aerosol (0.03 - 0.55 um) Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 4 ;
	}
#Dust Aerosol (0.55 - 0.9 um) Mixing Ratio
'Dust Aerosol (0.55 - 0.9 um) Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 5 ;
	}
#Dust Aerosol (0.9 - 20 um) Mixing Ratio
'Dust Aerosol (0.9 - 20 um) Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 6 ;
	}
#Hydrophilic Organic Matter Aerosol Mixing Ratio
'Hydrophilic Organic Matter Aerosol Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 7 ;
	}
#Hydrophobic Organic Matter Aerosol Mixing Ratio
'Hydrophobic Organic Matter Aerosol Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 8 ;
	}
#Hydrophilic Black Carbon Aerosol Mixing Ratio
'Hydrophilic Black Carbon Aerosol Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 9 ;
	}
#Hydrophobic Black Carbon Aerosol Mixing Ratio
'Hydrophobic Black Carbon Aerosol Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 10 ;
	}
#Sulphate Aerosol Mixing Ratio
'Sulphate Aerosol Mixing Ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 11 ;
	}
#SO2 precursor mixing ratio
'SO2 precursor mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 12 ;
	}
#Aerosol type 1 source/gain accumulated
'Aerosol type 1 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 16 ;
	}
#Aerosol type 2 source/gain accumulated
'Aerosol type 2 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 17 ;
	}
#Aerosol type 3 source/gain accumulated
'Aerosol type 3 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 18 ;
	}
#Aerosol type 4 source/gain accumulated
'Aerosol type 4 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 19 ;
	}
#Aerosol type 5 source/gain accumulated
'Aerosol type 5 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 20 ;
	}
#Aerosol type 6 source/gain accumulated
'Aerosol type 6 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 21 ;
	}
#Aerosol type 7 source/gain accumulated
'Aerosol type 7 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 22 ;
	}
#Aerosol type 8 source/gain accumulated
'Aerosol type 8 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 23 ;
	}
#Aerosol type 9 source/gain accumulated
'Aerosol type 9 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 24 ;
	}
#Aerosol type 10 source/gain accumulated
'Aerosol type 10 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 25 ;
	}
#Aerosol type 11 source/gain accumulated
'Aerosol type 11 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 26 ;
	}
#Aerosol type 12 source/gain accumulated
'Aerosol type 12 source/gain accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 27 ;
	}
#Aerosol type 1 sink/loss accumulated
'Aerosol type 1 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 31 ;
	}
#Aerosol type 2 sink/loss accumulated
'Aerosol type 2 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 32 ;
	}
#Aerosol type 3 sink/loss accumulated
'Aerosol type 3 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 33 ;
	}
#Aerosol type 4 sink/loss accumulated
'Aerosol type 4 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 34 ;
	}
#Aerosol type 5 sink/loss accumulated
'Aerosol type 5 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 35 ;
	}
#Aerosol type 6 sink/loss accumulated
'Aerosol type 6 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 36 ;
	}
#Aerosol type 7 sink/loss accumulated
'Aerosol type 7 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 37 ;
	}
#Aerosol type 8 sink/loss accumulated
'Aerosol type 8 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 38 ;
	}
#Aerosol type 9 sink/loss accumulated
'Aerosol type 9 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 39 ;
	}
#Aerosol type 10 sink/loss accumulated
'Aerosol type 10 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 40 ;
	}
#Aerosol type 11 sink/loss accumulated
'Aerosol type 11 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 41 ;
	}
#Aerosol type 12 sink/loss accumulated
'Aerosol type 12 sink/loss accumulated' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 42 ;
	}
#Aerosol precursor mixing ratio
'Aerosol precursor mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 46 ;
	}
#Aerosol small mode mixing ratio
'Aerosol small mode mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 47 ;
	}
#Aerosol large mode mixing ratio
'Aerosol large mode mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 48 ;
	}
#Aerosol precursor optical depth
'Aerosol precursor optical depth' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 49 ;
	}
#Aerosol small mode optical depth
'Aerosol small mode optical depth' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 50 ;
	}
#Aerosol large mode optical depth
'Aerosol large mode optical depth' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 51 ;
	}
#Dust emission potential
'Dust emission potential' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 52 ;
	}
#Lifting threshold speed
'Lifting threshold speed' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 53 ;
	}
#Soil clay content
'Soil clay content' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 54 ;
	}
#Carbon dioxide mass mixing ratio
'Carbon dioxide mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 61 ;
	}
#Methane
'Methane' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 62 ;
	}
#Nitrous oxide
'Nitrous oxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 63 ;
	}
#CO2 column-mean molar fraction
'CO2 column-mean molar fraction' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 64 ;
	}
#CH4 column-mean molar fraction
'CH4 column-mean molar fraction' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 65 ;
	}
#Total column Nitrous oxide
'Total column Nitrous oxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 66 ;
	}
#Ocean flux of Carbon Dioxide
'Ocean flux of Carbon Dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 67 ;
	}
#Natural biosphere flux of Carbon Dioxide
'Natural biosphere flux of Carbon Dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 68 ;
	}
#Anthropogenic emissions of Carbon Dioxide
'Anthropogenic emissions of Carbon Dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 69 ;
	}
#Methane Surface Fluxes
'Methane Surface Fluxes' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 70 ;
	}
#Methane loss rate due to radical hydroxyl (OH)
'Methane loss rate due to radical hydroxyl (OH)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 71 ;
	}
#Wildfire flux of Carbon Dioxide
'Wildfire flux of Carbon Dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 80 ;
	}
#Wildfire flux of Carbon Monoxide
'Wildfire flux of Carbon Monoxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 81 ;
	}
#Wildfire flux of Methane
'Wildfire flux of Methane' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 82 ;
	}
#Wildfire flux of Non-Methane Hydro-Carbons
'Wildfire flux of Non-Methane Hydro-Carbons' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 83 ;
	}
#Wildfire flux of Hydrogen
'Wildfire flux of Hydrogen' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 84 ;
	}
#Wildfire flux of Nitrogen Oxides NOx
'Wildfire flux of Nitrogen Oxides NOx' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 85 ;
	}
#Wildfire flux of Nitrous Oxide
'Wildfire flux of Nitrous Oxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 86 ;
	}
#Wildfire flux of Particulate Matter PM2.5
'Wildfire flux of Particulate Matter PM2.5' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 87 ;
	}
#Wildfire flux of Total Particulate Matter
'Wildfire flux of Total Particulate Matter' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 88 ;
	}
#Wildfire flux of Total Carbon in Aerosols
'Wildfire flux of Total Carbon in Aerosols' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 89 ;
	}
#Wildfire flux of Organic Carbon
'Wildfire flux of Organic Carbon' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 90 ;
	}
#Wildfire flux of Black Carbon
'Wildfire flux of Black Carbon' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 91 ;
	}
#Wildfire overall flux of burnt Carbon
'Wildfire overall flux of burnt Carbon' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 92 ;
	}
#Wildfire fraction of C4 plants
'Wildfire fraction of C4 plants' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 93 ;
	}
#Wildfire vegetation map index
'Wildfire vegetation map index' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 94 ;
	}
#Wildfire Combustion Completeness
'Wildfire Combustion Completeness' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 95 ;
	}
#Wildfire Fuel Load: Carbon per unit area
'Wildfire Fuel Load: Carbon per unit area' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 96 ;
	}
#Wildfire fraction of area observed
'Wildfire fraction of area observed' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 97 ;
	}
#Number of positive FRP pixels per grid cell
'Number of positive FRP pixels per grid cell' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 98 ;
	}
#Wildfire radiative power
'Wildfire radiative power' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 99 ;
	}
#Wildfire combustion rate
'Wildfire combustion rate' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 100 ;
	}
#Nitrogen dioxide mass mixing ratio
'Nitrogen dioxide mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 121 ;
	}
#Sulphur dioxide mass mixing ratio
'Sulphur dioxide mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 122 ;
	}
#Carbon monoxide mass mixing ratio
'Carbon monoxide mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 123 ;
	}
#Formaldehyde
'Formaldehyde' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 124 ;
	}
#Total column Nitrogen dioxide
'Total column Nitrogen dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 125 ;
	}
#Total column Sulphur dioxide
'Total column Sulphur dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 126 ;
	}
#Total column Carbon monoxide
'Total column Carbon monoxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 127 ;
	}
#Total column Formaldehyde
'Total column Formaldehyde' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 128 ;
	}
#Nitrogen Oxides
'Nitrogen Oxides' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 129 ;
	}
#Total Column Nitrogen Oxides
'Total Column Nitrogen Oxides' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 130 ;
	}
#Reactive tracer 1 mass mixing ratio
'Reactive tracer 1 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 131 ;
	}
#Total column GRG tracer 1
'Total column GRG tracer 1' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 132 ;
	}
#Reactive tracer 2 mass mixing ratio
'Reactive tracer 2 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 133 ;
	}
#Total column GRG tracer 2
'Total column GRG tracer 2' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 134 ;
	}
#Reactive tracer 3 mass mixing ratio
'Reactive tracer 3 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 135 ;
	}
#Total column GRG tracer 3
'Total column GRG tracer 3' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 136 ;
	}
#Reactive tracer 4 mass mixing ratio
'Reactive tracer 4 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 137 ;
	}
#Total column GRG tracer 4
'Total column GRG tracer 4' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 138 ;
	}
#Reactive tracer 5 mass mixing ratio
'Reactive tracer 5 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 139 ;
	}
#Total column GRG tracer 5
'Total column GRG tracer 5' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 140 ;
	}
#Reactive tracer 6 mass mixing ratio
'Reactive tracer 6 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 141 ;
	}
#Total column GRG tracer 6
'Total column GRG tracer 6' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 142 ;
	}
#Reactive tracer 7 mass mixing ratio
'Reactive tracer 7 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 143 ;
	}
#Total column GRG tracer 7
'Total column GRG tracer 7' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 144 ;
	}
#Reactive tracer 8 mass mixing ratio
'Reactive tracer 8 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 145 ;
	}
#Total column GRG tracer 8
'Total column GRG tracer 8' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 146 ;
	}
#Reactive tracer 9 mass mixing ratio
'Reactive tracer 9 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 147 ;
	}
#Total column GRG tracer 9
'Total column GRG tracer 9' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 148 ;
	}
#Reactive tracer 10 mass mixing ratio
'Reactive tracer 10 mass mixing ratio' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 149 ;
	}
#Total column GRG tracer 10
'Total column GRG tracer 10' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 150 ;
	}
#Surface flux Nitrogen oxides
'Surface flux Nitrogen oxides' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 151 ;
	}
#Surface flux Nitrogen dioxide
'Surface flux Nitrogen dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 152 ;
	}
#Surface flux Sulphur dioxide
'Surface flux Sulphur dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 153 ;
	}
#Surface flux Carbon monoxide
'Surface flux Carbon monoxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 154 ;
	}
#Surface flux Formaldehyde
'Surface flux Formaldehyde' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 155 ;
	}
#Surface flux GEMS Ozone
'Surface flux GEMS Ozone' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 156 ;
	}
#Surface flux reactive tracer 1
'Surface flux reactive tracer 1' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 157 ;
	}
#Surface flux reactive tracer 2
'Surface flux reactive tracer 2' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 158 ;
	}
#Surface flux reactive tracer 3
'Surface flux reactive tracer 3' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 159 ;
	}
#Surface flux reactive tracer 4
'Surface flux reactive tracer 4' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 160 ;
	}
#Surface flux reactive tracer 5
'Surface flux reactive tracer 5' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 161 ;
	}
#Surface flux reactive tracer 6
'Surface flux reactive tracer 6' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 162 ;
	}
#Surface flux reactive tracer 7
'Surface flux reactive tracer 7' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 163 ;
	}
#Surface flux reactive tracer 8
'Surface flux reactive tracer 8' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 164 ;
	}
#Surface flux reactive tracer 9
'Surface flux reactive tracer 9' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 165 ;
	}
#Surface flux reactive tracer 10
'Surface flux reactive tracer 10' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 166 ;
	}
#Radon
'Radon' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 181 ;
	}
#Sulphur Hexafluoride
'Sulphur Hexafluoride' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 182 ;
	}
#Total column Radon
'Total column Radon' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 183 ;
	}
#Total column Sulphur Hexafluoride
'Total column Sulphur Hexafluoride' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 184 ;
	}
#Anthropogenic Emissions of Sulphur Hexafluoride
'Anthropogenic Emissions of Sulphur Hexafluoride' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 185 ;
	}
#Ozone mass mixing ratio (full chemistry scheme)
'Ozone mass mixing ratio (full chemistry scheme)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 203 ;
	}
#GEMS Total column ozone
'GEMS Total column ozone' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 206 ;
	}
#Total Aerosol Optical Depth at 550nm
'Total Aerosol Optical Depth at 550nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 207 ;
	}
#Sea Salt Aerosol Optical Depth at 550nm
'Sea Salt Aerosol Optical Depth at 550nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 208 ;
	}
#Dust Aerosol Optical Depth at 550nm
'Dust Aerosol Optical Depth at 550nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 209 ;
	}
#Organic Matter Aerosol Optical Depth at 550nm
'Organic Matter Aerosol Optical Depth at 550nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 210 ;
	}
#Black Carbon Aerosol Optical Depth at 550nm
'Black Carbon Aerosol Optical Depth at 550nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 211 ;
	}
#Sulphate Aerosol Optical Depth at 550nm
'Sulphate Aerosol Optical Depth at 550nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 212 ;
	}
#Total Aerosol Optical Depth at 469nm
'Total Aerosol Optical Depth at 469nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 213 ;
	}
#Total Aerosol Optical Depth at 670nm
'Total Aerosol Optical Depth at 670nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 214 ;
	}
#Total Aerosol Optical Depth at 865nm
'Total Aerosol Optical Depth at 865nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 215 ;
	}
#Total Aerosol Optical Depth at 1240nm
'Total Aerosol Optical Depth at 1240nm' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 216 ;
	}
#Sea Salt Aerosol (0.03 - 0.5 um) Mixing Ratio
'Sea Salt Aerosol (0.03 - 0.5 um) Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 1 ;
	}
#Sea Salt Aerosol (0.5 - 5 um) Mixing Ratio
'Sea Salt Aerosol (0.5 - 5 um) Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 2 ;
	}
#Sea Salt Aerosol (5 - 20 um) Mixing Ratio
'Sea Salt Aerosol (5 - 20 um) Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 3 ;
	}
#Dust Aerosol (0.03 - 0.55 um) Mixing Ratio
'Dust Aerosol (0.03 - 0.55 um) Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 4 ;
	}
#Dust Aerosol (0.55 - 0.9 um) Mixing Ratio
'Dust Aerosol (0.55 - 0.9 um) Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 5 ;
	}
#Dust Aerosol (0.9 - 20 um) Mixing Ratio
'Dust Aerosol (0.9 - 20 um) Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 6 ;
	}
#Hydrophilic Organic Matter Aerosol Mixing Ratio
'Hydrophilic Organic Matter Aerosol Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 7 ;
	}
#Hydrophobic Organic Matter Aerosol Mixing Ratio
'Hydrophobic Organic Matter Aerosol Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 8 ;
	}
#Hydrophilic Black Carbon Aerosol Mixing Ratio
'Hydrophilic Black Carbon Aerosol Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 9 ;
	}
#Hydrophobic Black Carbon Aerosol Mixing Ratio
'Hydrophobic Black Carbon Aerosol Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 10 ;
	}
#Sulphate Aerosol Mixing Ratio
'Sulphate Aerosol Mixing Ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 11 ;
	}
#Aerosol type 12 mixing ratio
'Aerosol type 12 mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 12 ;
	}
#Aerosol type 1 source/gain accumulated
'Aerosol type 1 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 16 ;
	}
#Aerosol type 2 source/gain accumulated
'Aerosol type 2 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 17 ;
	}
#Aerosol type 3 source/gain accumulated
'Aerosol type 3 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 18 ;
	}
#Aerosol type 4 source/gain accumulated
'Aerosol type 4 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 19 ;
	}
#Aerosol type 5 source/gain accumulated
'Aerosol type 5 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 20 ;
	}
#Aerosol type 6 source/gain accumulated
'Aerosol type 6 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 21 ;
	}
#Aerosol type 7 source/gain accumulated
'Aerosol type 7 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 22 ;
	}
#Aerosol type 8 source/gain accumulated
'Aerosol type 8 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 23 ;
	}
#Aerosol type 9 source/gain accumulated
'Aerosol type 9 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 24 ;
	}
#Aerosol type 10 source/gain accumulated
'Aerosol type 10 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 25 ;
	}
#Aerosol type 11 source/gain accumulated
'Aerosol type 11 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 26 ;
	}
#Aerosol type 12 source/gain accumulated
'Aerosol type 12 source/gain accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 27 ;
	}
#Aerosol type 1 sink/loss accumulated
'Aerosol type 1 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 31 ;
	}
#Aerosol type 2 sink/loss accumulated
'Aerosol type 2 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 32 ;
	}
#Aerosol type 3 sink/loss accumulated
'Aerosol type 3 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 33 ;
	}
#Aerosol type 4 sink/loss accumulated
'Aerosol type 4 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 34 ;
	}
#Aerosol type 5 sink/loss accumulated
'Aerosol type 5 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 35 ;
	}
#Aerosol type 6 sink/loss accumulated
'Aerosol type 6 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 36 ;
	}
#Aerosol type 7 sink/loss accumulated
'Aerosol type 7 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 37 ;
	}
#Aerosol type 8 sink/loss accumulated
'Aerosol type 8 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 38 ;
	}
#Aerosol type 9 sink/loss accumulated
'Aerosol type 9 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 39 ;
	}
#Aerosol type 10 sink/loss accumulated
'Aerosol type 10 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 40 ;
	}
#Aerosol type 11 sink/loss accumulated
'Aerosol type 11 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 41 ;
	}
#Aerosol type 12 sink/loss accumulated
'Aerosol type 12 sink/loss accumulated' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 42 ;
	}
#Aerosol precursor mixing ratio
'Aerosol precursor mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 46 ;
	}
#Aerosol small mode mixing ratio
'Aerosol small mode mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 47 ;
	}
#Aerosol large mode mixing ratio
'Aerosol large mode mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 48 ;
	}
#Aerosol precursor optical depth
'Aerosol precursor optical depth' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 49 ;
	}
#Aerosol small mode optical depth
'Aerosol small mode optical depth' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 50 ;
	}
#Aerosol large mode optical depth
'Aerosol large mode optical depth' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 51 ;
	}
#Dust emission potential
'Dust emission potential' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 52 ;
	}
#Lifting threshold speed
'Lifting threshold speed' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 53 ;
	}
#Soil clay content
'Soil clay content' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 54 ;
	}
#Carbon Dioxide
'Carbon Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 61 ;
	}
#Methane
'Methane' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 62 ;
	}
#Nitrous oxide
'Nitrous oxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 63 ;
	}
#Total column Carbon Dioxide
'Total column Carbon Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 64 ;
	}
#Total column Methane
'Total column Methane' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 65 ;
	}
#Total column Nitrous oxide
'Total column Nitrous oxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 66 ;
	}
#Ocean flux of Carbon Dioxide
'Ocean flux of Carbon Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 67 ;
	}
#Natural biosphere flux of Carbon Dioxide
'Natural biosphere flux of Carbon Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 68 ;
	}
#Anthropogenic emissions of Carbon Dioxide
'Anthropogenic emissions of Carbon Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 69 ;
	}
#Methane Surface Fluxes
'Methane Surface Fluxes' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 70 ;
	}
#Methane loss rate due to radical hydroxyl (OH)
'Methane loss rate due to radical hydroxyl (OH)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 71 ;
	}
#Wildfire flux of Carbon Dioxide
'Wildfire flux of Carbon Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 80 ;
	}
#Wildfire flux of Carbon Monoxide
'Wildfire flux of Carbon Monoxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 81 ;
	}
#Wildfire flux of Methane
'Wildfire flux of Methane' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 82 ;
	}
#Wildfire flux of Non-Methane Hydro-Carbons
'Wildfire flux of Non-Methane Hydro-Carbons' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 83 ;
	}
#Wildfire flux of Hydrogen
'Wildfire flux of Hydrogen' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 84 ;
	}
#Wildfire flux of Nitrogen Oxides NOx
'Wildfire flux of Nitrogen Oxides NOx' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 85 ;
	}
#Wildfire flux of Nitrous Oxide
'Wildfire flux of Nitrous Oxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 86 ;
	}
#Wildfire flux of Particulate Matter PM2.5
'Wildfire flux of Particulate Matter PM2.5' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 87 ;
	}
#Wildfire flux of Total Particulate Matter
'Wildfire flux of Total Particulate Matter' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 88 ;
	}
#Wildfire flux of Total Carbon in Aerosols
'Wildfire flux of Total Carbon in Aerosols' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 89 ;
	}
#Wildfire flux of Organic Carbon
'Wildfire flux of Organic Carbon' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 90 ;
	}
#Wildfire flux of Black Carbon
'Wildfire flux of Black Carbon' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 91 ;
	}
#Wildfire overall flux of burnt Carbon
'Wildfire overall flux of burnt Carbon' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 92 ;
	}
#Wildfire fraction of C4 plants
'Wildfire fraction of C4 plants' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 93 ;
	}
#Wildfire vegetation map index
'Wildfire vegetation map index' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 94 ;
	}
#Wildfire Combustion Completeness
'Wildfire Combustion Completeness' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 95 ;
	}
#Wildfire Fuel Load: Carbon per unit area
'Wildfire Fuel Load: Carbon per unit area' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 96 ;
	}
#Wildfire fraction of area observed
'Wildfire fraction of area observed' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 97 ;
	}
#Wildfire observed area
'Wildfire observed area' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 98 ;
	}
#Wildfire radiative power
'Wildfire radiative power' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 99 ;
	}
#Wildfire combustion rate
'Wildfire combustion rate' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 100 ;
	}
#Nitrogen dioxide mass mixing ratio difference
'Nitrogen dioxide mass mixing ratio difference' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 121 ;
	}
#Sulphur dioxide mass mixing ratio difference
'Sulphur dioxide mass mixing ratio difference' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 122 ;
	}
#Carbon monoxide mass mixing ratio difference
'Carbon monoxide mass mixing ratio difference' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 123 ;
	}
#Formaldehyde
'Formaldehyde' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 124 ;
	}
#Total column Nitrogen dioxide
'Total column Nitrogen dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 125 ;
	}
#Total column Sulphur dioxide
'Total column Sulphur dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 126 ;
	}
#Total column Carbon monoxide
'Total column Carbon monoxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 127 ;
	}
#Total column Formaldehyde
'Total column Formaldehyde' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 128 ;
	}
#Nitrogen Oxides
'Nitrogen Oxides' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 129 ;
	}
#Total Column Nitrogen Oxides
'Total Column Nitrogen Oxides' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 130 ;
	}
#Reactive tracer 1 mass mixing ratio
'Reactive tracer 1 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 131 ;
	}
#Total column GRG tracer 1
'Total column GRG tracer 1' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 132 ;
	}
#Reactive tracer 2 mass mixing ratio
'Reactive tracer 2 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 133 ;
	}
#Total column GRG tracer 2
'Total column GRG tracer 2' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 134 ;
	}
#Reactive tracer 3 mass mixing ratio
'Reactive tracer 3 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 135 ;
	}
#Total column GRG tracer 3
'Total column GRG tracer 3' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 136 ;
	}
#Reactive tracer 4 mass mixing ratio
'Reactive tracer 4 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 137 ;
	}
#Total column GRG tracer 4
'Total column GRG tracer 4' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 138 ;
	}
#Reactive tracer 5 mass mixing ratio
'Reactive tracer 5 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 139 ;
	}
#Total column GRG tracer 5
'Total column GRG tracer 5' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 140 ;
	}
#Reactive tracer 6 mass mixing ratio
'Reactive tracer 6 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 141 ;
	}
#Total column GRG tracer 6
'Total column GRG tracer 6' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 142 ;
	}
#Reactive tracer 7 mass mixing ratio
'Reactive tracer 7 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 143 ;
	}
#Total column GRG tracer 7
'Total column GRG tracer 7' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 144 ;
	}
#Reactive tracer 8 mass mixing ratio
'Reactive tracer 8 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 145 ;
	}
#Total column GRG tracer 8
'Total column GRG tracer 8' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 146 ;
	}
#Reactive tracer 9 mass mixing ratio
'Reactive tracer 9 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 147 ;
	}
#Total column GRG tracer 9
'Total column GRG tracer 9' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 148 ;
	}
#Reactive tracer 10 mass mixing ratio
'Reactive tracer 10 mass mixing ratio' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 149 ;
	}
#Total column GRG tracer 10
'Total column GRG tracer 10' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 150 ;
	}
#Surface flux Nitrogen oxides
'Surface flux Nitrogen oxides' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 151 ;
	}
#Surface flux Nitrogen dioxide
'Surface flux Nitrogen dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 152 ;
	}
#Surface flux Sulphur dioxide
'Surface flux Sulphur dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 153 ;
	}
#Surface flux Carbon monoxide
'Surface flux Carbon monoxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 154 ;
	}
#Surface flux Formaldehyde
'Surface flux Formaldehyde' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 155 ;
	}
#Surface flux GEMS Ozone
'Surface flux GEMS Ozone' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 156 ;
	}
#Surface flux reactive tracer 1
'Surface flux reactive tracer 1' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 157 ;
	}
#Surface flux reactive tracer 2
'Surface flux reactive tracer 2' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 158 ;
	}
#Surface flux reactive tracer 3
'Surface flux reactive tracer 3' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 159 ;
	}
#Surface flux reactive tracer 4
'Surface flux reactive tracer 4' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 160 ;
	}
#Surface flux reactive tracer 5
'Surface flux reactive tracer 5' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 161 ;
	}
#Surface flux reactive tracer 6
'Surface flux reactive tracer 6' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 162 ;
	}
#Surface flux reactive tracer 7
'Surface flux reactive tracer 7' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 163 ;
	}
#Surface flux reactive tracer 8
'Surface flux reactive tracer 8' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 164 ;
	}
#Surface flux reactive tracer 9
'Surface flux reactive tracer 9' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 165 ;
	}
#Surface flux reactive tracer 10
'Surface flux reactive tracer 10' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 166 ;
	}
#Radon
'Radon' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 181 ;
	}
#Sulphur Hexafluoride
'Sulphur Hexafluoride' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 182 ;
	}
#Total column Radon
'Total column Radon' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 183 ;
	}
#Total column Sulphur Hexafluoride
'Total column Sulphur Hexafluoride' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 184 ;
	}
#Anthropogenic Emissions of Sulphur Hexafluoride
'Anthropogenic Emissions of Sulphur Hexafluoride' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 185 ;
	}
#Ozone mass mixing ratio difference (full chemistry scheme)
'Ozone mass mixing ratio difference (full chemistry scheme)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 203 ;
	}
#GEMS Total column ozone
'GEMS Total column ozone' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 206 ;
	}
#Total Aerosol Optical Depth at 550nm
'Total Aerosol Optical Depth at 550nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 207 ;
	}
#Sea Salt Aerosol Optical Depth at 550nm
'Sea Salt Aerosol Optical Depth at 550nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 208 ;
	}
#Dust Aerosol Optical Depth at 550nm
'Dust Aerosol Optical Depth at 550nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 209 ;
	}
#Organic Matter Aerosol Optical Depth at 550nm
'Organic Matter Aerosol Optical Depth at 550nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 210 ;
	}
#Black Carbon Aerosol Optical Depth at 550nm
'Black Carbon Aerosol Optical Depth at 550nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 211 ;
	}
#Sulphate Aerosol Optical Depth at 550nm
'Sulphate Aerosol Optical Depth at 550nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 212 ;
	}
#Total Aerosol Optical Depth at 469nm
'Total Aerosol Optical Depth at 469nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 213 ;
	}
#Total Aerosol Optical Depth at 670nm
'Total Aerosol Optical Depth at 670nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 214 ;
	}
#Total Aerosol Optical Depth at 865nm
'Total Aerosol Optical Depth at 865nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 215 ;
	}
#Total Aerosol Optical Depth at 1240nm
'Total Aerosol Optical Depth at 1240nm' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 216 ;
	}
#Total precipitation observation count
'Total precipitation observation count' = {
	 table2Version = 220 ;
	 indicatorOfParameter = 228 ;
	}
#Convective inhibition
'Convective inhibition' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 1 ;
	}
#Orography
'Orography' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 2 ;
	}
#Friction velocity
'Friction velocity' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 3 ;
	}
#Time-mean 2 metre temperature
'Time-mean 2 metre temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 4 ;
	}
#Time-mean 10 metre wind speed
'Time-mean 10 metre wind speed' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 5 ;
	}
#Mean total cloud cover
'Mean total cloud cover' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 6 ;
	}
#Lake total depth
'Lake total depth' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 7 ;
	}
#Lake mix-layer temperature
'Lake mix-layer temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 8 ;
	}
#Lake mix-layer depth
'Lake mix-layer depth' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 9 ;
	}
#Lake bottom temperature
'Lake bottom temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 10 ;
	}
#Lake total layer temperature
'Lake total layer temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 11 ;
	}
#Lake shape factor
'Lake shape factor' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 12 ;
	}
#Lake ice surface temperature
'Lake ice surface temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 13 ;
	}
#Lake ice total depth
'Lake ice total depth' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 14 ;
	}
#Minimum vertical gradient of refractivity inside trapping layer
'Minimum vertical gradient of refractivity inside trapping layer' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 15 ;
	}
#Mean vertical gradient of refractivity inside trapping layer
'Mean vertical gradient of refractivity inside trapping layer' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 16 ;
	}
#Duct base height
'Duct base height' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 17 ;
	}
#Trapping layer base height
'Trapping layer base height' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 18 ;
	}
#Trapping layer top height
'Trapping layer top height' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 19 ;
	}
#Soil moisture
'Soil moisture' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 39 ;
	}
#10 metre u-component of neutral wind
'10 metre u-component of neutral wind' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 131 ;
	}
#10 metre v-component of neutral wind
'10 metre v-component of neutral wind' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 132 ;
	}
#Soil temperature
'Soil temperature' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 139 ;
	}
#Snow depth water equivalent
'Snow depth water equivalent' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 141 ;
	}
#Snowfall water equivalent
'Snowfall water equivalent' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 144 ;
	}
#Total Cloud Cover
'Total Cloud Cover' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 164 ;
	}
#Field capacity
'Field capacity' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 170 ;
	}
#Wilting point
'Wilting point' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 171 ;
	}
#Total Precipitation
'Total Precipitation' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 228 ;
	}
#Snow evaporation (variable resolution)
'Snow evaporation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 44 ;
	}
#Snowmelt (variable resolution)
'Snowmelt (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 45 ;
	}
#Solar duration (variable resolution)
'Solar duration (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 46 ;
	}
#Downward UV radiation at the surface (variable resolution)
'Downward UV radiation at the surface (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 57 ;
	}
#Photosynthetically active radiation at the surface (variable resolution)
'Photosynthetically active radiation at the surface (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 58 ;
	}
#Stratiform precipitation (Large-scale precipitation) (variable resolution)
'Stratiform precipitation (Large-scale precipitation) (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 142 ;
	}
#Convective precipitation (variable resolution)
'Convective precipitation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 143 ;
	}
#Snowfall (convective + stratiform) (variable resolution)
'Snowfall (convective + stratiform) (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 144 ;
	}
#Boundary layer dissipation (variable resolution)
'Boundary layer dissipation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 145 ;
	}
#Surface sensible heat flux (variable resolution)
'Surface sensible heat flux (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 146 ;
	}
#Surface latent heat flux (variable resolution)
'Surface latent heat flux (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 147 ;
	}
#Surface solar radiation downwards (variable resolution)
'Surface solar radiation downwards (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 169 ;
	}
#Surface thermal radiation downwards (variable resolution)
'Surface thermal radiation downwards (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 175 ;
	}
#Surface net solar radiation (variable resolution)
'Surface net solar radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 176 ;
	}
#Surface net thermal radiation (variable resolution)
'Surface net thermal radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 177 ;
	}
#Top net solar radiation (variable resolution)
'Top net solar radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 178 ;
	}
#Top net thermal radiation (variable resolution)
'Top net thermal radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 179 ;
	}
#East-West surface stress (variable resolution)
'East-West surface stress (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 180 ;
	}
#North-South surface stress (variable resolution)
'North-South surface stress (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 181 ;
	}
#Evaporation (variable resolution)
'Evaporation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 182 ;
	}
#Sunshine duration (variable resolution)
'Sunshine duration (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 189 ;
	}
#Longitudinal component of gravity wave stress (variable resolution)
'Longitudinal component of gravity wave stress (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 195 ;
	}
#Meridional component of gravity wave stress (variable resolution)
'Meridional component of gravity wave stress (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 196 ;
	}
#Gravity wave dissipation (variable resolution)
'Gravity wave dissipation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 197 ;
	}
#Skin reservoir content (variable resolution)
'Skin reservoir content (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 198 ;
	}
#Runoff (variable resolution)
'Runoff (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 205 ;
	}
#Top net solar radiation, clear sky (variable resolution)
'Top net solar radiation, clear sky (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 208 ;
	}
#Top net thermal radiation, clear sky (variable resolution)
'Top net thermal radiation, clear sky (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 209 ;
	}
#Surface net solar radiation, clear sky (variable resolution)
'Surface net solar radiation, clear sky (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 210 ;
	}
#Surface net thermal radiation, clear sky (variable resolution)
'Surface net thermal radiation, clear sky (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 211 ;
	}
#TOA incident solar radiation (variable resolution)
'TOA incident solar radiation (variable resolution)' = {
	 table2Version = 230 ;
	 indicatorOfParameter = 212 ;
	}
#Surface temperature significance
'Surface temperature significance' = {
	 table2Version = 234 ;
	 indicatorOfParameter = 139 ;
	}
#Mean sea level pressure significance
'Mean sea level pressure significance' = {
	 table2Version = 234 ;
	 indicatorOfParameter = 151 ;
	}
#2 metre temperature significance
'2 metre temperature significance' = {
	 table2Version = 234 ;
	 indicatorOfParameter = 167 ;
	}
#Total precipitation significance
'Total precipitation significance' = {
	 table2Version = 234 ;
	 indicatorOfParameter = 228 ;
	}
#U-component surface stokes drift
'U-component surface stokes drift' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 215 ;
	}
#V-component surface stokes drift
'V-component surface stokes drift' = {
	 table2Version = 140 ;
	 indicatorOfParameter = 216 ;
	}
#Wildfire radiative power maximum
'Wildfire radiative power maximum' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 101 ;
	}
#Wildfire flux of Sulfur Dioxide
'Wildfire flux of Sulfur Dioxide' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 102 ;
	}
#Wildfire Flux of Methanol (CH3OH)
'Wildfire Flux of Methanol (CH3OH)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 103 ;
	}
#Wildfire Flux of Ethanol (C2H5OH)
'Wildfire Flux of Ethanol (C2H5OH)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 104 ;
	}
#Wildfire Flux of Propane (C3H8)
'Wildfire Flux of Propane (C3H8)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 105 ;
	}
#Wildfire Flux of Ethene (C2H4)
'Wildfire Flux of Ethene (C2H4)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 106 ;
	}
#Wildfire Flux of Propene (C3H6)
'Wildfire Flux of Propene (C3H6)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 107 ;
	}
#Wildfire Flux of Isoprene (C5H8)
'Wildfire Flux of Isoprene (C5H8)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 108 ;
	}
#Wildfire Flux of Terpenes (C5H8)n
'Wildfire Flux of Terpenes (C5H8)n' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 109 ;
	}
#Wildfire Flux of Toluene_lump (C7H8+ C6H6 + C8H10)
'Wildfire Flux of Toluene_lump (C7H8+ C6H6 + C8H10)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 110 ;
	}
#Wildfire Flux of Higher Alkenes (CnH2n, C>=4)
'Wildfire Flux of Higher Alkenes (CnH2n, C>=4)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 111 ;
	}
#Wildfire Flux of Higher Alkanes (CnH2n+2, C>=4)
'Wildfire Flux of Higher Alkanes (CnH2n+2, C>=4)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 112 ;
	}
#Wildfire Flux of Formaldehyde (CH2O)
'Wildfire Flux of Formaldehyde (CH2O)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 113 ;
	}
#Wildfire Flux of Acetaldehyde (C2H4O)
'Wildfire Flux of Acetaldehyde (C2H4O)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 114 ;
	}
#Wildfire Flux of Acetone (C3H6O)
'Wildfire Flux of Acetone (C3H6O)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 115 ;
	}
#Wildfire Flux of Ammonia (NH3)
'Wildfire Flux of Ammonia (NH3)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 116 ;
	}
#Wildfire Flux of Dimethyl Sulfide (DMS) (C2H6S)
'Wildfire Flux of Dimethyl Sulfide (DMS) (C2H6S)' = {
	 table2Version = 210 ;
	 indicatorOfParameter = 117 ;
	}
#Wildfire radiative power maximum
'Wildfire radiative power maximum' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 101 ;
	}
#Wildfire flux of Sulfur Dioxide
'Wildfire flux of Sulfur Dioxide' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 102 ;
	}
#Wildfire Flux of Methanol (CH3OH)
'Wildfire Flux of Methanol (CH3OH)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 103 ;
	}
#Wildfire Flux of Ethanol (C2H5OH)
'Wildfire Flux of Ethanol (C2H5OH)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 104 ;
	}
#Wildfire Flux of Propane (C3H8)
'Wildfire Flux of Propane (C3H8)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 105 ;
	}
#Wildfire Flux of Ethene (C2H4)
'Wildfire Flux of Ethene (C2H4)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 106 ;
	}
#Wildfire Flux of Propene (C3H6)
'Wildfire Flux of Propene (C3H6)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 107 ;
	}
#Wildfire Flux of Isoprene (C5H8)
'Wildfire Flux of Isoprene (C5H8)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 108 ;
	}
#Wildfire Flux of Terpenes (C5H8)n
'Wildfire Flux of Terpenes (C5H8)n' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 109 ;
	}
#Wildfire Flux of Toluene_lump (C7H8+ C6H6 + C8H10)
'Wildfire Flux of Toluene_lump (C7H8+ C6H6 + C8H10)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 110 ;
	}
#Wildfire Flux of Higher Alkenes (CnH2n, C>=4)
'Wildfire Flux of Higher Alkenes (CnH2n, C>=4)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 111 ;
	}
#Wildfire Flux of Higher Alkanes (CnH2n+2, C>=4)
'Wildfire Flux of Higher Alkanes (CnH2n+2, C>=4)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 112 ;
	}
#Wildfire Flux of Formaldehyde (CH2O)
'Wildfire Flux of Formaldehyde (CH2O)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 113 ;
	}
#Wildfire Flux of Acetaldehyde (C2H4O)
'Wildfire Flux of Acetaldehyde (C2H4O)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 114 ;
	}
#Wildfire Flux of Acetone (C3H6O)
'Wildfire Flux of Acetone (C3H6O)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 115 ;
	}
#Wildfire Flux of Ammonia (NH3)
'Wildfire Flux of Ammonia (NH3)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 116 ;
	}
#Wildfire Flux of Dimethyl Sulfide (DMS) (C2H6S)
'Wildfire Flux of Dimethyl Sulfide (DMS) (C2H6S)' = {
	 table2Version = 211 ;
	 indicatorOfParameter = 117 ;
	}
#V-tendency from non-orographic wave drag
'V-tendency from non-orographic wave drag' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 134 ;
	}
#U-tendency from non-orographic wave drag
'U-tendency from non-orographic wave drag' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 136 ;
	}
#100 metre U wind component
'100 metre U wind component' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 246 ;
	}
#100 metre V wind component
'100 metre V wind component' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 247 ;
	}
#ASCAT first soil moisture CDF matching parameter
'ASCAT first soil moisture CDF matching parameter' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 253 ;
	}
#ASCAT second soil moisture CDF matching parameter
'ASCAT second soil moisture CDF matching parameter' = {
	 table2Version = 228 ;
	 indicatorOfParameter = 254 ;
}
